Package: mlumr
Title: Multilevel Unanchored Meta-Regression for Indirect Treatment
        Comparisons
Version: 0.1.0
Authors@R: c(
    person("Ahmad", "Sofi-Mahmudi", , "a.sofimahmudi@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-6829-0823")),
    person("Conor", "Chandler", role = "aut",
           comment = c(ORCID = "0000-0002-1365-9002")))
Description: Bayesian multilevel unanchored meta-regression (ML-UMR) for
    indirect treatment comparisons using individual patient data (IPD) and
    aggregate data (AgD). Implements shared prognostic factor assumption
    (SPFA) and relaxed SPFA models for binary, continuous, and count
    outcomes via 'Stan'. Also
    provides simulated treatment comparison (STC) via parametric
    G-computation and naive unadjusted benchmarks. ML-UMR is an
    adaptation of the ML-NMR methodology (Phillippo et al. 2020,
    <doi:10.1111/rssa.12579>) implemented in the 'multinma' package
    (GPL-3) to the unanchored two-trial case; the public API
    deliberately mirrors multinma's so users can move between ML-NMR
    and ML-UMR with the same workflow.
License: GPL-3
URL: https://github.com/choxos/mlumr, https://choxos.github.io/mlumr/
BugReports: https://github.com/choxos/mlumr/issues
Encoding: UTF-8
Biarch: true
Depends: R (>= 4.1.0)
Imports: methods, stats, Rcpp (>= 0.12.0), RcppParallel (>= 5.0.1),
        rstan (>= 2.26.0), rstantools (>= 2.3.0), randtoolbox, copula
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.26.0), StanHeaders (>=
        2.26.0)
Suggests: testthat (>= 3.0.0), knitr, rmarkdown, posterior, bayesplot,
        loo, Matrix, cmdstanr, withr
Additional_repositories: https://stan-dev.r-universe.dev
SystemRequirements: GNU make
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-15 19:14:56 UTC; choxos
Author: Ahmad Sofi-Mahmudi [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6829-0823>),
  Conor Chandler [aut] (ORCID: <https://orcid.org/0000-0002-1365-9002>)
Maintainer: Ahmad Sofi-Mahmudi <a.sofimahmudi@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-20 08:30:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-25 04:28:53 UTC; unix
Archs: mlumr.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(predict,mlumr_fit)
S3method(print,mlumr_data)
S3method(print,mlumr_dic)
S3method(print,mlumr_fit)
S3method(print,mlumr_naive)
S3method(print,mlumr_stc)
S3method(prior_summary,default)
S3method(prior_summary,mlumr_fit)
S3method(summary,mlumr_fit)
S3method(summary,mlumr_naive)
S3method(summary,mlumr_stc)
export(add_integration)
export(calculate_dic)
export(calculate_loo)
export(calculate_waic)
export(check_integration)
export(combine_data)
export(compare_models)
export(conditional_effects)
export(conditional_predict)
export(dbern)
export(default_prior_beta)
export(default_prior_intercept)
export(default_prior_sigma)
export(distr)
export(marginal_effects)
export(mlumr)
export(mlumr_engine)
export(naive)
export(pbern)
export(prior_cauchy)
export(prior_exponential)
export(prior_normal)
export(prior_sensitivity)
export(prior_student_t)
export(prior_summary)
export(qbern)
export(set_agd)
export(set_ipd)
export(stc)
export(unnest_integration)
import(Rcpp)
import(methods)
importFrom(RcppParallel,RcppParallelLibs)
importFrom(rstantools,rstan_config)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,dbinom)
importFrom(stats,dnorm)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,pbinom)
importFrom(stats,plogis)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,qbinom)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
useDynLib(mlumr, .registration = TRUE)

## ----eval = TRUE--------------------------------------------------------------
library(mlumr)
set.seed(2026)

# Example: Trial A data (index treatment)
trial_a <- data.frame(
  patient_id = 1:500,
  treatment = "Drug_A",
  response = rbinom(500, 1, 0.6),
  age_group = rbinom(500, 1, 0.4),   # 0 = young, 1 = old
  sex = rbinom(500, 1, 0.55)          # 0 = male, 1 = female
)

ipd <- set_ipd(
  data = trial_a,
  treatment = "treatment",
  outcome = "response",
  covariates = c("age_group", "sex")
)

ipd

## ----eval = TRUE--------------------------------------------------------------
trial_a_normal <- data.frame(
  patient_id = 1:500,
  treatment = "Drug_A",
  score = rnorm(500, mean = 3.0, sd = 1.2),
  age_group = rbinom(500, 1, 0.4),
  sex = rbinom(500, 1, 0.55)
)

ipd_normal <- set_ipd(
  data = trial_a_normal,
  treatment = "treatment",
  outcome = "score",
  covariates = c("age_group", "sex"),
  family = "normal"
)

## ----eval = TRUE--------------------------------------------------------------
trial_a_poisson <- data.frame(
  patient_id = 1:500,
  treatment = "Drug_A",
  events = rpois(500, lambda = 0.8),
  person_years = runif(500, 0.5, 2.0),
  age_group = rbinom(500, 1, 0.4),
  sex = rbinom(500, 1, 0.55)
)

ipd_poisson <- set_ipd(
  data = trial_a_poisson,
  treatment = "treatment",
  outcome = "events",
  covariates = c("age_group", "sex"),
  family = "poisson",
  exposure = "person_years"
)

## ----eval = TRUE--------------------------------------------------------------
# Example: Trial B data (comparator treatment)
trial_b <- data.frame(
  study = "Trial_B",
  treatment = "Drug_B",
  n_total = 400,
  n_events = 160,
  age_group_mean = 0.35,   # proportion in "old" group
  sex_prop = 0.50           # proportion female
)

agd <- set_agd(
  data = trial_b,
  treatment = "treatment",
  outcome_n = "n_total",
  outcome_r = "n_events",
  cov_means = c("age_group_mean", "sex_prop"),
  cov_types = c("binary", "binary")
)

## ----eval = TRUE--------------------------------------------------------------
agd_normal <- set_agd(
  data = data.frame(
    trt = "Drug_B", y_mean = 3.2, se = 0.15, n = 400,
    age_group_mean = 0.35, sex_prop = 0.50
  ),
  treatment = "trt",
  family = "normal",
  outcome_mean = "y_mean",
  outcome_se = "se",
  outcome_n = "n",
  cov_means = c("age_group_mean", "sex_prop"),
  cov_types = c("binary", "binary")
)

## ----eval = TRUE--------------------------------------------------------------
agd_poisson <- set_agd(
  data = data.frame(
    trt = "Drug_B", n_events = 120, person_years = 800,
    age_group_mean = 0.35, sex_prop = 0.50
  ),
  treatment = "trt",
  family = "poisson",
  outcome_r = "n_events",
  outcome_E = "person_years",
  cov_means = c("age_group_mean", "sex_prop"),
  cov_types = c("binary", "binary")
)

## ----eval = TRUE--------------------------------------------------------------
agd_continuous <- set_agd(
  data = data.frame(
    trt = "B", n_total = 400, n_events = 160,
    bmi_mean = 25.3, bmi_sd = 4.2
  ),
  treatment = "trt",
  outcome_n = "n_total",
  outcome_r = "n_events",
  cov_means = "bmi_mean",
  cov_sds = "bmi_sd",
  cov_types = "continuous"
)

## ----eval = TRUE--------------------------------------------------------------
dat <- combine_data(ipd, agd)
print(dat)

## ----eval = TRUE--------------------------------------------------------------
dat <- add_integration(
  dat,
  n_int = 64,
  age_group = distr(qbern, prob = age_group_mean),
  sex = distr(qbern, prob = sex_mean)
)

## ----eval = TRUE--------------------------------------------------------------
# Default: Spearman correlation from IPD
dat <- add_integration(dat, n_int = 64,
                       age_group = distr(qbern, prob = age_group_mean),
                       sex = distr(qbern, prob = sex_mean))

# Supply your own correlation matrix
my_cor <- matrix(c(1, 0.3, 0.3, 1), 2, 2)
dat <- add_integration(dat, n_int = 64, cor = my_cor,
                       age_group = distr(qbern, prob = age_group_mean),
                       sex = distr(qbern, prob = sex_mean))

# No correlation adjustment
dat <- add_integration(dat, n_int = 64, cor_adjust = "none",
                       age_group = distr(qbern, prob = age_group_mean),
                       sex = distr(qbern, prob = sex_mean))

## ----eval = TRUE--------------------------------------------------------------
# Expand to long format
int_df <- unnest_integration(dat)
head(int_df)

## ----eval = TRUE--------------------------------------------------------------
# These work without add_integration()
dat_no_int <- combine_data(ipd, agd)
naive_result <- naive(dat_no_int)
stc_result <- stc(dat_no_int)

## ----eval = TRUE--------------------------------------------------------------
library(mlumr)
set.seed(2026)

# Toy IPD: trial A (index treatment, binary outcome)
n_a <- 300
trial_a_data <- data.frame(
  trt      = "Drug_A",
  response = rbinom(n_a, 1, 0.55),
  age_cat  = rbinom(n_a, 1, 0.40),
  sex      = rbinom(n_a, 1, 0.55)
)

# Toy AgD: trial B (comparator treatment)
trial_b_data <- data.frame(
  trt           = "Drug_B",
  n_total       = 400,
  n_events      = 160,
  age_cat_mean  = 0.35,
  sex_mean      = 0.50
)

## ----eval = TRUE--------------------------------------------------------------
# 1. Prepare IPD
ipd <- set_ipd(
  data = trial_a_data,
  treatment = "trt",
  outcome = "response",
  covariates = c("age_cat", "sex")
)

# 2. Prepare AgD
agd <- set_agd(
  data = trial_b_data,
  treatment = "trt",
  outcome_n = "n_total",
  outcome_r = "n_events",
  cov_means = c("age_cat_mean", "sex_mean"),
  cov_types = c("binary", "binary")
)

# 3. Combine
dat <- combine_data(ipd, agd)

# 4. Add integration points (needed for ML-UMR)
dat <- add_integration(
  dat,
  n_int = 64,
  age_cat = distr(qbern, prob = age_cat_mean),
  sex = distr(qbern, prob = sex_mean)
)

#' Set up individual patient data (IPD)
#'
#' Prepare IPD from the index treatment for an unanchored indirect comparison.
#'
#' @param data Data frame containing IPD
#' @param treatment Column name for treatment variable
#' @param outcome Column name for outcome variable. For `family = "binomial"`,
#'   must be binary (0/1). For `family = "normal"`, any numeric. For
#'   `family = "poisson"`, non-negative integer counts.
#' @param covariates Character vector of covariate column names
#' @param family Outcome family: `"binomial"`, `"normal"`, or `"poisson"`
#' @param exposure Column name for exposure/time-at-risk (required when
#'   `family = "poisson"`)
#' @param study Column name for study identifier (optional)
#'
#' @return An object of class `mlumr_ipd`
#' @export
#'
#' @examples
#' \dontrun{
#' # Binary outcome
#' ipd <- set_ipd(
#'   data = trial_a,
#'   treatment = "trt",
#'   outcome = "response",
#'   covariates = c("age", "sex")
#' )
#'
#' # Continuous outcome
#' ipd <- set_ipd(
#'   data = trial_a,
#'   treatment = "trt",
#'   outcome = "score",
#'   covariates = c("age", "sex"),
#'   family = "normal"
#' )
#'
#' # Count outcome with exposure
#' ipd <- set_ipd(
#'   data = trial_a,
#'   treatment = "trt",
#'   outcome = "events",
#'   covariates = c("age", "sex"),
#'   family = "poisson",
#'   exposure = "person_years"
#' )
#' }
set_ipd <- function(data, treatment, outcome, covariates,
                    family = c("binomial", "normal", "poisson"),
                    exposure = NULL, study = NULL) {

  family <- match.arg(family)

  if (!is.data.frame(data)) {
    stop("`data` must be a data frame", call. = FALSE)
  }
  .validate_non_empty_data(data, "IPD")
  .validate_required_covariates(covariates, "covariates")

  required_cols <- .ipd_required_columns(treatment, outcome, covariates,
                                         exposure, study)

  .check_required_columns(data, required_cols)
  .validate_ipd_outcome(data, outcome, family, exposure)
  .validate_reserved_internal_names(
    covariates,
    c(".study", ".trt", ".outcome", ".exposure"),
    "Covariate name(s)"
  )
  .validate_ipd_covariates(data, covariates)
  .validate_ipd_finite_columns(data, outcome, exposure, family)

  data <- .drop_missing_rows(data, required_cols)
  .validate_complete_rows_remain(data, "IPD")
  .warn_constant_ipd_covariates(data, covariates)
  .validate_single_treatment(data, treatment, "IPD")
  ipd_data <- .standardize_ipd_data(data, treatment, outcome, covariates,
                                    family, exposure, study)

  out <- c(
    list(
      data = ipd_data,
      n = nrow(ipd_data),
      treatment = unique(ipd_data$.trt),
      covariates = covariates,
      family = family,
      type = "ipd"
    ),
    .ipd_outcome_summary(ipd_data, family)
  )

  class(out) <- c("mlumr_ipd", "list")
  out
}

#' Required source columns for IPD setup
#' @keywords internal
.ipd_required_columns <- function(treatment, outcome, covariates,
                                  exposure = NULL, study = NULL) {
  cols <- c(treatment, outcome, covariates)
  if (!is.null(exposure)) cols <- c(cols, exposure)
  if (!is.null(study)) cols <- c(cols, study)
  cols
}

#' Validate that required columns are present
#' @keywords internal
.check_required_columns <- function(data, required_cols) {
  missing_cols <- setdiff(required_cols, names(data))
  if (length(missing_cols) > 0L) {
    stop(sprintf("Missing columns in data: %s",
                 paste(missing_cols, collapse = ", ")), call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate that input data contain at least one row
#' @keywords internal
.validate_non_empty_data <- function(data, label) {
  if (nrow(data) == 0L) {
    stop(sprintf("%s data must contain at least one row", label),
         call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate covariate argument shape before column lookup
#' @keywords internal
.validate_required_covariates <- function(covariates, arg_name) {
  if (!is.character(covariates) || length(covariates) == 0L) {
    stop(sprintf("`%s` must be a non-empty character vector", arg_name),
         call. = FALSE)
  }
  if (any(is.na(covariates)) || any(covariates == "")) {
    stop(sprintf("`%s` must not contain missing or empty names", arg_name),
         call. = FALSE)
  }
  dup_covariates <- unique(covariates[duplicated(covariates)])
  if (length(dup_covariates) > 0L) {
    stop(sprintf("Duplicate covariates: %s",
                 paste(dup_covariates, collapse = ", ")), call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate IPD outcome and exposure columns
#' @keywords internal
.validate_ipd_outcome <- function(data, outcome, family, exposure = NULL) {
  if (family == "binomial") {
    outcome_vals <- data[[outcome]]
    if (!is.numeric(outcome_vals) && !is.logical(outcome_vals)) {
      stop("`outcome` must be numeric or logical 0/1 for binomial family",
           call. = FALSE)
    }
    invalid <- !is.na(outcome_vals) & !(outcome_vals %in% c(0, 1))
    if (any(invalid)) {
      stop("`outcome` must be binary (0/1) for binomial family", call. = FALSE)
    }
  } else if (family == "normal") {
    if (!is.numeric(data[[outcome]])) {
      stop("`outcome` must be numeric for normal family", call. = FALSE)
    }
  } else {
    if (is.null(exposure)) {
      stop("`exposure` is required for poisson family", call. = FALSE)
    }
    outcome_vals <- data[[outcome]]
    if (!.is_whole_number_count(outcome_vals, allow_missing = TRUE)) {
      stop("`outcome` must be non-negative integer counts for poisson family",
           call. = FALSE)
    }
    exposure_vals <- data[[exposure]]
    if (!is.numeric(exposure_vals) || any(exposure_vals <= 0, na.rm = TRUE)) {
      stop("`exposure` must be positive numeric values", call. = FALSE)
    }
  }
  invisible(TRUE)
}

#' Validate a data source contains only one treatment
#' @keywords internal
.validate_single_treatment <- function(data, treatment, label) {
  trt_vals <- unique(data[[treatment]])
  if (length(trt_vals) > 1L) {
    stop(sprintf("%s should contain a single treatment. Found: %s",
                 label, paste(trt_vals, collapse = ", ")), call. = FALSE)
  }
  invisible(TRUE)
}

#' Reject user names that would overwrite standardized internal columns
#' @keywords internal
.validate_reserved_internal_names <- function(user_names, reserved, label) {
  reserved_hit <- intersect(user_names, reserved)
  if (length(reserved_hit) > 0L) {
    msg <- sprintf(
      "%s collide with reserved internal columns: %s. Please rename and retry.",
      label,
      paste(reserved_hit, collapse = ", ")
    )
    stop(msg, call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate IPD covariates
#' @keywords internal
.validate_ipd_covariates <- function(data, covariates) {
  for (cov in covariates) {
    if (!is.numeric(data[[cov]])) {
      stop(sprintf("Covariate '%s' must be numeric", cov), call. = FALSE)
    }
    vals <- data[[cov]][!is.na(data[[cov]])]
    if (any(!is.finite(vals))) {
      stop(sprintf("Covariate '%s' contains non-finite values (Inf, -Inf, or NaN)", cov),
           call. = FALSE)
    }
  }
  invisible(TRUE)
}

#' Validate finite IPD outcome and exposure values
#' @keywords internal
.validate_ipd_finite_columns <- function(data, outcome, exposure, family) {
  outcome_nona <- data[[outcome]][!is.na(data[[outcome]])]
  if (any(!is.finite(outcome_nona))) {
    stop("`outcome` contains non-finite values", call. = FALSE)
  }
  if (family == "poisson") {
    exposure_nona <- data[[exposure]][!is.na(data[[exposure]])]
    if (any(!is.finite(exposure_nona))) {
      stop("`exposure` contains non-finite values", call. = FALSE)
    }
  }
  invisible(TRUE)
}

#' Drop rows with missing setup inputs
#' @keywords internal
.drop_missing_rows <- function(data, required_cols) {
  complete <- complete.cases(data[, required_cols])
  if (!all(complete)) {
    n_missing <- sum(!complete)
    warning(sprintf("%d rows with missing values will be excluded", n_missing),
            call. = FALSE)
    data <- data[complete, ]
  }
  data
}

#' Validate that complete-case filtering left rows to analyze
#' @keywords internal
.validate_complete_rows_remain <- function(data, label) {
  if (nrow(data) == 0L) {
    stop(sprintf("No complete %s rows remain after excluding missing values",
                 label), call. = FALSE)
  }
  invisible(TRUE)
}

#' Warn when IPD covariates have no empirical variation
#' @keywords internal
.warn_constant_ipd_covariates <- function(data, covariates) {
  constant <- vapply(covariates, function(cov) {
    length(unique(data[[cov]])) <= 1L
  }, logical(1))

  if (any(constant)) {
    warning(
      paste(
        "IPD covariate(s) have zero empirical variation after missing-row",
        "filtering:",
        paste(covariates[constant], collapse = ", "),
        "Regression coefficients for constant covariates are not identified",
        "from the IPD; consider removing them or using informative priors."
      ),
      call. = FALSE
    )
  }
  invisible(TRUE)
}

#' Standardize IPD to mlumr's internal column contract
#' @keywords internal
.standardize_ipd_data <- function(data, treatment, outcome, covariates,
                                  family, exposure = NULL, study = NULL) {
  ipd_data <- data.frame(
    .study = if (!is.null(study)) data[[study]] else "IPD_Study",
    .trt = data[[treatment]],
    stringsAsFactors = FALSE
  )

  if (family == "normal") {
    ipd_data$.outcome <- as.numeric(data[[outcome]])
  } else {
    ipd_data$.outcome <- as.integer(data[[outcome]])
  }
  if (family == "poisson") {
    ipd_data$.exposure <- as.numeric(data[[exposure]])
  }

  for (cov in covariates) {
    ipd_data[[cov]] <- data[[cov]]
  }
  ipd_data
}

#' Summarize standardized IPD outcomes
#' @keywords internal
.ipd_outcome_summary <- function(ipd_data, family) {
  switch(family,
    binomial = list(n_events = sum(ipd_data$.outcome)),
    normal = list(mean_outcome = mean(ipd_data$.outcome),
                  sd_outcome = sd(ipd_data$.outcome)),
    poisson = list(total_events = sum(ipd_data$.outcome),
                   total_exposure = sum(ipd_data$.exposure))
  )
}


#' Set up aggregate data (AgD)
#'
#' Prepare AgD from the comparator treatment for an unanchored indirect
#' comparison.
#'
#' @param data Data frame containing AgD summary statistics
#' @param treatment Column name for treatment variable
#' @param family Outcome family: `"binomial"`, `"normal"`, or `"poisson"`
#' @param outcome_n Column name for sample size (required for binomial,
#'   optional for normal)
#' @param outcome_r Column name for number of events (required for binomial
#'   and poisson)
#' @param outcome_mean Column name for mean outcome (required for normal)
#' @param outcome_se Column name for standard error of outcome (required for
#'   normal)
#' @param outcome_E Column name for total exposure (required for poisson)
#' @param cov_means Character vector of column names for covariate means/proportions
#' @param cov_sds Character vector of column names for covariate SDs
#'   (`NA` for binary covariates)
#' @param cov_types Character vector specifying `"continuous"` or `"binary"` for
#'   each covariate. If `NULL`, inferred from presence of SD.
#' @param study Column name for study identifier (optional)
#'
#' @details
#' **Scale assumptions for `family = "normal"`.** The AgD likelihood is
#' `y_agd ~ normal(E[exp(eta)], se_agd)` under `link = "log"` and
#' `y_agd ~ normal(E[eta], se_agd)` under `link = "identity"`. In both
#' cases `outcome_mean` and `outcome_se` must be on the **arithmetic
#' (original, untransformed) scale**. If a publication reports only a
#' log-scale mean / SD or a geometric mean, back-transform before
#' calling `set_agd()` and propagate uncertainty via the delta method;
#' passing log-scale or geometric summaries silently misspecifies the
#' likelihood and biases the posterior.
#'
#' **Scale assumptions for `family = "poisson"`.** `outcome_r` is the
#' total count in each AgD row and `outcome_E` is the total
#' person-time (or other exposure). The Stan likelihood uses
#' `log(E_agd)` as an offset, so rates are modeled on the log scale
#' regardless of how `outcome_r` is tabulated.
#'
#' **Scale assumptions for `family = "binomial"`.** `outcome_r` /
#' `outcome_n` are counts of events and trials. The log-odds (or
#' probit / cloglog under alternative links) are formed from
#' `outcome_r / outcome_n`, so no scale conversion is required.
#'
#' @return An object of class `mlumr_agd`
#' @export
#'
#' @examples
#' \dontrun{
#' # Binary outcome
#' agd <- set_agd(
#'   data = trial_b,
#'   treatment = "trt",
#'   outcome_n = "n_total",
#'   outcome_r = "n_events",
#'   cov_means = c("age_mean", "sex_prop"),
#'   cov_sds = c("age_sd", NA),
#'   cov_types = c("continuous", "binary")
#' )
#'
#' # Continuous outcome
#' agd <- set_agd(
#'   data = trial_b,
#'   treatment = "trt",
#'   family = "normal",
#'   outcome_mean = "mean_score",
#'   outcome_se = "se_score",
#'   outcome_n = "n_total",
#'   cov_means = c("age_mean", "sex_prop")
#' )
#'
#' # Count outcome
#' agd <- set_agd(
#'   data = trial_b,
#'   treatment = "trt",
#'   family = "poisson",
#'   outcome_r = "n_events",
#'   outcome_E = "person_years",
#'   cov_means = c("age_mean", "sex_prop")
#' )
#' }
set_agd <- function(data, treatment,
                    family = c("binomial", "normal", "poisson"),
                    outcome_n = NULL, outcome_r = NULL,
                    outcome_mean = NULL, outcome_se = NULL,
                    outcome_E = NULL,
                    cov_means, cov_sds = NULL, cov_types = NULL,
                    study = NULL) {

  family <- match.arg(family)

  if (!is.data.frame(data)) {
    stop("`data` must be a data frame", call. = FALSE)
  }
  .validate_non_empty_data(data, "AgD")
  .validate_required_covariates(cov_means, "cov_means")

  .validate_agd_outcome_args(family, outcome_n, outcome_r, outcome_mean,
                             outcome_se, outcome_E)
  spec <- .agd_covariate_spec(cov_means, cov_sds, cov_types)
  cov_sds <- spec$cov_sds
  cov_types <- spec$cov_types
  cov_names <- spec$cov_names

  required_cols <- .agd_required_columns(
    treatment = treatment,
    cov_means = cov_means,
    cov_sds = cov_sds,
    outcome_n = outcome_n,
    outcome_r = outcome_r,
    outcome_mean = outcome_mean,
    outcome_se = outcome_se,
    outcome_E = outcome_E,
    study = study
  )

  .check_required_columns(data, required_cols)
  .validate_single_treatment(data, treatment, "AgD")
  .validate_reserved_internal_names(
    c(cov_means, cov_sds[!is.na(cov_sds)]),
    c(".study", ".trt", ".n", ".r", ".y", ".se", ".E"),
    "Column name(s)"
  )
  .validate_agd_covariate_names(cov_means)
  .validate_agd_outcomes(data, family, outcome_n, outcome_r, outcome_mean,
                         outcome_se, outcome_E)
  .validate_agd_covariates(data, cov_means, cov_sds)
  .validate_agd_cov_types(cov_types)
  .validate_agd_binary_covariates(data, cov_means, cov_sds, cov_types)

  agd_data <- .standardize_agd_data(
    data = data,
    treatment = treatment,
    family = family,
    outcome_n = outcome_n,
    outcome_r = outcome_r,
    outcome_mean = outcome_mean,
    outcome_se = outcome_se,
    outcome_E = outcome_E,
    cov_means = cov_means,
    cov_sds = cov_sds,
    cov_names = cov_names,
    study = study
  )
  cov_info <- .agd_cov_info(cov_names, cov_sds, cov_types)

  out <- c(
    list(
      data = agd_data,
      treatment = unique(agd_data$.trt),
      covariates = cov_names,
      cov_info = cov_info,
      family = family,
      type = "agd"
    ),
    .agd_outcome_summary(data, family, outcome_n, outcome_r,
                         outcome_mean, outcome_se, outcome_E)
  )

  class(out) <- c("mlumr_agd", "list")
  out
}

#' Validate AgD outcome column arguments
#' @keywords internal
.validate_agd_outcome_args <- function(family, outcome_n, outcome_r,
                                       outcome_mean, outcome_se, outcome_E) {
  if (family == "binomial" && (is.null(outcome_n) || is.null(outcome_r))) {
    stop("`outcome_n` and `outcome_r` are required for binomial family",
         call. = FALSE)
  }
  if (family == "normal" && (is.null(outcome_mean) || is.null(outcome_se))) {
    stop("`outcome_mean` and `outcome_se` are required for normal family",
         call. = FALSE)
  }
  if (family == "poisson" && (is.null(outcome_r) || is.null(outcome_E))) {
    stop("`outcome_r` and `outcome_E` are required for poisson family",
         call. = FALSE)
  }
  invisible(TRUE)
}

#' Normalize AgD covariate specification
#' @keywords internal
.agd_covariate_spec <- function(cov_means, cov_sds = NULL, cov_types = NULL) {
  n_cov <- length(cov_means)

  if (is.null(cov_sds)) {
    cov_sds <- rep(NA_character_, n_cov)
  } else if (length(cov_sds) != n_cov) {
    stop("`cov_sds` must have same length as `cov_means` or be NULL",
         call. = FALSE)
  }

  if (is.null(cov_types)) {
    cov_types <- ifelse(is.na(cov_sds), "binary", "continuous")
  } else if (length(cov_types) != n_cov) {
    stop("`cov_types` must have same length as `cov_means` or be NULL",
         call. = FALSE)
  }

  list(
    cov_sds = cov_sds,
    cov_types = cov_types,
    cov_names = .strip_agd_cov_suffix(cov_means)
  )
}

#' AgD source columns required for setup
#' @keywords internal
.agd_required_columns <- function(treatment, cov_means, cov_sds,
                                  outcome_n = NULL, outcome_r = NULL,
                                  outcome_mean = NULL, outcome_se = NULL,
                                  outcome_E = NULL, study = NULL) {
  cols <- c(treatment, cov_means)
  for (col in c(outcome_n, outcome_r, outcome_mean, outcome_se, outcome_E)) {
    if (!is.null(col)) cols <- c(cols, col)
  }
  sd_cols <- cov_sds[!is.na(cov_sds)]
  if (length(sd_cols) > 0L) cols <- c(cols, sd_cols)
  if (!is.null(study)) cols <- c(cols, study)
  cols
}

#' Strip AgD covariate suffixes
#' @keywords internal
.strip_agd_cov_suffix <- function(cov_means) {
  sub("_prop$", "", sub("_mean$", "", cov_means))
}

#' Validate AgD covariate names after suffix stripping
#' @keywords internal
.validate_agd_covariate_names <- function(cov_means) {
  stripped <- .strip_agd_cov_suffix(cov_means)
  dup_stripped <- unique(stripped[duplicated(stripped)])
  if (length(dup_stripped) > 0L) {
    msg <- sprintf(
      paste0(
        "Covariate-name collision after stripping _mean/_prop suffix: %s. ",
        "Pass distinct `cov_means` entries."
      ),
      paste(dup_stripped, collapse = ", ")
    )
    stop(msg, call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate AgD outcome summaries
#' @keywords internal
.validate_agd_outcomes <- function(data, family, outcome_n, outcome_r,
                                   outcome_mean, outcome_se, outcome_E) {
  if (family == "binomial") {
    .validate_agd_binomial_outcomes(data[[outcome_r]], data[[outcome_n]])
  } else if (family == "normal") {
    .validate_agd_normal_outcomes(data[[outcome_mean]], data[[outcome_se]])
    if (!is.null(outcome_n)) {
      .validate_agd_sample_size(data[[outcome_n]])
    }
  } else {
    .validate_agd_poisson_outcomes(data[[outcome_r]], data[[outcome_E]])
  }
  invisible(TRUE)
}

#' Validate binomial AgD outcomes
#' @keywords internal
.validate_agd_binomial_outcomes <- function(r_vals, n_vals) {
  if (any(is.na(r_vals)) || any(is.na(n_vals))) {
    stop("`outcome_r` and `outcome_n` must not contain NA values", call. = FALSE)
  }
  if (!is.numeric(r_vals) || !is.numeric(n_vals)) {
    stop("`outcome_r` and `outcome_n` must be numeric", call. = FALSE)
  }
  if (any(!is.finite(r_vals)) || any(!is.finite(n_vals))) {
    stop("`outcome_r` and `outcome_n` must be finite", call. = FALSE)
  }
  if (!.is_whole_number_count(r_vals)) {
    stop("`outcome_r` must be integer counts", call. = FALSE)
  }
  if (!.is_whole_number_count(n_vals)) {
    stop("`outcome_n` must be integer sample sizes", call. = FALSE)
  }
  if (any(r_vals < 0)) {
    stop("`outcome_r` must be non-negative", call. = FALSE)
  }
  if (any(n_vals <= 0)) {
    stop("`outcome_n` must be positive", call. = FALSE)
  }
  if (any(r_vals > n_vals)) {
    stop("`outcome_r` must not exceed `outcome_n`", call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate optional AgD sample sizes
#' @keywords internal
.validate_agd_sample_size <- function(n_vals) {
  if (any(is.na(n_vals))) {
    stop("`outcome_n` must not contain NA values", call. = FALSE)
  }
  if (!is.numeric(n_vals)) {
    stop("`outcome_n` must be numeric", call. = FALSE)
  }
  if (any(!is.finite(n_vals))) {
    stop("`outcome_n` must be finite", call. = FALSE)
  }
  if (!.is_whole_number_count(n_vals)) {
    stop("`outcome_n` must be integer sample sizes", call. = FALSE)
  }
  if (any(n_vals <= 0)) {
    stop("`outcome_n` must be positive", call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate normal AgD outcomes
#' @keywords internal
.validate_agd_normal_outcomes <- function(y_vals, se_vals) {
  if (any(is.na(y_vals)) || any(is.na(se_vals))) {
    stop("`outcome_mean` and `outcome_se` must not contain NA values",
         call. = FALSE)
  }
  if (!is.numeric(y_vals) || !is.numeric(se_vals)) {
    stop("`outcome_mean` and `outcome_se` must be numeric", call. = FALSE)
  }
  if (any(!is.finite(y_vals)) || any(!is.finite(se_vals))) {
    stop("`outcome_mean` and `outcome_se` must be finite", call. = FALSE)
  }
  if (any(se_vals <= 0)) {
    stop("`outcome_se` must be positive", call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate Poisson AgD outcomes
#' @keywords internal
.validate_agd_poisson_outcomes <- function(r_vals, E_vals) {
  if (any(is.na(r_vals)) || any(is.na(E_vals))) {
    stop("`outcome_r` and `outcome_E` must not contain NA values", call. = FALSE)
  }
  if (!is.numeric(r_vals) || !is.numeric(E_vals)) {
    stop("`outcome_r` and `outcome_E` must be numeric", call. = FALSE)
  }
  if (any(!is.finite(r_vals)) || any(!is.finite(E_vals))) {
    stop("`outcome_r` and `outcome_E` must be finite", call. = FALSE)
  }
  if (!.is_whole_number_count(r_vals)) {
    stop("`outcome_r` must be integer counts", call. = FALSE)
  }
  if (any(r_vals < 0)) {
    stop("`outcome_r` must be non-negative", call. = FALSE)
  }
  if (any(E_vals <= 0)) {
    stop("`outcome_E` must be positive", call. = FALSE)
  }
  invisible(TRUE)
}

#' Check whether values are valid non-negative whole-number counts
#' @keywords internal
.is_whole_number_count <- function(x, allow_missing = FALSE) {
  if (!is.numeric(x)) {
    return(FALSE)
  }
  vals <- x
  if (allow_missing) {
    vals <- vals[!is.na(vals)]
  } else if (any(is.na(vals))) {
    return(FALSE)
  }
  if (length(vals) == 0L) {
    return(TRUE)
  }
  all(is.finite(vals) &
        vals >= 0 &
        vals <= .Machine$integer.max &
        abs(vals - round(vals)) <= sqrt(.Machine$double.eps))
}

#' Validate AgD covariate summaries
#' @keywords internal
.validate_agd_covariates <- function(data, cov_means, cov_sds) {
  for (cm in cov_means) {
    if (!is.numeric(data[[cm]])) {
      stop(sprintf("Covariate mean column '%s' must be numeric", cm),
           call. = FALSE)
    }
    if (any(!is.finite(data[[cm]]))) {
      stop(sprintf("Covariate mean column '%s' contains non-finite values", cm),
           call. = FALSE)
    }
  }

  for (cs in cov_sds[!is.na(cov_sds)]) {
    if (!is.numeric(data[[cs]])) {
      stop(sprintf("Covariate SD column '%s' must be numeric", cs),
           call. = FALSE)
    }
    if (any(!is.finite(data[[cs]]))) {
      stop(sprintf("Covariate SD column '%s' contains non-finite values", cs),
           call. = FALSE)
    }
    if (any(data[[cs]] < 0)) {
      stop(sprintf("Covariate SD column '%s' must be non-negative", cs),
           call. = FALSE)
    }
  }
  invisible(TRUE)
}

#' Validate AgD covariate type labels
#' @keywords internal
.validate_agd_cov_types <- function(cov_types) {
  valid_cov_types <- c("binary", "continuous")
  bad_types <- setdiff(unique(cov_types), valid_cov_types)
  if (length(bad_types) > 0L) {
    stop(sprintf("Invalid cov_types: %s. Must be 'binary' or 'continuous'.",
                 paste(sQuote(bad_types), collapse = ", ")), call. = FALSE)
  }
  invisible(TRUE)
}

#' Validate binary AgD covariate summaries
#' @keywords internal
.validate_agd_binary_covariates <- function(data, cov_means, cov_sds, cov_types) {
  for (i in seq_along(cov_types)) {
    if (cov_types[[i]] != "binary") next

    mean_vals <- data[[cov_means[[i]]]]
    cov_label <- .strip_agd_cov_suffix(cov_means[[i]])

    if (any(mean_vals < 0 | mean_vals > 1)) {
      msg <- sprintf(
        "Binary covariate '%s': mean/proportion must be in [0, 1], got %s",
        cov_label,
        paste(mean_vals[mean_vals < 0 | mean_vals > 1], collapse = ", ")
      )
      stop(msg, call. = FALSE)
    }

    if (!is.na(cov_sds[[i]])) {
      sd_vals <- data[[cov_sds[[i]]]]
      max_sd <- sqrt(mean_vals * (1 - mean_vals))
      impossible <- sd_vals > max_sd + 1e-10
      if (any(impossible)) {
        msg <- sprintf(
          paste0(
            "Binary covariate '%s': SD exceeds Bernoulli maximum sqrt(p*(1-p)). ",
            "Got SD=%s for p=%s (max SD=%s)"
          ),
          cov_label,
          paste(round(sd_vals[impossible], 4), collapse = ", "),
          paste(round(mean_vals[impossible], 4), collapse = ", "),
          paste(round(max_sd[impossible], 4), collapse = ", ")
        )
        stop(msg, call. = FALSE)
      }
    }
  }
  invisible(TRUE)
}

#' Standardize AgD to mlumr's internal column contract
#' @keywords internal
.standardize_agd_data <- function(data, treatment, family, outcome_n,
                                  outcome_r, outcome_mean, outcome_se,
                                  outcome_E, cov_means, cov_sds,
                                  cov_names, study = NULL) {
  agd_data <- data.frame(
    .study = if (!is.null(study)) data[[study]] else "AgD_Study",
    .trt = data[[treatment]],
    stringsAsFactors = FALSE
  )

  if (family == "binomial") {
    agd_data$.n <- data[[outcome_n]]
    agd_data$.r <- data[[outcome_r]]
  } else if (family == "normal") {
    agd_data$.y <- data[[outcome_mean]]
    agd_data$.se <- data[[outcome_se]]
    if (!is.null(outcome_n)) agd_data$.n <- data[[outcome_n]]
  } else {
    agd_data$.r <- data[[outcome_r]]
    agd_data$.E <- data[[outcome_E]]
  }

  for (i in seq_along(cov_means)) {
    cov_name <- cov_names[[i]]
    agd_data[[paste0(cov_name, "_mean")]] <- data[[cov_means[[i]]]]
    if (!is.na(cov_sds[[i]])) {
      agd_data[[paste0(cov_name, "_sd")]] <- data[[cov_sds[[i]]]]
    }
  }
  agd_data
}

#' Build AgD covariate metadata
#' @keywords internal
.agd_cov_info <- function(cov_names, cov_sds, cov_types) {
  cov_info <- list()
  for (i in seq_along(cov_names)) {
    cov_name <- cov_names[[i]]
    cov_info[[cov_name]] <- list(
      mean_col = paste0(cov_name, "_mean"),
      sd_col = if (!is.na(cov_sds[[i]])) paste0(cov_name, "_sd") else NA,
      type = cov_types[[i]]
    )
  }
  cov_info
}

#' Summarize AgD outcomes
#' @keywords internal
.agd_outcome_summary <- function(data, family, outcome_n, outcome_r,
                                 outcome_mean, outcome_se, outcome_E) {
  switch(family,
    binomial = list(n = data[[outcome_n]], n_events = data[[outcome_r]]),
    normal = list(y = data[[outcome_mean]], se = data[[outcome_se]],
                  n = if (!is.null(outcome_n)) data[[outcome_n]] else NULL),
    poisson = list(n_events = data[[outcome_r]],
                   total_exposure = data[[outcome_E]])
  )
}


#' Combine IPD and AgD for unanchored comparison
#'
#' @param ipd An `mlumr_ipd` object from [set_ipd()]
#' @param agd An `mlumr_agd` object from [set_agd()]
#'
#' @return An object of class `mlumr_data`
#' @export
#'
#' @examples
#' \dontrun{
#' dat <- combine_data(ipd, agd)
#' }
combine_data <- function(ipd, agd) {

  if (!inherits(ipd, "mlumr_ipd")) {
    stop("`ipd` must be created with set_ipd()", call. = FALSE)
  }
  if (!inherits(agd, "mlumr_agd")) {
    stop("`agd` must be created with set_agd()", call. = FALSE)
  }

  if (ipd$family != agd$family) {
    stop(sprintf("Family mismatch: IPD uses '%s' but AgD uses '%s'",
                 ipd$family, agd$family), call. = FALSE)
  }

  if (!identical(sort(ipd$covariates), sort(agd$covariates))) {
    stop("Covariates must match between IPD and AgD", call. = FALSE)
  }

  if (length(intersect(ipd$treatment, agd$treatment)) > 0) {
    warning("IPD and AgD share treatments -- this looks like an anchored comparison",
            call. = FALSE)
  }

  out <- list(
    ipd = ipd,
    agd = agd,
    family = ipd$family,
    covariates = ipd$covariates,
    n_covariates = length(ipd$covariates),
    treatments = c(ipd$treatment, agd$treatment),
    index_treatment = ipd$treatment,
    comparator_treatment = agd$treatment,
    has_integration = FALSE
  )

  class(out) <- c("mlumr_data", "list")
  out
}


#' @method print mlumr_data
#' @export
print.mlumr_data <- function(x, ...) {
  family <- x$family %||% "binomial"
  family_label <- switch(family,
    binomial = "Binary",
    normal   = "Continuous",
    poisson  = "Count"
  )

  cat(sprintf("Unanchored Comparison Data (%s)\n", family_label))
  cat("====================================\n\n")
  cat("Index treatment (IPD):", x$index_treatment, "\n")
  cat("  N =", x$ipd$n, "\n")

  if (family == "binomial") {
    cat("  Events =", x$ipd$n_events,
        sprintf("(%.1f%%)", 100 * x$ipd$n_events / x$ipd$n), "\n\n")
  } else if (family == "normal") {
    cat(sprintf("  Mean outcome = %.3f (SD = %.3f)\n\n",
                x$ipd$mean_outcome, x$ipd$sd_outcome))
  } else {
    cat(sprintf("  Total events = %d, Total exposure = %.1f\n\n",
                x$ipd$total_events, x$ipd$total_exposure))
  }

  cat("Comparator treatment (AgD):", x$comparator_treatment, "\n")
  if (family == "binomial") {
    cat("  N =", x$agd$n, "\n")
    cat("  Events =", x$agd$n_events,
        sprintf("(%.1f%%)", 100 * x$agd$n_events / x$agd$n), "\n\n")
  } else if (family == "normal") {
    if (!is.null(x$agd$n)) cat("  N =", x$agd$n, "\n")
    cat(sprintf("  Mean outcome = %s, SE = %s\n\n",
                paste(round(x$agd$y, 3), collapse = ", "),
                paste(round(x$agd$se, 3), collapse = ", ")))
  } else {
    cat(sprintf("  Total events = %d, Total exposure = %.1f\n\n",
                sum(x$agd$n_events), sum(x$agd$total_exposure)))
  }

  cat("Covariates (", x$n_covariates, "):",
      paste(x$covariates, collapse = ", "), "\n")
  if (x$has_integration) {
    cat("Integration points:", x$n_int, "(QMC with Gaussian copula)\n")
  } else {
    cat("Integration points: not yet added (use add_integration())\n")
  }
  invisible(x)
}

test_that("add_integration generates valid integration points", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5), x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  dat <- add_integration(dat, n_int = 32, x1 = distr(qbern, prob = x1_mean))

  expect_true(dat$has_integration)
  expect_equal(dat$n_int, 32L)
  expect_equal(dim(dat$integration_points), c(1, 32, 1))
  # Bernoulli integration points should be 0 or 1
  expect_true(all(dat$integration_points %in% c(0, 1)))
})

test_that("add_integration can suppress progress messages", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5), x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_message(
    out <- add_integration(
      dat,
      n_int = 32,
      verbose = FALSE,
      x1 = distr(qbern, prob = x1_mean)
    ),
    NA
  )
  expect_true(out$has_integration)
})

test_that("add_integration works with two covariates and correlation", {
  set.seed(42)
  n <- 100
  ipd_df <- data.frame(
    trt = "A", outcome = rbinom(n, 1, 0.5),
    x1 = rbinom(n, 1, 0.4), x2 = rnorm(n, 5, 1)
  )
  agd_df <- data.frame(
    trt = "B", n_total = 200, n_events = 80,
    x1_mean = 0.3, x2_mean = 5.2, x2_sd = 0.9
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", c("x1", "x2"))
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events",
                 cov_means = c("x1_mean", "x2_mean"),
                 cov_sds = c(NA, "x2_sd"))
  dat <- combine_data(ipd, agd)

  dat <- add_integration(dat, n_int = 64,
                         x1 = distr(qbern, prob = x1_mean),
                         x2 = distr(qnorm, mean = x2_mean, sd = x2_sd))

  expect_equal(dim(dat$integration_points), c(1, 64, 2))
  expect_true(all(dat$integration_points[1, , 1] %in% c(0, 1)))
  # Continuous covariate should have reasonable range
  x2_pts <- dat$integration_points[1, , 2]
  expect_true(all(is.finite(x2_pts)))
  expect_true(mean(x2_pts) > 4 && mean(x2_pts) < 6)
})

test_that("add_integration keeps integration array in data covariate order", {
  set.seed(42)
  n <- 100
  ipd_df <- data.frame(
    trt = "A", outcome = rbinom(n, 1, 0.5),
    x1 = rbinom(n, 1, 0.4), x2 = rnorm(n, 5, 1)
  )
  agd_df <- data.frame(
    trt = "B", n_total = 200, n_events = 80,
    x1_mean = 0.3, x2_mean = 5.2, x2_sd = 0.9
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", c("x1", "x2"))
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events",
                 cov_means = c("x1_mean", "x2_mean"),
                 cov_sds = c(NA, "x2_sd"))
  dat <- combine_data(ipd, agd)

  dat <- add_integration(
    dat,
    n_int = 64,
    x2 = distr(qnorm, mean = x2_mean, sd = x2_sd),
    x1 = distr(qbern, prob = x1_mean)
  )

  expect_equal(dimnames(dat$integration_points)[[3]], c("x1", "x2"))
  expect_true(all(dat$integration_points[1, , "x1"] %in% c(0, 1)))
  expect_true(mean(dat$integration_points[1, , "x2"]) > 4)
})

test_that("add_integration reorders named correlation matrices to covariate order", {
  set.seed(42)
  n <- 100
  ipd_df <- data.frame(
    trt = "A", outcome = rbinom(n, 1, 0.5),
    x1 = rbinom(n, 1, 0.4), x2 = rnorm(n, 5, 1)
  )
  agd_df <- data.frame(
    trt = "B", n_total = 200, n_events = 80,
    x1_mean = 0.3, x2_mean = 5.2, x2_sd = 0.9
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", c("x1", "x2"))
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events",
                 cov_means = c("x1_mean", "x2_mean"),
                 cov_sds = c(NA, "x2_sd"))
  dat <- combine_data(ipd, agd)

  cor <- matrix(c(1, 0.2, 0.2, 1), nrow = 2,
                dimnames = list(c("x2", "x1"), c("x2", "x1")))
  dat <- add_integration(
    dat,
    n_int = 64,
    cor = cor,
    x1 = distr(qbern, prob = x1_mean),
    x2 = distr(qnorm, mean = x2_mean, sd = x2_sd)
  )

  expect_equal(rownames(dat$int_cor), c("x1", "x2"))
  expect_equal(colnames(dat$int_cor), c("x1", "x2"))
})

test_that("add_integration rejects missing distribution specs", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5),
                       x1 = rbinom(50, 1, 0.4), x2 = rnorm(50))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40,
                       x1_mean = 0.3, x2_mean = 0.1)

  ipd <- set_ipd(ipd_df, "trt", "outcome", c("x1", "x2"))
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events",
                 cov_means = c("x1_mean", "x2_mean"))
  dat <- combine_data(ipd, agd)

  # Only specifying one of two covariates
  expect_error(
    add_integration(dat, n_int = 32, x1 = distr(qbern, prob = x1_mean)),
    "Missing distribution"
  )
})

test_that("add_integration rejects invalid scalar arguments cleanly", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5),
                       x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40,
                       x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_error(add_integration(dat, n_int = Inf,
                               x1 = distr(qbern, prob = x1_mean)),
               "`n_int` should be a positive integer")
  expect_error(add_integration(dat, n_int = NA_real_,
                               x1 = distr(qbern, prob = x1_mean)),
               "`n_int` should be a positive integer")
  expect_error(add_integration(dat, n_int = 32,
                               cor_adjust = c("none", "pearson"),
                               x1 = distr(qbern, prob = x1_mean)),
               "`cor_adjust` must be one of")
  expect_error(add_integration(dat, n_int = 32,
                               cor_adjust = NA_character_,
                               x1 = distr(qbern, prob = x1_mean)),
               "`cor_adjust` must be one of")
})

test_that("add_integration rejects duplicate distribution names", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5),
                       x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40,
                       x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_error(
    add_integration(dat, n_int = 32,
                    x1 = distr(qbern, prob = x1_mean),
                    x1 = distr(qbern, prob = x1_mean)),
    "Duplicate distribution specifications for: x1"
  )
})

test_that("add_integration validates correlation names and computed correlations", {
  set.seed(42)
  n <- 100
  ipd_df <- data.frame(
    trt = "A", outcome = rbinom(n, 1, 0.5),
    x1 = rbinom(n, 1, 0.4), x2 = rnorm(n)
  )
  agd_df <- data.frame(
    trt = "B", n_total = 100, n_events = 40,
    x1_mean = 0.3, x2_mean = 0, x2_sd = 1
  )
  ipd <- set_ipd(ipd_df, "trt", "outcome", c("x1", "x2"))
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events",
                 cov_means = c("x1_mean", "x2_mean"),
                 cov_sds = c(NA, "x2_sd"))
  dat <- combine_data(ipd, agd)

  bad_cor <- matrix(c(1, 0.2, 0.2, 1), nrow = 2,
                    dimnames = list(c("x1", "wrong"), c("x1", "wrong")))
  expect_error(
    add_integration(dat, n_int = 64, cor = bad_cor,
                    x1 = distr(qbern, prob = x1_mean),
                    x2 = distr(qnorm, mean = x2_mean, sd = x2_sd)),
    "`cor` dimnames must match covariate names"
  )

  constant_ipd <- ipd_df
  constant_ipd$x2 <- 1
  expect_warning(
    ipd_const <- set_ipd(constant_ipd, "trt", "outcome", c("x1", "x2")),
    "zero empirical variation"
  )
  dat_const <- combine_data(ipd_const, agd)
  expect_error(
    add_integration(dat_const, n_int = 64,
                    x1 = distr(qbern, prob = x1_mean),
                    x2 = distr(qnorm, mean = x2_mean, sd = x2_sd)),
    "Computed IPD correlation matrix contains non-finite values"
  )
})

test_that("add_integration works with multi-row AgD", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5), x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(
    trt = c("B", "B"),
    n_total = c(100, 150),
    n_events = c(40, 55),
    x1_mean = c(0.3, 0.5)
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  dat <- add_integration(dat, n_int = 32, x1 = distr(qbern, prob = x1_mean))

  expect_equal(dim(dat$integration_points), c(2, 32, 1))
})

test_that("unnest_integration produces correct long format", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5), x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32, x1 = distr(qbern, prob = x1_mean))

  long <- unnest_integration(dat)
  expect_equal(nrow(long), 32)
  expect_true("x1" %in% names(long))
  expect_true(".int_id" %in% names(long))
  expect_true(".agd_row" %in% names(long))
})


test_that("check_integration errors without integration points", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5), x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)
  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_error(
    check_integration(dat, x1 = distr(qbern, prob = x1_mean)),
    "add_integration"
  )
})


test_that("check_integration returns marginals data frame", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(100, 1, 0.5),
                       x1 = rbinom(100, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)
  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 64, x1 = distr(qbern, prob = x1_mean))

  res <- suppressMessages(suppressWarnings(
    capture.output(out <- check_integration(dat, x1 = distr(qbern, prob = x1_mean),
                                            check_joint = FALSE))
  ))
  expect_type(out, "list")
  expect_s3_class(out$marginals, "data.frame")
  expect_true(all(c("covariate", "agd_row", "mean_current",
                    "rel_diff_mean") %in% names(out$marginals)))
  # No $correlations when check_joint = FALSE
  expect_null(out$correlations)
})

test_that("check_integration can suppress printed diagnostics", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(100, 1, 0.5),
                       x1 = rbinom(100, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)
  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 64, verbose = FALSE,
                         x1 = distr(qbern, prob = x1_mean))

  expect_output(
    out <- check_integration(
      dat,
      x1 = distr(qbern, prob = x1_mean),
      check_joint = FALSE,
      verbose = FALSE
    ),
    NA
  )
  expect_s3_class(out$marginals, "data.frame")
})

test_that("check_integration validates check_joint", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(100, 1, 0.5),
                       x1 = rbinom(100, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)
  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 64, verbose = FALSE,
                         x1 = distr(qbern, prob = x1_mean))

  expect_error(
    check_integration(dat, x1 = distr(qbern, prob = x1_mean),
                      check_joint = NA),
    "`check_joint` must be TRUE or FALSE"
  )
  expect_error(
    check_integration(dat, x1 = distr(qbern, prob = x1_mean),
                      check_joint = c(TRUE, FALSE)),
    "`check_joint` must be TRUE or FALSE"
  )
})


test_that("check_integration with check_joint returns pairwise correlations", {
  set.seed(42)
  n <- 100
  ipd_df <- data.frame(
    trt = "A", outcome = rbinom(n, 1, 0.5),
    x1 = rbinom(n, 1, 0.4), x2 = rnorm(n, 5, 1)
  )
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40,
                       x1_mean = 0.3, x2_mean = 5.2, x2_sd = 0.9)
  ipd <- set_ipd(ipd_df, "trt", "outcome", c("x1", "x2"))
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events",
                 cov_means = c("x1_mean", "x2_mean"),
                 cov_sds = c(NA, "x2_sd"))
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 64,
                         x1 = distr(qbern, prob = x1_mean),
                         x2 = distr(qnorm, mean = x2_mean, sd = x2_sd))

  res <- suppressMessages(suppressWarnings(
    capture.output(out <- check_integration(
      dat,
      x1 = distr(qbern, prob = x1_mean),
      x2 = distr(qnorm, mean = x2_mean, sd = x2_sd),
      check_joint = TRUE
    ))
  ))
  expect_s3_class(out$correlations, "data.frame")
  expect_true(all(c("agd_row", "pair", "cor_current", "cor_doubled",
                    "abs_diff") %in% names(out$correlations)))
})

test_that("set_ipd validates inputs", {
  df <- data.frame(trt = "A", outcome = c(0, 1, 1, 0), x1 = rnorm(4))

  ipd <- set_ipd(df, treatment = "trt", outcome = "outcome",
                 covariates = "x1")
  expect_s3_class(ipd, "mlumr_ipd")
  expect_equal(ipd$n, 4)
  expect_equal(ipd$n_events, 2)
  expect_equal(ipd$family, "binomial")

  # Missing column
  expect_error(set_ipd(df, "trt", "outcome", "nonexistent"), "Missing columns")

  # Non-binary outcome
  df_bad <- data.frame(trt = "A", outcome = c(0, 1, 2, 3), x1 = rnorm(4))
  expect_error(set_ipd(df_bad, "trt", "outcome", "x1"), "binary")
})


test_that("set_ipd rejects unsafe binomial outcome encodings", {
  df_factor <- data.frame(trt = "A", outcome = factor(c(0, 1, 1, 0)),
                          x1 = rnorm(4))
  expect_error(
    set_ipd(df_factor, "trt", "outcome", "x1"),
    "numeric or logical"
  )

  df_logical <- data.frame(trt = "A", outcome = c(TRUE, FALSE),
                           x1 = c(0, 1))
  ipd <- set_ipd(df_logical, "trt", "outcome", "x1")
  expect_equal(ipd$data$.outcome, c(1L, 0L))
})


test_that("set_ipd requires non-empty unique covariates", {
  df <- data.frame(trt = "A", outcome = c(0, 1), x1 = c(0, 1))

  expect_error(
    set_ipd(df, "trt", "outcome", character(0)),
    "non-empty character vector"
  )
  expect_error(
    set_ipd(df, "trt", "outcome", c("x1", "x1")),
    "Duplicate covariates"
  )
})


test_that("set_ipd drops missing treatment rows before treatment validation", {
  df <- data.frame(trt = c("A", NA, "A"), outcome = c(0, 1, 1),
                   x1 = c(0, 1, 1))

  expect_warning(
    ipd <- set_ipd(df, "trt", "outcome", "x1"),
    "missing values"
  )
  expect_equal(ipd$n, 2)
  expect_equal(ipd$treatment, "A")
})


test_that("set_ipd warns on zero-variation covariates after filtering", {
  df <- data.frame(trt = "A", outcome = c(0, 1, 0, 1), x1 = c(1, 1, 1, 1))

  expect_warning(
    ipd <- set_ipd(df, "trt", "outcome", "x1"),
    "zero empirical variation"
  )
  expect_s3_class(ipd, "mlumr_ipd")
})


test_that("set_ipd does not warn on varying covariates", {
  df <- data.frame(trt = "A", outcome = c(0, 1, 0, 1), x1 = c(0, 1, 0, 1))

  expect_warning(
    ipd <- set_ipd(df, "trt", "outcome", "x1"),
    NA
  )
  expect_s3_class(ipd, "mlumr_ipd")
})


test_that("set_ipd errors when no complete rows remain", {
  df <- data.frame(trt = "A", outcome = c(NA_real_, NA_real_),
                   x1 = c(NA_real_, NA_real_))

  expect_error(
    suppressWarnings(set_ipd(df, "trt", "outcome", "x1", family = "normal")),
    "No complete IPD rows remain"
  )
})


test_that("set_agd validates inputs", {
  df <- data.frame(trt = "B", n_total = 200, n_events = 60, x1_mean = 0.3)

  agd <- set_agd(df, treatment = "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  expect_s3_class(agd, "mlumr_agd")
  expect_equal(agd$covariates, "x1")
  expect_equal(agd$family, "binomial")

  # Missing columns
  expect_error(
    set_agd(df, "trt", outcome_n = "n_total", outcome_r = "missing_col",
            cov_means = "x1_mean"),
    "Missing columns"
  )
})


test_that("set_agd rejects non-integer aggregate counts", {
  df_n <- data.frame(trt = "B", n_total = 200.5, n_events = 60,
                     x1_mean = 0.3)
  expect_error(
    set_agd(df_n, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean"),
    "integer sample sizes"
  )

  df_r <- data.frame(trt = "B", n_total = 200, n_events = 60.5,
                     x1_mean = 0.3)
  expect_error(
    set_agd(df_r, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean"),
    "integer counts"
  )

  df_p <- data.frame(trt = "B", n_events = 60.5, pyears = 100,
                     x1_mean = 0.3)
  expect_error(
    set_agd(df_p, "trt", family = "poisson", outcome_r = "n_events",
            outcome_E = "pyears", cov_means = "x1_mean"),
    "integer counts"
  )
})


test_that("set_agd requires non-empty covariate summaries", {
  df <- data.frame(trt = "B", n_total = 200, n_events = 60, x1_mean = 0.3)

  expect_error(
    set_agd(df, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = character(0)),
    "non-empty character vector"
  )
})


test_that("combine_data validates and combines", {
  ipd_df <- data.frame(trt = "A", outcome = c(0, 1, 1, 0),
                       x1 = c(1, 0, 1, 0))
  agd_df <- data.frame(trt = "B", n_total = 200, n_events = 60,
                       x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")

  dat <- combine_data(ipd, agd)
  expect_s3_class(dat, "mlumr_data")
  expect_equal(dat$n_covariates, 1)
  expect_equal(dat$index_treatment, "A")
  expect_equal(dat$comparator_treatment, "B")
  expect_equal(dat$family, "binomial")
  expect_false(dat$has_integration)

  # Wrong class
  expect_error(combine_data("not_ipd", agd), "set_ipd")
  expect_error(combine_data(ipd, "not_agd"), "set_agd")
})


test_that("set_agd handles multi-row AgD", {
  agd_df <- data.frame(
    trt = c("B", "B"),
    n_total = c(100, 200),
    n_events = c(40, 80),
    x1_mean = c(0.3, 0.5)
  )

  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  expect_s3_class(agd, "mlumr_agd")
  expect_equal(nrow(agd$data), 2)
})


test_that("set_ipd with study column", {
  df <- data.frame(trt = "A", outcome = c(0, 1, 1, 0), x1 = rnorm(4),
                   study = c("S1", "S1", "S2", "S2"))

  ipd <- set_ipd(df, "trt", "outcome", "x1", study = "study")
  expect_s3_class(ipd, "mlumr_ipd")
  expect_true(".study" %in% names(ipd$data))
  expect_equal(unique(ipd$data$.study), c("S1", "S2"))
})


test_that("set_ipd warns about missing values", {
  df <- data.frame(trt = "A", outcome = c(0, 1, NA, 0), x1 = rnorm(4))
  expect_warning(set_ipd(df, "trt", "outcome", "x1"), "missing values")
})


test_that("set_ipd rejects non-finite covariates and outcomes", {
  # Infinite covariate
  df_inf <- data.frame(trt = "A", outcome = c(0, 1, 1, 0),
                       x1 = c(1, Inf, 0, 1))
  expect_error(set_ipd(df_inf, "trt", "outcome", "x1"), "non-finite")

  # -Inf covariate
  df_neginf <- data.frame(trt = "A", outcome = c(0, 1, 1, 0),
                          x1 = c(1, -Inf, 0, 1))
  expect_error(set_ipd(df_neginf, "trt", "outcome", "x1"), "non-finite")

  # Infinite Poisson exposure
  df_exp <- data.frame(trt = "A", events = c(1L, 0L, 2L, 1L),
                       pyears = c(1.5, Inf, 1.0, 3.0), x1 = rnorm(4))
  expect_error(
    set_ipd(df_exp, "trt", "events", "x1", family = "poisson",
            exposure = "pyears"),
    "non-finite"
  )
})


test_that("set_agd rejects NA and non-finite outcomes", {
  # NA in binomial outcome_r
  df_na <- data.frame(trt = "B", n_total = 200, n_events = NA, x1_mean = 0.3)
  expect_error(
    set_agd(df_na, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean"),
    "must not contain NA"
  )

  # NA in normal outcome_se
  df_na_se <- data.frame(trt = "B", y_mean = 2.0, se = NA, x1_mean = 0.3)
  expect_error(
    set_agd(df_na_se, "trt", family = "normal", outcome_mean = "y_mean",
            outcome_se = "se", cov_means = "x1_mean"),
    "must not contain NA"
  )

  # NA in Poisson outcome_E
  df_na_E <- data.frame(trt = "B", n_events = 50, pyears = NA, x1_mean = 0.4)
  expect_error(
    set_agd(df_na_E, "trt", family = "poisson", outcome_r = "n_events",
            outcome_E = "pyears", cov_means = "x1_mean"),
    "must not contain NA"
  )

  # Infinite covariate mean
  df_inf <- data.frame(trt = "B", n_total = 200, n_events = 60, x1_mean = Inf)
  expect_error(
    set_agd(df_inf, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean"),
    "non-finite"
  )

  # Negative covariate SD
  df_neg_sd <- data.frame(trt = "B", n_total = 200, n_events = 60,
                          x1_mean = 0.3, x1_sd = -2)
  expect_error(
    set_agd(df_neg_sd, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean", cov_sds = "x1_sd"),
    "non-negative"
  )
})


test_that("set_agd validates binary covariate domains", {
  # Binary mean > 1
  df_hi <- data.frame(trt = "B", n_total = 200, n_events = 60, x1_mean = 1.2)
  expect_error(
    set_agd(df_hi, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean", cov_types = "binary"),
    "Binary covariate.*must be in \\[0, 1\\]"
  )

  # Binary mean < 0
  df_lo <- data.frame(trt = "B", n_total = 200, n_events = 60, x1_mean = -0.1)
  expect_error(
    set_agd(df_lo, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean", cov_types = "binary"),
    "Binary covariate.*must be in \\[0, 1\\]"
  )

  # Impossible binary SD: p=0.9, max SD = sqrt(0.09) ~ 0.3, but SD=2
  df_sd <- data.frame(trt = "B", n_total = 200, n_events = 60,
                      x1_mean = 0.9, x1_sd = 2)
  expect_error(
    set_agd(df_sd, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean", cov_sds = "x1_sd", cov_types = "binary"),
    "SD exceeds Bernoulli maximum"
  )

  # Invalid cov_types value
  df_ok <- data.frame(trt = "B", n_total = 200, n_events = 60, x1_mean = 0.5)
  expect_error(
    set_agd(df_ok, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean", cov_types = "ordinal"),
    "Invalid cov_types"
  )

  # Valid binary mean without SD passes
  expect_no_error(
    set_agd(df_ok, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = "x1_mean", cov_types = "binary")
  )
})


test_that("print.mlumr_data works", {
  ipd_df <- data.frame(trt = "A", outcome = c(0, 1, 1, 0), x1 = c(1, 0, 1, 0))
  agd_df <- data.frame(trt = "B", n_total = 200, n_events = 60, x1_mean = 0.3)
  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_output(print(dat), "Unanchored Comparison")
  expect_output(print(dat), "add_integration")
})


# ---- Normal family tests ----

test_that("set_ipd works with normal family", {
  df <- data.frame(trt = "A", score = c(2.1, 3.5, 1.8, 4.2), x1 = rnorm(4))

  ipd <- set_ipd(df, "trt", "score", "x1", family = "normal")
  expect_s3_class(ipd, "mlumr_ipd")
  expect_equal(ipd$family, "normal")
  expect_equal(ipd$n, 4)
  expect_equal(ipd$mean_outcome, mean(c(2.1, 3.5, 1.8, 4.2)))
  expect_true(!is.null(ipd$sd_outcome))
  expect_true(is.numeric(ipd$data$.outcome))

  # Non-numeric outcome should error
  df_bad <- data.frame(trt = "A", score = c("a", "b", "c", "d"), x1 = rnorm(4))
  expect_error(set_ipd(df_bad, "trt", "score", "x1", family = "normal"), "numeric")
})


test_that("set_agd works with normal family", {
  df <- data.frame(trt = "B", y_mean = 5.2, se = 0.3, n = 200, x1_mean = 0.5)

  agd <- set_agd(df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 outcome_n = "n", cov_means = "x1_mean")
  expect_s3_class(agd, "mlumr_agd")
  expect_equal(agd$family, "normal")
  expect_equal(agd$data$.y, 5.2)
  expect_equal(agd$data$.se, 0.3)
  expect_equal(agd$data$.n, 200)

  # Missing required columns
  expect_error(
    set_agd(df, "trt", family = "normal", cov_means = "x1_mean"),
    "outcome_mean.*outcome_se.*required"
  )

  df_bad_n <- data.frame(trt = "B", y_mean = 5.2, se = 0.3, n = 200.5,
                         x1_mean = 0.5)
  expect_error(
    set_agd(df_bad_n, "trt", family = "normal",
            outcome_mean = "y_mean", outcome_se = "se",
            outcome_n = "n", cov_means = "x1_mean"),
    "integer sample sizes"
  )
})


test_that("combine_data with normal family", {
  ipd_df <- data.frame(trt = "A", score = rnorm(10), x1 = rnorm(10))
  agd_df <- data.frame(trt = "B", y_mean = 2.0, se = 0.5, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "score", "x1", family = "normal")
  agd <- set_agd(agd_df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 cov_means = "x1_mean")

  dat <- combine_data(ipd, agd)
  expect_equal(dat$family, "normal")
})


test_that("combine_data rejects family mismatch", {
  ipd_df <- data.frame(trt = "A", outcome = c(0, 1, 1, 0), x1 = rnorm(4))
  agd_df <- data.frame(trt = "B", y_mean = 2.0, se = 0.5, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1", family = "binomial")
  agd <- set_agd(agd_df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 cov_means = "x1_mean")

  expect_error(combine_data(ipd, agd), "Family mismatch")
})


test_that("print.mlumr_data works for normal family", {
  ipd_df <- data.frame(trt = "A", score = rnorm(10), x1 = rnorm(10))
  agd_df <- data.frame(trt = "B", y_mean = 2.0, se = 0.5, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "score", "x1", family = "normal")
  agd <- set_agd(agd_df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_output(print(dat), "Continuous")
  expect_output(print(dat), "Mean outcome")
})


# ---- Poisson family tests ----

test_that("set_ipd works with poisson family", {
  df <- data.frame(trt = "A", events = c(2L, 0L, 3L, 1L),
                   pyears = c(1.5, 2.0, 1.0, 3.0), x1 = rnorm(4))

  ipd <- set_ipd(df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  expect_s3_class(ipd, "mlumr_ipd")
  expect_equal(ipd$family, "poisson")
  expect_equal(ipd$total_events, 6L)
  expect_equal(ipd$total_exposure, 7.5)
  expect_true(".exposure" %in% names(ipd$data))

  # Missing exposure
  expect_error(
    set_ipd(df, "trt", "events", "x1", family = "poisson"),
    "exposure.*required"
  )

  # Negative counts
  df_bad <- data.frame(trt = "A", events = c(-1L, 0L, 3L, 1L),
                       pyears = c(1.5, 2.0, 1.0, 3.0), x1 = rnorm(4))
  expect_error(
    set_ipd(df_bad, "trt", "events", "x1", family = "poisson",
            exposure = "pyears"),
    "non-negative"
  )
})


test_that("set_agd works with poisson family", {
  df <- data.frame(trt = "B", n_events = 50, pyears = 500, x1_mean = 0.4)

  agd <- set_agd(df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")
  expect_s3_class(agd, "mlumr_agd")
  expect_equal(agd$family, "poisson")
  expect_equal(agd$data$.r, 50)
  expect_equal(agd$data$.E, 500)

  # Missing required
  expect_error(
    set_agd(df, "trt", family = "poisson", cov_means = "x1_mean"),
    "outcome_r.*outcome_E.*required"
  )
})


test_that("combine_data with poisson family", {
  ipd_df <- data.frame(trt = "A", events = c(2L, 0L, 3L, 1L),
                       pyears = c(1.5, 2.0, 1.0, 3.0), x1 = rnorm(4))
  agd_df <- data.frame(trt = "B", n_events = 50, pyears = 500, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  agd <- set_agd(agd_df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")

  dat <- combine_data(ipd, agd)
  expect_equal(dat$family, "poisson")
})


test_that("print.mlumr_data works for poisson family", {
  ipd_df <- data.frame(trt = "A", events = c(2L, 0L, 3L, 1L),
                       pyears = c(1.5, 2.0, 1.0, 3.0), x1 = rnorm(4))
  agd_df <- data.frame(trt = "B", n_events = 50, pyears = 500, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  agd <- set_agd(agd_df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_output(print(dat), "Count")
  expect_output(print(dat), "Total events")
})


test_that("set_ipd rejects covariate names that collide with internal columns", {
  df <- data.frame(trt = "A", outcome = c(0, 1, 1, 0),
                   .outcome = rnorm(4), .trt = rnorm(4),
                   .study = rnorm(4), .exposure = rnorm(4))

  for (bad in c(".outcome", ".trt", ".study", ".exposure")) {
    expect_error(
      set_ipd(df, "trt", "outcome", covariates = bad),
      "reserved internal columns"
    )
  }
})


test_that("set_agd rejects mean/SD column names that collide with internal columns", {
  df <- data.frame(trt = "B", n_total = 200, n_events = 60,
                   .r = 0.3, .n = 0.3, .y = 0.3, .se = 0.1,
                   .study = 0.1, .trt = 0.1)

  for (bad in c(".r", ".n", ".y", ".se", ".study", ".trt")) {
    expect_error(
      set_agd(df, "trt", outcome_n = "n_total", outcome_r = "n_events",
              cov_means = bad),
      "reserved internal columns"
    )
  }
})


test_that("set_agd rejects duplicate cov_names after suffix-stripping", {
  df <- data.frame(trt = "B", n_total = 200, n_events = 60,
                   age_mean = 55, age = 55)

  # "age_mean" strips to "age"; "age" stays "age" -> duplicate
  expect_error(
    set_agd(df, "trt", outcome_n = "n_total", outcome_r = "n_events",
            cov_means = c("age_mean", "age")),
    "collision after stripping"
  )
})


test_that("set_ipd accepts ordinary covariate names (sanity check)", {
  df <- data.frame(trt = "A", outcome = c(0, 1, 1, 0),
                   age = c(50, 60, 55, 65), bmi = c(22, 25, 27, 30))
  expect_s3_class(
    set_ipd(df, "trt", "outcome", covariates = c("age", "bmi")),
    "mlumr_ipd"
  )
})

test_that("naive computes correct LOR", {
  # Create simple data
  set.seed(42)
  n <- 100
  ipd_df <- data.frame(
    trt = "A",
    outcome = c(rep(1, 60), rep(0, 40)),
    x1 = rnorm(n)
  )
  agd_df <- data.frame(
    trt = "B", n_total = 200, n_events = 80, x1_mean = 0.5
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- naive(dat)

  expect_s3_class(result, "mlumr_naive")

  # Manual check: p_A = 0.6, p_B = 0.4
  p_A <- 0.6
  p_B <- 0.4
  expected_lor <- qlogis(p_A) - qlogis(p_B)
  expect_equal(result$estimate, expected_lor, tolerance = 1e-4)

  # SE should be positive
  expect_gt(result$se, 0)

  # CI should contain the estimate
  expect_lt(result$ci_lower, result$estimate)
  expect_gt(result$ci_upper, result$estimate)

  # Print method works
  expect_output(print(result), "Naive")
})


test_that("naive handles extreme rates", {
  # All events
  ipd_df <- data.frame(trt = "A", outcome = rep(1, 50), x1 = rnorm(50))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 1, x1_mean = 0.5)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- naive(dat)
  expect_true(is.finite(result$estimate))
  expect_true(is.finite(result$se))
  expect_gte(result$p_index_lower, 0)
  expect_lte(result$p_index_upper, 1)
  expect_gte(result$p_comparator_lower, 0)
  expect_lte(result$p_comparator_upper, 1)
})


test_that("naive respects custom conf_level", {
  ipd_df <- data.frame(trt = "A", outcome = c(rep(1, 60), rep(0, 40)),
                        x1 = rnorm(100))
  agd_df <- data.frame(trt = "B", n_total = 200, n_events = 80, x1_mean = 0.5)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  r95 <- naive(dat, conf_level = 0.95)
  r90 <- naive(dat, conf_level = 0.90)

  # 90% CI should be narrower than 95% CI
  expect_lt(r90$ci_upper - r90$ci_lower, r95$ci_upper - r95$ci_lower)
  expect_equal(r90$conf_level, 0.90)
})


test_that("naive validates data and conf_level", {
  ipd_df <- data.frame(trt = "A", outcome = c(1, 0, 1, 0), x1 = c(0, 1, 0, 1))
  agd_df <- data.frame(trt = "B", n_total = 10, n_events = 4, x1_mean = 0.5)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_error(naive(data.frame()), "combine_data")
  expect_error(naive(dat, conf_level = 1), "`conf_level`")
  expect_error(naive(dat, conf_level = c(0.90, 0.95)), "`conf_level`")
  expect_error(naive(dat, conf_level = NA_real_), "`conf_level`")
})


test_that("naive works with equal rates", {
  ipd_df <- data.frame(trt = "A", outcome = c(rep(1, 50), rep(0, 50)),
                        x1 = rnorm(100))
  agd_df <- data.frame(trt = "B", n_total = 200, n_events = 100, x1_mean = 0.5)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- naive(dat)
  expect_equal(result$estimate, 0, tolerance = 1e-10)
})


# ---- Normal family ----

test_that("naive works with normal family", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", score = rnorm(100, mean = 5, sd = 2),
                       x1 = rnorm(100))
  agd_df <- data.frame(trt = "B", y_mean = 3.0, se = 0.2, x1_mean = 0.5)

  ipd <- set_ipd(ipd_df, "trt", "score", "x1", family = "normal")
  agd <- set_agd(agd_df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- naive(dat)
  expect_s3_class(result, "mlumr_naive")
  expect_equal(result$family, "normal")
  expect_true(is.finite(result$estimate))
  expect_gt(result$se, 0)

  # Mean difference should be approximately 5 - 3 = 2
  expect_equal(result$estimate, mean(ipd_df$score) - 3.0, tolerance = 1e-10)

  # Print works
  expect_output(print(result), "Mean Difference")
})


# ---- Poisson family ----

test_that("naive works with poisson family", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", events = rpois(50, 3),
                       pyears = runif(50, 0.5, 2), x1 = rnorm(50))
  agd_df <- data.frame(trt = "B", n_events = 100, pyears = 500, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  agd <- set_agd(agd_df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- naive(dat)
  expect_s3_class(result, "mlumr_naive")
  expect_equal(result$family, "poisson")
  expect_true(is.finite(result$estimate))
  expect_gt(result$se, 0)

  # Manual check
  rate_A <- sum(ipd_df$events) / sum(ipd_df$pyears)
  rate_B <- 100 / 500
  expected_lrr <- log(rate_A) - log(rate_B)
  expect_equal(result$estimate, expected_lrr, tolerance = 1e-10)

  # Print works
  expect_output(print(result), "Log Rate Ratio")
})

# Tests for conditional_effects() and conditional_predict()
# These require a fitted model so they are slow.

make_conditional_fit <- function() {
  draws <- data.frame(
    mu_index = c(1000, 1),
    mu_comparator = c(-1000, 0),
    check.names = FALSE
  )
  draws[["beta[1]"]] <- c(0, 0.5)

  structure(
    list(
      draws = draws,
      family = "binomial",
      link = "logit",
      model = "spfa",
      data = list(
        covariates = "x",
        ipd = list(data = data.frame(x = c(0, 2))),
        index_treatment = "A",
        comparator_treatment = "B"
      )
    ),
    class = c("mlumr_fit", "list")
  )
}


test_that("conditional summaries validate arguments and newdata", {
  fit <- make_conditional_fit()

  expect_error(conditional_effects(list()), "`object` must be an mlumr_fit object")
  expect_error(conditional_effects(fit, effect = c("rd", "rr")),
               "`effect` must be a single non-missing string")
  expect_error(conditional_effects(fit, summary = NA),
               "`summary` must be TRUE or FALSE")
  expect_error(conditional_effects(fit, probs = c(0.5, 0.5)),
               "`probs` must be unique")

  expect_error(conditional_predict(fit, type = "r"), "`type` must be one of")
  expect_error(conditional_predict(fit, newdata = list(x = 0)),
               "`newdata` must be a data frame")
  expect_error(conditional_predict(fit, newdata = data.frame(x = numeric(0))),
               "`newdata` must contain at least one row")
  expect_error(conditional_predict(fit, newdata = data.frame(z = 0)),
               "Missing covariates")
  expect_error(conditional_predict(fit, newdata = data.frame(x = "0")),
               "must be numeric")
  expect_error(conditional_predict(fit, newdata = data.frame(x = Inf)),
               "must be finite")

  broken <- fit
  broken$draws[["beta[1]"]] <- NULL
  expect_error(conditional_effects(broken, newdata = data.frame(x = 0)),
               "Missing conditional parameter draw column")
})


test_that("conditional link effects use eta differences directly", {
  fit <- make_conditional_fit()

  effects <- conditional_effects(
    fit,
    newdata = data.frame(x = 0),
    effect = "link_effect",
    summary = FALSE
  )
  expect_equal(effects$link_effect, c(2000, 1))

  lor_alias <- conditional_effects(
    fit,
    newdata = data.frame(x = 0),
    effect = "lor",
    summary = FALSE
  )
  expect_equal(lor_alias$link_effect, c(2000, 1))

  rr <- conditional_effects(
    fit,
    newdata = data.frame(x = 0),
    effect = "rr",
    summary = FALSE
  )
  expect_true(all(is.finite(rr$rr)))

  link_pred <- conditional_predict(
    fit,
    newdata = data.frame(x = 0),
    type = "link",
    summary = FALSE
  )
  expect_equal(link_pred$index, c(1000, 1))
  expect_equal(link_pred$comparator, c(-1000, 0))
})


test_that("conditional_effects works with default and custom newdata", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 80
  x1 <- rbinom(n, 1, 0.5)
  outcome <- rbinom(n, 1, plogis(-0.3 + 0.8 * x1))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1)
  agd_df <- data.frame(trt = "B", n_total = 200, n_events = 70, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  fit <- mlumr(dat, model = "spfa", chains = 2, iter = 500,
               warmup = 250, refresh = 0, seed = 123)

  # Default newdata (IPD means)
  ce <- conditional_effects(fit)
  expect_s3_class(ce, "data.frame")
  expect_true(nrow(ce) > 0)
  expect_true(all(c("profile", "effect", "mean", "sd") %in% names(ce)))

  # Custom newdata
  profiles <- data.frame(x1 = c(0, 1))
  ce2 <- conditional_effects(fit, newdata = profiles)
  expect_s3_class(ce2, "data.frame")
  expect_true(nrow(ce2) >= 2)

  # summary = FALSE returns draws
  ce_draws <- conditional_effects(fit, summary = FALSE)
  expect_s3_class(ce_draws, "data.frame")
  expect_true("profile" %in% names(ce_draws))
  expect_true(nrow(ce_draws) > nrow(ce))
})


test_that("conditional_predict works with response and link types", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 80
  x1 <- rbinom(n, 1, 0.5)
  outcome <- rbinom(n, 1, plogis(-0.3 + 0.8 * x1))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1)
  agd_df <- data.frame(trt = "B", n_total = 200, n_events = 70, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  fit <- mlumr(dat, model = "spfa", chains = 2, iter = 500,
               warmup = 250, refresh = 0, seed = 123)

  # Response type (default)
  cp <- conditional_predict(fit, type = "response")
  expect_s3_class(cp, "data.frame")
  expect_true(nrow(cp) > 0)
  expect_true(all(c("treatment", "mean") %in% names(cp)))

  # Link type
  cp_link <- conditional_predict(fit, type = "link")
  expect_s3_class(cp_link, "data.frame")
  expect_true(nrow(cp_link) > 0)

  # Custom newdata
  profiles <- data.frame(x1 = c(0, 1))
  cp2 <- conditional_predict(fit, newdata = profiles)
  expect_true(nrow(cp2) >= 4)  # 2 profiles x 2 treatments

  # summary = FALSE
  cp_draws <- conditional_predict(fit, summary = FALSE)
  expect_s3_class(cp_draws, "data.frame")
  expect_true(nrow(cp_draws) > nrow(cp))
})

# These tests require Stan compilation and are slow.
# Skip on CRAN and when rstan is not available.

test_that("mlumr fits SPFA model", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 100
  x1 <- rbinom(n, 1, 0.5)
  outcome <- rbinom(n, 1, plogis(-0.3 + 0.8 * x1))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1)
  agd_df <- data.frame(
    trt = "B", n_total = 200, n_events = 70, x1_mean = 0.4
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  fit <- mlumr(dat, model = "spfa", chains = 2, iter = 500,
               warmup = 250, refresh = 0, seed = 123)

  expect_s3_class(fit, "mlumr_fit")
  expect_equal(fit$model, "spfa")
  expect_equal(fit$family, "binomial")
  expect_true("lor_index" %in% names(fit$draws))
  expect_true("lor_comparator" %in% names(fit$draws))

  # Convergence checks
  expect_true(all(fit$summary$Rhat < 1.05, na.rm = TRUE),
              info = sprintf("max Rhat = %.3f", max(fit$summary$Rhat, na.rm = TRUE)))
  expect_true(min(fit$summary$n_eff, na.rm = TRUE) > 50,
              info = sprintf("min ESS = %.1f", min(fit$summary$n_eff, na.rm = TRUE)))
  expect_equal(fit$diagnostics$n_divergent, 0)

  # predict works
  preds <- predict(fit, population = "both")
  expect_s3_class(preds, "data.frame")
  expect_equal(nrow(preds), 4)
  expect_true(all(c("mean", "sd", "treatment", "population") %in% names(preds)))

  # marginal_effects works
  me <- marginal_effects(fit, effect = "lor")
  expect_s3_class(me, "data.frame")
  expect_true(all(me$effect == "LOR"))

  # DIC works
  dic <- calculate_dic(fit)
  expect_s3_class(dic, "mlumr_dic")
  expect_true(is.finite(dic$DIC))
})


test_that("mlumr fits Relaxed model", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 100
  x1 <- rbinom(n, 1, 0.5)
  outcome <- rbinom(n, 1, plogis(-0.3 + 0.8 * x1))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1)
  agd_df <- data.frame(
    trt = "B", n_total = 200, n_events = 70, x1_mean = 0.4
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  # Relaxed model with a single AgD row and one covariate is weakly
  # identified for beta_comparator (see the warning in mlumr()). The
  # settings below — a mildly informative prior on beta plus stronger
  # adaptation — are what a user would realistically use in this regime
  # and they produce stable convergence in short runs.
  fit <- suppressWarnings(mlumr(
    dat, model = "relaxed",
    prior_beta = prior_normal(0, 2),
    chains = 2, iter = 1000, warmup = 500,
    adapt_delta = 0.99, refresh = 0, seed = 123
  ))

  expect_s3_class(fit, "mlumr_fit")
  expect_equal(fit$model, "relaxed")
  expect_true("delta_beta[1]" %in% fit$summary$variable)

  # Convergence checks
  expect_true(all(fit$summary$Rhat < 1.05, na.rm = TRUE),
              info = sprintf("max Rhat = %.3f", max(fit$summary$Rhat, na.rm = TRUE)))
  expect_true(min(fit$summary$n_eff, na.rm = TRUE) > 50,
              info = sprintf("min ESS = %.1f", min(fit$summary$n_eff, na.rm = TRUE)))
  expect_equal(fit$diagnostics$n_divergent, 0)

  # Print and summary work
  expect_output(print(fit), "Relaxed SPFA")
  expect_output(summary(fit), "Relaxed SPFA")
})


test_that("mlumr rejects data without integration points", {
  set.seed(42)
  ipd_df <- data.frame(trt = "A", outcome = rbinom(50, 1, 0.5), x1 = rbinom(50, 1, 0.4))
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 40, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_error(mlumr(dat), "Integration points not found")
})


# ---- Normal family Stan tests ----

test_that("mlumr fits Normal SPFA model", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 100
  x1 <- rbinom(n, 1, 0.5)
  score <- 2.0 + 0.5 * x1 + rnorm(n, 0, 1)

  ipd_df <- data.frame(trt = "A", score = score, x1 = x1)
  agd_df <- data.frame(trt = "B", y_mean = 1.5, se = 0.2, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "score", "x1", family = "normal")
  agd <- set_agd(agd_df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  fit <- mlumr(dat, model = "spfa", chains = 2, iter = 500,
               warmup = 250, refresh = 0, seed = 123)

  expect_s3_class(fit, "mlumr_fit")
  expect_equal(fit$family, "normal")
  expect_equal(fit$model_name, "mlumr_normal_spfa")
  expect_true("delta_index" %in% names(fit$draws))
  expect_true("delta_comparator" %in% names(fit$draws))
  expect_true("sigma" %in% names(fit$draws))
  expect_true("y_index_index" %in% names(fit$draws))

  # Convergence checks
  expect_true(all(fit$summary$Rhat < 1.05, na.rm = TRUE),
              info = sprintf("max Rhat = %.3f", max(fit$summary$Rhat, na.rm = TRUE)))
  expect_true(min(fit$summary$n_eff, na.rm = TRUE) > 50,
              info = sprintf("min ESS = %.1f", min(fit$summary$n_eff, na.rm = TRUE)))
  expect_equal(fit$diagnostics$n_divergent, 0)

  # predict works
  preds <- predict(fit, population = "both")
  expect_equal(nrow(preds), 4)

  # marginal effects (normal family: canonical label is MD, not DELTA)
  me <- marginal_effects(fit)
  expect_true("MD" %in% me$effect)

  # Print works
  expect_output(print(fit), "Normal")
  expect_output(summary(fit), "Mean Differences")
})


test_that("mlumr fits Normal Relaxed model", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 100
  x1 <- rbinom(n, 1, 0.5)
  score <- 2.0 + 0.5 * x1 + rnorm(n, 0, 1)

  ipd_df <- data.frame(trt = "A", score = score, x1 = x1)
  agd_df <- data.frame(trt = "B", y_mean = 1.5, se = 0.2, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "score", "x1", family = "normal")
  agd <- set_agd(agd_df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  fit <- mlumr(dat, model = "relaxed", chains = 2, iter = 500,
               warmup = 250, refresh = 0, seed = 123)

  expect_s3_class(fit, "mlumr_fit")
  expect_equal(fit$model_name, "mlumr_normal_relaxed")
  expect_true("delta_beta[1]" %in% fit$summary$variable)
  expect_true("sigma" %in% names(fit$draws))

  # Convergence checks
  expect_true(all(fit$summary$Rhat < 1.05, na.rm = TRUE),
              info = sprintf("max Rhat = %.3f", max(fit$summary$Rhat, na.rm = TRUE)))
  expect_true(min(fit$summary$n_eff, na.rm = TRUE) > 50,
              info = sprintf("min ESS = %.1f", min(fit$summary$n_eff, na.rm = TRUE)))
  expect_equal(fit$diagnostics$n_divergent, 0)
})


# ---- Poisson family Stan tests ----

test_that("mlumr fits Poisson SPFA model", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 100
  x1 <- rbinom(n, 1, 0.5)
  exposure <- runif(n, 0.5, 2.0)
  events <- rpois(n, exp(0.5 + 0.3 * x1) * exposure)

  ipd_df <- data.frame(trt = "A", events = events, pyears = exposure, x1 = x1)
  agd_df <- data.frame(trt = "B", n_events = 80, pyears = 400, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  agd <- set_agd(agd_df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  fit <- mlumr(dat, model = "spfa", chains = 2, iter = 500,
               warmup = 250, refresh = 0, seed = 123)

  expect_s3_class(fit, "mlumr_fit")
  expect_equal(fit$family, "poisson")
  expect_equal(fit$model_name, "mlumr_poisson_spfa")
  expect_true("delta_index" %in% names(fit$draws))
  expect_true("rate_index_index" %in% names(fit$draws))

  # Convergence checks
  expect_true(all(fit$summary$Rhat < 1.05, na.rm = TRUE),
              info = sprintf("max Rhat = %.3f", max(fit$summary$Rhat, na.rm = TRUE)))
  expect_true(min(fit$summary$n_eff, na.rm = TRUE) > 50,
              info = sprintf("min ESS = %.1f", min(fit$summary$n_eff, na.rm = TRUE)))
  expect_equal(fit$diagnostics$n_divergent, 0)

  # predict works
  preds <- predict(fit, population = "both")
  expect_equal(nrow(preds), 4)

  # marginal effects (poisson family: canonical label is RR, not DELTA)
  me <- marginal_effects(fit)
  expect_true("RR" %in% me$effect)

  # Print works
  expect_output(print(fit), "Poisson")
  expect_output(summary(fit), "Rate Ratios")
})


test_that("mlumr fits Poisson Relaxed model", {
  skip_on_cran()
  skip_if_not_installed("rstan")

  set.seed(42)
  n <- 100
  x1 <- rbinom(n, 1, 0.5)
  exposure <- runif(n, 0.5, 2.0)
  events <- rpois(n, exp(0.5 + 0.3 * x1) * exposure)

  ipd_df <- data.frame(trt = "A", events = events, pyears = exposure, x1 = x1)
  agd_df <- data.frame(trt = "B", n_events = 80, pyears = 400, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  agd <- set_agd(agd_df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  # Tighter priors for relaxed model: beta_comparator is weakly identified

  # with sparse AgD, so prior_beta = normal(0, 10) causes funnel geometry.
  fit <- mlumr(dat, model = "relaxed", chains = 2, iter = 1000,
               warmup = 500, refresh = 0, seed = 123,
               prior_beta = prior_normal(0, 2))

  expect_s3_class(fit, "mlumr_fit")
  expect_equal(fit$model_name, "mlumr_poisson_relaxed")
  expect_true("delta_beta[1]" %in% fit$summary$variable)
  expect_true("rate_index_index" %in% names(fit$draws))

  # Convergence checks
  expect_true(all(fit$summary$Rhat < 1.05, na.rm = TRUE),
              info = sprintf("max Rhat = %.3f", max(fit$summary$Rhat, na.rm = TRUE)))
  expect_true(min(fit$summary$n_eff, na.rm = TRUE) > 50,
              info = sprintf("min ESS = %.1f", min(fit$summary$n_eff, na.rm = TRUE)))
  expect_equal(fit$diagnostics$n_divergent, 0)
})

test_that("stc computes LOR with delta-method SE", {
  set.seed(123)
  n <- 200

  # Generate IPD with known effect
  x1 <- rbinom(n, 1, 0.4)
  x2 <- rbinom(n, 1, 0.6)
  logit_p <- -0.5 + 1.0 * x1 - 0.5 * x2
  outcome <- rbinom(n, 1, plogis(logit_p))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1, x2 = x2)
  agd_df <- data.frame(
    trt = "B", n_total = 300, n_events = 120,
    x1_mean = 0.3, x2_mean = 0.7
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", c("x1", "x2"))
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = c("x1_mean", "x2_mean"))
  dat <- combine_data(ipd, agd)

  result <- stc(dat)

  expect_s3_class(result, "mlumr_stc")
  expect_true(is.finite(result$estimate))
  expect_gt(result$se, 0)
  expect_lt(result$ci_lower, result$estimate)
  expect_gt(result$ci_upper, result$estimate)

  # GLM fit should be stored
  expect_s3_class(result$glm_fit, "glm")

  # Print method works
  expect_output(print(result), "Simulated Treatment Comparison")
})


test_that("stc works with integration points", {
  set.seed(42)
  n <- 100
  x1 <- rbinom(n, 1, 0.5)
  outcome <- rbinom(n, 1, plogis(-0.3 + 0.8 * x1))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1)
  agd_df <- data.frame(
    trt = "B", n_total = 200, n_events = 70, x1_mean = 0.4
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  # Add integration points
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  result <- stc(dat)
  expect_s3_class(result, "mlumr_stc")
  expect_true(is.finite(result$estimate))
})


test_that("stc respects custom conf_level", {
  set.seed(123)
  n <- 200
  x1 <- rbinom(n, 1, 0.4)
  outcome <- rbinom(n, 1, plogis(-0.5 + 1.0 * x1))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1)
  agd_df <- data.frame(trt = "B", n_total = 300, n_events = 120, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  r95 <- stc(dat, conf_level = 0.95)
  r90 <- stc(dat, conf_level = 0.90)

  expect_lt(r90$ci_upper - r90$ci_lower, r95$ci_upper - r95$ci_lower)
})


test_that("stc validates data and conf_level", {
  ipd_df <- data.frame(trt = "A", outcome = c(1, 0, 1, 0, 1, 0),
                       x1 = c(0, 1, 0, 1, 0, 1))
  agd_df <- data.frame(trt = "B", n_total = 20, n_events = 8, x1_mean = 0.5)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  expect_error(stc(data.frame()), "combine_data")
  expect_error(stc(dat, conf_level = 1), "`conf_level`")
  expect_error(stc(dat, conf_level = c(0.90, 0.95)), "`conf_level`")
  expect_error(stc(dat, conf_level = NA_real_), "`conf_level`")
})


test_that("stc supports non-syntactic covariate names", {
  set.seed(12)
  n <- 120
  x <- rbinom(n, 1, 0.4)
  outcome <- rbinom(n, 1, plogis(-0.2 + 0.7 * x))
  ipd_df <- data.frame(trt = "A", outcome = outcome, check.names = FALSE)
  ipd_df[["x one"]] <- x
  agd_df <- data.frame(trt = "B", n_total = 180, n_events = 70,
                       check.names = FALSE)
  agd_df[["x one_mean"]] <- 0.3

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x one")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x one_mean")
  dat <- combine_data(ipd, agd)

  result <- stc(dat)
  expect_s3_class(result, "mlumr_stc")
  expect_true(is.finite(result$estimate))
  expect_true(is.finite(result$se))
})


test_that("stc handles multi-row AgD with weighted integration", {
  set.seed(42)
  n <- 150
  x1 <- rbinom(n, 1, 0.5)
  outcome <- rbinom(n, 1, plogis(-0.3 + 0.8 * x1))

  ipd_df <- data.frame(trt = "A", outcome = outcome, x1 = x1)
  agd_df <- data.frame(
    trt = c("B", "B"),
    n_total = c(100, 200),
    n_events = c(35, 80),
    x1_mean = c(0.3, 0.5)
  )

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  # Without integration points
  result_no_int <- stc(dat)
  expect_s3_class(result_no_int, "mlumr_stc")
  expect_true(is.finite(result_no_int$estimate))

  # With integration points
  dat <- add_integration(dat, n_int = 32, x1 = distr(qbern, prob = x1_mean))
  result_int <- stc(dat)
  expect_s3_class(result_int, "mlumr_stc")
  expect_true(is.finite(result_int$estimate))
})


# ---- Normal family ----

test_that("stc works with normal family", {
  set.seed(123)
  n <- 200
  x1 <- rnorm(n, 0, 1)
  score <- 2.0 + 0.5 * x1 + rnorm(n, 0, 1)

  ipd_df <- data.frame(trt = "A", score = score, x1 = x1)
  agd_df <- data.frame(trt = "B", y_mean = 1.5, se = 0.2, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "score", "x1", family = "normal")
  agd <- set_agd(agd_df, "trt", family = "normal",
                 outcome_mean = "y_mean", outcome_se = "se",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- stc(dat)
  expect_s3_class(result, "mlumr_stc")
  expect_equal(result$family, "normal")
  expect_true(is.finite(result$estimate))
  expect_gt(result$se, 0)

  # Print works
  expect_output(print(result), "Mean Difference")
})


# ---- Poisson family ----

test_that("stc works with poisson family", {
  set.seed(123)
  n <- 200
  x1 <- rbinom(n, 1, 0.4)
  exposure <- runif(n, 0.5, 2.0)
  events <- rpois(n, exp(0.5 + 0.3 * x1) * exposure)

  ipd_df <- data.frame(trt = "A", events = events, pyears = exposure, x1 = x1)
  agd_df <- data.frame(trt = "B", n_events = 80, pyears = 400, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  agd <- set_agd(agd_df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- stc(dat)
  expect_s3_class(result, "mlumr_stc")
  expect_equal(result$family, "poisson")
  expect_true(is.finite(result$estimate))
  expect_gt(result$se, 0)

  # Print works
  expect_output(print(result), "Log Rate Ratio")
})


test_that("stc handles zero-event Poisson comparator data", {
  set.seed(321)
  n <- 200
  x1 <- rbinom(n, 1, 0.4)
  exposure <- runif(n, 0.5, 2.0)
  events <- rpois(n, exp(0.3 + 0.2 * x1) * exposure)

  ipd_df <- data.frame(trt = "A", events = events, pyears = exposure, x1 = x1)
  agd_df <- data.frame(trt = "B", n_events = 0, pyears = 400, x1_mean = 0.3)

  ipd <- set_ipd(ipd_df, "trt", "events", "x1", family = "poisson",
                 exposure = "pyears")
  agd <- set_agd(agd_df, "trt", family = "poisson",
                 outcome_r = "n_events", outcome_E = "pyears",
                 cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)

  result <- stc(dat)
  expect_true(is.finite(result$estimate))
  expect_true(is.finite(result$se))
  expect_equal(result$rate_comparator, 0)
  expect_equal(result$events_comparator_adjusted, 0.5)
})

test_that("default engine is rstan", {
  withr::local_options(list(mlumr.stan_engine = NULL))
  expect_equal(mlumr_engine(), "rstan")
})

test_that("setting engine to rstan works", {
  withr::local_options(list(mlumr.stan_engine = NULL))
  mlumr_engine("rstan")
  expect_equal(mlumr_engine(), "rstan")
})

test_that("setting engine to invalid value errors", {
  expect_error(mlumr_engine("jags"), "`engine` must be 'rstan' or 'cmdstanr'")
  expect_error(mlumr_engine("c"), "`engine` must be 'rstan' or 'cmdstanr'")
  expect_error(mlumr_engine(c("rstan", "cmdstanr")),
               "`engine` must be 'rstan' or 'cmdstanr'")
})

test_that("get_engine returns option value", {
  withr::local_options(list(mlumr.stan_engine = "cmdstanr"))
  expect_equal(mlumr:::get_engine(), "cmdstanr")
})

test_that("get_engine rejects invalid option values", {
  withr::local_options(list(mlumr.stan_engine = "bad"))
  expect_error(mlumr:::get_engine(), "`engine` must be 'rstan' or 'cmdstanr'")
})

test_that("unchanged-engine message tolerates invalid option values", {
  withr::local_options(list(mlumr.stan_engine = "bad"))
  expect_message(value <- mlumr:::.message_engine_unchanged(),
                 "Engine unchanged \\(invalid current option\\)")
  expect_true(is.na(value))
})

test_that("get_engine defaults to rstan when option is NULL", {
  withr::local_options(list(mlumr.stan_engine = NULL))
  expect_equal(mlumr:::get_engine(), "rstan")
})


test_that("cmdstanr backend fits a model end-to-end", {
  skip_on_cran()
  skip_if_not_installed("cmdstanr")
  skip_if_not(
    tryCatch({ cmdstanr::cmdstan_path(); TRUE }, error = function(e) FALSE),
    "CmdStan not available"
  )

  set.seed(42)
  n <- 50
  x1 <- rbinom(n, 1, 0.5)
  y <- rbinom(n, 1, plogis(-0.5 + 0.8 * x1))
  ipd_df <- data.frame(trt = "A", outcome = y, x1 = x1)
  agd_df <- data.frame(trt = "B", n_total = 100, n_events = 35, x1_mean = 0.4)

  ipd <- set_ipd(ipd_df, "trt", "outcome", "x1")
  agd <- set_agd(agd_df, "trt", outcome_n = "n_total",
                 outcome_r = "n_events", cov_means = "x1_mean")
  dat <- combine_data(ipd, agd)
  dat <- add_integration(dat, n_int = 32,
                         x1 = distr(qbern, prob = x1_mean))

  fit <- mlumr(dat, model = "spfa", engine = "cmdstanr",
               chains = 2, iter = 500, warmup = 250, refresh = 0, seed = 123)

  expect_s3_class(fit, "mlumr_fit")
  expect_equal(fit$family, "binomial")
  expect_equal(fit$engine, "cmdstanr")
  expect_true("mu_index" %in% names(fit$draws))
  expect_true("variable" %in% names(fit$summary))
  expect_true("mean" %in% names(fit$summary))
  expect_true("Rhat" %in% names(fit$summary))
  expect_true(is.numeric(fit$diagnostics$n_divergent))
  expect_true(is.numeric(fit$diagnostics$n_max_treedepth))
  expect_false(file.exists(file.path("inst", "stan", "mlumr_binary_spfa")))
})

#' Simulated treatment comparison via G-computation
#'
#' Perform an unanchored simulated treatment comparison (STC) using parametric
#' G-computation. Fits a regression on IPD, predicts counterfactual
#' outcomes in both the index and comparator populations, and computes
#' marginal treatment effects with delta-method standard errors.
#'
#' For binomial outcomes, returns the treatment effect on the link scale plus
#' event probabilities, risk difference, and log risk ratio with SEs and CIs
#' for both populations. Event-probability intervals use Wald standard errors
#' and are bounded to `[0, 1]`. For Poisson outcomes, the comparator log rate
#' uses a 0.5 continuity correction when the observed event count is zero.
#'
#' @param data An `mlumr_data` object from [combine_data()]. Integration points
#'   are not required for STC but covariate information from the AgD is used.
#' @param link Link function. For binomial: `"logit"` (default), `"probit"`,
#'   or `"cloglog"`. For normal: `"identity"` (default) or `"log"`. For
#'   poisson: `"log"` (default). If `NULL`, uses the canonical default.
#' @param conf_level Confidence level for the interval (default 0.95)
#'
#' @return An object of class `mlumr_stc`
#' @importFrom stats gaussian poisson dnorm
#' @export
#'
#' @details
#' The STC procedure is:
#' 1. Fit a GLM on IPD (binomial/gaussian/poisson as appropriate).
#' 2. Predict on comparator-population covariates (from integration points or
#'    AgD covariate means).
#' 3. Marginalize predictions over the comparator population.
#' 4. Predict on index-population covariates (IPD).
#' 5. Compute treatment effects and SEs via the delta method.
#'
#' STC is a parametric G-computation benchmark. It relies on the IPD outcome
#' model being correctly specified and transportable to the comparator
#' population. It does not model posterior uncertainty in population
#' covariate distributions or relax treatment-specific covariate effects.
#' When clinically meaningful effect modification is plausible, prefer
#' `mlumr(..., model = "relaxed")` as the primary analysis and use STC as a
#' sensitivity or benchmarking analysis.
#'
#' For binomial outcomes, the comparator-population treatment contrast is
#' transported to the index population **assuming the treatment contrast is
#' constant on the fitted link scale** (i.e., no effect modification on that
#' scale). Under this assumption, the index-population comparator
#' probability is computed as
#' `inv_link(link(p_A_index) - (link(p_A_comp) - link(p_B)))`, and its
#' uncertainty is propagated through the delta method. If effect modification
#' is expected, fit a Bayesian relaxed model with
#' `mlumr(..., model = "relaxed")` and use `predict(..., population =
#' "index")` instead, which does not require this assumption.
#'
#' @examples
#' \dontrun{
#' result <- stc(dat)
#' print(result)
#' }
stc <- function(data, link = NULL, conf_level = 0.95) {

  .validate_mlumr_data_object(data)

  family <- data$family %||% "binomial"
  ipd <- data$ipd$data
  agd <- data$agd$data
  cov_names <- data$covariates
  z <- .z_from_conf_level(conf_level)

  link_info <- check_link(family, link)
  link_resolved <- link_info$link

  if (family == "binomial") {
    glm_family <- binomial(link = link_resolved)
  } else if (family == "normal") {
    glm_family <- gaussian(link = link_resolved)
  } else {
    glm_family <- poisson(link = link_resolved)
  }

  fit <- glm(.stc_formula(cov_names, family), family = glm_family, data = ipd)
  glm_params <- .stc_glm_parameters(fit)

  comparator <- .stc_comparator_data(data, cov_names, family)
  newdata <- comparator$newdata
  n_int <- comparator$n_int

  beta_hat <- glm_params$beta_hat
  V <- glm_params$V

  out <- switch(family,
    binomial = .stc_binomial(data, fit, ipd, agd, newdata, link_resolved,
                             conf_level, z, beta_hat, V, n_int),
    normal = .stc_normal(data, fit, ipd, agd, newdata, link_resolved,
                         conf_level, z, beta_hat, V, n_int),
    poisson = .stc_poisson(data, fit, ipd, agd, newdata, link_resolved,
                           conf_level, z, beta_hat, V, n_int)
  )

  class(out) <- c("mlumr_stc", "list")
  out
}

#' Build the STC GLM formula without assuming syntactic covariate names
#' @keywords internal
.stc_formula <- function(cov_names, family) {
  terms <- lapply(cov_names, as.name)
  if (family == "poisson") {
    terms <- c(terms, list(call("offset", call("log", as.name(".exposure")))))
  }
  rhs <- Reduce(function(left, right) call("+", left, right), terms)
  stats::as.formula(call("~", as.name(".outcome"), rhs),
                    env = parent.frame())
}

#' Validate fitted STC GLM parameters before delta-method calculations
#' @keywords internal
.stc_glm_parameters <- function(fit) {
  if (!isTRUE(fit$converged)) {
    stop("STC GLM did not converge; check the IPD model or use mlumr().",
         call. = FALSE)
  }
  beta_hat <- coef(fit)
  V <- vcov(fit)
  if (anyNA(beta_hat) || anyNA(V)) {
    stop(
      paste(
        "STC GLM produced aliased coefficients or covariance terms.",
        "Check for collinear covariates or insufficient variation."
      ),
      call. = FALSE
    )
  }
  if (any(!is.finite(beta_hat)) || any(!is.finite(V))) {
    stop(
      paste(
        "STC GLM produced non-finite coefficients or covariance terms.",
        "This can occur with separation, collinearity, or insufficient",
        "outcome variation; consider mlumr() for a Bayesian fit."
      ),
      call. = FALSE
    )
  }
  list(beta_hat = beta_hat, V = V)
}

#' Build a model matrix aligned with fitted GLM coefficients
#' @keywords internal
.stc_model_matrix <- function(fit, newdata) {
  X <- stats::model.matrix(stats::delete.response(stats::terms(fit)), newdata)
  beta_names <- names(coef(fit))
  missing_cols <- setdiff(beta_names, colnames(X))
  if (length(missing_cols) > 0L) {
    stop(sprintf("Cannot build STC model matrix columns: %s",
                 paste(missing_cols, collapse = ", ")), call. = FALSE)
  }
  X[, beta_names, drop = FALSE]
}

#' Binomial STC estimator
#' @keywords internal
.stc_binomial <- function(data, fit, ipd, agd, newdata, link_resolved,
                          conf_level, z, beta_hat, V, n_int) {
  pred_probs <- predict(fit, newdata = newdata, type = "response")
  weights <- if (data$has_integration) rep(agd$.n, each = n_int) else agd$.n

  p_hat_A_comp <- weighted.mean(pred_probs, weights)
  n_B <- sum(agd$.n)
  p_B <- sum(agd$.r) / n_B

  p_hat_A_comp <- bound_probability(p_hat_A_comp, n_B)
  p_B <- bound_probability(p_B, n_B)

  comp_delta <- .stc_binomial_comparator_delta(
    fit, newdata, weights, beta_hat, V, link_resolved,
    p_hat_A_comp, p_B, n_B
  )
  index_delta <- .stc_binomial_index_delta(
    fit, ipd, beta_hat, V, link_resolved,
    p_hat_A_comp, p_B, comp_delta
  )

  estimate <- comp_delta$link_effect
  se <- comp_delta$link_effect_se
  var_p_B <- p_B * (1 - p_B) / n_B
  p_hat_A_comp_se <- .sqrt_variance(comp_delta$var_p_A,
                                    "comparator probability variance")
  p_B_se <- sqrt(var_p_B)
  p_A_index_se <- .sqrt_variance(index_delta$var_p_A,
                                 "index probability variance")
  p_B_index_se <- .sqrt_variance(index_delta$var_p_B,
                                 "index comparator probability variance")
  p_hat_A_comp_ci <- .bounded_wald_interval(p_hat_A_comp, p_hat_A_comp_se, z,
                                            lower = 0, upper = 1)
  p_B_ci <- .bounded_wald_interval(p_B, p_B_se, z, lower = 0, upper = 1)
  p_A_index_ci <- .bounded_wald_interval(index_delta$p_A, p_A_index_se, z,
                                         lower = 0, upper = 1)
  p_B_index_ci <- .bounded_wald_interval(index_delta$p_B, p_B_index_se, z,
                                         lower = 0, upper = 1)

  rd <- p_hat_A_comp - p_B
  se_rd <- .sqrt_variance(comp_delta$var_p_A + var_p_B,
                          "risk-difference variance")

  log_rr <- log(p_hat_A_comp) - log(p_B)
  se_log_rr <- .sqrt_variance(
    comp_delta$var_p_A / (p_hat_A_comp^2) +
      (1 - p_B) / (n_B * p_B),
    "log-risk-ratio variance"
  )

  list(
    estimate = estimate,
    link_effect = estimate,
    se = se,
    ci_lower = estimate - z * se,
    ci_upper = estimate + z * se,
    conf_level = conf_level,
    family = "binomial",
    link = link_resolved,
    p_hat_index = p_hat_A_comp,
    p_hat_index_se = p_hat_A_comp_se,
    p_hat_index_lower = p_hat_A_comp_ci$lower,
    p_hat_index_upper = p_hat_A_comp_ci$upper,
    p_comparator = p_B,
    p_comparator_se = p_B_se,
    p_comparator_lower = p_B_ci$lower,
    p_comparator_upper = p_B_ci$upper,
    p_index_index = index_delta$p_A,
    p_index_index_se = p_A_index_se,
    p_index_index_lower = p_A_index_ci$lower,
    p_index_index_upper = p_A_index_ci$upper,
    p_comparator_index = index_delta$p_B,
    p_comparator_index_se = p_B_index_se,
    p_comparator_index_lower = p_B_index_ci$lower,
    p_comparator_index_upper = p_B_index_ci$upper,
    rd = rd,
    rd_se = se_rd,
    rd_lower = rd - z * se_rd,
    rd_upper = rd + z * se_rd,
    log_rr = log_rr,
    log_rr_se = se_log_rr,
    log_rr_lower = log_rr - z * se_log_rr,
    log_rr_upper = log_rr + z * se_log_rr,
    glm_fit = fit,
    data = data
  )
}

#' Comparator-population delta-method terms for binomial STC
#' @keywords internal
.stc_binomial_comparator_delta <- function(fit, newdata, weights,
                                           beta_hat, V, link_resolved,
                                           p_A, p_B, n_B) {
  X_comp_design <- .stc_model_matrix(fit, newdata)
  eta_comp <- as.vector(X_comp_design %*% beta_hat)
  p_comp <- inverse_link(eta_comp, link_resolved)
  w_norm <- weights / sum(weights)

  dp_dbeta <- inverse_link_derivative(eta_comp, p_comp, link_resolved)
  grad_mean <- colSums(w_norm * dp_dbeta * X_comp_design)
  grad_link <- grad_mean * link_derivative_response(
    sum(w_norm * p_comp), link_resolved
  )

  var_link_A <- as.numeric(t(grad_link) %*% V %*% grad_link)
  var_link_B <- binomial_link_variance(p_B, n_B, link_resolved)
  var_link_effect <- .nonnegative_variance(var_link_A + var_link_B,
                                           "comparator link-effect variance")
  var_p_A <- .nonnegative_variance(as.numeric(t(grad_mean) %*% V %*% grad_mean),
                                   "comparator probability variance")

  list(
    link_effect = link_fun(p_A, link_resolved) - link_fun(p_B, link_resolved),
    link_effect_se = sqrt(var_link_effect),
    var_link_effect = var_link_effect,
    var_p_A = var_p_A
  )
}

#' Index-population delta-method terms for binomial STC
#' @keywords internal
.stc_binomial_index_delta <- function(fit, ipd, beta_hat, V,
                                      link_resolved, p_A_comp, p_B_comp,
                                      comp_delta) {
  n_ipd <- nrow(ipd)
  p_A_index <- mean(predict(fit, newdata = ipd, type = "response"))
  p_A_index <- bound_probability(p_A_index, n_ipd)

  link_effect_comp <- link_fun(p_A_comp, link_resolved) -
    link_fun(p_B_comp, link_resolved)
  eta_B_index <- link_fun(p_A_index, link_resolved) - link_effect_comp
  p_B_index <- inverse_link(eta_B_index, link_resolved)
  p_B_index <- bound_probability(p_B_index, n_ipd)

  X_ipd_design <- .stc_model_matrix(fit, ipd)
  eta_ipd <- as.vector(X_ipd_design %*% beta_hat)
  p_ipd <- inverse_link(eta_ipd, link_resolved)
  dp_dbeta_ipd <- inverse_link_derivative(eta_ipd, p_ipd, link_resolved)
  grad_mean <- colMeans(dp_dbeta_ipd * X_ipd_design)
  var_p_A_index <- .nonnegative_variance(
    as.numeric(t(grad_mean) %*% V %*% grad_mean),
    "index probability variance"
  )

  dg_dp_A <- link_derivative_response(p_A_index, link_resolved)
  d_inv_link <- inverse_link_derivative(eta_B_index, p_B_index, link_resolved)
  var_p_B_index <- .nonnegative_variance(
    (d_inv_link * dg_dp_A)^2 * var_p_A_index +
      d_inv_link^2 * comp_delta$var_link_effect,
    "index comparator probability variance"
  )

  list(
    p_A = p_A_index,
    p_B = p_B_index,
    var_p_A = var_p_A_index,
    var_p_B = var_p_B_index
  )
}

#' Normal-outcome STC estimator
#' @keywords internal
.stc_normal <- function(data, fit, ipd, agd, newdata, link_resolved,
                        conf_level, z, beta_hat, V, n_int) {
  pred_y <- predict(fit, newdata = newdata, type = "response")
  weights <- if (data$has_integration) {
    rep(1 / agd$.se^2, each = n_int)
  } else {
    1 / (agd$.se^2)
  }
  y_hat_A <- weighted.mean(pred_y, weights)
  y_B <- weighted.mean(agd$.y, 1 / agd$.se^2)
  estimate <- y_hat_A - y_B

  X_comp_design <- .stc_model_matrix(fit, newdata)
  w_norm <- weights / sum(weights)
  grad <- if (link_resolved == "identity") {
    colSums(w_norm * X_comp_design)
  } else {
    eta_comp <- as.vector(X_comp_design %*% beta_hat)
    colSums(w_norm * exp(eta_comp) * X_comp_design)
  }

  var_A <- .nonnegative_variance(as.numeric(t(grad) %*% V %*% grad),
                                 "normal STC index variance")
  var_B <- 1 / sum(1 / agd$.se^2)
  se <- .sqrt_variance(var_A + var_B, "normal STC contrast variance")
  y_hat_A_index <- mean(predict(fit, newdata = ipd, type = "response"))

  list(
    estimate = estimate,
    se = se,
    ci_lower = estimate - z * se,
    ci_upper = estimate + z * se,
    conf_level = conf_level,
    family = "normal",
    link = link_resolved,
    y_hat_index = y_hat_A,
    y_hat_index_se = sqrt(var_A),
    y_comparator = y_B,
    y_comparator_se = sqrt(var_B),
    y_hat_index_index = y_hat_A_index,
    glm_fit = fit,
    data = data
  )
}

#' Poisson-outcome STC estimator
#' @keywords internal
.stc_poisson <- function(data, fit, ipd, agd, newdata, link_resolved,
                         conf_level, z, beta_hat, V, n_int) {
  pred_rate <- predict(fit, newdata = newdata, type = "response")
  weights <- if (data$has_integration) rep(agd$.E, each = n_int) else agd$.E
  rate_hat_A <- weighted.mean(pred_rate, weights)

  events_B <- sum(agd$.r)
  exposure_B <- sum(agd$.E)
  rate_B <- events_B / exposure_B
  events_B_adjusted <- max(events_B, 0.5)
  rate_B_for_log <- events_B_adjusted / exposure_B
  estimate <- log(rate_hat_A) - log(rate_B_for_log)

  X_comp_design <- .stc_model_matrix(fit, newdata)
  eta_comp <- as.vector(X_comp_design %*% beta_hat)
  lambda_comp <- exp(eta_comp)
  w_norm <- weights / sum(weights)

  grad_log_rate <- colSums(w_norm * lambda_comp * X_comp_design) /
    sum(w_norm * lambda_comp)
  var_lrr_A <- .nonnegative_variance(
    as.numeric(t(grad_log_rate) %*% V %*% grad_log_rate),
    "poisson STC log-rate variance"
  )
  var_lrr_B <- 1 / events_B_adjusted
  se <- .sqrt_variance(var_lrr_A + var_lrr_B,
                       "poisson STC contrast variance")

  grad_rate <- colSums(w_norm * lambda_comp * X_comp_design)
  var_rate_A <- .nonnegative_variance(
    as.numeric(t(grad_rate) %*% V %*% grad_rate),
    "poisson STC rate variance"
  )
  rate_hat_A_index <- mean(predict(fit, newdata = ipd, type = "response"))

  list(
    estimate = estimate,
    se = se,
    ci_lower = estimate - z * se,
    ci_upper = estimate + z * se,
    conf_level = conf_level,
    family = "poisson",
    link = link_resolved,
    rate_hat_index = rate_hat_A,
    rate_hat_index_se = sqrt(var_rate_A),
    rate_comparator = rate_B,
    rate_comparator_se = sqrt(rate_B / exposure_B),
    rate_hat_index_index = rate_hat_A_index,
    events_comparator = events_B,
    exposure_comparator = exposure_B,
    events_comparator_adjusted = events_B_adjusted,
    glm_fit = fit,
    data = data
  )
}

#' Build comparator-population covariates for STC prediction
#' @keywords internal
.stc_comparator_data <- function(data, cov_names, family) {
  agd <- data$agd$data
  n_int <- NULL

  if (data$has_integration) {
    X_int <- data$integration_points
    n_agd_rows <- dim(X_int)[1]
    n_int <- dim(X_int)[2]

    X_comp <- matrix(NA_real_,
                     nrow = n_agd_rows * n_int,
                     ncol = length(cov_names))
    colnames(X_comp) <- cov_names
    for (k in seq_len(n_agd_rows)) {
      rows <- ((k - 1) * n_int + 1):(k * n_int)
      X_comp[rows, ] <- X_int[k, , ]
    }
    newdata <- as.data.frame(X_comp)
  } else {
    newdata <- .stc_agd_mean_newdata(agd, cov_names)
  }

  if (family == "poisson") {
    newdata$.exposure <- 1
  }

  list(newdata = newdata, n_int = n_int)
}

#' Build comparator-population covariates from AgD means
#' @keywords internal
.stc_agd_mean_newdata <- function(agd, cov_names) {
  newdata <- data.frame(row.names = seq_len(nrow(agd)))
  for (cov in cov_names) {
    mean_col <- paste0(cov, "_mean")
    if (!mean_col %in% names(agd)) {
      stop(sprintf(
        paste0(
          "Cannot find mean for covariate '%s' in AgD. ",
          "Either add integration points or ensure AgD has '%s' column."
        ),
        cov, mean_col
      ), call. = FALSE)
    }
    newdata[[cov]] <- agd[[mean_col]]
  }
  newdata
}

#' Fit a Stan model using cmdstanr
#' @keywords internal
fit_cmdstanr <- function(model_name, stan_data, chains, iter, warmup,
                         seed, adapt_delta, max_treedepth, refresh, ...) {

  if (!requireNamespace("cmdstanr", quietly = TRUE)) {
    stop("cmdstanr is required but not installed. Run mlumr_engine('cmdstanr') to set up.",
         call. = FALSE)
  }

  # Locate .stan file from installed package
  stan_file <- system.file("stan", paste0(model_name, ".stan"), package = "mlumr")
  if (stan_file == "") {
    # Development mode fallback
    stan_file <- file.path("inst", "stan", paste0(model_name, ".stan"))
  }
  if (!file.exists(stan_file)) {
    stop(sprintf("Cannot find Stan model file: %s.stan", model_name), call. = FALSE)
  }

  # Keep CmdStan executables out of inst/stan. cmdstanr's default executable
  # path is next to the .stan file, which pollutes the source tree in
  # development and installed package directories in some workflows.
  compile_dir <- .cmdstanr_compile_dir(model_name, stan_file)
  mod <- cmdstanr::cmdstan_model(stan_file, dir = compile_dir)

  # Sample (note arg name differences from rstan).
  # Default parallel_chains to mirror rstan's behavior (which uses mc.cores).
  # cmdstanr defaults parallel_chains = 1, so chains run serially unless we
  # set it explicitly. Cap at `chains` so we never request more workers than
  # chains. Honor any user override passed through `...`.
  dots <- list(...)
  if (!"parallel_chains" %in% names(dots)) {
    dots$parallel_chains <- min(chains,
                                max(1L, as.integer(getOption("mc.cores", 1L))))
  }

  sample_args <- c(
    list(
      data = stan_data,
      chains = chains,
      iter_sampling = iter - warmup,
      iter_warmup = warmup,
      seed = seed,
      adapt_delta = adapt_delta,
      max_treedepth = max_treedepth,
      refresh = refresh
    ),
    dots
  )
  fit <- do.call(mod$sample, sample_args)

  # Extract draws as plain data.frame (drop metadata columns)
  draws_df <- as.data.frame(fit$draws(format = "df"))
  meta_cols <- c(".chain", ".iteration", ".draw")
  draws_df <- draws_df[, !names(draws_df) %in% meta_cols, drop = FALSE]

  # Build summary matching rstan column names:
  # variable, mean, se_mean, sd, 2.5%, 25%, 50%, 75%, 97.5%, n_eff, Rhat
  if (requireNamespace("posterior", quietly = TRUE)) {
    cmdstan_summ <- fit$summary(
      variables = NULL,
      mean = mean,
      se_mean = function(.x) stats::sd(.x) / sqrt(posterior::ess_bulk(.x)),
      sd = stats::sd,
      `2.5%` = function(.x) stats::quantile(.x, 0.025),
      `25%` = function(.x) stats::quantile(.x, 0.25),
      `50%` = function(.x) stats::quantile(.x, 0.50),
      `75%` = function(.x) stats::quantile(.x, 0.75),
      `97.5%` = function(.x) stats::quantile(.x, 0.975),
      n_eff = posterior::ess_bulk,
      Rhat = posterior::rhat
    )
    summary_df <- as.data.frame(cmdstan_summ)
  } else {
    # Fallback when posterior is not installed: basic summary with NA for
    # ESS and Rhat. We deliberately do NOT impute n_eff with the total
    # draw count (or any other plug-in) because that would silently report
    # wildly optimistic sample sizes and mask autocorrelation. Installing
    # 'posterior' is required for valid convergence diagnostics.
    warning(
      "posterior package not installed. ESS (n_eff) and Rhat will be NA. ",
      "Install with: install.packages('posterior') for valid diagnostics.",
      call. = FALSE
    )
    var_names <- colnames(draws_df)
    summary_df <- data.frame(
      variable = var_names,
      mean = vapply(draws_df, mean, numeric(1)),
      se_mean = rep(NA_real_, length(var_names)),
      sd = vapply(draws_df, stats::sd, numeric(1)),
      `2.5%` = vapply(draws_df, function(.x) stats::quantile(.x, 0.025), numeric(1)),
      `25%` = vapply(draws_df, function(.x) stats::quantile(.x, 0.25), numeric(1)),
      `50%` = vapply(draws_df, function(.x) stats::quantile(.x, 0.50), numeric(1)),
      `75%` = vapply(draws_df, function(.x) stats::quantile(.x, 0.75), numeric(1)),
      `97.5%` = vapply(draws_df, function(.x) stats::quantile(.x, 0.975), numeric(1)),
      n_eff = rep(NA_real_, length(var_names)),
      Rhat = rep(NA_real_, length(var_names)),
      check.names = FALSE, stringsAsFactors = FALSE
    )
  }

  # Diagnostics
  diag <- fit$diagnostic_summary(quiet = TRUE)
  n_divergent <- sum(diag$num_divergent)
  n_max_td <- sum(diag$num_max_treedepth)

  list(
    native_fit = fit,
    draws = draws_df,
    summary_df = summary_df,
    n_divergent = n_divergent,
    n_max_td = n_max_td
  )
}


#' Cache directory for cmdstanr-compiled model executables
#' @keywords internal
.cmdstanr_compile_dir <- function(model_name, stan_file) {
  stan_dir <- dirname(stan_file)
  include_dir <- file.path(stan_dir, "include")
  include_files <- if (dir.exists(include_dir)) {
    sort(list.files(include_dir, pattern = "[.]stan$", full.names = TRUE))
  } else {
    character()
  }

  source_files <- c(stan_file, include_files)
  source_hash <- paste(unname(tools::md5sum(source_files)), collapse = "")
  cache_key <- substr(source_hash, 1L, 32L)
  cache_dir <- file.path(.cmdstanr_cache_root(), paste0(model_name, "-", cache_key))

  if (!dir.exists(cache_dir)) {
    dir.create(cache_dir, recursive = TRUE, showWarnings = FALSE)
  }
  if (!dir.exists(cache_dir) || file.access(cache_dir, mode = 2L) != 0L) {
    stop("Could not create a writable cmdstanr cache directory.", call. = FALSE)
  }
  cache_dir
}


#' Resolve a writable cmdstanr cache root
#' @keywords internal
.cmdstanr_cache_root <- function() {
  candidates <- c(
    file.path(tools::R_user_dir("mlumr", "cache"), "cmdstanr"),
    file.path(tempdir(), "mlumr", "cmdstanr")
  )

  selected <- NULL
  for (cache_root in candidates) {
    if (!dir.exists(cache_root)) {
      dir.create(cache_root, recursive = TRUE, showWarnings = FALSE)
    }
    if (dir.exists(cache_root) && file.access(cache_root, mode = 2L) == 0L) {
      selected <- cache_root
      break
    }
  }

  if (!is.null(selected)) {
    selected
  } else {
    stop("Could not create a writable cmdstanr cache root.", call. = FALSE)
  }
}

.onLoad <- function(libname, pkgname) {
  op <- options()
  op_mlumr <- list(mlumr.stan_engine = "rstan")
  toset <- !(names(op_mlumr) %in% names(op))
  if (any(toset)) options(op_mlumr[toset])
  invisible()
}

.onAttach <- function(libname, pkgname) {
  version <- utils::packageVersion(pkgname)
  packageStartupMessage(sprintf(
    "mlumr %s: Bayesian multilevel unanchored meta-regression. GitHub: https://github.com/choxos/mlumr",
    version
  ))
  invisible()
}

#' Fit a Stan model using rstan
#' @keywords internal
fit_rstan <- function(model_name, stan_data, chains, iter, warmup,
                      seed, adapt_delta, max_treedepth, refresh, ...) {

  fit <- rstan::sampling(
    stanmodels[[model_name]],
    data = stan_data,
    chains = chains,
    iter = iter,
    warmup = warmup,
    seed = seed,
    control = list(adapt_delta = adapt_delta, max_treedepth = max_treedepth),
    refresh = refresh,
    ...
  )

  draws <- as.data.frame(fit)

  summary_stats <- rstan::summary(fit)$summary
  summary_df <- as.data.frame(summary_stats)
  summary_df$variable <- rownames(summary_stats)
  summary_df <- summary_df[, c("variable", setdiff(names(summary_df), "variable"))]

  sp <- rstan::get_sampler_params(fit, inc_warmup = FALSE)
  n_divergent <- sum(vapply(sp, function(x) sum(x[, "divergent__"]), numeric(1)))
  n_max_td <- sum(vapply(sp, function(x) sum(x[, "treedepth__"] >= max_treedepth), numeric(1)))

  list(
    native_fit = fit,
    draws = draws,
    summary_df = summary_df,
    n_divergent = n_divergent,
    n_max_td = n_max_td
  )
}

#' Prior sensitivity analysis for an ML-UMR fit
#'
#' Refit an [mlumr()] model across a grid of `prior_beta` scales (keeping the
#' family, mean, and df fixed) and summarize how the posterior for the
#' marginal treatment effects (`delta_index`, `delta_comparator`) moves. This
#' is the workflow recommended by Vehtari et al.'s prior-choice wiki for
#' judging how much of the posterior is driven by the data versus the prior.
#'
#' @param fit A fitted `mlumr_fit` object to re-fit under alternative priors.
#' @param prior_beta_scales Numeric vector of scales for `prior_beta`.
#'   Default `c(0.5, 1, 2.5, 5, 10)`.
#' @param probs Quantiles for summarizing each posterior
#'   (default `c(0.025, 0.5, 0.975)`).
#' @param verbose Logical; if `FALSE`, suppresses progress messages and final
#'   printed summary table.
#' @param ... Additional arguments forwarded to [mlumr()] on each refit
#'   (e.g. `chains`, `iter`, `refresh`). Sampling defaults otherwise inherit
#'   from the original fit.
#'
#' @details
#' Only the scale of the `prior_beta` family is varied; its distribution
#' (normal / student_t) and mean are preserved so comparisons are apples to
#' apples. `prior_intercept` and `prior_sigma` are carried through
#' unchanged from the original fit. Each value in `prior_beta_scales` is
#' used as the **absolute** scale for every coefficient at that refit --- if
#' the original fit used per-coefficient priors, all coefficients are set
#' to the same scale (the sweep is deliberately homogeneous so the grid
#' reflects a single level of prior informativeness per refit, not a
#' rescaling of existing relative differences). If the original
#' `prior_beta` used an exponential family, it is swapped for a
#' `prior_normal(0, scale)` at each grid point since exponential has no
#' scale parameter to vary.
#'
#' @return A data frame (tibble-style) with one row per
#'   (scale, population, quantile) combination and columns `scale`,
#'   `parameter`, `mean`, `sd`, and the requested quantiles. Side effect:
#'   prints a summary table at the end.
#' @seealso [prior_summary()] for a one-shot description of the priors on
#'   a fit; [marginal_effects()] for the posterior summary quantities this
#'   sweep tracks.
#' @export
#' @examples
#' \dontrun{
#' sens <- prior_sensitivity(fit_spfa, prior_beta_scales = c(1, 2.5, 5))
#' }
prior_sensitivity <- function(fit,
                              prior_beta_scales = c(0.5, 1, 2.5, 5, 10),
                              probs = c(0.025, 0.5, 0.975),
                              verbose = TRUE,
                              ...) {

  if (!inherits(fit, "mlumr_fit")) {
    stop("`fit` must be an mlumr_fit object", call. = FALSE)
  }
  invalid_prior_scales <- !is.numeric(prior_beta_scales) ||
    any(!is.finite(prior_beta_scales)) ||
    any(prior_beta_scales <= 0)
  if (invalid_prior_scales) {
    stop("`prior_beta_scales` must be positive finite numbers", call. = FALSE)
  }
  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("`verbose` must be TRUE or FALSE.", call. = FALSE)
  }

  # Extract the base prior so we can reconstruct scaled variants.
  base_beta <- fit$priors$beta
  if (is.null(base_beta)) {
    stop("Original `prior_beta` not found on the fit. ",
         "Was the model fitted with an older version of mlumr?", call. = FALSE)
  }

  # Inherit the original sampling args unless overridden by ...
  sa <- fit$sampling_args %||% list()

  # Re-use the original data object (has_integration is already TRUE)
  data <- fit$data

  results <- vector("list", length(prior_beta_scales))

  for (i in seq_along(prior_beta_scales)) {
    s <- prior_beta_scales[[i]]
    prior_beta_i <- .rescale_prior_beta(base_beta, s)

    mlumr_message(sprintf("Prior sensitivity: refit %d/%d with scale %g",
                          i, length(prior_beta_scales), s),
                  verbose = verbose)

    fit_i <- mlumr(
      data = data,
      model = fit$model,
      link = fit$link,
      prior_intercept = fit$priors$intercept,
      prior_beta = prior_beta_i,
      prior_sigma = fit$priors$sigma %||% default_prior_sigma(),
      chains       = sa$chains       %||% 4,
      iter         = sa$iter         %||% 2000,
      warmup       = sa$warmup       %||% 1000,
      seed         = sa$seed,
      adapt_delta  = sa$adapt_delta  %||% 0.95,
      max_treedepth = sa$max_treedepth %||% 15,
      refresh = 0,
      engine = fit$engine,
      verbose = verbose,
      ...
    )

    results[[i]] <- .summarize_sensitivity(fit_i, scale = s, probs = probs)
  }

  out <- do.call(rbind, results)
  rownames(out) <- NULL

  if (verbose) {
    cat("\nPrior sensitivity: posterior of marginal treatment effects\n")
    cat("=========================================================\n\n")
    print(out, row.names = FALSE)
    cat("\nInterpretation: if posterior summaries are approximately constant\n")
    cat("across scales, the inference is data-driven rather than prior-driven.\n")
  }

  invisible(out)
}


# ---- Helpers ---------------------------------------------------------------

#' Rescale a prior_beta's scale, preserving family / mean / df.
#' @keywords internal
.rescale_prior_beta <- function(prior, new_scale) {
  if (is_single_prior(prior)) {
    # Replace the scalar sd field. For exponential (not supported on beta
    # but defensively) fall back to a normal(0, new_scale).
    if (prior$distribution == "exponential") {
      return(prior_normal(mean = 0, sd = new_scale))
    }
    prior$sd <- new_scale
    # Strip default/version tags since this is a user-generated variant.
    prior$default <- NULL
    prior$version <- NULL
    return(prior)
  }
  # Per-coefficient list: rescale each element to new_scale (absolute, not
  # ratio — we want a homogeneous sensitivity sweep).
  lapply(prior, function(p) {
    p$sd <- new_scale
    p$default <- NULL
    p$version <- NULL
    p
  })
}

#' Summarize a sensitivity refit.
#' @keywords internal
.summarize_sensitivity <- function(fit, scale, probs) {
  draws <- fit$draws
  delta_names <- intersect(c("delta_index", "delta_comparator"), colnames(draws))
  if (length(delta_names) == 0L) return(NULL)

  qnames <- paste0("q", round(100 * probs))

  rows <- lapply(delta_names, function(nm) {
    x <- draws[, nm]
    qs <- stats::quantile(x, probs = probs, names = FALSE)
    row <- data.frame(
      scale = scale,
      parameter = nm,
      mean = mean(x),
      sd = stats::sd(x),
      stringsAsFactors = FALSE
    )
    for (j in seq_along(probs)) row[[qnames[[j]]]] <- qs[[j]]
    row
  })
  do.call(rbind, rows)
}

#' Predictions from ML-UMR model
#'
#' Generate population-average absolute-outcome predictions in the index and
#' comparator populations.
#'
#' @param object An `mlumr_fit` object
#' @param population Which population: `"both"`, `"index"`, or `"comparator"`
#' @param type Prediction type: `"response"` or `"link"`. For `"response"`:
#'   probabilities (binomial), means (normal), or rates (poisson). For
#'   `"link"`: mean linear predictor on the fitted link scale (logit, probit,
#'   cloglog, log, or identity). The link-scale values are computed directly
#'   from parameter draws as `E[eta]`, not as `link(E[g^{-1}(eta)])`, to
#'   avoid Jensen's inequality bias.
#' @param summary Return summary statistics (`TRUE`) or full posterior draws (`FALSE`)
#' @param probs Quantiles for summary (default `c(0.025, 0.5, 0.975)`)
#' @param ... Additional arguments (unused)
#'
#' @details
#' **Marginalization on non-identity links.** For `type = "response"` the
#' reported values are `E[g^{-1}(eta)]` — the posterior expectation of the
#' inverse-link-transformed linear predictor — *not* `g^{-1}(E[eta])`. The
#' two differ whenever `g` is non-linear (logit, probit, cloglog, log) by
#' Jensen's inequality. In the index population the expectation is taken
#' over IPD individuals; in the comparator population it is taken over the
#' integration points constructed by [add_integration()] from the AgD
#' moments. This is the correct population-average prediction for an
#' individual randomly drawn from that population, and it matches what the
#' Stan `generated quantities` block computes. For the link scale
#' (`type = "link"`) the reported value is `E[eta]`, a linear functional,
#' and the two interpretations coincide.
#'
#' @return A data frame with predictions. When `type = "link"`, values are
#'   mean linear predictors computed directly from parameter draws (avoiding
#'   Jensen's inequality bias).
#' @seealso [marginal_effects()] for treatment-effect summaries;
#'   [conditional_predict()] and [conditional_effects()] for predictions
#'   at specific covariate profiles.
#' @export
predict.mlumr_fit <- function(object,
                              population = c("both", "index", "comparator"),
                              type = c("response", "link"),
                              summary = TRUE,
                              probs = c(0.025, 0.5, 0.975),
                              ...) {

  .validate_mlumr_fit_object(object)
  population <- .validate_predict_choice(population, c("both", "index", "comparator"),
                                         "population")
  type <- .validate_predict_choice(type, c("response", "link"), "type")
  summary <- .validate_summary_flag(summary)
  .validate_probs(probs)

  family <- object$family %||% "binomial"

  cfg <- get_family_config(family)
  prefix <- cfg$predict_prefix

  all_vars <- paste0(prefix, "_",
                     c("index_index", "comparator_index",
                       "index_comparator", "comparator_comparator"))

  if (population == "index") {
    pred_cols <- all_vars[1:2]
    labels <- data.frame(
      treatment = c(object$data$index_treatment, object$data$comparator_treatment),
      population = "Index",
      stringsAsFactors = FALSE
    )
  } else if (population == "comparator") {
    pred_cols <- all_vars[3:4]
    labels <- data.frame(
      treatment = c(object$data$index_treatment, object$data$comparator_treatment),
      population = "Comparator",
      stringsAsFactors = FALSE
    )
  } else {
    pred_cols <- all_vars
    labels <- data.frame(
      treatment = rep(c(object$data$index_treatment, object$data$comparator_treatment), 2),
      population = rep(c("Index", "Comparator"), each = 2),
      stringsAsFactors = FALSE
    )
  }

  .require_draw_columns(object$draws, pred_cols, "prediction")
  pred_draws <- object$draws[, pred_cols, drop = FALSE]

  if (type == "link") {
    lnk <- object$link %||% cfg$link_default
    if (lnk != "identity") {
      # Compute mean linear predictors directly from parameter draws to avoid
      # Jensen's bias from link(E[inv_link(eta)]) != E[eta].
      pred_draws <- .compute_mean_lp(object, pred_cols)
    }
  }

  if (!summary) return(as.data.frame(pred_draws))

  summary_df <- .summarize_draw_matrix(pred_draws, probs)
  cbind(labels, summary_df, row.names = NULL)
}


#' Marginal treatment effects
#'
#' Extract marginal treatment effects from a fitted ML-UMR model. For
#' binomial: log odds ratio, risk difference, risk ratio. For normal:
#' mean difference. For poisson: rate ratio.
#'
#' @param object An `mlumr_fit` object
#' @param population Which population: `"both"`, `"index"`, or `"comparator"`
#' @param effect Which effect measure. For binomial: `"all"`, `"lor"`, `"rd"`,
#'   or `"rr"`. For normal: `"all"` or `"md"` (mean difference). For poisson:
#'   `"all"` or `"rr"` (rate ratio).
#' @param summary Return summary (`TRUE`) or full draws (`FALSE`)
#' @param probs Quantiles for summary
#'
#' @return A data frame
#' @seealso [predict.mlumr_fit()] for absolute predictions;
#'   [conditional_effects()] for covariate-conditional effects at specific
#'   profiles; [prior_sensitivity()] to check how strongly the marginal
#'   effect depends on `prior_beta`.
#' @export
#' @examples
#' \dontrun{
#' # All effect measures for both populations
#' marginal_effects(fit)
#'
#' # Only the log odds ratio in the index population
#' marginal_effects(fit, population = "index", effect = "lor")
#'
#' # Full posterior draws rather than summary statistics
#' marginal_effects(fit, summary = FALSE)
#' }
marginal_effects <- function(object,
                             population = c("both", "index", "comparator"),
                             effect = "all",
                             summary = TRUE,
                             probs = c(0.025, 0.5, 0.975)) {

  .validate_mlumr_fit_object(object)
  population <- .validate_predict_choice(population, c("both", "index", "comparator"),
                                         "population")
  effect <- .validate_effect_choice(effect)
  summary <- .validate_summary_flag(summary)
  .validate_probs(probs)

  family <- object$family %||% "binomial"
  cfg <- get_family_config(family)

  valid_effects <- c("all", cfg$effect_measures)
  if (!effect %in% valid_effects) {
    stop(sprintf("For %s family, `effect` must be one of: %s",
                 family, paste(valid_effects, collapse = ", ")), call. = FALSE)
  }
  if (effect == "all") {
    vars <- unlist(cfg$marginal_effect_vars, use.names = FALSE)
  } else {
    vars <- cfg$marginal_effect_vars[[effect]]
  }

  if (population == "index") {
    vars <- grep("_index$", vars, value = TRUE)
  } else if (population == "comparator") {
    vars <- grep("_comparator$", vars, value = TRUE)
  }

  .require_draw_columns(object$draws, vars, "marginal effect")
  effect_draws <- object$draws[, vars, drop = FALSE]

  if (!summary) return(as.data.frame(effect_draws))

  summary_df <- .summarize_draw_matrix(effect_draws, probs)

  # Invert marginal_effect_vars to map Stan variable name -> canonical
  # effect label. This is family-aware, so normal yields "MD" and poisson
  # yields "RR" (both use delta_* columns under different effect names)
  # rather than a regex that would collapse both to "DELTA".
  vmap <- cfg$marginal_effect_vars
  var_to_effect <- setNames(
    rep(names(vmap), lengths(vmap)),
    unlist(vmap, use.names = FALSE)
  )
  vars_row <- rownames(summary_df)
  pop_raw <- sub("^.*_(index|comparator)$", "\\1", vars_row)
  labels <- data.frame(
    variable = vars_row,
    effect = toupper(unname(var_to_effect[vars_row])),
    population = paste0(toupper(substring(pop_raw, 1, 1)),
                        substring(pop_raw, 2)),
    stringsAsFactors = FALSE
  )

  cbind(labels, summary_df, row.names = NULL)
}


#' Compute mean linear predictors from parameter draws
#'
#' Internal helper for predict.mlumr_fit with type="link". Computes
#' mean(eta_i) directly from parameter draws instead of applying the link
#' to marginalized response-scale draws, avoiding Jensen's inequality bias.
#'
#' @param object An mlumr_fit object
#' @param pred_cols Character vector of column names to compute
#' @return Data frame with the same columns as pred_cols, on the link scale
#' @keywords internal
.compute_mean_lp <- function(object, pred_cols) {
  draws <- object$draws
  family <- object$family %||% "binomial"
  covariates <- object$data$covariates
  n_cov <- length(covariates)
  is_relaxed <- object$model == "relaxed"
  n_draws <- nrow(draws)

  .require_draw_columns(draws, c("mu_index", "mu_comparator"), "linear predictor")
  mu_idx <- draws$mu_index
  mu_cmp <- draws$mu_comparator

  if (is_relaxed) {
    beta_idx_cols <- paste0("beta_index[", seq_len(n_cov), "]")
    beta_cmp_cols <- paste0("beta_comparator[", seq_len(n_cov), "]")
    .require_draw_columns(draws, c(beta_idx_cols, beta_cmp_cols), "linear predictor")
    beta_idx <- as.matrix(draws[, beta_idx_cols, drop = FALSE])
    beta_cmp <- as.matrix(draws[, beta_cmp_cols, drop = FALSE])
  } else {
    beta_cols <- paste0("beta[", seq_len(n_cov), "]")
    .require_draw_columns(draws, beta_cols, "linear predictor")
    beta_shared <- as.matrix(draws[, beta_cols, drop = FALSE])
    beta_idx <- beta_shared
    beta_cmp <- beta_shared
  }

  # Index population: mean LP over IPD covariates
  X_ipd <- as.matrix(object$data$ipd$data[, covariates, drop = FALSE])
  mean_X_ipd <- colMeans(X_ipd)

  lp_idx_index <- mu_idx + as.vector(beta_idx %*% mean_X_ipd)
  lp_cmp_index <- mu_cmp + as.vector(beta_cmp %*% mean_X_ipd)

  # Comparator population: weighted mean LP over AgD integration points
  X_int <- object$data$integration_points  # [n_agd_rows, n_int, n_cov]
  n_agd_rows <- dim(X_int)[1]

  # Family-appropriate weights (see family_config$comp_weight_field)
  stan_data <- object$stan_data
  wfield <- get_family_config(family)$comp_weight_field
  if (is.null(wfield)) {
    w <- rep(1, n_agd_rows)
  } else {
    w <- as.numeric(stan_data[[wfield]])
  }
  total_w <- sum(w)

  lp_idx_comp <- rep(0, n_draws)
  lp_cmp_comp <- rep(0, n_draws)
  for (k in seq_len(n_agd_rows)) {
    mean_X_k <- colMeans(matrix(X_int[k, , ], ncol = n_cov))
    lp_idx_comp <- lp_idx_comp + (mu_idx + as.vector(beta_idx %*% mean_X_k)) * w[k]
    lp_cmp_comp <- lp_cmp_comp + (mu_cmp + as.vector(beta_cmp %*% mean_X_k)) * w[k]
  }
  lp_idx_comp <- lp_idx_comp / total_w
  lp_cmp_comp <- lp_cmp_comp / total_w

  prefix <- get_family_config(family)$predict_prefix
  all_lp <- data.frame(
    lp_idx_index, lp_cmp_index, lp_idx_comp, lp_cmp_comp
  )
  names(all_lp) <- paste0(
    prefix, "_",
    c("index_index", "comparator_index",
      "index_comparator", "comparator_comparator")
  )

  all_lp[, pred_cols, drop = FALSE]
}


#' Validate an mlumr fit object
#' @keywords internal
.validate_mlumr_fit_object <- function(object) {
  if (!inherits(object, "mlumr_fit")) {
    stop("`object` must be an mlumr_fit object.", call. = FALSE)
  }
  invisible(TRUE)
}


#' Validate an exact scalar prediction argument
#' @keywords internal
.validate_predict_choice <- function(x, choices, name) {
  if (identical(x, choices)) {
    return(choices[[1L]])
  }

  valid <- is.character(x) &&
    length(x) == 1L &&
    !is.na(x) &&
    nzchar(x) &&
    x %in% choices

  if (!valid) {
    stop(sprintf("`%s` must be one of: %s.",
                 name, paste(sprintf("'%s'", choices), collapse = ", ")),
         call. = FALSE)
  }

  x
}


#' Validate a scalar summary flag
#' @keywords internal
.validate_summary_flag <- function(summary) {
  valid <- is.logical(summary) && length(summary) == 1L && !is.na(summary)
  if (!valid) {
    stop("`summary` must be TRUE or FALSE.", call. = FALSE)
  }
  summary
}


#' Validate quantile probabilities
#' @keywords internal
.validate_probs <- function(probs) {
  valid <- is.numeric(probs) &&
    length(probs) > 0L &&
    all(is.finite(probs)) &&
    all(probs >= 0 & probs <= 1) &&
    !anyDuplicated(probs)

  if (!valid) {
    stop("`probs` must be unique finite numeric values between 0 and 1.",
         call. = FALSE)
  }

  invisible(TRUE)
}


#' Validate a scalar marginal-effect choice
#' @keywords internal
.validate_effect_choice <- function(effect) {
  valid <- is.character(effect) &&
    length(effect) == 1L &&
    !is.na(effect) &&
    nzchar(effect)

  if (!valid) {
    stop("`effect` must be a single non-missing string.", call. = FALSE)
  }

  effect
}


#' Require posterior draw columns
#' @keywords internal
.require_draw_columns <- function(draws, columns, context) {
  missing <- setdiff(columns, names(draws))
  if (length(missing) > 0L) {
    stop(sprintf("Missing %s draw column(s): %s.",
                 context, paste(missing, collapse = ", ")),
         call. = FALSE)
  }
  invisible(TRUE)
}

make_predict_fit <- function() {
  draws <- data.frame(
    mu_index = c(0, 1),
    mu_comparator = c(-1, 0),
    check.names = FALSE
  )
  draws[["beta[1]"]] <- c(0.5, 0.5)

  draws[["p_index_index"]] <- c(0.60, 0.70)
  draws[["p_comparator_index"]] <- c(0.40, 0.50)
  draws[["p_index_comparator"]] <- c(0.65, 0.75)
  draws[["p_comparator_comparator"]] <- c(0.45, 0.55)

  draws[["lor_index"]] <- qlogis(draws[["p_index_index"]]) -
    qlogis(draws[["p_comparator_index"]])
  draws[["lor_comparator"]] <- qlogis(draws[["p_index_comparator"]]) -
    qlogis(draws[["p_comparator_comparator"]])
  draws[["rd_index"]] <- draws[["p_index_index"]] -
    draws[["p_comparator_index"]]
  draws[["rd_comparator"]] <- draws[["p_index_comparator"]] -
    draws[["p_comparator_comparator"]]
  draws[["rr_index"]] <- draws[["p_index_index"]] /
    draws[["p_comparator_index"]]
  draws[["rr_comparator"]] <- draws[["p_index_comparator"]] /
    draws[["p_comparator_comparator"]]

  structure(
    list(
      draws = draws,
      family = "binomial",
      link = "logit",
      model = "spfa",
      data = list(
        covariates = "x",
        ipd = list(data = data.frame(x = c(0, 2))),
        integration_points = array(c(1, 3), dim = c(1, 2, 1)),
        index_treatment = "A",
        comparator_treatment = "B"
      ),
      stan_data = list(n_agd = 10L)
    ),
    class = c("mlumr_fit", "list")
  )
}


test_that("predict summarizes response-scale generated quantities", {
  fit <- make_predict_fit()

  pred <- predict(fit, population = "both", probs = 0.5)

  expect_s3_class(pred, "data.frame")
  expect_equal(nrow(pred), 4)
  expect_equal(pred$treatment, c("A", "B", "A", "B"))
  expect_equal(pred$population, c("Index", "Index", "Comparator", "Comparator"))
  expect_equal(pred$mean, c(0.65, 0.45, 0.70, 0.50))
  expect_equal(pred$q50, c(0.65, 0.45, 0.70, 0.50))
})


test_that("predict computes link-scale marginal linear predictors directly", {
  fit <- make_predict_fit()

  link_draws <- predict(fit, type = "link", summary = FALSE)

  expect_equal(link_draws[["p_index_index"]], c(0.5, 1.5))
  expect_equal(link_draws[["p_comparator_index"]], c(-0.5, 0.5))
  expect_equal(link_draws[["p_index_comparator"]], c(1, 2))
  expect_equal(link_draws[["p_comparator_comparator"]], c(0, 1))
})


test_that("predict validates arguments and draw columns", {
  fit <- make_predict_fit()

  expect_error(predict.mlumr_fit(list()), "`object` must be an mlumr_fit object")
  expect_error(predict(fit, population = "i"), "`population` must be one of")
  expect_error(predict(fit, type = "r"), "`type` must be one of")
  expect_error(predict(fit, summary = NA), "`summary` must be TRUE or FALSE")
  expect_error(predict(fit, probs = c(0.5, 0.5)), "`probs` must be unique")

  broken <- fit
  broken$draws["p_index_index"] <- NULL
  expect_error(predict(broken), "Missing prediction draw column")
})


test_that("marginal_effects validates inputs and returns labeled effects", {
  fit <- make_predict_fit()

  me <- marginal_effects(fit, effect = "lor", summary = FALSE)
  expect_equal(names(me), c("lor_index", "lor_comparator"))

  me_summary <- marginal_effects(fit, effect = "rd", probs = 0.5)
  expect_equal(me_summary$effect, c("RD", "RD"))
  expect_equal(me_summary$population, c("Index", "Comparator"))

  expect_error(marginal_effects(fit, effect = c("lor", "rd")),
               "`effect` must be a single non-missing string")
  expect_error(marginal_effects(fit, effect = "bad"),
               "For binomial family")
})

#' Naive unadjusted indirect comparison
#'
#' Compute an unadjusted (naive) indirect treatment comparison by comparing
#' crude outcomes from the IPD and AgD without any covariate adjustment.
#' Returns treatment effect on the link scale plus absolute predictions
#' (event probabilities, risk difference, risk ratio) for both populations.
#'
#' For binomial outcomes, event-probability intervals use Wald standard errors
#' and are bounded to `[0, 1]`. For Poisson outcomes, the log-rate contrast
#' uses a 0.5 continuity correction when an observed event count is zero.
#'
#' @param data An `mlumr_data` object from [combine_data()]
#' @param link Link function. For binomial: `"logit"` (default), `"probit"`,
#'   or `"cloglog"`. For normal/poisson: ignored (identity/log always used).
#'   If `NULL`, uses the canonical default.
#' @param conf_level Confidence level for the interval (default 0.95)
#'
#' @return An object of class `mlumr_naive`
#' @export
#'
#' @examples
#' \dontrun{
#' result <- naive(dat)
#' print(result)
#' }
naive <- function(data, link = NULL, conf_level = 0.95) {

  .validate_mlumr_data_object(data)

  family <- data$family %||% "binomial"
  ipd <- data$ipd$data
  agd <- data$agd$data
  z <- .z_from_conf_level(conf_level)

  out <- switch(
    family,
    binomial = .naive_binomial(data, ipd, agd, link, conf_level, z),
    normal = .naive_normal(data, ipd, agd, conf_level, z),
    poisson = .naive_poisson(data, ipd, agd, conf_level, z),
    stop("Unsupported outcome family.", call. = FALSE)
  )

  class(out) <- c("mlumr_naive", "list")
  out
}


#' Naive comparison for binomial outcomes
#' @keywords internal
.naive_binomial <- function(data, ipd, agd, link, conf_level, z) {
  link_info <- check_link("binomial", link)
  link_resolved <- link_info$link

  n_index <- nrow(ipd)
  n_comparator <- sum(agd$.n)
  p_index <- bound_probability(mean(ipd$.outcome), n_index)
  p_comparator <- bound_probability(sum(agd$.r) / n_comparator, n_comparator)

  estimate <- link_fun(p_index, link_resolved) -
    link_fun(p_comparator, link_resolved)
  se <- sqrt(
    binomial_link_variance(p_index, n_index, link_resolved) +
      binomial_link_variance(p_comparator, n_comparator, link_resolved)
  )

  p_index_se <- sqrt(p_index * (1 - p_index) / n_index)
  p_comparator_se <- sqrt(p_comparator * (1 - p_comparator) / n_comparator)
  p_index_ci <- .bounded_wald_interval(p_index, p_index_se, z,
                                       lower = 0, upper = 1)
  p_comparator_ci <- .bounded_wald_interval(p_comparator, p_comparator_se, z,
                                            lower = 0, upper = 1)
  rd <- p_index - p_comparator
  rd_se <- sqrt(p_index_se^2 + p_comparator_se^2)
  log_rr <- log(p_index) - log(p_comparator)
  log_rr_se <- sqrt(
    (1 - p_index) / (n_index * p_index) +
      (1 - p_comparator) / (n_comparator * p_comparator)
  )

  list(
    estimate = estimate,
    link_effect = estimate,
    se = se,
    ci_lower = estimate - z * se,
    ci_upper = estimate + z * se,
    conf_level = conf_level,
    family = "binomial",
    link = link_resolved,
    p_index = p_index,
    p_index_se = p_index_se,
    p_index_lower = p_index_ci$lower,
    p_index_upper = p_index_ci$upper,
    p_comparator = p_comparator,
    p_comparator_se = p_comparator_se,
    p_comparator_lower = p_comparator_ci$lower,
    p_comparator_upper = p_comparator_ci$upper,
    rd = rd,
    rd_se = rd_se,
    rd_lower = rd - z * rd_se,
    rd_upper = rd + z * rd_se,
    log_rr = log_rr,
    log_rr_se = log_rr_se,
    log_rr_lower = log_rr - z * log_rr_se,
    log_rr_upper = log_rr + z * log_rr_se,
    n_index = n_index,
    n_comparator = n_comparator,
    data = data
  )
}


#' Naive comparison for normal outcomes
#' @keywords internal
.naive_normal <- function(data, ipd, agd, conf_level, z) {
  mean_index <- mean(ipd$.outcome)
  n_index <- nrow(ipd)
  var_index <- var(ipd$.outcome) / n_index

  iv_weights <- 1 / (agd$.se^2)
  mean_comparator <- weighted.mean(agd$.y, iv_weights)
  var_comparator <- 1 / sum(iv_weights)

  estimate <- mean_index - mean_comparator
  se <- sqrt(var_index + var_comparator)
  mean_index_se <- sqrt(var_index)
  mean_comparator_se <- sqrt(var_comparator)

  list(
    estimate = estimate,
    se = se,
    ci_lower = estimate - z * se,
    ci_upper = estimate + z * se,
    conf_level = conf_level,
    family = "normal",
    mean_index = mean_index,
    mean_index_se = mean_index_se,
    mean_index_lower = mean_index - z * mean_index_se,
    mean_index_upper = mean_index + z * mean_index_se,
    mean_comparator = mean_comparator,
    mean_comparator_se = mean_comparator_se,
    mean_comparator_lower = mean_comparator - z * mean_comparator_se,
    mean_comparator_upper = mean_comparator + z * mean_comparator_se,
    n_index = n_index,
    data = data
  )
}


#' Naive comparison for Poisson outcomes
#' @keywords internal
.naive_poisson <- function(data, ipd, agd, conf_level, z) {
  n_index <- nrow(ipd)
  events_index <- sum(ipd$.outcome)
  exposure_index <- sum(ipd$.exposure)
  rate_index <- events_index / exposure_index

  events_comparator <- sum(agd$.r)
  exposure_comparator <- sum(agd$.E)
  rate_comparator <- events_comparator / exposure_comparator

  events_index_adjusted <- max(events_index, 0.5)
  events_comparator_adjusted <- max(events_comparator, 0.5)
  log_rate_index <- log(events_index_adjusted / exposure_index)
  log_rate_comparator <- log(events_comparator_adjusted / exposure_comparator)

  estimate <- log_rate_index - log_rate_comparator
  se <- sqrt(1 / events_index_adjusted + 1 / events_comparator_adjusted)
  rate_index_se <- sqrt(rate_index / exposure_index)
  rate_comparator_se <- sqrt(rate_comparator / exposure_comparator)
  log_rate_index_se <- sqrt(1 / events_index_adjusted)
  log_rate_comparator_se <- sqrt(1 / events_comparator_adjusted)

  list(
    estimate = estimate,
    se = se,
    ci_lower = estimate - z * se,
    ci_upper = estimate + z * se,
    conf_level = conf_level,
    family = "poisson",
    rate_index = rate_index,
    rate_index_se = rate_index_se,
    rate_index_lower = exp(log_rate_index - z * log_rate_index_se),
    rate_index_upper = exp(log_rate_index + z * log_rate_index_se),
    rate_comparator = rate_comparator,
    rate_comparator_se = rate_comparator_se,
    rate_comparator_lower = exp(log_rate_comparator - z * log_rate_comparator_se),
    rate_comparator_upper = exp(log_rate_comparator + z * log_rate_comparator_se),
    n_index = n_index,
    events_index = events_index,
    exposure_index = exposure_index,
    events_comparator = events_comparator,
    exposure_comparator = exposure_comparator,
    data = data
  )
}

#' Numerical guards in the Stan models
#'
#' `mlumr`'s Stan models clip a few quantities to avoid undefined arithmetic
#' in `generated quantities`. This page documents the exact behavior so
#' users can assess when the guard matters for their analysis.
#'
#' @section Guards currently in place:
#'
#' \describe{
#'   \item{`safe_logit(p)` (binomial models)}{Returns
#'     `logit(clamp(p, 1e-10, 1 - 1e-10))`. The logit and log-odds-ratio
#'     posterior summaries use this rather than the raw `logit(p)` so that
#'     finite samples with `p` at the boundaries do not propagate
#'     `Inf`/`NaN` into the posterior.}
#'   \item{`safe_divide(num, denom)` (binomial and Poisson models)}{Returns
#'     `num / max(denom, 1e-10)`. Used for risk ratios and rate ratios to
#'     avoid division-by-zero when a comparator posterior draw has a
#'     near-zero mean outcome.}
#'   \item{Stan `<lower=0>` constraints on `sigma` (normal family)}{Truncate
#'     the prior to the positive half-line — the reason why
#'     `prior_sigma = prior_normal(0, 2.5)` is interpreted as a
#'     half-normal and why [prior_exponential()] is also valid.}
#' }
#'
#' @section When the guards introduce bias:
#'
#' The guards are one-sided: they replace invalid values with extreme but
#' finite ones rather than with a symmetric distribution. In practice:
#'
#' \itemize{
#'   \item For binomial outcomes with event rates in the range
#'     `[1e-4, 1 - 1e-4]` the clipped output is indistinguishable from the
#'     unclipped value at double precision.
#'   \item For event rates outside that range — or Poisson models with
#'     expected rates below `1e-4` events per unit exposure — the summary
#'     of the posterior for LOR, RR, and related ratios may be biased
#'     toward the clip boundary. The point estimate for the treatment
#'     effect on the link scale (`delta_index` on log-odds, log-rate, or
#'     mean-difference scale) is \emph{not} affected since it does not
#'     go through the guards.
#'   \item **Direction of `safe_divide` bias.** When the comparator rate
#'     (the denominator) is smaller than `1e-10`, the ratio is clipped
#'     to `num / 1e-10`. Because rates and probabilities in mlumr are
#'     non-negative, the clipped value is always \emph{larger} (more
#'     positive) than the mathematical limit — so risk ratios and rate
#'     ratios in near-zero-comparator regimes are \emph{upward-biased},
#'     never downward. Expect the clipped posterior summary to
#'     overstate, not understate, the relative treatment effect in
#'     those draws.
#'   \item For very-rare-event data, consider reframing the model in terms
#'     of rates: an event count with exposure via [set_agd()] `family
#'     = "poisson"` is numerically better behaved than Bernoulli /
#'     Binomial at extreme probabilities.
#' }
#'
#' @section How to diagnose guard activation:
#'
#' Posterior draws of `p_*_index` / `p_*_comparator` or `rate_*_*` are
#' returned by the Stan models and are available in `fit$draws`. If any
#' posterior draw sits below `5e-10` or above `1 - 5e-10` for binomial
#' probabilities (respectively below `5e-10` for Poisson rates), the
#' guards are active for that draw. A quick check:
#' \preformatted{
#'   any(fit$draws[, "p_comparator_comparator"] < 5e-10)
#' }
#'
#' @name mlumr-numerical-guards
#' @aliases safe_logit safe_divide
#' @keywords internal
NULL

#' Specify a normal prior
#'
#' Construct a normal prior for passing to [mlumr()] via `prior_intercept`,
#' `prior_beta`, or `prior_sigma`. The resulting list is consumed by the
#' Stan models.
#'
#' @section Choosing a scale:
#'
#' The Stan community's prior-choice wiki (Vehtari et al., 2025) describes
#' five broad categories, from least to most informative:
#'
#' \enumerate{
#'   \item Flat prior — not recommended.
#'   \item Super-vague proper prior, e.g., `normal(0, 1e6)` — not
#'     recommended.
#'   \item Weakly informative, **very weak**, e.g., `normal(0, 10)`.
#'   \item Generic weakly informative, e.g., `normal(0, 1)`.
#'   \item Specific informative, e.g., `normal(0.4, 0.2)`.
#' }
#'
#' Those scales assume parameters are on roughly unit scale. In ML-UMR
#' models the natural scales are:
#'
#' \describe{
#'   \item{Treatment intercepts}{On the linear-predictor (link) scale. For
#'     a binary outcome with logit link, the intercept is a baseline log
#'     odds; `normal(0, 10)` spans ±20 log-odds at 95 percent and is
#'     "very weak". It is the default because the data usually constrain
#'     the intercept strongly. Tightening to `normal(0, 5)` is reasonable
#'     when the expected event rate is far from the extremes.}
#'   \item{Regression coefficients (`beta`)}{On the link scale, per unit
#'     of covariate. For logistic regression with predictors on unit
#'     scale, Gelman et al. (2008) and the Stan wiki recommend
#'     `student_t(df, 0, 2.5)` with `df` in `3:7`, or — as a practical
#'     approximation — `normal(0, 2.5)`. That is the default used by
#'     [mlumr()]. Use `normal(0, 1)` if you expect small effects (e.g.,
#'     standardized predictors in a normal-outcome model). If predictors
#'     are on very different scales, set `autoscale = TRUE` so the scale
#'     is divided by each covariate's SD.}
#'   \item{Residual SD (`sigma`, normal family only)}{`prior_sigma` is
#'     interpreted as a half-normal via the Stan `<lower=0>` constraint.
#'     The default `normal(0, 2.5)` (i.e., `half-normal(0, 2.5)`) is
#'     weakly informative for residual SDs on the scale of the outcome.
#'     Scale to the outcome if it is far from unit scale, or use
#'     [prior_exponential()].}
#' }
#'
#' Prior sensitivity is especially important for the relaxed model,
#' where `beta_comparator` is identified only by the AgD likelihood.
#' Run [prior_sensitivity()] to quantify how much conclusions move under
#' alternative scales; see `vignette("mlumr-models")`.
#'
#' @param mean Prior mean (default 0).
#' @param sd Prior standard deviation (default 10). The default matches
#'   the historical "very weak" scale; explicit tighter values are
#'   recommended for regression coefficients (see Details).
#' @param autoscale If `TRUE` and this prior is passed as `prior_beta`,
#'   the scale is divided by each covariate's empirical SD so the prior
#'   is weakly-informative regardless of predictor scaling. Default
#'   `FALSE` to preserve backward-compatible behavior; set to `TRUE`
#'   explicitly when passing unstandardized predictors. Ignored for
#'   `prior_intercept` and `prior_sigma`.
#'
#' @return A list with components `distribution`, `mean`, `sd`, `df`,
#'   `autoscale`.
#' @export
#'
#' @references
#' Gelman, A., Jakulin, A., Pittau, M. G., & Su, Y.-S. (2008). A weakly
#' informative default prior distribution for logistic and other
#' regression models. *Annals of Applied Statistics*, 2(4), 1360–1383.
#'
#' Vehtari, A. et al. Prior Choice Recommendations (Stan wiki):
#' <https://github.com/stan-dev/stan/wiki/Prior-Choice-Recommendations>.
#'
#' @examples
#' # Default weakly-very-weak intercept prior
#' prior_normal(mean = 0, sd = 10)
#'
#' # Gelman 2008 default for logistic-regression coefficients
#' prior_normal(mean = 0, sd = 2.5)
#'
#' # Autoscaled coefficient prior (dividing 2.5 by each covariate's SD)
#' prior_normal(mean = 0, sd = 2.5, autoscale = TRUE)
prior_normal <- function(mean = 0, sd = 10, autoscale = FALSE) {
  .validate_prior_number(mean, "mean")
  .validate_prior_number(sd, "sd", positive = TRUE)
  autoscale <- .validate_prior_autoscale(autoscale)
  list(
    distribution = "normal",
    mean = mean,
    sd = sd,
    df = NA_real_,
    autoscale = isTRUE(autoscale)
  )
}

#' Specify a Student-t prior
#'
#' Heavier-tailed alternative to [prior_normal()]. For logistic-regression
#' coefficients with unit-scale predictors, the Stan community prior-choice
#' recommendations suggest Student-t with `df` between 3 and 7 as a robust
#' weakly informative prior (Gelman et al., 2008).
#'
#' @param df Degrees of freedom (must be positive). Values in `3:7` are
#'   recommended for logistic-regression coefficients.
#' @param mean Prior location (default 0).
#' @param sd Prior scale (default 2.5).
#' @param autoscale See [prior_normal()]. Default `FALSE`.
#'
#' @return A list with components `distribution = "student_t"`, `df`,
#'   `mean`, `sd`, `autoscale`.
#' @export
#'
#' @examples
#' # Gelman et al. 2008 recommendation for logistic-regression coefficients
#' prior_student_t(df = 5, mean = 0, sd = 2.5)
prior_student_t <- function(df = 5, mean = 0, sd = 2.5, autoscale = FALSE) {
  .validate_prior_number(df, "df", positive = TRUE)
  .validate_prior_number(mean, "mean")
  .validate_prior_number(sd, "sd", positive = TRUE)
  autoscale <- .validate_prior_autoscale(autoscale)
  list(
    distribution = "student_t",
    mean = mean,
    sd = sd,
    df = df,
    autoscale = isTRUE(autoscale)
  )
}

#' Specify a Cauchy prior
#'
#' Cauchy is Student-t with `df = 1`; this constructor is a convenience
#' wrapper around [prior_student_t()]. It has very heavy tails and should
#' be used with care — modern recommendations generally prefer
#' `prior_student_t(df in 3:7, ...)` over Cauchy for regression
#' coefficients to keep sampling well-behaved (see Piironen & Vehtari on
#' the horseshoe; Ghosh et al. 2015).
#'
#' @param mean Prior location (default 0).
#' @param sd Prior scale (default 2.5).
#' @param autoscale See [prior_normal()]. Default `FALSE`.
#'
#' @return A list with components `distribution = "student_t"`, `df = 1`,
#'   `mean`, `sd`, `autoscale`.
#' @export
#'
#' @examples
#' prior_cauchy(mean = 0, sd = 2.5)
prior_cauchy <- function(mean = 0, sd = 2.5, autoscale = FALSE) {
  prior_student_t(df = 1, mean = mean, sd = sd, autoscale = autoscale)
}

#' Specify an exponential prior
#'
#' Exponential prior on a positive scalar. Currently supported for
#' [`prior_sigma`][mlumr] (normal-family residual SD) only; rejected for
#' unconstrained intercepts and regression coefficients.
#' `prior_exponential(rate = 1)` has mean 1 and is a reasonable
#' weakly-informative choice when the outcome is standardized.
#'
#' @param rate Rate parameter (default 1). Larger `rate` = tighter prior
#'   concentrated near zero.
#' @return A list with components `distribution = "exponential"`, `rate`,
#'   and placeholders (`mean = 0`, `sd = 1 / rate`) so the same
#'   Stan-field translation works.
#' @export
#' @examples
#' prior_exponential(rate = 1)
prior_exponential <- function(rate = 1) {
  .validate_prior_number(rate, "rate", positive = TRUE)
  list(
    distribution = "exponential",
    rate = rate,
    mean = 0,
    sd = 1 / rate,
    df = NA_real_,
    autoscale = FALSE
  )
}


# ---- Default priors (carry $default = TRUE and $version) -------------------

#' Default priors used by [mlumr()]
#'
#' These accessors return the current default priors used by [mlumr()],
#' tagged with `$default = TRUE` and the package version. [prior_summary()]
#' prints the version so cross-release reproducibility is diagnosable: if a
#' later release changes a default, fits produced with an older version
#' will still carry the correct `$version` tag.
#'
#' @return A prior list (see [prior_normal()]).
#' @name default_priors
#' @examples
#' default_prior_intercept()
#' default_prior_beta()
#' default_prior_sigma()
NULL

#' @rdname default_priors
#' @export
default_prior_intercept <- function() {
  .tag_default(prior_normal(mean = 0, sd = 10))
}

#' @rdname default_priors
#' @export
default_prior_beta <- function() {
  .tag_default(prior_normal(mean = 0, sd = 2.5))
}

#' @rdname default_priors
#' @export
default_prior_sigma <- function() {
  .tag_default(prior_normal(mean = 0, sd = 2.5))
}

#' @keywords internal
.tag_default <- function(prior) {
  prior$default <- TRUE
  prior$version <- as.character(utils::packageVersion("mlumr"))
  prior
}


# ---- Validation ------------------------------------------------------------

#' Validate prior specification
#' @param prior Prior specification list
#' @param param_name Parameter name for error messages
#' @keywords internal
validate_prior <- function(prior, param_name = "parameter") {
  if (!is.list(prior)) {
    stop(sprintf("Prior for %s must be a list", param_name), call. = FALSE)
  }
  valid_distribution <- is.character(prior$distribution) &&
    length(prior$distribution) == 1L &&
    prior$distribution %in% c("normal", "student_t", "exponential")
  if (!valid_distribution) {
    msg <- paste0(
      "Unsupported prior distribution '%s' for %s. ",
      "Use prior_normal(), prior_student_t(), prior_cauchy(), ",
      "or prior_exponential()."
    )
    stop(sprintf(
      msg,
      .prior_distribution_label(prior$distribution),
      param_name
    ), call. = FALSE)
  }
  if (prior$distribution == "exponential") {
    .validate_prior_number(prior$rate, "rate", positive = TRUE,
                           param_name = param_name)
    return(invisible(TRUE))
  }
  .validate_prior_number(prior$mean, "mean", param_name = param_name)
  .validate_prior_number(prior$sd, "sd", positive = TRUE,
                         param_name = param_name)
  if (prior$distribution == "student_t") {
    .validate_prior_number(prior$df, "df", positive = TRUE,
                           param_name = param_name)
  }
  invisible(TRUE)
}

.validate_prior_number <- function(x, field, positive = FALSE,
                                   param_name = NULL) {
  valid <- is.numeric(x) &&
    length(x) == 1L &&
    is.finite(x) &&
    (!positive || x > 0)
  if (!valid) {
    qualifier <- if (positive) "positive finite" else "finite"
    if (is.null(param_name)) {
      stop(sprintf("`%s` must be a single %s number", field, qualifier),
           call. = FALSE)
    }
    stop(sprintf("Prior %s must be a single %s number for %s",
                 field, qualifier, param_name),
         call. = FALSE)
  }
  invisible(TRUE)
}

.validate_prior_autoscale <- function(autoscale) {
  if (!is.logical(autoscale) || length(autoscale) != 1L || is.na(autoscale)) {
    stop("`autoscale` must be TRUE or FALSE", call. = FALSE)
  }
  autoscale
}

.prior_distribution_label <- function(distribution) {
  if (is.null(distribution)) {
    return("<missing>")
  }
  paste(distribution, collapse = ", ")
}

#' Is this object a single prior (vs a list of priors)?
#' @keywords internal
is_single_prior <- function(x) {
  is.list(x) && !is.null(x$distribution)
}


# ---- Stan-data translation -------------------------------------------------

#' Translate a scalar prior spec to the Stan data fields
#'
#' Stan scalar-prior fields: `prior_*_mean`, `prior_*_sd`,
#' `prior_*_dist` (0 = normal, 1 = student_t, 2 = exponential for sigma
#' only), `prior_*_df` (used only when dist == 1; positive placeholder
#' otherwise).
#'
#' @param prior A prior list.
#' @return A list with `mean`, `sd`, `dist`, `df`.
#' @keywords internal
stan_prior_fields <- function(prior) {
  validate_prior(prior)
  dist_code <- switch(prior$distribution,
    normal      = 0L,
    student_t   = 1L,
    exponential = 2L,
    stop("Unsupported distribution: ", prior$distribution)
  )
  df_value <- if (prior$distribution == "student_t") prior$df else 3
  # For exponential, Stan interprets `scale` as 1 / rate (see priors_functions.stan).
  location <- if (prior$distribution == "exponential") 0 else prior$mean
  scale    <- if (prior$distribution == "exponential") 1 / prior$rate else prior$sd
  list(mean = location, sd = scale, dist = dist_code, df = df_value)
}


#' Translate a `prior_beta` spec to vector-valued Stan data fields
#'
#' The Stan models declare `prior_beta_mean` and `prior_beta_sd` as
#' `vector[n_cov]` so the same data contract supports:
#'
#' \itemize{
#'   \item (i) a single prior broadcast to all covariates (scalar user input),
#'   \item (ii) per-coefficient priors (a list of prior lists, length `n_cov`),
#'   \item (iii) autoscaling: each covariate's scale is divided by `sd(x_j)`
#'     so the prior is weakly-informative on the standardized scale.
#' }
#'
#' For a list of per-coefficient priors, all elements must use the same
#' `distribution` family and `df` (Stan branches on a single dist code).
#'
#' @param prior A prior list from [prior_normal()] / [prior_student_t()] /
#'   [prior_cauchy()], OR a list of such priors of length `n_cov`.
#' @param n_cov Number of covariates.
#' @param sd_x Optional numeric vector of covariate SDs (length `n_cov`).
#'   Required when any prior has `autoscale = TRUE`; ignored otherwise.
#' @param covariate_names Optional character vector of covariate names
#'   (length `n_cov`). Used only to produce informative warnings when
#'   `autoscale = TRUE` meets a zero-SD covariate.
#' @return A list with numeric vectors `mean` and `sd` (length `n_cov`)
#'   and scalars `dist`, `df`, and a logical vector `autoscale` recording
#'   which elements were autoscaled (for `prior_summary()`).
#' @keywords internal
stan_prior_fields_beta <- function(prior, n_cov, sd_x = NULL,
                                   covariate_names = NULL) {

  # Expand to a list of n_cov single-priors
  if (is_single_prior(prior)) {
    validate_prior(prior, "beta")
    prior_list <- rep(list(prior), n_cov)
  } else if (is.list(prior)) {
    if (length(prior) != n_cov) {
      stop(sprintf(
        "Per-coefficient prior list has length %d but n_cov = %d.",
        length(prior), n_cov
      ), call. = FALSE)
    }
    # Validate each
    for (i in seq_along(prior)) {
      validate_prior(prior[[i]], sprintf("beta[%d]", i))
    }
    prior_list <- prior
  } else {
    stop("`prior_beta` must be a single prior or a list of priors", call. = FALSE)
  }

  # Check all elements share a distribution family and df
  dists <- vapply(prior_list, function(p) p$distribution, character(1))
  if (length(unique(dists)) > 1L) {
    stop("All per-coefficient priors must use the same distribution family ",
         "(mixed normal / student_t is not supported by the Stan dispatch).",
         call. = FALSE)
  }
  if (dists[[1L]] == "exponential") {
    stop("Exponential priors are not supported for regression coefficients.",
         call. = FALSE)
  }
  dfs <- vapply(prior_list, function(p) if (p$distribution == "student_t") p$df else NA_real_,
                numeric(1))
  if (dists[[1L]] == "student_t" && length(unique(dfs)) > 1L) {
    stop("All per-coefficient Student-t priors must share the same df.",
         call. = FALSE)
  }

  means <- vapply(prior_list, function(p) as.numeric(p$mean)[[1L]], numeric(1))
  sds   <- vapply(prior_list, function(p) as.numeric(p$sd)[[1L]],   numeric(1))
  autos <- vapply(prior_list, function(p) isTRUE(p$autoscale),      logical(1))

  if (any(autos)) {
    if (is.null(sd_x)) {
      stop("`autoscale = TRUE` requires covariate SDs; did you call ",
           "stan_prior_fields_beta() without sd_x?", call. = FALSE)
    }
    if (length(sd_x) != n_cov) {
      stop("`sd_x` must have length n_cov", call. = FALSE)
    }
    # Protect against sd_x == 0 (constant covariate): fall back to unscaled
    # and warn, so the user knows the autoscale contract has been broken
    # for those columns. Naming the offending covariates makes the warning
    # actionable.
    zero_var <- autos & sd_x <= 0
    if (any(zero_var)) {
      if (is.null(covariate_names) || length(covariate_names) != n_cov) {
        bad <- sprintf("column %d", which(zero_var))
      } else {
        bad <- covariate_names[zero_var]
      }
      warning(sprintf(
        paste0("`autoscale = TRUE` is requested for covariate(s) with zero ",
               "empirical SD in the IPD: %s. The prior on these coefficients ",
               "falls back to the un-autoscaled scale (sd as supplied). ",
               "Consider removing the constant covariate or turning ",
               "autoscale off for it."),
        paste(bad, collapse = ", ")
      ), call. = FALSE)
    }
    sd_x_safe <- ifelse(sd_x > 0, sd_x, 1)
    sds <- ifelse(autos, sds / sd_x_safe, sds)
  }

  dist_code <- switch(dists[[1L]], normal = 0L, student_t = 1L)
  df_value <- if (dists[[1L]] == "student_t") dfs[[1L]] else 3

  list(
    mean = means,
    sd = sds,
    dist = dist_code,
    df = df_value,
    autoscale = autos
  )
}

# Tests for prior_sensitivity() — exercises the argument-validation path
# and the internal .rescale_prior_beta helper. The full refit path calls
# mlumr() and requires Stan, which is covered by the smoke-level tests in
# test-mlumr.R; here we stay pure-R so the suite stays fast.

test_that("prior_sensitivity rejects non-mlumr_fit input", {
  expect_error(prior_sensitivity(list()), "mlumr_fit")
  expect_error(prior_sensitivity(NULL), "mlumr_fit")
})


test_that("prior_sensitivity rejects invalid prior_beta_scales", {
  fit <- structure(list(priors = list(beta = prior_normal(0, 2.5))),
                   class = "mlumr_fit")
  expect_error(prior_sensitivity(fit, prior_beta_scales = c(-1, 2)),
               "positive finite")
  expect_error(prior_sensitivity(fit, prior_beta_scales = c(0, 1)),
               "positive finite")
  expect_error(prior_sensitivity(fit, prior_beta_scales = c(NA, 1)),
               "positive finite")
  expect_error(prior_sensitivity(fit, prior_beta_scales = "2.5"),
               "positive finite")
})


test_that("prior_sensitivity errors when priors$beta is missing", {
  fit <- structure(list(priors = list()), class = "mlumr_fit")
  expect_error(prior_sensitivity(fit, prior_beta_scales = 2.5),
               "Original `prior_beta` not found")
})


test_that(".rescale_prior_beta preserves family and replaces scale", {
  p <- prior_normal(0, 2.5)
  r <- mlumr:::.rescale_prior_beta(p, new_scale = 5)
  expect_equal(r$sd, 5)
  # Mean and distribution preserved
  expect_equal(r$mean, 0)
  expect_equal(r$distribution, "normal")
  # Default tags are stripped
  expect_null(r$default)
  expect_null(r$version)
})


test_that(".rescale_prior_beta preserves student_t df", {
  p <- prior_student_t(df = 5, mean = 0, sd = 2.5)
  r <- mlumr:::.rescale_prior_beta(p, new_scale = 1)
  expect_equal(r$sd, 1)
  expect_equal(r$df, 5)
  expect_equal(r$distribution, "student_t")
})


test_that(".rescale_prior_beta falls back to normal for exponential input", {
  # Exponential on beta is not supported at fit time, but the helper
  # should not blow up if a pathological fit carries one.
  p <- prior_exponential(rate = 1)
  r <- mlumr:::.rescale_prior_beta(p, new_scale = 2.5)
  expect_equal(r$distribution, "normal")
  expect_equal(r$sd, 2.5)
})


test_that(".rescale_prior_beta rescales per-coefficient priors element-wise", {
  priors <- list(prior_normal(0, 2.5), prior_normal(0, 1))
  r <- mlumr:::.rescale_prior_beta(priors, new_scale = 3)
  expect_length(r, 2L)
  expect_true(all(vapply(r, function(x) x$sd, numeric(1)) == 3))
  expect_null(r[[1]]$default)
})

test_that("prior constructors validate scalar arguments", {
  expect_error(prior_normal(mean = c(0, 1)),
               "`mean` must be a single finite number")
  expect_error(prior_normal(sd = c(1, 2)),
               "`sd` must be a single positive finite number")
  expect_error(prior_normal(sd = 0),
               "`sd` must be a single positive finite number")
  expect_error(prior_normal(autoscale = "TRUE"),
               "`autoscale` must be TRUE or FALSE")
  expect_error(prior_normal(autoscale = NA),
               "`autoscale` must be TRUE or FALSE")

  expect_error(prior_student_t(df = c(3, 4)),
               "`df` must be a single positive finite number")
  expect_error(prior_student_t(mean = NA_real_),
               "`mean` must be a single finite number")
  expect_error(prior_student_t(sd = Inf),
               "`sd` must be a single positive finite number")

  expect_error(prior_exponential(rate = c(1, 2)),
               "`rate` must be a single positive finite number")
  expect_error(prior_exponential(rate = 0),
               "`rate` must be a single positive finite number")
})


test_that("validate_prior rejects malformed hand-built prior lists cleanly", {
  expect_error(
    validate_prior(list(distribution = "normal", mean = c(0, 1), sd = 1),
                   "beta"),
    "Prior mean must be a single finite number for beta"
  )
  expect_error(
    validate_prior(list(distribution = "normal", mean = 0, sd = c(1, 2)),
                   "beta"),
    "Prior sd must be a single positive finite number for beta"
  )
  expect_error(
    validate_prior(list(distribution = "student_t", mean = 0, sd = 1,
                        df = c(3, 4)), "beta"),
    "Prior df must be a single positive finite number for beta"
  )
  expect_error(
    validate_prior(list(distribution = "exponential", rate = c(1, 2)),
                   "sigma"),
    "Prior rate must be a single positive finite number for sigma"
  )
})


test_that("Stan prior translators reject vector scalar priors", {
  vector_prior <- list(
    distribution = "normal",
    mean = c(0, 1),
    sd = c(2, 3),
    df = NA_real_,
    autoscale = FALSE
  )

  expect_error(stan_prior_fields(vector_prior),
               "Prior mean must be a single finite number")
  expect_error(stan_prior_fields_beta(vector_prior, n_cov = 2),
               "Prior mean must be a single finite number for beta")
})


test_that("per-coefficient priors still support heterogeneous means and scales", {
  fields <- stan_prior_fields_beta(
    list(prior_normal(0, 2), prior_normal(1, 3)),
    n_cov = 2
  )

  expect_equal(fields$mean, c(0, 1))
  expect_equal(fields$sd, c(2, 3))
  expect_equal(fields$dist, 0L)
  expect_equal(fields$df, 3)
})

# Tests for prior_summary.mlumr_fit()
#
# Pure-R tests — no Stan, no sampling. The method inspects a stored
# `priors` list on the fit object, so we construct minimal fit stubs
# that exercise the three branches the method cares about:
#
#   1. Broadcast homogeneous prior (all coefficients share mean/sd,
#      no autoscaling).
#   2. Per-coefficient prior table (heterogeneous scales OR autoscaled).
#   3. Default-prior tag footer (written when $default = TRUE).

make_fit_stub <- function(priors) {
  structure(list(priors = priors), class = "mlumr_fit")
}

resolved_beta <- function(mean, sd, dist = 0L, df = 3,
                          autoscale = rep(FALSE, length(mean)),
                          sd_x = rep(1, length(mean)),
                          covariate_names = paste0("x", seq_along(mean))) {
  list(
    mean = as.numeric(mean),
    sd   = as.numeric(sd),
    dist = as.integer(dist),
    df   = df,
    autoscale = as.logical(autoscale),
    sd_x = as.numeric(sd_x),
    covariate_names = covariate_names
  )
}

tag_default <- function(prior) {
  prior$default <- TRUE
  prior$version <- "0.1.0"
  prior
}


test_that("broadcast summary is used when priors are homogeneous", {
  priors <- list(
    intercept = prior_normal(0, 10),
    beta = prior_normal(0, 2.5),
    beta_resolved = resolved_beta(mean = rep(0, 3), sd = rep(2.5, 3))
  )
  fit <- make_fit_stub(priors)

  out <- capture.output(prior_summary(fit))

  # Broadcast line names the prior and the covariate count
  expect_true(any(grepl("normal\\(0, 2\\.5\\) applied to all 3 covariate", out)))
  # Per-coefficient table should NOT be printed in the broadcast branch
  expect_false(any(grepl("coefficient\\s+mean\\s+scale", out)))
})


test_that("per-coefficient table is used when resolved scales differ", {
  priors <- list(
    intercept = prior_normal(0, 10),
    beta = list(prior_normal(0, 2.5), prior_normal(0, 1)),
    beta_resolved = resolved_beta(mean = c(0, 0), sd = c(2.5, 1),
                                  covariate_names = c("age", "sex"))
  )
  fit <- make_fit_stub(priors)

  out <- capture.output(prior_summary(fit))

  # Header identifies the family
  expect_true(any(grepl("Family: normal", out)))
  # Table header and at least one coefficient row
  expect_true(any(grepl("coefficient", out)))
  expect_true(any(grepl("\\bage\\b", out)))
  expect_true(any(grepl("\\bsex\\b", out)))
})


test_that("autoscaled coefficients are flagged and the footnote is printed", {
  # Coefficient 2 autoscaled from 2.5 by sd_x = 2 -> scale 1.25.
  priors <- list(
    intercept = prior_normal(0, 10),
    beta = list(prior_normal(0, 2.5), prior_normal(0, 2.5, autoscale = TRUE)),
    beta_resolved = resolved_beta(
      mean = c(0, 0),
      sd   = c(2.5, 1.25),
      autoscale = c(FALSE, TRUE),
      sd_x = c(1, 2),
      covariate_names = c("age", "bmi")
    )
  )
  fit <- make_fit_stub(priors)

  out <- capture.output(prior_summary(fit))

  # The autoscaled row is flagged in the table
  expect_true(any(grepl("TRUE", out)))
  # The footnote explains the scale = user_scale / sd_x convention
  expect_true(any(grepl("scale = user_scale / sd_x for autoscaled rows", out)))
})


test_that("default priors are tagged with (package default, mlumr <version>)", {
  priors <- list(
    intercept = tag_default(prior_normal(0, 10)),
    beta = tag_default(prior_normal(0, 2.5)),
    beta_resolved = resolved_beta(mean = rep(0, 2), sd = rep(2.5, 2))
  )
  fit <- make_fit_stub(priors)

  out <- capture.output(prior_summary(fit))

  # Both the intercept and beta blocks carry the default tag
  default_lines <- grep("package default, mlumr", out, value = TRUE)
  expect_gte(length(default_lines), 2L)
  expect_true(all(grepl("0\\.1\\.0", default_lines)))
})


test_that("sigma prior block is printed only when sigma is present", {
  priors_no_sigma <- list(
    intercept = prior_normal(0, 10),
    beta = prior_normal(0, 2.5),
    beta_resolved = resolved_beta(mean = 0, sd = 2.5)
  )
  out_no <- capture.output(prior_summary(make_fit_stub(priors_no_sigma)))
  expect_false(any(grepl("Residual SD", out_no)))

  priors_sigma <- priors_no_sigma
  priors_sigma$sigma <- prior_exponential(rate = 1)
  out_yes <- capture.output(prior_summary(make_fit_stub(priors_sigma)))
  expect_true(any(grepl("Residual SD", out_yes)))
  # Exponential priors are rendered by rate, not (mean, sd)
  expect_true(any(grepl("exponential\\(rate = 1\\)", out_yes)))
})


test_that("prior_summary errors on fits missing the priors slot", {
  fit <- structure(list(), class = "mlumr_fit")
  expect_error(prior_summary(fit), "No prior information stored")
})


test_that("prior_summary validates digits", {
  priors <- list(
    intercept = prior_normal(0, 10),
    beta = prior_normal(0, 2.5),
    beta_resolved = resolved_beta(mean = 0, sd = 2.5)
  )
  fit <- make_fit_stub(priors)

  expect_error(prior_summary(fit, digits = 0), "`digits`")
  expect_error(prior_summary(fit, digits = c(2, 3)), "`digits`")
  expect_error(prior_summary(fit, digits = NA_real_), "`digits`")
})


test_that("prior_summary rejects malformed resolved beta metadata", {
  priors <- list(
    intercept = prior_normal(0, 10),
    beta = prior_normal(0, 2.5),
    beta_resolved = list(mean = 0, sd = 2.5)
  )
  fit <- make_fit_stub(priors)

  expect_error(prior_summary(fit), "Resolved beta prior metadata")
})


test_that("prior_summary prints beta prior lists for older fit metadata", {
  priors <- list(
    intercept = prior_normal(0, 10),
    beta = list(prior_normal(0, 2.5), prior_normal(0, 1))
  )
  fit <- make_fit_stub(priors)

  out <- capture.output(prior_summary(fit))

  expect_true(any(grepl("beta\\[1\\]: normal\\(0, 2\\.5\\)", out)))
  expect_true(any(grepl("beta\\[2\\]: normal\\(0, 1\\)", out)))
})


test_that("prior_summary default method rejects unsupported classes", {
  expect_error(prior_summary(list()), "no method for class")
})


test_that("Student-t priors render their df", {
  priors <- list(
    intercept = prior_student_t(df = 7, 0, 10),
    beta = prior_student_t(df = 5, 0, 2.5),
    beta_resolved = resolved_beta(
      mean = rep(0, 2), sd = rep(2.5, 2), dist = 1L, df = 5
    )
  )
  out <- capture.output(prior_summary(make_fit_stub(priors)))
  expect_true(any(grepl("student_t\\(df = 7", out)))
  expect_true(any(grepl("normal|student_t", out)))  # broadcast line
})

#' Conditional treatment effects
#'
#' Compute conditional (individual-level) treatment effects at specific
#' covariate values from a fitted ML-UMR model. Unlike [marginal_effects()],
#' which averages over a population's covariate distribution, conditional
#' effects evaluate the treatment effect at a particular covariate profile.
#'
#' @param object An `mlumr_fit` object
#' @param newdata Data frame of covariate values at which to compute effects.
#'   Each row defines one covariate profile. Column names must match the
#'   covariates used in fitting. If `NULL` (default), uses the covariate means
#'   from the IPD as a single reference profile.
#' @param effect Which effect measure. For binomial: `"all"`, `"link_effect"`,
#'   `"rd"`, or `"rr"`. For normal: `"all"` or `"md"`. For Poisson: `"all"`
#'   or `"rr"`. The legacy value `"lor"` is accepted as an alias for
#'   `"link_effect"` when the fitted link is logit.
#' @param summary Return summary statistics (`TRUE`) or full posterior draws
#'   (`FALSE`)
#' @param probs Quantiles for summary (default `c(0.025, 0.5, 0.975)`)
#'
#' @details
#' For SPFA models, the conditional link-scale treatment effect is constant
#' across all covariate values because the shared beta cancels in the treatment
#' contrast on the fitted link scale. However, risk difference (RD) and risk
#' ratio (RR) still vary with covariates because they depend on absolute
#' probability levels. For relaxed models, all conditional effects vary with
#' covariate values because the index and comparator treatments have different
#' regression coefficients.
#'
#' The conditional link-scale effect is computed directly as
#' `eta_index - eta_comparator`. This avoids numerical distortion from
#' transforming extreme response-scale probabilities back through the link
#' function.
#'
#' **Conditional vs marginal on non-identity links.** Conditional effects are
#' evaluated at a single covariate profile, so there is no averaging over a
#' population and no Jensen's-inequality gap between the conditional and
#' marginal response. Compare with [marginal_effects()] and
#' [predict.mlumr_fit()], which average over either the IPD individuals
#' (index population) or the AgD integration points (comparator population)
#' and therefore return `E[g^{-1}(eta)]`, not `g^{-1}(E[eta])`.
#'
#' @return A data frame. If `summary = TRUE`, contains columns `profile`,
#'   `effect`, `mean`, `sd`, and quantile columns. If `summary = FALSE`,
#'   returns a single combined data frame of full posterior draws with a
#'   `profile` column indicating which covariate profile each draw belongs
#'   to.
#' @seealso [marginal_effects()] for population-averaged treatment effects;
#'   [conditional_predict()] for absolute predictions at specific profiles;
#'   [predict.mlumr_fit()] for population-level predictions.
#' @export
#'
#' @examples
#' \dontrun{
#' # Conditional effects at IPD covariate means (default)
#' conditional_effects(fit)
#'
#' # At specific covariate values
#' conditional_effects(fit, newdata = data.frame(age = 60, sex = 1))
#'
#' # Multiple profiles
#' profiles <- data.frame(age = c(50, 60, 70), sex = c(0, 0, 1))
#' conditional_effects(fit, newdata = profiles)
#' }
conditional_effects <- function(object,
                                newdata = NULL,
                                effect = "all",
                                summary = TRUE,
                                probs = c(0.025, 0.5, 0.975)) {

  .validate_mlumr_fit_object(object)
  effect <- .validate_effect_choice(effect)
  summary <- .validate_summary_flag(summary)
  .validate_probs(probs)

  family <- object$family %||% "binomial"
  cfg_family <- get_family_config(family)
  lnk <- object$link %||% cfg_family$link_default

  if (effect == "lor") {
    if (family == "binomial" && lnk == "logit") {
      effect <- "link_effect"
    } else if (family == "binomial") {
      stop(sprintf(
        "'lor' is only valid when link is 'logit'. For link '%s', use 'link_effect' instead.",
        lnk
      ), call. = FALSE)
    }
  }

  valid_effects <- .conditional_effect_choices(family)
  if (!effect %in% valid_effects) {
    stop(sprintf("For %s family, `effect` must be one of: %s",
                 family, paste(valid_effects, collapse = ", ")), call. = FALSE)
  }

  profiles <- .conditional_profiles(object, newdata)
  X <- profiles$X
  n_profiles <- nrow(X)
  params <- .conditional_parameters(object, profiles$covariates)
  results <- vector("list", n_profiles)

  for (i in seq_len(n_profiles)) {
    eta <- .conditional_eta(params, X[i, , drop = FALSE])
    eta_idx <- eta$index
    eta_cmp <- eta$comparator

    if (family == "binomial") {
      p_idx <- inverse_link(eta_idx, lnk)
      p_cmp <- inverse_link(eta_cmp, lnk)
      profile_draws <- data.frame(
        link_effect = eta_idx - eta_cmp,
        rd = p_idx - p_cmp,
        rr = .safe_positive_ratio(p_idx, p_cmp)
      )
    } else if (family == "normal") {
      if (lnk == "identity") {
        profile_draws <- data.frame(
          md = eta_idx - eta_cmp
        )
      } else {
        val_idx <- inverse_link(eta_idx, lnk)
        val_cmp <- inverse_link(eta_cmp, lnk)
        profile_draws <- data.frame(
          md = val_idx - val_cmp
        )
      }
    } else {
      profile_draws <- data.frame(
        rr = exp(eta_idx - eta_cmp)
      )
    }

    results[[i]] <- profile_draws
  }

  # Filter to requested effects
  if (family == "binomial") {
    keep_cols <- if (effect == "all") c("link_effect", "rd", "rr") else effect
  } else if (family == "normal") {
    keep_cols <- "md"
  } else {
    keep_cols <- "rr"
  }

  if (!summary) {
    out <- lapply(seq_along(results), function(i) {
      d <- results[[i]][, keep_cols, drop = FALSE]
      d$profile <- i
      d
    })
    return(do.call(rbind, out))
  }

  summary_list <- lapply(seq_along(results), function(i) {
    d <- results[[i]][, keep_cols, drop = FALSE]
    summary_df <- .summarize_draw_matrix(d, probs)
    summary_df$profile <- i
    summary_df$effect <- toupper(rownames(summary_df))
    summary_df
  })

  out <- do.call(rbind, summary_list)
  out <- out[, c("profile", "effect", "mean", "sd",
                 paste0("q", probs * 100)), drop = FALSE]
  rownames(out) <- NULL
  out
}

#' Build covariate profiles for conditional summaries
#' @keywords internal
.conditional_profiles <- function(object, newdata = NULL) {
  covariates <- object$data$covariates

  if (is.null(newdata)) {
    X_ipd <- as.matrix(object$data$ipd$data[, covariates, drop = FALSE])
    col_means <- colMeans(X_ipd)
    newdata <- as.data.frame(
      matrix(col_means, nrow = 1, dimnames = list(NULL, covariates))
    )
  }

  if (!is.data.frame(newdata)) {
    stop("`newdata` must be a data frame.", call. = FALSE)
  }
  if (nrow(newdata) == 0L) {
    stop("`newdata` must contain at least one row.", call. = FALSE)
  }
  if (!all(covariates %in% names(newdata))) {
    missing <- setdiff(covariates, names(newdata))
    stop(sprintf("Missing covariates in `newdata`: %s",
                 paste(missing, collapse = ", ")), call. = FALSE)
  }

  profile_data <- newdata[, covariates, drop = FALSE]
  non_numeric <- covariates[
    !vapply(profile_data, is.numeric, logical(1))
  ]
  if (length(non_numeric) > 0L) {
    stop(sprintf("Covariates in `newdata` must be numeric: %s",
                 paste(non_numeric, collapse = ", ")), call. = FALSE)
  }

  X <- as.matrix(profile_data)
  storage.mode(X) <- "double"
  if (any(!is.finite(X))) {
    stop("`newdata` covariates must be finite.", call. = FALSE)
  }

  list(
    X = X,
    covariates = covariates
  )
}

#' Extract posterior parameter draws for conditional summaries
#' @keywords internal
.conditional_parameters <- function(object, covariates) {
  draws <- object$draws
  n_cov <- length(covariates)
  is_relaxed <- object$model == "relaxed"

  .require_draw_columns(draws, c("mu_index", "mu_comparator"),
                        "conditional parameter")
  out <- list(
    mu_index = draws$mu_index,
    mu_comparator = draws$mu_comparator,
    is_relaxed = is_relaxed
  )

  if (is_relaxed) {
    beta_index_cols <- paste0("beta_index[", seq_len(n_cov), "]")
    beta_comparator_cols <- paste0("beta_comparator[", seq_len(n_cov), "]")
    .require_draw_columns(draws, c(beta_index_cols, beta_comparator_cols),
                          "conditional parameter")
    out$beta_index <- as.matrix(
      draws[, beta_index_cols, drop = FALSE]
    )
    out$beta_comparator <- as.matrix(
      draws[, beta_comparator_cols, drop = FALSE]
    )
  } else {
    beta_cols <- paste0("beta[", seq_len(n_cov), "]")
    .require_draw_columns(draws, beta_cols, "conditional parameter")
    out$beta <- as.matrix(
      draws[, beta_cols, drop = FALSE]
    )
  }

  out
}

#' Compute conditional linear predictors for one profile
#' @keywords internal
.conditional_eta <- function(params, x) {
  if (params$is_relaxed) {
    eta_idx <- params$mu_index + as.vector(params$beta_index %*% t(x))
    eta_cmp <- params$mu_comparator +
      as.vector(params$beta_comparator %*% t(x))
  } else {
    eta_idx <- params$mu_index + as.vector(params$beta %*% t(x))
    eta_cmp <- params$mu_comparator + as.vector(params$beta %*% t(x))
  }

  list(index = eta_idx, comparator = eta_cmp)
}

#' Conditional effect choices by family
#' @keywords internal
.conditional_effect_choices <- function(family) {
  if (family == "binomial") {
    c("all", "link_effect", "rd", "rr")
  } else if (family == "normal") {
    c("all", "md")
  } else {
    c("all", "rr")
  }
}

#' Ratio with the same positive-denominator guard used by Stan generated quantities
#' @keywords internal
.safe_positive_ratio <- function(num, denom) {
  num / pmax(denom, 1e-10)
}


#' Conditional predictions
#'
#' Generate absolute predictions at specific covariate values for both
#' treatments.
#'
#' @param object An `mlumr_fit` object
#' @param newdata Data frame of covariate values. If `NULL`, uses IPD covariate
#'   means.
#' @param type `"response"` for probabilities, means, or rates; `"link"` for
#'   the fitted linear-predictor scale.
#' @param summary Return summary (`TRUE`) or full draws (`FALSE`)
#' @param probs Quantiles for summary
#'
#' @return A data frame with predictions for each treatment at each profile
#' @seealso [conditional_effects()] for covariate-conditional treatment
#'   *effects*; [predict.mlumr_fit()] for population-level predictions.
#' @export
#'
#' @examples
#' \dontrun{
#' conditional_predict(fit)
#' conditional_predict(fit, newdata = data.frame(age = 60, sex = 1))
#' }
conditional_predict <- function(object,
                                newdata = NULL,
                                type = c("response", "link"),
                                summary = TRUE,
                                probs = c(0.025, 0.5, 0.975)) {

  .validate_mlumr_fit_object(object)
  type <- .validate_predict_choice(type, c("response", "link"), "type")
  summary <- .validate_summary_flag(summary)
  .validate_probs(probs)

  profiles <- .conditional_profiles(object, newdata)
  X <- profiles$X
  n_profiles <- nrow(X)
  params <- .conditional_parameters(object, profiles$covariates)
  family <- object$family %||% "binomial"
  lnk <- object$link %||% get_family_config(family)$link_default

  summarize_col <- function(x) .summarize_draw_vector(x, probs)

  results <- vector("list", n_profiles)

  for (i in seq_len(n_profiles)) {
    eta <- .conditional_eta(params, X[i, , drop = FALSE])
    eta_idx <- eta$index
    eta_cmp <- eta$comparator

    if (type == "response" && lnk != "identity") {
      val_idx <- inverse_link(eta_idx, lnk)
      val_cmp <- inverse_link(eta_cmp, lnk)
    } else {
      val_idx <- eta_idx
      val_cmp <- eta_cmp
    }

    if (!summary) {
      results[[i]] <- data.frame(
        profile = i,
        index = val_idx,
        comparator = val_cmp
      )
    } else {
      s_idx <- summarize_col(val_idx)
      s_cmp <- summarize_col(val_cmp)
      qcols <- paste0("q", probs * 100)

      results[[i]] <- data.frame(
        profile = c(i, i),
        treatment = c(object$data$index_treatment,
                      object$data$comparator_treatment),
        mean = c(s_idx["mean"], s_cmp["mean"]),
        sd = c(s_idx["sd"], s_cmp["sd"]),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
      for (j in seq_along(probs)) {
        qname <- qcols[j]
        pname <- paste0(probs[j] * 100, "%")
        results[[i]][[qname]] <- c(s_idx[pname], s_cmp[pname])
      }
    }
  }

  out <- do.call(rbind, results)
  rownames(out) <- NULL
  out
}

#' Fit ML-UMR Model
#'
#' Fit a Bayesian multilevel unanchored meta-regression model using individual
#' patient data (IPD) and aggregate data (AgD). Supports binary, continuous,
#' and count outcomes.
#'
#' @param data An `mlumr_data` object with integration points (from
#'   [add_integration()])
#' @param model Model type: `"spfa"` (shared prognostic factor assumption) or
#'   `"relaxed"` (treatment-specific coefficients). Default `"spfa"`.
#' @param link Link function. For binomial: `"logit"` (default), `"probit"`,
#'   or `"cloglog"`. For normal: `"identity"` (default) or `"log"`. For
#'   poisson: `"log"` (default, only option). If `NULL`, uses the canonical
#'   default for the family.
#' @param prior_intercept Prior for treatment intercepts. Default from
#'   [default_prior_intercept()] (`prior_normal(0, 10)`, weakly informative
#'   on the linear-predictor scale). See [prior_normal()] for guidance.
#' @param prior_beta Prior for regression coefficients. May be a single
#'   prior broadcast to all covariates, or a `list` of priors of length
#'   `n_cov` for per-coefficient specification. All per-coefficient priors
#'   must share the same family and (for Student-t) df. Default from
#'   [default_prior_beta()] (`prior_normal(0, 2.5)`; Gelman et al. 2008;
#'   Stan community prior-choice wiki). Set `autoscale = TRUE` on the
#'   prior to divide the scale by each covariate's empirical SD — useful
#'   when predictors are on very different scales.
#' @param prior_sigma Prior for residual SD (normal family only). Default
#'   from [default_prior_sigma()] (`prior_normal(0, 2.5)`, half-normal via
#'   the Stan `<lower=0>` constraint). [prior_exponential()] is also
#'   supported for sigma.
#' @details
#' The model assumes that all AgD rows come from the same comparator treatment
#' and that, conditional on covariates, there is no between-study heterogeneity.
#' If AgD rows come from multiple studies with different designs or unmeasured
#' confounders, this assumption may not hold. No random effects for study-level
#' heterogeneity are included.
#'
#' **AgD scale assumptions (family = `"normal"`).** The AgD likelihood is
#' `y_agd ~ normal(E[exp(eta)], se_agd)` under `link = "log"` and
#' `y_agd ~ normal(E[eta], se_agd)` under `link = "identity"`. In both
#' cases `set_agd()` expects `outcome_mean` and `outcome_se` on the
#' **arithmetic (original, untransformed) scale**, not log-scale or
#' geometric. Passing log-scale summaries silently misspecifies the
#' likelihood. See [set_agd()] for details.
#'
#' **Comparator-population weighting is family-dependent.** Integrated
#' marginal predictions in the comparator population (`*_comparator`
#' generated quantities) are weighted by:
#' \itemize{
#'   \item **binomial**: `n_agd[k]` (AgD sample size), so larger
#'     AgD rows contribute more to the marginal mean.
#'   \item **normal**: equal weights across AgD rows (`/ n_agd_rows`),
#'     reflecting the normal likelihood's treatment of each row as a
#'     single summary statistic.
#'   \item **poisson**: `E_agd[k]` (AgD exposure), matching the
#'     rate-based likelihood.
#' }
#' Each weighting is natural for the corresponding likelihood; users
#' comparing marginal effects across families should be aware they are
#' not identically weighted.
#'
#' **Weakly-identified coefficients in the relaxed model** —
#' `beta_comparator` is identified only through AgD, so the relaxed
#' model needs informative priors (or many AgD rows) to estimate
#' effect modification reliably. [prior_sensitivity()] is the
#' recommended diagnostic.
#'
#' @seealso [prior_sensitivity()] for sensitivity of the posterior
#'   to `prior_beta`; [set_agd()] for AgD scale requirements;
#'   [prior_summary()] for introspection of the priors actually used.
#'
#' @param chains Number of MCMC chains (default 4)
#' @param iter Total iterations per chain (default 2000)
#' @param warmup Number of warmup iterations (default 1000)
#' @param seed Random seed for reproducibility
#' @param adapt_delta Target acceptance rate (default 0.95)
#' @param max_treedepth Maximum tree depth for NUTS (default 15)
#' @param refresh How often to print progress (0 = silent, default 200)
#' @param engine Stan backend: `"rstan"` (default) or `"cmdstanr"`. If `NULL`,
#'   uses the engine set by [mlumr_engine()]. See [mlumr_engine()] for setup.
#' @param verbose Logical; if `FALSE`, suppresses mlumr progress messages.
#'   Stan sampler progress is still controlled by `refresh`.
#' @param ... Additional arguments passed to the Stan sampling function
#'   ([rstan::sampling()] or cmdstanr's `$sample()` method)
#'
#' @return An object of class `mlumr_fit`
#' @export
#'
#' @examples
#' \dontrun{
#' # Binary SPFA model
#' fit_spfa <- mlumr(dat, model = "spfa")
#'
#' # Relaxed SPFA (allows effect modification)
#' fit_relaxed <- mlumr(dat, model = "relaxed")
#' }
mlumr <- function(data,
                  model = c("spfa", "relaxed"),
                  link = NULL,
                  prior_intercept = default_prior_intercept(),
                  prior_beta = default_prior_beta(),
                  prior_sigma = default_prior_sigma(),
                  chains = 4,
                  iter = 2000,
                  warmup = 1000,
                  seed = NULL,
                  adapt_delta = 0.95,
                  max_treedepth = 15,
                  refresh = 200,
                  engine = NULL,
                  verbose = TRUE,
                  ...) {

  model <- match.arg(model)

  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("`verbose` must be TRUE or FALSE.", call. = FALSE)
  }
  .validate_mlumr_sampling_args(
    chains = chains,
    iter = iter,
    warmup = warmup,
    seed = seed,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    refresh = refresh
  )
  engine <- .resolve_mlumr_engine(engine)

  if (!inherits(data, "mlumr_data")) {
    stop("`data` must be created with combine_data()", call. = FALSE)
  }
  if (!data$has_integration) {
    stop("Integration points not found. Use add_integration() first.", call. = FALSE)
  }

  validate_prior(prior_intercept, "intercept")
  if (prior_intercept$distribution == "exponential") {
    stop("prior_intercept does not support exponential priors ",
         "(treatment intercepts are unconstrained on the link scale). ",
         "Use prior_normal(), prior_student_t(), or prior_cauchy().",
         call. = FALSE)
  }
  # prior_beta may be a single prior or a list of per-coefficient priors;
  # per-coefficient validation happens inside stan_prior_fields_beta().
  if (is_single_prior(prior_beta)) {
    validate_prior(prior_beta, "beta")
    if (prior_beta$distribution == "exponential") {
      stop("prior_beta does not support exponential priors ",
           "(coefficients are unconstrained on the link scale). ",
           "Use prior_normal(), prior_student_t(), or prior_cauchy().",
           call. = FALSE)
    }
  } else if (!is.list(prior_beta)) {
    stop("`prior_beta` must be a prior list or a list of priors.", call. = FALSE)
  }

  # C.3: autoscale is only consumed by prior_beta. Warn if the user set it
  # on prior_intercept or prior_sigma, because it will be silently ignored.
  if (isTRUE(prior_intercept$autoscale)) {
    warning("`autoscale = TRUE` on prior_intercept is ignored; ",
            "autoscaling is only applied to prior_beta.", call. = FALSE)
  }

  family <- data$family %||% "binomial"
  link_info <- check_link(family, link)

  if (family == "normal") {
    validate_prior(prior_sigma, "sigma")
    if (isTRUE(prior_sigma$autoscale)) {
      warning("`autoscale = TRUE` on prior_sigma is ignored; ",
              "autoscaling is only applied to prior_beta.", call. = FALSE)
    }
  } else if (!is.null(prior_sigma) && !isTRUE(prior_sigma$default)) {
    warning("`prior_sigma` is ignored for non-normal families.",
            call. = FALSE)
  }

  # Warn about weak identifiability in relaxed models with sparse AgD
  if (model == "relaxed") {
    n_agd_rows_check <- nrow(data$agd$data)
    n_cov_check <- data$n_covariates
    if (n_agd_rows_check < 2 * n_cov_check) {
      warning(sprintf(
        paste0("Relaxed model with %d AgD row(s) and %d covariate(s): ",
               "beta_comparator may be weakly identified. ",
               "Consider using more informative priors on beta or the SPFA model."),
        n_agd_rows_check, n_cov_check
      ), call. = FALSE)
    }
  }

  prepared <- .mlumr_build_stan_data(
    data = data,
    family = family,
    link_info = link_info,
    prior_intercept = prior_intercept,
    prior_beta = prior_beta,
    prior_sigma = prior_sigma
  )
  stan_data <- prepared$stan_data

  # Select Stan model (family_config is the single source of truth)
  model_name <- paste0(get_family_config(family)$stan_prefix, "_", model)

  .mlumr_log_fit_start(model_name, family, link_info$link, stan_data,
                       engine, verbose)

  # Fit model
  result <- .mlumr_fit_backend(
    engine = engine,
    model_name = model_name,
    stan_data = stan_data,
    chains = chains,
    iter = iter,
    warmup = warmup,
    seed = seed,
    adapt_delta = adapt_delta,
    max_treedepth = max_treedepth,
    refresh = refresh,
    ...
  )

  fit <- result$native_fit
  draws <- result$draws
  summary_df <- result$summary_df
  n_divergent <- result$n_divergent
  n_max_td <- result$n_max_td

  # Resolved prior info for prior_summary(): carries both the user-specified
  # priors and the Stan-scale values actually used (post-autoscale), plus the
  # covariate SDs used for autoscaling, so the summary can reconstruct the
  # effective prior that was sampled.
  priors <- .mlumr_prior_metadata(
    data = data,
    family = family,
    prior_intercept = prior_intercept,
    prior_beta = prior_beta,
    prior_sigma = prior_sigma,
    beta_fields = prepared$beta_fields,
    sd_x = prepared$sd_x
  )

  out <- list(
    stanfit = fit,
    draws = draws,
    summary = summary_df,
    diagnostics = list(
      n_divergent = n_divergent,
      n_max_treedepth = n_max_td
    ),
    data = data,
    family = family,
    link = link_info$link,
    link_code = link_info$code,
    model = model,
    model_name = model_name,
    stan_data = stan_data,
    engine = engine,
    priors = priors,
    sampling_args = list(
      chains = chains,
      iter = iter,
      warmup = warmup,
      seed = seed,
      adapt_delta = adapt_delta,
      max_treedepth = max_treedepth
    )
  )

  class(out) <- c("mlumr_fit", "list")

  check_diagnostics(out)

  mlumr_message("Fitting complete!", verbose = verbose)
  out
}

#' Build the Stan data list for mlumr()
#' @keywords internal
.mlumr_build_stan_data <- function(data, family, link_info, prior_intercept,
                                   prior_beta, prior_sigma) {
  ipd_data <- data$ipd$data
  agd_data <- data$agd$data
  X_ipd <- as.matrix(ipd_data[, data$covariates])

  # Autoscaling is based on the IPD covariate scale because the regression
  # coefficients are estimated from individual observations.
  sd_x <- apply(X_ipd, 2, stats::sd)
  intercept_fields <- stan_prior_fields(prior_intercept)
  beta_fields <- stan_prior_fields_beta(
    prior_beta,
    data$n_covariates,
    sd_x = sd_x,
    covariate_names = data$covariates
  )

  stan_data <- list(
    n_ipd = nrow(ipd_data),
    n_cov = data$n_covariates,
    X_ipd = X_ipd,
    n_agd_rows = nrow(agd_data),
    n_int = data$n_int,
    X_int = data$integration_points,
    prior_intercept_mean = intercept_fields$mean,
    prior_intercept_sd = intercept_fields$sd,
    prior_intercept_dist = intercept_fields$dist,
    prior_intercept_df = intercept_fields$df,
    prior_beta_mean = as.array(beta_fields$mean),
    prior_beta_sd = as.array(beta_fields$sd),
    prior_beta_dist = beta_fields$dist,
    prior_beta_df = beta_fields$df,
    link = link_info$code
  )

  if (family == "binomial") {
    stan_data$y_ipd <- as.integer(ipd_data$.outcome)
    stan_data$n_agd <- array(as.integer(agd_data$.n))
    stan_data$r_agd <- array(as.integer(agd_data$.r))
  } else if (family == "normal") {
    sigma_fields <- stan_prior_fields(prior_sigma)
    stan_data$y_ipd <- as.numeric(ipd_data$.outcome)
    stan_data$y_agd <- array(as.numeric(agd_data$.y))
    stan_data$se_agd <- array(as.numeric(agd_data$.se))
    stan_data$prior_sigma_location <- sigma_fields$mean
    stan_data$prior_sigma_scale <- sigma_fields$sd
    stan_data$prior_sigma_dist <- sigma_fields$dist
    stan_data$prior_sigma_df <- sigma_fields$df
  } else {
    stan_data$y_ipd <- as.integer(ipd_data$.outcome)
    stan_data$E_ipd <- as.numeric(ipd_data$.exposure)
    stan_data$r_agd <- array(as.integer(agd_data$.r))
    stan_data$E_agd <- array(as.numeric(agd_data$.E))
  }

  list(stan_data = stan_data, beta_fields = beta_fields, sd_x = sd_x)
}

#' Validate mlumr() sampler controls before backend dispatch
#' @keywords internal
.validate_mlumr_sampling_args <- function(chains, iter, warmup, seed,
                                          adapt_delta, max_treedepth,
                                          refresh) {
  .validate_mlumr_integer(chains, "chains", lower = 1L)
  .validate_mlumr_integer(iter, "iter", lower = 1L)
  .validate_mlumr_integer(warmup, "warmup", lower = 0L)
  if (warmup >= iter) {
    stop("`warmup` must be smaller than `iter` so post-warmup draws remain.",
         call. = FALSE)
  }
  if (!is.null(seed)) {
    .validate_mlumr_integer(seed, "seed", lower = 0L)
  }
  if (!is.numeric(adapt_delta) || length(adapt_delta) != 1L ||
        !is.finite(adapt_delta) || adapt_delta <= 0 || adapt_delta >= 1) {
    stop("`adapt_delta` must be a single finite number between 0 and 1.",
         call. = FALSE)
  }
  .validate_mlumr_integer(max_treedepth, "max_treedepth", lower = 1L)
  .validate_mlumr_integer(refresh, "refresh", lower = 0L)
  invisible(TRUE)
}


#' Validate an integer-like mlumr() argument
#' @keywords internal
.validate_mlumr_integer <- function(x, name, lower) {
  valid <- is.numeric(x) &&
    length(x) == 1L &&
    is.finite(x) &&
    x == floor(x) &&
    x >= lower
  if (!valid) {
    stop(sprintf("`%s` must be a single integer >= %d.", name, lower),
         call. = FALSE)
  }
  invisible(TRUE)
}


#' Resolve and validate mlumr() backend engine
#' @keywords internal
.resolve_mlumr_engine <- function(engine) {
  .validate_engine_name(engine %||% get_engine())
}

#' Log mlumr() fit metadata
#' @keywords internal
.mlumr_log_fit_start <- function(model_name, family, link, stan_data,
                                 engine, verbose) {
  mlumr_message(sprintf("Fitting ML-UMR (%s, %s, link=%s)...",
                        model_name, family, link),
                verbose = verbose)
  mlumr_message(sprintf("  IPD: n = %d", stan_data$n_ipd),
                verbose = verbose)

  if (family == "binomial") {
    mlumr_message(sprintf("  AgD: %d rows, total n = %d",
                          stan_data$n_agd_rows, sum(stan_data$n_agd)),
                  verbose = verbose)
  } else if (family == "normal") {
    mlumr_message(sprintf("  AgD: %d rows", stan_data$n_agd_rows),
                  verbose = verbose)
  } else {
    mlumr_message(sprintf("  AgD: %d rows, total exposure = %.1f",
                          stan_data$n_agd_rows, sum(stan_data$E_agd)),
                  verbose = verbose)
  }

  mlumr_message(sprintf("  Covariates: %d, Integration points: %d",
                        stan_data$n_cov, stan_data$n_int),
                verbose = verbose)
  mlumr_message(sprintf("  Engine: %s", engine), verbose = verbose)
}

#' Dispatch mlumr() sampling to the selected backend
#' @keywords internal
.mlumr_fit_backend <- function(engine, model_name, stan_data, chains, iter,
                               warmup, seed, adapt_delta, max_treedepth,
                               refresh, ...) {
  if (engine == "cmdstanr") {
    fit_cmdstanr(model_name, stan_data, chains, iter, warmup,
                 seed, adapt_delta, max_treedepth, refresh, ...)
  } else {
    fit_rstan(model_name, stan_data, chains, iter, warmup,
              seed, adapt_delta, max_treedepth, refresh, ...)
  }
}

#' Store user priors plus the resolved Stan-scale beta prior
#' @keywords internal
.mlumr_prior_metadata <- function(data, family, prior_intercept, prior_beta,
                                  prior_sigma, beta_fields, sd_x) {
  priors <- list(
    intercept = prior_intercept,
    beta = prior_beta,
    beta_resolved = list(
      covariate_names = data$covariates,
      mean = beta_fields$mean,
      sd = beta_fields$sd,
      dist = beta_fields$dist,
      df = beta_fields$df,
      autoscale = beta_fields$autoscale,
      sd_x = sd_x
    )
  )

  if (family == "normal") {
    priors$sigma <- prior_sigma
  }

  priors
}

test_that("mlumr sampler controls are validated before backend dispatch", {
  expect_silent(.validate_mlumr_sampling_args(
    chains = 2,
    iter = 100,
    warmup = 50,
    seed = 123,
    adapt_delta = 0.95,
    max_treedepth = 15,
    refresh = 0
  ))

  expect_error(
    .validate_mlumr_sampling_args(0, 100, 50, NULL, 0.95, 15, 0),
    "`chains` must be a single integer >= 1"
  )
  expect_error(
    .validate_mlumr_sampling_args(2, 0, 0, NULL, 0.95, 15, 0),
    "`iter` must be a single integer >= 1"
  )
  expect_error(
    .validate_mlumr_sampling_args(2, 100, -1, NULL, 0.95, 15, 0),
    "`warmup` must be a single integer >= 0"
  )
  expect_error(
    .validate_mlumr_sampling_args(2, 100, 100, NULL, 0.95, 15, 0),
    "`warmup` must be smaller than `iter`"
  )
  expect_error(
    .validate_mlumr_sampling_args(2, 100, 50, -1, 0.95, 15, 0),
    "`seed` must be a single integer >= 0"
  )
  expect_error(
    .validate_mlumr_sampling_args(2, 100, 50, NULL, 1, 15, 0),
    "`adapt_delta` must be a single finite number between 0 and 1"
  )
  expect_error(
    .validate_mlumr_sampling_args(2, 100, 50, NULL, 0.95, 0, 0),
    "`max_treedepth` must be a single integer >= 1"
  )
  expect_error(
    .validate_mlumr_sampling_args(2, 100, 50, NULL, 0.95, 15, -1),
    "`refresh` must be a single integer >= 0"
  )
})


test_that("mlumr engine is scalar and supported", {
  old_engine <- getOption("mlumr.stan_engine")
  on.exit(options(mlumr.stan_engine = old_engine), add = TRUE)
  options(mlumr.stan_engine = "rstan")

  expect_equal(.resolve_mlumr_engine(NULL), "rstan")
  expect_equal(.resolve_mlumr_engine("cmdstanr"), "cmdstanr")
  expect_error(.resolve_mlumr_engine(c("rstan", "cmdstanr")),
               "`engine` must be 'rstan' or 'cmdstanr'")
  expect_error(.resolve_mlumr_engine(NA_character_),
               "`engine` must be 'rstan' or 'cmdstanr'")
  expect_error(.resolve_mlumr_engine("bad"),
               "`engine` must be 'rstan' or 'cmdstanr'")
  expect_error(.resolve_mlumr_engine("c"),
               "`engine` must be 'rstan' or 'cmdstanr'")
})


test_that("check_diagnostics ignores unavailable Rhat and ESS cleanly", {
  fit <- list(
    diagnostics = list(n_divergent = NA_real_, n_max_treedepth = NA_real_),
    summary = data.frame(variable = "x", Rhat = NA_real_, n_eff = NA_real_),
    sampling_args = list(adapt_delta = 0.95, max_treedepth = 15)
  )
  class(fit) <- c("mlumr_fit", "list")

  expect_warning(check_diagnostics(fit), NA)
})


test_that("check_diagnostics still warns on finite diagnostic problems", {
  fit <- list(
    diagnostics = list(n_divergent = 2, n_max_treedepth = 3),
    summary = data.frame(variable = "x", Rhat = 1.06, n_eff = 100),
    sampling_args = list(adapt_delta = 0.8, max_treedepth = 10)
  )
  class(fit) <- c("mlumr_fit", "list")

  expect_warning(check_diagnostics(fit), "divergent transitions")
  fit$diagnostics$n_divergent <- 0
  expect_warning(check_diagnostics(fit), "max treedepth")
  fit$diagnostics$n_max_treedepth <- 0
  expect_warning(check_diagnostics(fit), "Rhat values > 1.05")
  fit$summary$Rhat <- 1
  expect_warning(check_diagnostics(fit), "ESS values < 400")
})

#' Extract the full pointwise log-likelihood matrix from an mlumr_fit
#'
#' Combines the IPD and AgD per-observation log-likelihood draws into a
#' single matrix with one row per posterior draw and one column per
#' observation. The result is suitable for direct use with
#' [loo::loo()] / [loo::waic()].
#'
#' @param object An `mlumr_fit` object.
#' @return A numeric matrix of dimension `n_draws x (n_ipd + n_agd_rows)`.
#'   IPD columns come first, then AgD rows.
#' @keywords internal
extract_log_lik <- function(object) {
  .validate_mlumr_fit_object(object)

  draws <- object$draws
  if (is.null(draws) || is.null(colnames(draws))) {
    stop("`object$draws` must contain named posterior draw columns.",
         call. = FALSE)
  }

  ipd_cols <- .ordered_log_lik_columns(draws, "ipd")
  agd_cols <- .ordered_log_lik_columns(draws, "agd")
  if (length(ipd_cols) == 0L && length(agd_cols) == 0L) {
    stop(
      "Pointwise log-likelihood columns not found in draws. ",
      "This package requires Stan models that generate `log_lik_ipd` and ",
      "`log_lik_agd` as vectors (per-observation). If you have a fit from ",
      "a pre-v0.2 version of mlumr, refit with the current version.",
      call. = FALSE
    )
  }

  selected <- draws[, c(ipd_cols, agd_cols), drop = FALSE]
  numeric_cols <- vapply(selected, is.numeric, logical(1))
  if (!all(numeric_cols)) {
    stop("Pointwise log-likelihood columns must be numeric.", call. = FALSE)
  }

  log_lik <- as.matrix(selected)
  .validate_log_lik_matrix(log_lik)
  log_lik
}


#' Ordered pointwise log-likelihood columns for one data source
#' @keywords internal
.ordered_log_lik_columns <- function(draws, source) {
  pattern <- sprintf("^log_lik_%s\\[[0-9]+\\]$", source)
  cols <- grep(pattern, colnames(draws), value = TRUE)
  if (length(cols) == 0L) {
    return(character())
  }
  cols[order(.log_lik_column_index(cols, source))]
}


#' Extract integer indexes from Stan vector column names
#' @keywords internal
.log_lik_column_index <- function(cols, source) {
  pattern <- sprintf("^log_lik_%s\\[|\\]$", source)
  as.integer(gsub(pattern, "", cols))
}


#' Validate a pointwise log-likelihood matrix
#' @keywords internal
.validate_log_lik_matrix <- function(log_lik) {
  if (!is.matrix(log_lik) || !is.numeric(log_lik)) {
    stop("Pointwise log-likelihood must be a numeric matrix.", call. = FALSE)
  }
  if (nrow(log_lik) == 0L || ncol(log_lik) == 0L) {
    stop("Pointwise log-likelihood matrix must be non-empty.", call. = FALSE)
  }
  if (anyNA(log_lik) || any(!is.finite(log_lik))) {
    stop("Pointwise log-likelihood values must be finite.", call. = FALSE)
  }
  invisible(TRUE)
}


#' Calculate DIC for model comparison
#'
#' Computes the Deviance Information Criterion using the variance-based
#' effective-parameters formula from Gelman et al. (2004): `pD = 0.5 * Var(D)`.
#' This is more stable than the plug-in alternative for multimodal posteriors.
#'
#' DIC is retained for backward compatibility and rough comparison. For
#' principled Bayesian model comparison, prefer [calculate_loo()] or
#' [calculate_waic()] (Vehtari, Gelman, Gabry 2017).
#'
#' @param object An `mlumr_fit` object
#'
#' @return A list of class `mlumr_dic` with components `DIC`, `pD`, `D_bar`
#' @export
#'
#' @examples
#' \dontrun{
#' dic_spfa <- calculate_dic(fit_spfa)
#' dic_relaxed <- calculate_dic(fit_relaxed)
#' }
calculate_dic <- function(object) {

  log_lik <- extract_log_lik(object)
  if (nrow(log_lik) < 2L) {
    stop("DIC requires at least two posterior draws.", call. = FALSE)
  }
  log_lik_total <- rowSums(log_lik)
  D <- -2 * log_lik_total
  D_bar <- mean(D)
  # Effective number of parameters (Gelman et al. 2004, variance-based):
  # pD = 0.5 * Var(D). More stable than the plug-in alternative (D_bar - D_hat)
  # when the posterior is multimodal.
  pD <- 0.5 * var(D)
  DIC <- D_bar + pD

  out <- list(
    DIC = DIC,
    pD = pD,
    D_bar = D_bar,
    model = .mlumr_model_label(object)
  )

  class(out) <- "mlumr_dic"
  out
}


#' @method print mlumr_dic
#' @export
print.mlumr_dic <- function(x, ...) {
  cat("DIC for ML-UMR Model\n")
  cat("====================\n\n")
  cat("Model:", x$model, "\n")
  cat("DIC:", round(x$DIC, 2), "\n")
  cat("pD:", round(x$pD, 2), "\n")
  cat("D_bar:", round(x$D_bar, 2), "\n")
  invisible(x)
}


#' Calculate LOO-CV for an mlumr_fit
#'
#' Computes approximate leave-one-out cross-validation (PSIS-LOO, Vehtari,
#' Gelman, Gabry 2017) using the pointwise log-likelihoods stored by the
#' Stan models. Returns a `loo` object from the `loo` package.
#'
#' Pareto-k diagnostics: values > 0.7 indicate observations for which the
#' PSIS approximation is unreliable; the printed output flags these.
#' Typical remedies are running more iterations, using `moment_match = TRUE`,
#' or (for highly influential AgD rows) refitting without the offending
#' observation to check sensitivity.
#'
#' @note
#' **AgD rows are treated as independent observations.** Each AgD row
#' contributes one column to the pointwise `log_lik` matrix. If two or
#' more AgD rows come from the same study (e.g. subgroup summaries
#' within a single trial) the PSIS-LOO approximation does not account
#' for the within-study clustering; effective sample sizes are
#' inflated and Pareto-k warnings are understated. For clustered AgD,
#' corroborate with [prior_sensitivity()] or refit omitting suspect
#' rows to check the influence on the posterior.
#'
#' @param object An `mlumr_fit` object.
#' @param ... Additional arguments passed to [loo::loo.array()].
#'
#' @return An object of class `psis_loo` (see [loo::loo()]).
#' @export
#' @examples
#' \dontrun{
#' loo_spfa <- calculate_loo(fit_spfa)
#' print(loo_spfa)
#' }
calculate_loo <- function(object, ...) {
  if (!requireNamespace("loo", quietly = TRUE)) {
    stop("The 'loo' package is required for calculate_loo(). ",
         "Install with install.packages('loo').", call. = FALSE)
  }
  log_lik <- extract_log_lik(object)
  r_eff <- .relative_eff_from_log_lik(log_lik, .chain_id(object))
  loo::loo(log_lik, r_eff = r_eff, ...)
}


#' Calculate WAIC for an mlumr_fit
#'
#' Watanabe-Akaike Information Criterion (Watanabe 2010) based on the
#' pointwise log-likelihoods. WAIC is asymptotically equivalent to
#' LOO-CV; prefer [calculate_loo()] when Pareto-k is well-behaved.
#'
#' @note
#' As with [calculate_loo()], each AgD row is treated as an
#' independent observation. WAIC will be optimistic for AgD rows
#' that share a study (see the note on `calculate_loo()`).
#'
#' @param object An `mlumr_fit` object.
#' @param ... Additional arguments passed to [loo::waic()].
#'
#' @return An object of class `waic` (see [loo::waic()]).
#' @export
#' @examples
#' \dontrun{
#' waic_spfa <- calculate_waic(fit_spfa)
#' }
calculate_waic <- function(object, ...) {
  if (!requireNamespace("loo", quietly = TRUE)) {
    stop("The 'loo' package is required for calculate_waic(). ",
         "Install with install.packages('loo').", call. = FALSE)
  }
  log_lik <- extract_log_lik(object)
  loo::waic(log_lik, ...)
}


#' Compare fitted ML-UMR models
#'
#' Compare two or more `mlumr_fit` objects by DIC (default), LOO, or WAIC.
#' For LOO/WAIC, [loo::loo_compare()] is used under the hood; the output
#' is the standard `loo_compare` table. For DIC the return is a data
#' frame ordered by DIC.
#'
#' DIC is the default for backward compatibility and because it has no
#' additional package dependencies. For principled Bayesian model
#' comparison, LOO (Vehtari, Gelman, Gabry 2017) is preferred and
#' requires the optional `loo` package.
#'
#' @param ... Two or more `mlumr_fit` objects. For DIC, `mlumr_dic`
#'   objects are also accepted.
#' @param criterion One of `"dic"` (default), `"loo"`, or `"waic"`.
#'   LOO and WAIC require the optional `loo` package.
#'
#' @return For `"loo"` / `"waic"`: a `compare.loo` table from
#'   [loo::loo_compare()]. For `"dic"`: a data frame (invisibly) with
#'   columns `Model`, `DIC`, `pD`, `Delta_DIC`.
#' @export
compare_models <- function(..., criterion = c("dic", "loo", "waic")) {

  criterion <- .validate_diagnostic_choice(criterion, c("dic", "loo", "waic"),
                                           "criterion")
  models <- list(...)
  .validate_model_count(models)

  if (criterion == "dic") {
    dics <- lapply(models, function(m) {
      if (inherits(m, "mlumr_fit")) {
        calculate_dic(m)
      } else if (inherits(m, "mlumr_dic")) {
        m
      } else {
        stop("For criterion = 'dic', arguments must be mlumr_fit or mlumr_dic objects",
             call. = FALSE)
      }
    })

    model_names <- vapply(dics, function(d) d$model, character(1))
    dic_vals <- vapply(dics, function(d) d$DIC, numeric(1))
    pD_vals <- vapply(dics, function(d) d$pD, numeric(1))
    model_names <- .comparison_names(models, model_names)

    out <- data.frame(
      Model = model_names,
      DIC = round(dic_vals, 2),
      pD = round(pD_vals, 2),
      Delta_DIC = round(dic_vals - min(dic_vals), 2)
    )

    out <- out[order(out$DIC), ]
    rownames(out) <- NULL

    cat("\nModel Comparison (DIC)\n")
    cat("======================\n\n")
    print(out, row.names = FALSE)
    cat("\nLower DIC = better fit. Delta_DIC > 5 is a rough heuristic for\n")
    cat("meaningful difference, not a formally calibrated threshold.\n")
    cat("DIC should not be the sole basis for model selection.\n")

    return(invisible(out))
  }

  # LOO / WAIC path
  if (!requireNamespace("loo", quietly = TRUE)) {
    stop("The 'loo' package is required for LOO/WAIC comparison. ",
         "Install with install.packages('loo').", call. = FALSE)
  }
  if (!all(vapply(models, inherits, logical(1), what = "mlumr_fit"))) {
    stop("For criterion = 'loo' or 'waic', all arguments must be mlumr_fit objects",
         call. = FALSE)
  }

  calc_fn <- if (criterion == "loo") calculate_loo else calculate_waic
  ic_list <- lapply(models, calc_fn)
  names(ic_list) <- .comparison_names(
    models,
    vapply(models, .mlumr_model_label, character(1))
  )

  cat(sprintf("\nModel Comparison (%s)\n", toupper(criterion)))
  cat(strrep("=", 22), "\n\n", sep = "")
  cmp <- loo::loo_compare(ic_list)
  print(cmp)
  cat("\nelpd_diff is the difference in expected log pointwise predictive\n")
  cat("density vs the best model. se_diff > 2 is the conventional threshold\n")
  cat("for a meaningful difference (|elpd_diff| > 4 * se_diff is stronger).\n")
  invisible(cmp)
}


# Helper: recover chain ids from draws ordering.
# rstan stores draws in chain-major order (post-warmup iterations per chain
# contiguous). cmdstanr's format = "df" is similar. If sampling_args is
# unavailable we fall back to all-one (treated as a single chain, which
# inflates r_eff but does not produce incorrect elpd).
.chain_id <- function(object) {
  n_draws <- nrow(object$draws)
  if (!is.numeric(n_draws) || length(n_draws) != 1L || n_draws < 1L) {
    return(integer())
  }
  chains <- object$sampling_args$chains %||% 1L
  valid_chains <- is.numeric(chains) &&
    length(chains) == 1L &&
    is.finite(chains) &&
    chains >= 1 &&
    chains == as.integer(chains)
  if (!valid_chains) {
    return(rep(1L, n_draws))
  }
  chains <- as.integer(chains)
  iter_per_chain <- n_draws %/% chains
  if (iter_per_chain * chains != n_draws) return(rep(1L, n_draws))
  rep(seq_len(chains), each = iter_per_chain)
}


#' Compute stable relative effective sample sizes from log-likelihoods
#' @keywords internal
.relative_eff_from_log_lik <- function(log_lik, chain_id) {
  .validate_log_lik_matrix(log_lik)
  col_max <- apply(log_lik, 2L, max)
  stabilized <- sweep(log_lik, 2L, col_max, "-")
  loo::relative_eff(exp(stabilized), chain_id = chain_id)
}


#' Validate a diagnostics/model-comparison choice
#' @keywords internal
.validate_diagnostic_choice <- function(x, choices, name) {
  if (identical(x, choices)) {
    return(choices[[1L]])
  }

  valid <- is.character(x) &&
    length(x) == 1L &&
    !is.na(x) &&
    nzchar(x) &&
    x %in% choices

  if (!valid) {
    stop(sprintf("`%s` must be one of: %s.",
                 name, paste(sprintf("'%s'", choices), collapse = ", ")),
         call. = FALSE)
  }

  x
}


#' Validate that model comparison has at least two candidates
#' @keywords internal
.validate_model_count <- function(models) {
  if (length(models) < 2L) {
    stop("compare_models() requires at least two models.", call. = FALSE)
  }
  invisible(TRUE)
}


#' Human-readable model label for an mlumr fit or DIC object
#' @keywords internal
.mlumr_model_label <- function(object) {
  if (inherits(object, "mlumr_dic") && !is.null(object$model)) {
    return(as.character(object$model)[[1L]])
  }

  model <- object$model %||% NA_character_
  if (!is.character(model) || length(model) != 1L || is.na(model)) {
    return("ML-UMR")
  }

  switch(model,
    spfa = "SPFA",
    relaxed = "Relaxed SPFA",
    model
  )
}


#' Use user-supplied comparison names when available
#' @keywords internal
.comparison_names <- function(models, fallback) {
  user_names <- names(models)
  if (is.null(user_names)) {
    return(fallback)
  }
  use_user_name <- nzchar(user_names)
  out <- fallback
  out[use_user_name] <- user_names[use_user_name]
  out
}


#' Check MCMC diagnostics and warn if issues found
#' @param fit An `mlumr_fit` object
#' @keywords internal
check_diagnostics <- function(fit) {

  .validate_mlumr_fit_object(fit)
  diag <- fit$diagnostics %||% list()
  sampling_args <- fit$sampling_args %||% list()

  n_divergent <- .diagnostic_count(diag$n_divergent)
  if (n_divergent > 0) {
    warning(sprintf(
      "%d divergent transitions detected. Consider increasing adapt_delta (currently %s).",
      n_divergent, .diagnostic_value(sampling_args$adapt_delta)
    ), call. = FALSE)
  }

  n_max_treedepth <- .diagnostic_count(diag$n_max_treedepth)
  if (n_max_treedepth > 0) {
    warning(sprintf(
      "%d iterations hit max treedepth. Consider increasing max_treedepth (currently %s).",
      n_max_treedepth, .diagnostic_value(sampling_args$max_treedepth)
    ), call. = FALSE)
  }

  rhat_vals <- .finite_numeric_values(fit$summary$Rhat)
  if (length(rhat_vals) > 0L) {
    max_rhat <- max(rhat_vals)
    if (max_rhat > 1.05) {
      warning(sprintf(
        "Some Rhat values > 1.05 (max = %.3f). Chains have likely NOT converged.",
        max_rhat
      ), call. = FALSE)
    } else if (max_rhat > 1.01) {
      warning(sprintf(
        "Some Rhat values > 1.01 (max = %.3f). Chains may not have fully converged.",
        max_rhat
      ), call. = FALSE)
    }
  }

  ess_vals <- .finite_numeric_values(fit$summary$n_eff)
  if (length(ess_vals) > 0L) {
    min_ess <- min(ess_vals)
    if (min_ess < 400) {
      warning(sprintf(
        "Some ESS values < 400 (min = %.1f). Consider running more iterations.",
        min_ess
      ), call. = FALSE)
    }
  }

  invisible(NULL)
}


#' Read a non-negative scalar diagnostic count
#' @keywords internal
.diagnostic_count <- function(x) {
  if (!is.numeric(x) || length(x) != 1L || !is.finite(x) || x < 0) {
    return(0)
  }
  as.integer(x)
}


#' Format a diagnostic setting for warning messages
#' @keywords internal
.diagnostic_value <- function(x) {
  if (is.null(x) || length(x) != 1L || is.na(x)) {
    return("unknown")
  }
  as.character(x)
}


#' Return finite numeric values from a summary column
#' @keywords internal
.finite_numeric_values <- function(x) {
  if (!is.numeric(x)) {
    return(numeric())
  }
  x[is.finite(x)]
}

# Test DIC and model comparison using mock fit objects
# (no Stan compilation needed)

make_mock_fit <- function(model = "spfa", n_draws = 100,
                           n_ipd = 10L, n_agd = 3L) {
  set.seed(42)
  # Stan models produce pointwise log_lik as vectors: log_lik_ipd[1..n_ipd]
  # and log_lik_agd[1..n_agd]. The test mock mirrors that contract so
  # extract_log_lik() / calculate_dic() parse the columns correctly.
  ll_ipd <- matrix(rnorm(n_draws * n_ipd, -5, 0.5), nrow = n_draws)
  ll_agd <- matrix(rnorm(n_draws * n_agd, -6, 0.5), nrow = n_draws)
  colnames(ll_ipd) <- sprintf("log_lik_ipd[%d]", seq_len(n_ipd))
  colnames(ll_agd) <- sprintf("log_lik_agd[%d]", seq_len(n_agd))

  draws <- data.frame(
    ll_ipd,
    ll_agd,
    lor_index = rnorm(n_draws, 0.5, 0.2),
    lor_comparator = rnorm(n_draws, 0.4, 0.2),
    check.names = FALSE
  )

  summary_df <- data.frame(
    variable = c("mu_index", "mu_comparator", "lor_index", "lor_comparator"),
    mean = c(-0.5, -0.8, 0.5, 0.4),
    sd = c(0.1, 0.1, 0.2, 0.2),
    Rhat = c(1.001, 1.002, 1.003, 1.002),
    n_eff = c(800, 750, 600, 650),
    stringsAsFactors = FALSE
  )

  out <- list(
    draws = draws,
    summary = summary_df,
    model = model,
    diagnostics = list(n_divergent = 0, n_max_treedepth = 0),
    sampling_args = list(adapt_delta = 0.95, max_treedepth = 15, chains = 4)
  )
  class(out) <- c("mlumr_fit", "list")
  out
}


test_that("calculate_dic returns correct structure", {
  fit <- make_mock_fit()
  dic <- calculate_dic(fit)

  expect_s3_class(dic, "mlumr_dic")
  expect_true(is.numeric(dic$DIC))
  expect_true(is.numeric(dic$pD))
  expect_true(is.numeric(dic$D_bar))
  expect_true(dic$pD >= 0)
  expect_equal(dic$model, "SPFA")
})

test_that("calculate_dic uses variance-based pD", {
  fit <- make_mock_fit()
  dic <- calculate_dic(fit)

  # Manually compute expected values from the vector-indexed columns
  ipd_cols <- grep("^log_lik_ipd\\[", colnames(fit$draws), value = TRUE)
  agd_cols <- grep("^log_lik_agd\\[", colnames(fit$draws), value = TRUE)
  ll_total <- rowSums(fit$draws[, c(ipd_cols, agd_cols), drop = FALSE])
  D <- -2 * ll_total
  expected_pD <- 0.5 * var(D)
  expected_D_bar <- mean(D)

  expect_equal(dic$pD, expected_pD)
  expect_equal(dic$D_bar, expected_D_bar)
  expect_equal(dic$DIC, expected_D_bar + expected_pD)
})

test_that("extract_log_lik orders shuffled double-digit columns", {
  fit <- make_mock_fit(n_ipd = 12L, n_agd = 2L)
  set.seed(99)
  fit$draws <- fit$draws[, sample(names(fit$draws)), drop = FALSE]

  ll <- mlumr:::extract_log_lik(fit)

  expect_equal(colnames(ll)[1], "log_lik_ipd[1]")
  expect_equal(colnames(ll)[10], "log_lik_ipd[10]")
  expect_equal(colnames(ll)[13], "log_lik_agd[1]")
})

test_that("extract_log_lik rejects malformed pointwise values", {
  fit <- make_mock_fit()
  fit$draws[1, "log_lik_ipd[1]"] <- NA_real_
  expect_error(mlumr:::extract_log_lik(fit), "finite")

  fit <- make_mock_fit()
  fit$draws[1, "log_lik_ipd[1]"] <- -Inf
  expect_error(mlumr:::extract_log_lik(fit), "finite")
})

test_that("calculate_dic requires at least two posterior draws", {
  fit <- make_mock_fit(n_draws = 1L)
  expect_error(calculate_dic(fit), "at least two posterior draws")
})

test_that("calculate_dic rejects non-mlumr_fit objects", {
  expect_error(calculate_dic(list()), "mlumr_fit")
})

test_that("print.mlumr_dic works", {
  fit <- make_mock_fit()
  dic <- calculate_dic(fit)
  expect_output(print(dic), "DIC for ML-UMR")
  expect_output(print(dic), "SPFA")
})

test_that("compare_models compares two models", {
  fit1 <- make_mock_fit("spfa")
  fit2 <- make_mock_fit("relaxed")

  expect_output(result <- compare_models(fit1, fit2), "Model Comparison")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 2)
  expect_true("Delta_DIC" %in% names(result))
  # Best model should have Delta_DIC = 0
  expect_equal(min(result$Delta_DIC), 0)
})

test_that("compare_models validates model count and criterion", {
  fit1 <- make_mock_fit("spfa")
  fit2 <- make_mock_fit("relaxed")

  expect_error(compare_models(fit1), "at least two models")
  expect_error(compare_models(fit1, fit2, criterion = "l"), "`criterion`")
})

test_that("compare_models uses supplied names when present", {
  fit1 <- make_mock_fit("spfa")
  fit2 <- make_mock_fit("relaxed")

  expect_output(
    result <- compare_models(reference = fit1, alternative = fit2),
    "Model Comparison"
  )
  expect_setequal(result$Model, c("reference", "alternative"))
})

test_that("compare_models accepts mlumr_dic objects", {
  fit1 <- make_mock_fit("spfa")
  dic1 <- calculate_dic(fit1)

  fit2 <- make_mock_fit("relaxed")

  # Mix of fit and dic objects
  expect_output(compare_models(dic1, fit2), "Model Comparison")
})

test_that(".chain_id falls back to one chain when metadata is invalid", {
  fit <- make_mock_fit(n_draws = 40)
  fit$sampling_args$chains <- NA_real_

  ids <- mlumr:::.chain_id(fit)

  expect_equal(length(ids), 40L)
  expect_true(all(ids == 1L))
})

# Pure-R tests for calculate_loo(), calculate_waic(), and mixed-criterion
# compare_models(). No Stan compilation — we construct mock mlumr_fit
# objects whose $draws carries the vector-indexed `log_lik_ipd[i]` and
# `log_lik_agd[i]` columns that all six shipping Stan models emit.

make_ll_fit <- function(model = "spfa", n_draws = 400,
                        n_ipd = 12L, n_agd = 4L, seed = 1L) {
  set.seed(seed)
  ll_ipd <- matrix(rnorm(n_draws * n_ipd, -5, 0.5), nrow = n_draws)
  ll_agd <- matrix(rnorm(n_draws * n_agd, -6, 0.5), nrow = n_draws)
  colnames(ll_ipd) <- sprintf("log_lik_ipd[%d]", seq_len(n_ipd))
  colnames(ll_agd) <- sprintf("log_lik_agd[%d]", seq_len(n_agd))

  draws <- data.frame(
    ll_ipd, ll_agd,
    lor_index = rnorm(n_draws, 0.5, 0.2),
    lor_comparator = rnorm(n_draws, 0.4, 0.2),
    check.names = FALSE
  )

  out <- list(
    draws = draws,
    summary = data.frame(
      variable = c("mu_index", "lor_index"),
      mean = c(-0.5, 0.5), sd = c(0.1, 0.2),
      Rhat = c(1.001, 1.002), n_eff = c(800, 700),
      stringsAsFactors = FALSE
    ),
    model = model,
    diagnostics = list(n_divergent = 0, n_max_treedepth = 0),
    sampling_args = list(adapt_delta = 0.95, max_treedepth = 15, chains = 4)
  )
  class(out) <- c("mlumr_fit", "list")
  out
}


# extract_log_lik is internal but we can probe it via mlumr:::
test_that("extract_log_lik orders columns and returns a numeric matrix", {
  fit <- make_ll_fit(n_ipd = 5L, n_agd = 2L)
  ll <- mlumr:::extract_log_lik(fit)
  expect_true(is.matrix(ll))
  expect_equal(dim(ll), c(400L, 7L))
  # The last IPD column (index 5) should precede the first AgD column
  expect_equal(colnames(ll)[5], "log_lik_ipd[5]")
  expect_equal(colnames(ll)[6], "log_lik_agd[1]")
})


test_that("extract_log_lik rejects fits with no pointwise columns", {
  fit <- make_ll_fit()
  fit$draws <- fit$draws[, c("lor_index", "lor_comparator")]
  expect_error(mlumr:::extract_log_lik(fit),
               "Pointwise log-likelihood columns not found")
})


test_that("calculate_loo returns a psis_loo object", {
  skip_if_not_installed("loo")
  fit <- make_ll_fit()
  result <- suppressWarnings(calculate_loo(fit))
  expect_s3_class(result, "psis_loo")
  expect_s3_class(result, "loo")
  # The `loo` object must expose `elpd_loo`, p_loo, looic
  expect_true(all(c("elpd_loo", "p_loo", "looic") %in%
                    rownames(result$estimates)))
})


test_that("calculate_loo uses stable relative efficiency for tiny likelihoods", {
  skip_if_not_installed("loo")
  fit <- make_ll_fit()
  ll_cols <- grep("^log_lik_", names(fit$draws), value = TRUE)
  fit$draws[, ll_cols] <- fit$draws[, ll_cols] - 10000

  result <- suppressWarnings(calculate_loo(fit))

  expect_s3_class(result, "psis_loo")
})


test_that("calculate_waic returns a waic object", {
  skip_if_not_installed("loo")
  fit <- make_ll_fit()
  result <- suppressWarnings(calculate_waic(fit))
  expect_s3_class(result, "waic")
  expect_s3_class(result, "loo")
  expect_true(all(c("elpd_waic", "p_waic", "waic") %in%
                    rownames(result$estimates)))
})


test_that("calculate_loo errors informatively without pointwise log_lik", {
  fit <- make_ll_fit()
  fit$draws <- fit$draws[, c("lor_index", "lor_comparator")]
  expect_error(calculate_loo(fit),
               "Pointwise log-likelihood columns not found")
})


test_that("compare_models with criterion = 'loo' returns a compare.loo", {
  skip_if_not_installed("loo")
  fit1 <- make_ll_fit("spfa", seed = 1L)
  fit2 <- make_ll_fit("relaxed", seed = 2L)
  out <- suppressWarnings(
    capture.output(cmp <- compare_models(fit1, fit2, criterion = "loo"))
  )
  expect_s3_class(cmp, "compare.loo")
  # Two rows, one per model
  expect_equal(nrow(cmp), 2L)
  expect_true(any(grepl("Model Comparison \\(LOO\\)", out)))
})


test_that("compare_models with criterion = 'waic' returns a compare.loo", {
  skip_if_not_installed("loo")
  fit1 <- make_ll_fit("spfa", seed = 1L)
  fit2 <- make_ll_fit("relaxed", seed = 2L)
  out <- suppressWarnings(
    capture.output(cmp <- compare_models(fit1, fit2, criterion = "waic"))
  )
  expect_s3_class(cmp, "compare.loo")
  expect_equal(nrow(cmp), 2L)
})


test_that("compare_models rejects mixed mlumr_fit + mlumr_dic for LOO/WAIC", {
  skip_if_not_installed("loo")
  fit <- make_ll_fit("spfa")
  dic <- calculate_dic(fit)
  expect_error(
    compare_models(fit, dic, criterion = "loo"),
    "all arguments must be mlumr_fit"
  )
})


test_that(".chain_id recovers chain ids in chain-major order", {
  fit <- make_ll_fit(n_draws = 400)  # chains = 4 -> 100 per chain
  ids <- mlumr:::.chain_id(fit)
  expect_equal(length(ids), 400L)
  expect_equal(sort(unique(ids)), 1:4)
  # First and last 100 draws belong to different chains
  expect_equal(ids[1], 1L)
  expect_equal(ids[400], 4L)
})

# Null-coalescing operator (available in base R >= 4.4.0, but we support >= 4.1.0)
`%||%` <- function(x, y) if (is.null(x)) y else x


#' Specify a marginal distribution
#'
#' Used to specify marginal distributions for covariates when adding integration
#' points. Wraps an inverse CDF (quantile) function with its parameters.
#'
#' @param qfun Inverse CDF function (e.g., `qnorm`, `qbern`)
#' @param ... Parameters of the distribution, can reference column names in AgD
#'
#' @return A list with class `"mlumr_distr"` containing the distribution specification
#' @export
#'
#' @examples
#' # Normal distribution
#' distr(qnorm, mean = 0, sd = 1)
#'
#' # Bernoulli distribution with probability 0.3
#' distr(qbern, prob = 0.3)
distr <- function(qfun, ...) {
  qfun_resolved <- match.fun(qfun)
  qfun_name <- tryCatch(deparse(substitute(qfun)), error = function(e) "user_function")

  # Capture arguments as unevaluated expressions
  args <- as.list(match.call(expand.dots = FALSE))[["..."]]

  if (!"p" %in% names(formals(qfun_resolved))) {
    stop("`qfun` should be an inverse CDF function with a formal argument `p`",
         call. = FALSE)
  }

  d <- list(
    qfun = qfun_resolved,
    args = args,
    qfun_name = qfun_name
  )

  class(d) <- "mlumr_distr"
  d
}

#' Evaluate a mlumr_distr object with data context
#'
#' @param d A `mlumr_distr` object
#' @param p Vector of probabilities
#' @param data A named list or data frame to evaluate expressions in
#' @return Numeric vector of quantiles
#' @keywords internal
eval_distr <- function(d, p, data = list()) {
  # Build the call: d$qfun(p = p, arg1 = val1, arg2 = val2, ...)
  call_args <- list(p = p)
  for (nm in names(d$args)) {
    call_args[[nm]] <- tryCatch(
      eval(d$args[[nm]], envir = data, enclos = parent.frame(2)),
      error = function(e) {
        stop(sprintf(
          "Error evaluating distribution argument '%s': %s\nAvailable data columns: %s",
          nm, e$message, paste(names(data), collapse = ", ")
        ), call. = FALSE)
      }
    )
  }
  do.call(d$qfun, call_args)
}


#' Summarize a single draw vector into mean / sd / quantiles.
#'
#' Internal helper. Centralizes the (mean, sd, quantile) triplet used by
#' [predict.mlumr_fit()], [marginal_effects()], [conditional_effects()] and
#' [conditional_predict()] so that any later change to the canonical posterior
#' summary only needs to happen in one place.
#'
#' @param x Numeric vector of posterior draws.
#' @param probs Quantile probabilities.
#' @return Named numeric vector: `c(mean, sd, <named quantiles>)`.
#' @keywords internal
.summarize_draw_vector <- function(x, probs) {
  c(mean = mean(x, na.rm = TRUE),
    sd   = stats::sd(x, na.rm = TRUE),
    stats::quantile(x, probs = probs, na.rm = TRUE))
}

#' Summarize a draws matrix column-wise into a tidy data frame.
#'
#' Applies `.summarize_draw_vector()` across columns of `draws` and renames
#' the quantile columns to `qNN` form (e.g., `q2.5`, `q50`, `q97.5`).
#'
#' @param draws A numeric matrix or data frame of posterior draws (one column
#'   per quantity, one row per draw).
#' @param probs Quantile probabilities.
#' @return Data frame with columns `mean`, `sd`, and one `qNN` column per
#'   element of `probs`.
#' @keywords internal
.summarize_draw_matrix <- function(draws, probs) {
  summary_mat <- t(apply(draws, 2, .summarize_draw_vector, probs = probs))
  summary_df <- as.data.frame(summary_mat)
  colnames(summary_df) <- c("mean", "sd", paste0("q", probs * 100))
  summary_df
}


#' Validate an mlumr_data object
#' @keywords internal
.validate_mlumr_data_object <- function(data) {
  if (!inherits(data, "mlumr_data")) {
    stop("`data` must be created with combine_data().", call. = FALSE)
  }
  invisible(TRUE)
}


#' Validate a confidence level
#' @keywords internal
.validate_conf_level <- function(conf_level) {
  valid <- is.numeric(conf_level) &&
    length(conf_level) == 1L &&
    is.finite(conf_level) &&
    conf_level > 0 &&
    conf_level < 1
  if (!valid) {
    stop("`conf_level` must be a single finite number between 0 and 1.",
         call. = FALSE)
  }
  invisible(TRUE)
}


#' Convert a confidence level to a two-sided normal critical value
#' @keywords internal
.z_from_conf_level <- function(conf_level) {
  .validate_conf_level(conf_level)
  stats::qnorm(1 - (1 - conf_level) / 2)
}


#' Bound a Wald interval to a valid numerical range
#' @keywords internal
.bounded_wald_interval <- function(center, se, z,
                                   lower = -Inf, upper = Inf) {
  .validate_numeric_vector(center, "center")
  .validate_numeric_vector(se, "se")
  .validate_numeric_vector(z, "z")
  if (length(z) != 1L || !is.finite(z) || z < 0) {
    stop("`z` must be a single non-negative finite value.", call. = FALSE)
  }
  if (any(!is.finite(center)) || any(!is.finite(se)) || any(se < 0)) {
    stop("`center` and `se` must be finite, with non-negative `se`.",
         call. = FALSE)
  }
  if (!is.numeric(lower) || !is.numeric(upper) ||
      length(lower) != 1L || length(upper) != 1L ||
      is.na(lower) || is.na(upper) || lower > upper) {
    stop("`lower` and `upper` must define a valid interval.", call. = FALSE)
  }
  list(
    lower = pmax(lower, center - z * se),
    upper = pmin(upper, center + z * se)
  )
}


#' Truncate tiny negative variance estimates caused by numerical noise
#' @keywords internal
.nonnegative_variance <- function(x, name = "variance", tol = 1e-10) {
  .validate_numeric_vector(x, name)
  if (any(!is.finite(x))) {
    stop(sprintf("`%s` must be finite.", name), call. = FALSE)
  }
  if (any(x < -tol)) {
    stop(sprintf("`%s` must be non-negative.", name), call. = FALSE)
  }
  pmax(x, 0)
}


#' Square root of a variance estimate with numerical guarding
#' @keywords internal
.sqrt_variance <- function(x, name = "variance", tol = 1e-10) {
  sqrt(.nonnegative_variance(x, name, tol))
}


# -----------------------------------------------------------------------------
# Bernoulli wrappers (qbern / pbern / dbern)
#
# These three one-line wrappers around stats::qbinom/pbinom/dbinom with
# size = 1 are identical to the corresponding functions in the multinma
# package (Phillippo et al., GPL-3,
# <https://github.com/dmphillippo/multinma>, R/integration.R). They
# are reproduced here as part of mlumr's adaptation of the ML-NMR
# workflow so that mlumr users can pass `qbern` to `distr()` without
# having multinma attached. Both packages are GPL-3-licensed.
# -----------------------------------------------------------------------------

#' Bernoulli quantile function
#'
#' @param p Vector of probabilities
#' @param prob Success probability
#' @param lower.tail Logical; if TRUE, probabilities are P(X <= x)
#' @param log.p Logical; if TRUE, probabilities are given as log(p)
#'
#' @return Integer vector of 0s and 1s
#' @export
qbern <- function(p, prob, lower.tail = TRUE, log.p = FALSE) {
  qbinom(p, size = 1, prob = prob, lower.tail = lower.tail, log.p = log.p)
}

#' Bernoulli CDF
#'
#' @param q Vector of quantiles
#' @param prob Success probability
#' @param lower.tail Logical
#' @param log.p Logical
#'
#' @return Numeric vector
#' @export
pbern <- function(q, prob, lower.tail = TRUE, log.p = FALSE) {
  pbinom(q, size = 1, prob = prob, lower.tail = lower.tail, log.p = log.p)
}

#' Bernoulli PMF
#'
#' @param x Vector of values
#' @param prob Success probability
#' @param log Logical; if TRUE, return log-density
#'
#' @return Numeric vector
#' @export
dbern <- function(x, prob, log = FALSE) {
  dbinom(x, size = 1, prob = prob, log = log)
}

#' Get distribution type (continuous, discrete, or binary)
#' @param ... distr() objects
#' @param data Sample data for evaluation
#' @return Named character vector
#' @keywords internal
get_distribution_type <- function(..., data = list()) {
  ds <- list(...)
  dnames <- names(ds)

  out <- vector("character", length = length(ds))
  names(out) <- dnames

  known_continuous <- c("qbeta", "qcauchy", "qchisq", "qexp", "qf", "qgamma",
                        "qlnorm", "qnorm", "qt", "qunif", "qweibull")
  known_discrete <- c("qgeom", "qnbinom", "qpois")
  known_binary <- "qbern"

  for (i in seq_along(ds)) {
    di <- ds[[i]]
    if (di$qfun_name %in% known_continuous) {
      out[i] <- "continuous"
    } else if (di$qfun_name %in% known_discrete) {
      out[i] <- "discrete"
    } else if (di$qfun_name %in% known_binary) {
      out[i] <- "binary"
    } else if (di$qfun_name == "qbinom") {
      size_val <- eval_distr_arg(di$args$size, data)
      out[i] <- if (all(size_val == 1)) "binary" else "discrete"
    } else {
      # Test distribution on a grid
      ps <- 1:99 / 100
      support <- eval_distr(di, ps, data)
      is_int <- all(abs(support - round(support)) < .Machine$double.eps^0.5,
                    na.rm = TRUE)
      if (is_int) {
        out[i] <- if (all(abs(support) < 1.5, na.rm = TRUE)) "binary" else "discrete"
      } else {
        out[i] <- "continuous"
      }
    }
  }

  out
}

#' Evaluate a single mlumr_distr argument expression
#' @param expr An unevaluated expression
#' @param data Data context
#' @return Evaluated value
#' @keywords internal
eval_distr_arg <- function(expr, data) {
  eval(expr, envir = data, enclos = parent.frame(2))
}

#' Convert Spearman correlations to Gaussian copula correlations
#'
#' Applies theoretical relationships between Spearman's rho and the
#' Gaussian copula parameter (Kurowicka & Cooke, 2006; Lebrun & Dutfoy, 2009):
#'   - Continuous-continuous: rho_copula = 2 * sin(pi * rho_S / 6) (exact)
#'   - Binary-binary: rho_copula = sin(pi * rho_S / 2) (heuristic; the exact
#'     relationship depends on marginal prevalences, not accounted for here)
#'   - Continuous-binary: rho_copula = sqrt(2) * sin(pi * rho_S / (2*sqrt(3)))
#'     (heuristic)
#'
#' @param X Correlation matrix (Spearman)
#' @param types Character vector of distribution types
#' @return Adjusted correlation matrix for Gaussian copula
#' @keywords internal
cor_adjust_spearman <- function(X, types) {
  if (length(types) != nrow(X)) {
    stop("`types` length must match correlation matrix dimensions", call. = FALSE)
  }
  bin <- types == "binary"
  cont <- !bin

  X[cont, cont] <- 2 * sin(pi * X[cont, cont] / 6)
  X[bin, bin] <- sin(pi * X[bin, bin] / 2)
  X[cont, bin] <- sqrt(2) * sin(pi * X[cont, bin] / (2 * sqrt(3)))
  X[bin, cont] <- sqrt(2) * sin(pi * X[bin, cont] / (2 * sqrt(3)))

  diag(X) <- 1
  X
}

#' Convert Pearson correlations to Gaussian copula correlations
#'
#' For continuous-continuous pairs, Pearson rho equals the Gaussian copula
#' parameter only under normality of both margins. For non-normal continuous
#' covariates, this no-adjustment assumption introduces approximation error.
#' For binary and mixed pairs:
#'   - Binary-binary: rho_copula = sin(pi * rho_P / 2)
#'   - Continuous-binary: rho_copula = sqrt(pi/2) * rho_P
#'
#' @param X Correlation matrix (Pearson)
#' @param types Character vector of distribution types
#' @return Adjusted correlation matrix for Gaussian copula
#' @keywords internal
cor_adjust_pearson <- function(X, types) {
  if (length(types) != nrow(X)) {
    stop("`types` length must match correlation matrix dimensions", call. = FALSE)
  }
  bin <- types == "binary"
  cont <- !bin

  X[bin, bin] <- sin(pi * X[bin, bin] / 2)
  X[cont, bin] <- sqrt(pi / 2) * X[cont, bin]
  X[bin, cont] <- sqrt(pi / 2) * X[bin, cont]

  diag(X) <- 1
  X
}

test_that("qbern returns correct values", {
  expect_equal(qbern(0.3, prob = 0.5), qbinom(0.3, size = 1, prob = 0.5))
  expect_equal(qbern(0, prob = 0.5), 0L)
  expect_equal(qbern(1, prob = 0.5), 1L)
  # P(X <= 0) = 1 - prob = 0.8, so qbern(0.7, 0.2) = 0 (0.7 < 0.8)
  expect_equal(qbern(0.7, prob = 0.2), 0L)
  # qbern(0.9, 0.2) = 1 (0.9 > 0.8)
  expect_equal(qbern(0.9, prob = 0.2), 1L)
})

test_that("pbern returns correct values", {
  expect_equal(pbern(0, prob = 0.3), 1 - 0.3)
  expect_equal(pbern(1, prob = 0.3), 1)
})

test_that("dbern returns correct values", {
  expect_equal(dbern(1, prob = 0.4), 0.4)
  expect_equal(dbern(0, prob = 0.4), 0.6)
  expect_equal(dbern(1, prob = 0.4, log = TRUE), log(0.4))
})

test_that("distr creates valid distribution specification", {
  d <- distr(qnorm, mean = 0, sd = 1)
  expect_s3_class(d, "mlumr_distr")
  expect_true(is.function(d$qfun))
  expect_equal(d$qfun_name, "qnorm")
  expect_true("mean" %in% names(d$args))
  expect_true("sd" %in% names(d$args))
})

test_that("distr rejects non-quantile functions", {
  expect_error(distr(function(x) x), "inverse CDF")
})

test_that("eval_distr evaluates correctly with data context", {
  d <- distr(qbern, prob = my_prob)
  result <- eval_distr(d, p = c(0.3, 0.7), data = list(my_prob = 0.5))
  expect_equal(result, qbern(c(0.3, 0.7), prob = 0.5))
})

test_that("eval_distr gives informative error on missing column", {
  d <- distr(qbern, prob = nonexistent_col)
  expect_error(
    eval_distr(d, p = 0.5, data = list(x_mean = 0.3)),
    "Error evaluating distribution argument"
  )
  expect_error(
    eval_distr(d, p = 0.5, data = list(x_mean = 0.3)),
    "Available data columns"
  )
})

test_that("get_distribution_type classifies correctly", {
  d_norm <- distr(qnorm, mean = 0, sd = 1)
  d_bern <- distr(qbern, prob = 0.5)

  types <- get_distribution_type(x1 = d_norm, x2 = d_bern)
  expect_equal(types[["x1"]], "continuous")
  expect_equal(types[["x2"]], "binary")
})

test_that("cor_adjust_spearman returns valid correlation matrix", {
  rho <- matrix(c(1, 0.3, 0.3, 1), nrow = 2)
  types <- c("continuous", "binary")
  result <- cor_adjust_spearman(rho, types)

  expect_equal(diag(result), c(1, 1))
  expect_true(isSymmetric(result))
  # Adjusted values should differ from input
  expect_false(result[1, 2] == 0.3)
})

test_that("cor_adjust_pearson returns valid correlation matrix", {
  rho <- matrix(c(1, 0.3, 0.3, 1), nrow = 2)
  types <- c("continuous", "binary")
  result <- cor_adjust_pearson(rho, types)

  expect_equal(diag(result), c(1, 1))
  expect_true(isSymmetric(result))
})

#' Add numerical integration points
#'
#' Generate quasi-Monte Carlo integration points using Sobol sequences and a
#' Gaussian copula to account for correlations between covariates in the AgD.
#'
#' @param data An `mlumr_data` object from [combine_data()]
#' @param n_int Number of integration points (default 64, powers of 2 recommended)
#' @param cor Correlation matrix for covariates. If `NULL`, computed from IPD.
#' @param cor_adjust Adjustment method: `"spearman"`, `"pearson"`, or `"none"`
#' @param verbose Logical; if `FALSE`, suppresses progress messages.
#' @param ... Distribution specifications for each covariate using [distr()]
#'
#' @return An `mlumr_data` object with integration points added
#' @export
#'
#' @examples
#' \dontrun{
#' dat <- add_integration(
#'   dat,
#'   n_int = 64,
#'   x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
#'   x2 = distr(qbern, prob = x2_mean)
#' )
#' }
add_integration <- function(data, n_int = 64, cor = NULL,
                            cor_adjust = NULL, verbose = TRUE, ...) {
  ds <- list(...)
  .validate_integration_args(data, n_int, cor_adjust, verbose, ds)
  .validate_integration_distributions(ds, data)

  # Stan receives integration arrays without dimnames, so the third array
  # dimension must use the same covariate order as the IPD design matrix.
  ds <- ds[data$covariates]
  cov_names <- data$covariates
  n_cov <- length(ds)
  n_int <- as.integer(n_int)

  .warn_integration_size(n_int, n_cov)

  cor_info <- .resolve_integration_cor(
    data = data,
    cov_names = cov_names,
    n_cov = n_cov,
    cor = cor,
    cor_adjust = cor_adjust,
    ds = ds,
    verbose = verbose
  )

  u_cor <- .generate_copula_uniforms(n_int, n_cov, cor_info$copula_cor)
  integration_points <- .transform_integration_points(
    u_cor = u_cor,
    data = data,
    ds = ds,
    cov_names = cov_names,
    n_int = n_int,
    n_cov = n_cov
  )

  data <- .attach_integration_points(
    data = data,
    X_int_array = integration_points,
    n_int = n_int,
    cor = cor_info$cor,
    copula_cor = cor_info$copula_cor,
    cor_adjust = cor_info$cor_adjust
  )

  mlumr_message(sprintf("Added %d integration points for AgD", n_int),
                verbose = verbose)
  data
}


#' Validate top-level integration arguments
#' @keywords internal
.validate_integration_args <- function(data, n_int, cor_adjust, verbose, ds) {
  if (!inherits(data, "mlumr_data")) {
    stop("`data` must be created with combine_data()", call. = FALSE)
  }
  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("`verbose` must be TRUE or FALSE.", call. = FALSE)
  }
  valid_cor_adjust <- c("spearman", "pearson", "none")
  invalid_cor_adjust <- !is.null(cor_adjust) &&
    (!is.character(cor_adjust) ||
       length(cor_adjust) != 1L ||
       is.na(cor_adjust) ||
       !cor_adjust %in% valid_cor_adjust)
  if (invalid_cor_adjust) {
    stop(sprintf("`cor_adjust` must be one of: %s",
                 paste(valid_cor_adjust, collapse = ", ")), call. = FALSE)
  }
  invalid_n_int <- !is.numeric(n_int) || length(n_int) != 1 ||
    !is.finite(n_int) || n_int <= 0 || n_int != floor(n_int)
  if (invalid_n_int) {
    stop("`n_int` should be a positive integer", call. = FALSE)
  }
  if (length(ds) == 0) {
    stop("No covariate distributions specified. Use distr() to specify distributions.",
         call. = FALSE)
  }
  invalid_ds <- any(vapply(ds, function(x) !inherits(x, "mlumr_distr"), logical(1))) ||
    is.null(names(ds)) || any(!nzchar(names(ds)))
  if (invalid_ds) {
    stop("Covariate distributions should be specified as named arguments using distr()",
         call. = FALSE)
  }
  if (anyDuplicated(names(ds)) > 0L) {
    duplicates <- unique(names(ds)[duplicated(names(ds))])
    stop(sprintf("Duplicate distribution specifications for: %s",
                 paste(duplicates, collapse = ", ")), call. = FALSE)
  }
  invisible(TRUE)
}


#' Validate integration distributions against the combined data
#' @keywords internal
.validate_integration_distributions <- function(ds, data) {
  cov_names <- names(ds)
  if (anyDuplicated(cov_names) > 0L) {
    duplicates <- unique(cov_names[duplicated(cov_names)])
    stop(sprintf("Duplicate distribution specifications for: %s",
                 paste(duplicates, collapse = ", ")), call. = FALSE)
  }

  if (!all(cov_names %in% data$covariates)) {
    missing <- setdiff(cov_names, data$covariates)
    stop(sprintf("Unknown covariates: %s", paste(missing, collapse = ", ")),
         call. = FALSE)
  }

  if (length(cov_names) != length(data$covariates)) {
    missing <- setdiff(data$covariates, cov_names)
    stop(sprintf("Missing distribution specifications for: %s",
                 paste(missing, collapse = ", ")), call. = FALSE)
  }
  invisible(TRUE)
}


#' Warn when integration resolution is low for the covariate dimension
#' @keywords internal
.warn_integration_size <- function(n_int, n_cov) {
  min_recommended <- 2^(n_cov + 4)
  if (n_int < min_recommended) {
    warning(sprintf(
      paste0("n_int = %d may be insufficient for %d covariate(s). ",
             "Recommended minimum: %d (= 2^(n_cov+4)). ",
             "Consider increasing n_int or using check_integration() to assess accuracy."),
      n_int, n_cov, min_recommended
    ), call. = FALSE)
  }
  invisible(TRUE)
}


#' Resolve raw and copula correlation matrices for integration
#' @keywords internal
.resolve_integration_cor <- function(data, cov_names, n_cov, cor, cor_adjust,
                                     ds, verbose) {
  if (n_cov == 1L) {
    list(cor = matrix(1), copula_cor = matrix(1), cor_adjust = "none")
  } else {
    if (is.null(cor)) {
      mlumr_message("Computing correlation matrix from IPD...", verbose = verbose)
      if (is.null(cor_adjust)) cor_adjust <- "spearman"
      cor_method <- if (cor_adjust == "none") "pearson" else cor_adjust
      cor <- suppressWarnings(stats::cor(
        data$ipd$data[, cov_names],
        method = cor_method,
        use = "complete.obs"
      ))
      .validate_computed_integration_cor(cor)
    } else {
      cor <- .validate_integration_cor(cor, n_cov, cov_names = cov_names)
      if (is.null(cor_adjust)) cor_adjust <- "pearson"
    }

    dtypes <- do.call(
      get_distribution_type,
      c(ds, list(data = utils::head(data$agd$data)))
    )
    copula_cor <- .adjust_integration_cor(cor, cor_adjust, dtypes)
    copula_cor <- .ensure_positive_definite_cor(copula_cor, n_cov)

    list(cor = cor, copula_cor = copula_cor, cor_adjust = cor_adjust)
  }
}


#' Validate a user-supplied integration correlation matrix
#' @keywords internal
.validate_integration_cor <- function(cor, n_cov, cov_names = NULL) {
  cor <- as.matrix(cor)
  if (!is.numeric(cor) || nrow(cor) != n_cov || ncol(cor) != n_cov) {
    stop("Correlation matrix dimensions don't match number of covariates",
         call. = FALSE)
  }
  if (!is.null(cov_names)) {
    row_names <- rownames(cor)
    col_names <- colnames(cor)
    has_dimnames <- !is.null(row_names) || !is.null(col_names)
    if (has_dimnames) {
      dimnames_match <- !is.null(row_names) &&
        !is.null(col_names) &&
        setequal(row_names, cov_names) &&
        setequal(col_names, cov_names)
      if (!dimnames_match) {
        stop("`cor` dimnames must match covariate names when provided",
             call. = FALSE)
      }
      cor <- cor[cov_names, cov_names, drop = FALSE]
    }
  }
  valid <- all(is.finite(cor)) &&
    isSymmetric(cor) &&
    max(abs(diag(cor) - 1)) <= sqrt(.Machine$double.eps) &&
    all(eigen(cor, symmetric = TRUE)$values > 0)

  if (!valid) {
    stop("`cor` must be a valid correlation matrix", call. = FALSE)
  }
  cor
}


#' Validate an IPD-derived correlation matrix before copula adjustment
#' @keywords internal
.validate_computed_integration_cor <- function(cor) {
  if (any(!is.finite(cor))) {
    stop("Computed IPD correlation matrix contains non-finite values. ",
         "Check for constant or missing covariates, or provide `cor` manually.",
         call. = FALSE)
  }
  if (!isSymmetric(cor) ||
        max(abs(diag(cor) - 1)) > sqrt(.Machine$double.eps)) {
    stop("Computed IPD correlation matrix is invalid. Provide `cor` manually.",
         call. = FALSE)
  }
  invisible(TRUE)
}


#' Adjust correlations to Gaussian-copula scale
#' @keywords internal
.adjust_integration_cor <- function(cor, cor_adjust, dtypes) {
  if (cor_adjust == "spearman") {
    cor_adjust_spearman(cor, types = dtypes)
  } else if (cor_adjust == "pearson") {
    cor_adjust_pearson(cor, types = dtypes)
  } else {
    cor
  }
}


#' Ensure adjusted integration correlation is positive definite
#' @keywords internal
.ensure_positive_definite_cor <- function(copula_cor, n_cov) {
  eigen_tol <- .Machine$double.eps * max(dim(copula_cor)) * 100
  if (all(eigen(copula_cor, symmetric = TRUE)$values > eigen_tol)) {
    copula_cor
  } else {
    warning("Adjusted correlation matrix not positive definite; applying nearPD correction.",
            call. = FALSE)
    if (requireNamespace("Matrix", quietly = TRUE)) {
      as.matrix(Matrix::nearPD(copula_cor, corr = TRUE)$mat)
    } else {
      copula_cor <- 0.99 * copula_cor + 0.01 * diag(n_cov)
      diag(copula_cor) <- 1
      copula_cor
    }
  }
}


#' Generate correlated uniform quasi-Monte Carlo points
#' @keywords internal
.generate_copula_uniforms <- function(n_int, n_cov, copula_cor) {
  u <- randtoolbox::sobol(n = n_int, dim = n_cov)
  if (n_cov == 1L) {
    as.matrix(u)
  } else {
    u_cor <- tryCatch({
      cop <- copula::normalCopula(copula::P2p(copula_cor), dim = n_cov,
                                  dispstr = "un")
      copula::cCopula(u, copula = cop, inverse = TRUE)
    }, error = function(e) {
      stop(paste0(
        "Gaussian copula construction failed: ", e$message, "\n",
        "Try using `cor_adjust = \"none\"` or check covariate distributions."
      ), call. = FALSE)
    })

    as.matrix(u_cor)
  }
}


#' Transform uniform integration points to covariate scales
#' @keywords internal
.transform_integration_points <- function(u_cor, data, ds, cov_names, n_int,
                                          n_cov) {
  agd_data <- data$agd$data
  n_agd_rows <- nrow(agd_data)

  X_int_array <- array(NA_real_, dim = c(n_agd_rows, n_int, n_cov))
  dimnames(X_int_array)[[3]] <- cov_names

  for (j in 1:n_agd_rows) {
    row_data <- as.list(agd_data[j, , drop = FALSE])
    for (i in seq_along(cov_names)) {
      cov <- cov_names[i]
      u_i <- as.vector(u_cor[, i])
      X_int_array[j, , i] <- eval_distr(ds[[cov]], u_i, row_data)
    }
  }

  if (any(is.na(X_int_array) | is.infinite(X_int_array) | is.nan(X_int_array))) {
    stop("Invalid integration points generated. Check covariate distribution parameters.",
         call. = FALSE)
  }
  X_int_array
}


#' Attach generated integration points to an mlumr_data object
#' @keywords internal
.attach_integration_points <- function(data, X_int_array, n_int, cor, copula_cor,
                                       cor_adjust) {
  data$integration_points <- X_int_array
  data$n_int <- n_int
  data$int_cor <- cor
  data$int_copula_cor <- copula_cor
  data$int_cor_adjust <- cor_adjust
  data$has_integration <- TRUE
  data
}


#' Expand integration points into a long-format data frame
#'
#' @param data An `mlumr_data` object with integration points
#'
#' @return A data frame with columns for each covariate plus `.int_id` and `.agd_row`
#' @export
unnest_integration <- function(data) {

  if (!inherits(data, "mlumr_data")) {
    stop("`data` must be an mlumr_data object", call. = FALSE)
  }
  if (!data$has_integration) {
    stop("No integration points found. Use add_integration() first.", call. = FALSE)
  }

  X_int <- data$integration_points
  dims <- dim(X_int)
  n_agd <- dims[1]
  n_int <- dims[2]
  cov_names <- dimnames(X_int)[[3]]

  out_list <- vector("list", n_agd)
  for (i in 1:n_agd) {
    mat <- X_int[i, , , drop = FALSE]
    dim(mat) <- dims[2:3]
    colnames(mat) <- cov_names
    df <- as.data.frame(mat)
    df$.int_id <- seq_len(n_int)
    df$.agd_row <- i
    out_list[[i]] <- df
  }

  do.call(rbind, out_list)
}


#' Check integration point adequacy
#'
#' Compare integration results at the current `n_int` against a doubled
#' resolution to assess numerical accuracy. Large discrepancies indicate
#' that `n_int` should be increased.
#'
#' @param data An `mlumr_data` object with integration points
#' @param ... Distribution specifications (same as passed to
#'   [add_integration()])
#' @param cor Correlation matrix (same as passed to [add_integration()])
#' @param cor_adjust Adjustment method (same as passed to [add_integration()])
#' @param check_joint If `TRUE` (default), also compare pairwise correlation
#'   matrices between the current and doubled `n_int`, and the maximum
#'   per-AgD-row absolute deviation from the user-supplied `cor`. The
#'   pairwise comparison catches cases where marginals converge but joint
#'   dependence structure does not (rare in practice for QMC with sensible
#'   `cor_adjust` but worth flagging when `n_int` is small).
#'
#' @return A list with components `marginals` (the original data frame
#'   returned by previous versions) and, if `check_joint = TRUE`,
#'   `correlations` -- a data frame of pairwise covariate correlations at
#'   the current and doubled `n_int` for each AgD row. Printed with a
#'   pass/warn verdict.
#' @param verbose Logical; if `FALSE`, suppresses printed diagnostic messages.
#' @export
check_integration <- function(data, ..., cor = NULL, cor_adjust = NULL,
                              check_joint = TRUE, verbose = TRUE) {

  if (!inherits(data, "mlumr_data")) {
    stop("`data` must be an mlumr_data object", call. = FALSE)
  }
  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("`verbose` must be TRUE or FALSE.", call. = FALSE)
  }
  if (!is.logical(check_joint) || length(check_joint) != 1L ||
        is.na(check_joint)) {
    stop("`check_joint` must be TRUE or FALSE.", call. = FALSE)
  }
  if (!data$has_integration) {
    stop("No integration points found. Use add_integration() first.", call. = FALSE)
  }

  n_int_orig <- data$n_int
  n_int_double <- n_int_orig * 2

  # Compute stats at current resolution
  X_orig <- data$integration_points
  cov_names <- dimnames(X_orig)[[3]]
  n_agd <- dim(X_orig)[1]

  stats_orig <- .int_stats(X_orig, cov_names, n_agd)

  # Re-run at doubled resolution (temporarily)
  data_copy <- data
  data_copy$has_integration <- FALSE
  data_doubled <- suppressMessages(suppressWarnings(
    add_integration(data_copy, n_int = n_int_double, cor = cor,
                    cor_adjust = cor_adjust %||% data$int_cor_adjust,
                    verbose = FALSE, ...)
  ))
  X_double <- data_doubled$integration_points
  stats_double <- .int_stats(X_double, cov_names, n_agd)

  # Compare
  rel_diff_mean <- abs(stats_orig$mean - stats_double$mean) /
    (abs(stats_double$mean) + 1e-10)
  rel_diff_sd <- abs(stats_orig$sd - stats_double$sd) /
    (abs(stats_double$sd) + 1e-10)

  result <- data.frame(
    covariate = stats_orig$covariate,
    agd_row = stats_orig$agd_row,
    mean_current = round(stats_orig$mean, 6),
    mean_doubled = round(stats_double$mean, 6),
    rel_diff_mean = round(rel_diff_mean, 6),
    sd_current = round(stats_orig$sd, 6),
    sd_doubled = round(stats_double$sd, 6),
    rel_diff_sd = round(rel_diff_sd, 6),
    stringsAsFactors = FALSE
  )

  max_diff <- max(c(rel_diff_mean, rel_diff_sd), na.rm = TRUE)

  if (verbose) {
    cat(sprintf("Integration check: n_int = %d vs %d\n", n_int_orig, n_int_double))
    cat(sprintf("Marginals -- max relative difference: %.4f\n", max_diff))
    if (max_diff > 0.05) {
      cat("WARN: >5%% marginal relative difference. Increase n_int.\n")
    } else if (max_diff > 0.01) {
      cat("CAUTION: 1-5%% marginal relative difference. Consider increasing n_int.\n")
    } else {
      cat("OK marginals: <1%% relative difference.\n")
    }
  }

  out <- list(marginals = result)

  if (isTRUE(check_joint) && length(cov_names) >= 2L) {
    cor_result <- .int_cor_stats(X_orig, X_double, cov_names, n_agd,
                                 cor_target = cor)
    max_cor_diff <- max(cor_result$diff$abs_diff, na.rm = TRUE)
    if (verbose) {
      cat(sprintf("Joint -- max |cor(current) - cor(doubled)|: %.4f\n",
                  max_cor_diff))
      if (max_cor_diff > 0.05) {
        cat("WARN: pairwise correlations differ by > 0.05 between resolutions.\n")
      } else {
        cat("OK joint: pairwise correlations agree within 0.05.\n")
      }
      if (!is.null(cor)) {
        cat(sprintf("Target -- max |cor(doubled) - cor_target|: %.4f\n",
                    max(cor_result$diff$abs_diff_target, na.rm = TRUE)))
      }
    }
    out$correlations <- cor_result$diff
  }

  invisible(out)
}


#' Pairwise-correlation diagnostics for integration points
#' @keywords internal
.int_cor_stats <- function(X_orig, X_double, cov_names, n_agd, cor_target = NULL) {
  K <- length(cov_names)
  pairs <- utils::combn(seq_len(K), 2, simplify = FALSE)
  rows <- vector("list", length(pairs) * n_agd)
  idx <- 1L
  for (k in seq_len(n_agd)) {
    Xo <- X_orig[k, , , drop = FALSE]
    Xd <- X_double[k, , , drop = FALSE]
    dim(Xo) <- dim(X_orig)[2:3]
    dim(Xd) <- dim(X_double)[2:3]
    colnames(Xo) <- cov_names
    colnames(Xd) <- cov_names
    cor_o <- stats::cor(Xo)
    cor_d <- stats::cor(Xd)
    for (ij in pairs) {
      i <- ij[[1L]]
      j <- ij[[2L]]
      rho_o <- cor_o[i, j]
      rho_d <- cor_d[i, j]
      rho_t <- if (!is.null(cor_target)) cor_target[i, j] else NA_real_
      rows[[idx]] <- data.frame(
        agd_row = k,
        pair = sprintf("%s~%s", cov_names[i], cov_names[j]),
        cor_current = round(rho_o, 4),
        cor_doubled = round(rho_d, 4),
        cor_target = round(rho_t, 4),
        abs_diff = round(abs(rho_o - rho_d), 4),
        abs_diff_target = round(abs(rho_d - rho_t), 4),
        stringsAsFactors = FALSE
      )
      idx <- idx + 1L
    }
  }
  list(diff = do.call(rbind, rows))
}


#' Compute summary statistics for integration points
#' @keywords internal
.int_stats <- function(X_int, cov_names, n_agd) {
  rows <- vector("list", n_agd * length(cov_names))
  idx <- 1
  for (k in seq_len(n_agd)) {
    for (j in seq_along(cov_names)) {
      vals <- X_int[k, , j]
      rows[[idx]] <- data.frame(
        covariate = cov_names[j], agd_row = k,
        mean = mean(vals), sd = sd(vals),
        stringsAsFactors = FALSE
      )
      idx <- idx + 1
    }
  }
  do.call(rbind, rows)
}

#' Family metadata registry
#'
#' Internal single-source-of-truth for family-specific Stan model names,
#' AgD weighting, prediction-variable prefixes, and supported links and
#' effect measures. Every R-side call site that hard-coded a per-family
#' branch now looks up the relevant field here. This file is deliberately
#' pure-R and makes no Stan calls; keeping it a data registry makes
#' rstantools regeneration of `R/stanmodels.R` independent.
#'
#' Fields:
#' \describe{
#'   \item{`stan_prefix`}{Prefix for the Stan model name (the full name
#'     is `<stan_prefix>_{spfa,relaxed}`).}
#'   \item{`predict_prefix`}{Column prefix for generated-quantity variables
#'     in [predict.mlumr_fit()] (e.g. `"p"`, `"y"`, `"rate"`).}
#'   \item{`link_default`}{The default link when the user passes
#'     `link = NULL`.}
#'   \item{`links`}{Vector of supported links. Should match the branches
#'     in [check_link()].}
#'   \item{`effect_measures`}{Supported values of the `effect` argument in
#'     [marginal_effects()] (excluding `"all"`).}
#'   \item{`marginal_effect_vars`}{Generated-quantity column names for each
#'     effect measure, per population. Expanded in [marginal_effects()].}
#'   \item{`comp_weight_field`}{Name of the Stan-data field used to
#'     weight the comparator-population marginal predictions
#'     (`n_agd` for binomial, `E_agd` for poisson, `NULL` for normal
#'     = equal weights).}
#' }
#'
#' @keywords internal
#' @name family_config
NULL

#' @keywords internal
family_config <- list(
  binomial = list(
    stan_prefix          = "mlumr_binary",
    predict_prefix       = "p",
    link_default         = "logit",
    links                = c("logit", "probit", "cloglog"),
    effect_measures      = c("lor", "rd", "rr"),
    marginal_effect_vars = list(
      lor = c("lor_index", "lor_comparator"),
      rd  = c("rd_index",  "rd_comparator"),
      rr  = c("rr_index",  "rr_comparator")
    ),
    comp_weight_field    = "n_agd"
  ),
  normal = list(
    stan_prefix          = "mlumr_normal",
    predict_prefix       = "y",
    link_default         = "identity",
    links                = c("identity", "log"),
    effect_measures      = c("md"),
    marginal_effect_vars = list(
      md = c("delta_index", "delta_comparator")
    ),
    comp_weight_field    = NULL
  ),
  poisson = list(
    stan_prefix          = "mlumr_poisson",
    predict_prefix       = "rate",
    link_default         = "log",
    links                = c("log"),
    effect_measures      = c("rr"),
    marginal_effect_vars = list(
      rr = c("delta_index", "delta_comparator")
    ),
    comp_weight_field    = "E_agd"
  )
)

#' Lookup helper for the family registry
#'
#' Fails fast with an informative message when a caller requests an
#' unregistered family, rather than returning `NULL` and letting a
#' downstream `$` chain produce a cryptic error.
#'
#' @param family A single family string.
#' @return The corresponding entry from [family_config].
#' @keywords internal
get_family_config <- function(family) {
  if (!is.character(family) || length(family) != 1L) {
    stop("`family` must be a single string", call. = FALSE)
  }
  cfg <- family_config[[family]]
  if (is.null(cfg)) {
    stop(sprintf("Unknown family '%s'. Supported: %s.",
                 family, paste(names(family_config), collapse = ", ")),
         call. = FALSE)
  }
  cfg
}

#' @method print mlumr_fit
#' @export
print.mlumr_fit <- function(x, ...) {
  cat("ML-UMR Fit\n")
  cat("==========\n\n")

  family <- x$family %||% "binomial"
  family_label <- switch(family,
    binomial = "Binary",
    normal   = "Continuous (Normal)",
    poisson  = "Count (Poisson)"
  )

  model_label <- if (x$model == "spfa") {
    "SPFA (Shared Prognostic Factors)"
  } else {
    "Relaxed SPFA (Treatment-specific effects)"
  }

  link_label <- x$link %||% switch(family, binomial = "logit", normal = "identity", poisson = "log")

  cat("Model:", model_label, "\n")
  cat("Family:", family_label, "\n")
  cat("Link:", link_label, "\n")
  if (!is.null(x$engine)) cat("Engine:", x$engine, "\n")
  cat("Treatments:\n")
  cat("  Index (IPD):", x$data$index_treatment, "\n")
  cat("  Comparator (AgD):", x$data$comparator_treatment, "\n\n")

  cat("Data:\n")
  cat("  IPD: n =", x$stan_data$n_ipd, "observations\n")
  cat("  AgD:", x$stan_data$n_agd_rows, "rows\n")
  cat("  Covariates:", x$stan_data$n_cov, "\n")
  cat("  Integration points:", x$stan_data$n_int, "\n\n")

  cat("Sampling:\n")
  cat("  Chains:", x$sampling_args$chains, "\n")
  cat("  Iterations:", x$sampling_args$iter,
      "(warmup:", x$sampling_args$warmup, ")\n\n")

  # Key parameters depend on family
  if (family == "binomial") {
    params <- c("mu_index", "mu_comparator",
                "lor_index", "lor_comparator",
                "rd_index", "rd_comparator")
  } else if (family == "normal") {
    params <- c("mu_index", "mu_comparator", "sigma",
                "delta_index", "delta_comparator")
  } else {
    params <- c("mu_index", "mu_comparator",
                "delta_index", "delta_comparator")
  }

  idx <- x$summary$variable %in% params
  if (any(idx)) {
    cat("Key Parameters:\n")
    sub_df <- x$summary[idx, c("variable", "mean", "sd", "2.5%", "97.5%", "Rhat")]
    print(sub_df, row.names = FALSE)
  }

  cat("\nUse summary() for full results\n")
  invisible(x)
}


#' @method summary mlumr_fit
#' @export
summary.mlumr_fit <- function(object, ...) {

  cat("ML-UMR Model Summary\n")
  cat("====================\n\n")

  family <- object$family %||% "binomial"
  model_label <- if (object$model == "spfa") "SPFA" else "Relaxed SPFA"
  family_label <- switch(family,
    binomial = "Binary",
    normal   = "Continuous (Normal)",
    poisson  = "Count (Poisson)"
  )

  link_label <- object$link %||% switch(family, binomial = "logit", normal = "identity", poisson = "log")

  cat("Model:", model_label, "\n")
  cat("Family:", family_label, "\n")
  cat("Link:", link_label, "\n")
  if (!is.null(object$engine)) cat("Engine:", object$engine, "\n")
  cat("Treatments:", object$data$index_treatment, "(IPD) vs",
      object$data$comparator_treatment, "(AgD)\n\n")

  # Diagnostics
  cat("MCMC Diagnostics:\n")
  cat("  Divergent transitions:", object$diagnostics$n_divergent, "\n")
  cat("  Max treedepth hits:", object$diagnostics$n_max_treedepth, "\n")
  if (!is.null(object$summary$Rhat)) {
    cat("  Max Rhat:", round(max(object$summary$Rhat, na.rm = TRUE), 3), "\n")
  }
  if (!is.null(object$summary$n_eff)) {
    cat("  Min ESS:", round(min(object$summary$n_eff, na.rm = TRUE), 0), "\n")
  }
  cat("\n")

  # Intercepts
  scale_label <- paste0(link_label, " scale")
  cat(sprintf("Intercepts (%s):\n", scale_label))
  mu_idx <- grep("^mu_", object$summary$variable)
  print(object$summary[mu_idx, c("variable", "mean", "sd", "2.5%", "97.5%", "Rhat")],
        row.names = FALSE)

  # Residual SD (normal only)
  if (family == "normal") {
    sigma_idx <- which(object$summary$variable == "sigma")
    if (length(sigma_idx) > 0) {
      cat("\nResidual SD:\n")
      print(object$summary[sigma_idx, c("variable", "mean", "sd", "2.5%", "97.5%", "Rhat")],
            row.names = FALSE)
    }
  }

  # Regression coefficients
  cat("\nRegression Coefficients:\n")
  if (object$model == "spfa") {
    beta_idx <- grep("^beta\\[", object$summary$variable)
  } else {
    beta_idx <- grep("^beta_(index|comparator)\\[", object$summary$variable)
  }
  if (length(beta_idx) > 0) {
    print(object$summary[beta_idx, c("variable", "mean", "sd", "2.5%", "97.5%", "Rhat")],
          row.names = FALSE)
  }

  # Marginal effects
  cat("\nMarginal Treatment Effects:\n")
  if (family == "binomial") {
    lor_idx <- grep("^lor_", object$summary$variable)
    if (length(lor_idx) > 0) {
      cat("  Log Odds Ratios:\n")
      print(object$summary[lor_idx, c("variable", "mean", "sd", "2.5%", "97.5%")],
            row.names = FALSE)
    }

    rd_idx <- grep("^rd_", object$summary$variable)
    if (length(rd_idx) > 0) {
      cat("  Risk Differences:\n")
      print(object$summary[rd_idx, c("variable", "mean", "sd", "2.5%", "97.5%")],
            row.names = FALSE)
    }

    rr_idx <- grep("^rr_", object$summary$variable)
    if (length(rr_idx) > 0) {
      cat("  Risk Ratios:\n")
      print(object$summary[rr_idx, c("variable", "mean", "sd", "2.5%", "97.5%")],
            row.names = FALSE)
    }
  } else if (family == "normal") {
    delta_idx <- grep("^delta_(index|comparator)$", object$summary$variable)
    if (length(delta_idx) > 0) {
      cat("  Mean Differences:\n")
      print(object$summary[delta_idx, c("variable", "mean", "sd", "2.5%", "97.5%")],
            row.names = FALSE)
    }
  } else {
    delta_idx <- grep("^delta_(index|comparator)$", object$summary$variable)
    if (length(delta_idx) > 0) {
      cat("  Rate Ratios:\n")
      print(object$summary[delta_idx, c("variable", "mean", "sd", "2.5%", "97.5%")],
            row.names = FALSE)
    }
  }

  cat("\n")
  invisible(object)
}


#' @method print mlumr_naive
#' @export
print.mlumr_naive <- function(x, ...) {
  family <- x$family %||% "binomial"
  cat("Naive Unadjusted Indirect Comparison\n")
  cat("=====================================\n\n")
  cat("Treatments:", x$data$index_treatment, "vs", x$data$comparator_treatment, "\n\n")

  if (family == "binomial") {
    cat("Event rates:\n")
    cat(sprintf("  Index (IPD):      %.3f (%d/%d)\n",
                x$p_index, round(x$p_index * x$n_index), x$n_index))
    cat(sprintf("  Comparator (AgD): %.3f (%d/%d)\n",
                x$p_comparator, round(x$p_comparator * x$n_comparator), x$n_comparator))
    link_label <- switch(x$link %||% "logit",
      logit   = "Log Odds Ratio",
      probit  = "Probit Difference",
      cloglog = "Cloglog Difference"
    )
    cat(sprintf("\n%s: %.4f (SE: %.4f)\n", link_label, x$estimate, x$se))
  } else if (family == "normal") {
    cat("Mean outcomes:\n")
    cat(sprintf("  Index (IPD):      %.4f\n", x$mean_index))
    cat(sprintf("  Comparator (AgD): %.4f\n", x$mean_comparator))
    cat(sprintf("\nMean Difference: %.4f (SE: %.4f)\n", x$estimate, x$se))
  } else {
    cat("Rates:\n")
    cat(sprintf("  Index (IPD):      %.4f\n", x$rate_index))
    cat(sprintf("  Comparator (AgD): %.4f\n", x$rate_comparator))
    cat(sprintf("\nLog Rate Ratio: %.4f (SE: %.4f)\n", x$estimate, x$se))
  }

  cat(sprintf("%.0f%% CI: [%.4f, %.4f]\n",
              x$conf_level * 100, x$ci_lower, x$ci_upper))
  invisible(x)
}


#' @method summary mlumr_naive
#' @export
summary.mlumr_naive <- function(object, ...) {
  print.mlumr_naive(object, ...)
}


#' @method print mlumr_stc
#' @export
print.mlumr_stc <- function(x, ...) {
  family <- x$family %||% "binomial"
  cat("Simulated Treatment Comparison (G-computation)\n")
  cat("===============================================\n\n")
  cat("Treatments:", x$data$index_treatment, "vs", x$data$comparator_treatment, "\n\n")

  if (family == "binomial") {
    cat(sprintf("Marginalized P(Y=1|index trt, comp pop): %.4f\n", x$p_hat_index))
    cat(sprintf("Observed P(Y=1|comp trt, comp pop):      %.4f\n", x$p_comparator))
    link_label <- switch(x$link %||% "logit",
      logit   = "Log Odds Ratio",
      probit  = "Probit Difference",
      cloglog = "Cloglog Difference"
    )
    cat(sprintf("\n%s: %.4f (SE: %.4f)\n", link_label, x$estimate, x$se))
  } else if (family == "normal") {
    cat(sprintf("Marginalized E[Y|index trt, comp pop]: %.4f\n", x$y_hat_index))
    cat(sprintf("Observed E[Y|comp trt, comp pop]:      %.4f\n", x$y_comparator))
    cat(sprintf("\nMean Difference: %.4f (SE: %.4f)\n", x$estimate, x$se))
  } else {
    cat(sprintf("Marginalized rate (index trt, comp pop): %.4f\n", x$rate_hat_index))
    cat(sprintf("Observed rate (comp trt, comp pop):      %.4f\n", x$rate_comparator))
    cat(sprintf("\nLog Rate Ratio: %.4f (SE: %.4f)\n", x$estimate, x$se))
  }

  cat(sprintf("%.0f%% CI: [%.4f, %.4f]\n",
              x$conf_level * 100, x$ci_lower, x$ci_upper))

  cat("\nOutcome model coefficients:\n")
  print(round(coef(x$glm_fit), 4))
  invisible(x)
}


#' @method summary mlumr_stc
#' @export
summary.mlumr_stc <- function(object, ...) {
  print.mlumr_stc(object, ...)
  cat("\nFull GLM summary:\n")
  print(summary(object$glm_fit))
  invisible(object)
}

#' Get or Set the Stan Engine
#'
#' Control which Stan backend mlumr uses for model fitting. The default engine
#' is `"rstan"` (compiled C++ models via rstantools). Users who prefer cmdstanr
#' can switch engines after installation.
#'
#' @param engine Character string: `"rstan"` or `"cmdstanr"`. If `NULL`
#'   (default), returns the current engine without changing it.
#'
#' @details
#' When switching to `"cmdstanr"`, this function checks whether the cmdstanr
#' package and CmdStan toolchain are installed. If either is missing, it offers
#' to install them interactively.
#'
#' Engine names must be matched exactly. Partial strings such as `"c"` are not
#' accepted.
#'
#' The engine preference is stored as `options(mlumr.stan_engine = ...)` and
#' persists for the current R session. To set a permanent default, add to your
#' `.Rprofile`:
#'
#' ```
#' options(mlumr.stan_engine = "cmdstanr")
#' ```
#'
#' @return The current engine (character), returned invisibly when setting.
#' @export
#'
#' @examples
#' # Check current engine
#' mlumr_engine()
#'
#' # Switch to cmdstanr (interactive)
#' \dontrun{
#' mlumr_engine("cmdstanr")
#' }
mlumr_engine <- function(engine = NULL) {
  if (is.null(engine)) {
    return(get_engine())
  }

  engine <- .validate_engine_name(engine)

  if (engine == "rstan") {
    options(mlumr.stan_engine = "rstan")
    message("mlumr engine set to: rstan")
    return(invisible(engine))
  }

  if (!requireNamespace("cmdstanr", quietly = TRUE)) {
    message("cmdstanr is not installed.")
    if (interactive()) {
      choice <- utils::menu(
        c("Yes", "No"),
        title = "Install cmdstanr from r-universe?"
      )
      if (choice == 1L) {
        install_ok <- tryCatch(
          {
            utils::install.packages(
              "cmdstanr",
              repos = c("https://stan-dev.r-universe.dev", getOption("repos"))
            )
            TRUE
          },
          error = function(e) {
            message("cmdstanr installation failed: ", conditionMessage(e))
            FALSE
          }
        )
        if (!install_ok) {
          return(.message_engine_unchanged())
        }
      } else {
        return(.message_engine_unchanged())
      }
    } else {
      message(
        "Install cmdstanr with:\n",
        '  install.packages("cmdstanr", repos = c("https://stan-dev.r-universe.dev", getOption("repos")))'
      )
      return(.message_engine_unchanged())
    }

    if (!requireNamespace("cmdstanr", quietly = TRUE)) {
      message("cmdstanr installation failed.")
      return(.message_engine_unchanged())
    }
  }

  if (!.cmdstan_available()) {
    message("CmdStan is not installed.")
    if (interactive()) {
      choice <- utils::menu(
        c("Yes", "No"),
        title = "Install CmdStan via cmdstanr::install_cmdstan()?"
      )
      if (choice == 1L) {
        install_ok <- tryCatch(
          {
            cmdstanr::install_cmdstan()
            TRUE
          },
          error = function(e) {
            message("CmdStan installation failed: ", conditionMessage(e))
            FALSE
          }
        )
        if (!install_ok) {
          return(.message_engine_unchanged())
        }
      } else {
        return(.message_engine_unchanged())
      }
    } else {
      message(
        "Install CmdStan with:\n",
        "  cmdstanr::install_cmdstan()"
      )
      return(.message_engine_unchanged())
    }

    if (!.cmdstan_available()) {
      message("CmdStan installation failed.")
      return(.message_engine_unchanged())
    }
  }

  options(mlumr.stan_engine = "cmdstanr")
  message("mlumr engine set to: cmdstanr")
  invisible(engine)
}


#' Get the current Stan engine (internal)
#' @keywords internal
get_engine <- function() {
  .validate_engine_name(getOption("mlumr.stan_engine", "rstan"))
}


#' Validate a Stan engine name
#' @keywords internal
.validate_engine_name <- function(engine) {
  valid <- is.character(engine) &&
    length(engine) == 1L &&
    !is.na(engine) &&
    nzchar(engine)

  if (!valid || !engine %in% .supported_engines()) {
    stop("`engine` must be 'rstan' or 'cmdstanr'.", call. = FALSE)
  }

  engine
}


#' Supported Stan engines
#' @keywords internal
.supported_engines <- function() {
  c("rstan", "cmdstanr")
}


#' Check whether CmdStan is configured for cmdstanr
#' @keywords internal
.cmdstan_available <- function() {
  if (!requireNamespace("cmdstanr", quietly = TRUE)) {
    return(FALSE)
  }

  isTRUE(tryCatch(
    nzchar(cmdstanr::cmdstan_path()),
    error = function(e) FALSE
  ))
}


#' Report that the current engine was not changed
#' @keywords internal
.message_engine_unchanged <- function() {
  current <- tryCatch(
    get_engine(),
    error = function(e) NA_character_
  )
  label <- if (is.na(current)) "invalid current option" else current
  message(sprintf("Engine unchanged (%s).", label))
  invisible(current)
}

#' Summary of priors used by a fitted ML-UMR model
#'
#' Print a human-readable summary of every prior that was used to fit an
#' [mlumr()] model, including the effective per-coefficient scales after
#' autoscaling. Mirrors the spirit of `rstanarm::prior_summary()`.
#'
#' @param object An `mlumr_fit` object.
#' @param digits Number of significant digits for numeric values (default 3).
#' @param ... Unused.
#'
#' @return Invisibly returns a list describing the priors; the side effect is
#'   printing a formatted summary.
#' @seealso [prior_sensitivity()] to quantify how much the posterior moves
#'   under alternative `prior_beta` scales; [prior_normal()],
#'   [prior_student_t()], [prior_cauchy()], [prior_exponential()] for the
#'   prior constructors themselves.
#' @export
#' @examples
#' \dontrun{
#' fit <- mlumr(dat)
#' prior_summary(fit)
#' }
prior_summary <- function(object, ...) {
  UseMethod("prior_summary")
}

#' @rdname prior_summary
#' @export
prior_summary.default <- function(object, ...) {
  stop("prior_summary() has no method for class ",
       paste(class(object), collapse = "/"),
       call. = FALSE)
}

#' @rdname prior_summary
#' @method prior_summary mlumr_fit
#' @export
prior_summary.mlumr_fit <- function(object, digits = 3, ...) {

  .validate_mlumr_fit_object(object)
  digits <- .validate_prior_summary_digits(digits)

  priors <- object$priors
  if (is.null(priors)) {
    stop("No prior information stored on this fit (was it fitted with an ",
         "older version of mlumr?).", call. = FALSE)
  }
  if (!is.null(priors$beta_resolved)) {
    priors$beta_resolved <- .validate_resolved_beta_prior(priors$beta_resolved)
  }

  cat("Priors for ML-UMR Fit\n")
  cat("=====================\n\n")

  # Intercepts
  cat("Intercepts (mu_index, mu_comparator):\n")
  cat("  ", .format_prior(priors$intercept, digits = digits), "\n", sep = "")
  .print_default_tag(priors$intercept)
  cat("\n")

  # Beta (regression coefficients)
  cat("Regression coefficients (beta):\n")
  br <- priors$beta_resolved
  if (is.null(br)) {
    # Fallback for older fits: just print the user-specified prior.
    cat(paste0("  ", .format_prior_collection(priors$beta, digits = digits)),
        sep = "\n")
    cat("\n")
  } else {
    # Detect whether the resolved per-coefficient priors are homogeneous.
    means_eq <- length(unique(round(br$mean, 12))) == 1L
    sds_eq   <- length(unique(round(br$sd,   12))) == 1L
    autos_any <- any(br$autoscale)

    family_label <- .resolved_prior_family_label(br$dist)
    broadcast_label <- .resolved_prior_broadcast_label(br, digits)

    if (means_eq && sds_eq && !autos_any) {
      # Broadcast summary
      cat(sprintf("  %s applied to all %d covariate(s)\n",
                  broadcast_label, length(br$mean)))
    } else {
      # Per-coefficient table
      tbl <- data.frame(
        coefficient = br$covariate_names,
        mean = round(br$mean, digits),
        scale = round(br$sd,  digits),
        autoscaled = br$autoscale,
        sd_x = round(br$sd_x, digits),
        stringsAsFactors = FALSE
      )
      cat(sprintf("  Family: %s%s\n", family_label,
                  if (br$dist == 1L) sprintf(" (df = %g)", br$df) else ""))
      print(tbl, row.names = FALSE)
      if (autos_any) {
        cat("  (scale = user_scale / sd_x for autoscaled rows)\n")
      }
    }
  }
  .print_default_tag(priors$beta)
  cat("\n")

  # Sigma (normal family only)
  if (!is.null(priors$sigma)) {
    cat("Residual SD (sigma, half-distribution via <lower=0>):\n")
    cat("  ", .format_prior(priors$sigma, digits = digits), "\n", sep = "")
    .print_default_tag(priors$sigma)
    cat("\n")
  }

  invisible(priors)
}


#' @keywords internal
.format_prior <- function(prior, digits = 3) {
  if (is.null(prior$distribution)) {
    return("<missing prior>")
  }
  switch(prior$distribution,
    normal = sprintf("normal(%s, %s)%s",
                     format(prior$mean, digits = digits),
                     format(prior$sd,   digits = digits),
                     if (isTRUE(prior$autoscale)) " [autoscale]" else ""),
    student_t = sprintf("student_t(df = %g, %s, %s)%s",
                        prior$df,
                        format(prior$mean, digits = digits),
                        format(prior$sd,   digits = digits),
                        if (isTRUE(prior$autoscale)) " [autoscale]" else ""),
    exponential = sprintf("exponential(rate = %s)",
                          format(prior$rate, digits = digits)),
    sprintf("%s(...)", prior$distribution)
  )
}

#' @keywords internal
.format_prior_collection <- function(prior, digits = 3) {
  if (is_single_prior(prior) || is.null(prior)) {
    return(.format_prior(prior, digits = digits))
  }
  if (is.list(prior) && all(vapply(prior, is_single_prior, logical(1)))) {
    return(vapply(seq_along(prior), function(i) {
      sprintf("beta[%d]: %s", i, .format_prior(prior[[i]], digits = digits))
    }, character(1)))
  }
  "<missing prior>"
}

#' @keywords internal
.print_default_tag <- function(prior) {
  if (isTRUE(prior$default) && !is.null(prior$version)) {
    cat("  (package default, mlumr ", prior$version, ")\n", sep = "")
  }
}


#' Validate prior_summary digits
#' @keywords internal
.validate_prior_summary_digits <- function(digits) {
  valid <- is.numeric(digits) &&
    length(digits) == 1L &&
    is.finite(digits) &&
    digits >= 1 &&
    digits <= 22 &&
    digits == as.integer(digits)
  if (!valid) {
    stop("`digits` must be a single integer between 1 and 22.",
         call. = FALSE)
  }
  as.integer(digits)
}


#' Validate resolved beta-prior metadata stored on a fit
#' @keywords internal
.validate_resolved_beta_prior <- function(br) {
  required <- c("mean", "sd", "dist", "df", "autoscale", "sd_x",
                "covariate_names")
  missing <- setdiff(required, names(br))
  if (length(missing) > 0L) {
    stop(sprintf("Resolved beta prior metadata is missing: %s.",
                 paste(missing, collapse = ", ")), call. = FALSE)
  }

  n_beta <- length(br$mean)
  valid <- is.numeric(br$mean) &&
    is.numeric(br$sd) &&
    is.logical(br$autoscale) &&
    is.numeric(br$sd_x) &&
    is.character(br$covariate_names) &&
    n_beta > 0L &&
    length(br$sd) == n_beta &&
    length(br$autoscale) == n_beta &&
    length(br$sd_x) == n_beta &&
    length(br$covariate_names) == n_beta &&
    all(is.finite(br$mean)) &&
    all(is.finite(br$sd)) &&
    all(br$sd > 0) &&
    all(is.finite(br$sd_x)) &&
    all(!is.na(br$autoscale)) &&
    all(!is.na(br$covariate_names)) &&
    all(nzchar(br$covariate_names))

  if (!valid) {
    stop("Resolved beta prior metadata is malformed.", call. = FALSE)
  }
  if (!is.numeric(br$dist) || length(br$dist) != 1L ||
      !is.finite(br$dist)) {
    stop("Resolved beta prior distribution code is malformed.", call. = FALSE)
  }
  if (!is.numeric(br$df) || length(br$df) != 1L || !is.finite(br$df)) {
    stop("Resolved beta prior degrees of freedom is malformed.",
         call. = FALSE)
  }

  br$dist <- as.integer(br$dist)
  br
}


#' Label a resolved Stan prior family code
#' @keywords internal
.resolved_prior_family_label <- function(dist) {
  switch(as.character(dist),
    "0" = "normal",
    "1" = "student_t",
    sprintf("dist=%s", dist)
  )
}


#' Format a homogeneous resolved beta prior
#' @keywords internal
.resolved_prior_broadcast_label <- function(br, digits) {
  switch(as.character(br$dist),
    "0" = sprintf("normal(%s, %s)",
                  format(br$mean[[1L]], digits = digits),
                  format(br$sd[[1L]], digits = digits)),
    "1" = sprintf("student_t(df = %g, %s, %s)",
                  br$df,
                  format(br$mean[[1L]], digits = digits),
                  format(br$sd[[1L]], digits = digits)),
    sprintf("dist=%s(...)", br$dist)
  )
}

#' Validate and resolve link function for a given family
#'
#' Checks that `link` is valid for `family` and returns the resolved link name
#' plus an integer code for Stan. Accepts canonical family names (`"binomial"`,
#' `"normal"`, `"poisson"`) and data-type aliases (`"binary"`, `"count"`,
#' `"rate"`, `"continuous"`).
#'
#' The V1 likelihood/link matrix is:
#'
#' \tabular{llll}{
#'   \strong{Data type} \tab \strong{Family} \tab \strong{Likelihoods} \tab \strong{Link functions} \cr
#'   Binary  \tab binomial \tab bernoulli (IPD), binomial (AgD)  \tab logit, probit, cloglog \cr
#'   Count*  \tab binomial \tab bernoulli (IPD), binomial (AgD)  \tab logit, probit, cloglog \cr
#'   Rate    \tab poisson  \tab poisson                          \tab log                    \cr
#'   Continuous \tab normal \tab normal                           \tab identity, log          \cr
#' }
#'
#' *`"count"` refers to count/total (binomial denominator) data, not Poisson
#' event counts. For Poisson rate or count outcomes, use `"poisson"` or `"rate"`.
#'
#' @param family Character: canonical (`"binomial"`, `"normal"`, `"poisson"`)
#'   or alias (`"binary"`, `"count"`, `"rate"`, `"continuous"`).
#' @param link Character or `NULL`. If `NULL`, uses default for family.
#' @return List with components:
#' \describe{
#'   \item{family}{Canonical family name (e.g. `"binomial"`)}
#'   \item{link}{Resolved link name (e.g. `"probit"`)}
#'   \item{code}{Integer code for Stan data block}
#' }
#' @keywords internal
check_link <- function(family, link = NULL) {

  family <- .validate_link_string(family, "family")
  family <- tolower(family)

  # Resolve data-type aliases to canonical family names (user-facing
  # ergonomics; valid links / defaults come from family_config)
  family_aliases <- c(
    binary     = "binomial",
    count      = "binomial",
    rate       = "poisson",
    continuous = "normal"
  )
  canonical <- unname(family_aliases[family])
  if (!is.na(canonical)) {
    family <- canonical
  }

  if (!family %in% names(family_config)) {
    stop(sprintf("Unknown family '%s'. Valid: %s",
                 family,
                 paste(c(names(family_config), names(family_aliases)),
                       collapse = ", ")),
         call. = FALSE)
  }

  cfg <- family_config[[family]]

  if (is.null(link)) {
    link <- cfg$link_default
  } else {
    link <- .validate_link_string(link, "link")
  }
  link <- tolower(link)

  if (!link %in% cfg$links) {
    stop(sprintf("Link '%s' is not valid for family '%s'. Valid: %s",
                 link, family, paste(cfg$links, collapse = ", ")),
         call. = FALSE)
  }

  list(
    family = family,
    link = link,
    code = as.integer(match(link, cfg$links))
  )
}

#' Inverse link function (linear predictor -> response scale)
#' @param x Numeric vector
#' @param link Character: link function name
#' @return Numeric vector on response scale
#' @importFrom stats pnorm
#' @keywords internal
inverse_link <- function(x, link = c("identity", "log", "logit", "probit", "cloglog")) {
  link <- match.arg(link)
  .validate_numeric_vector(x, "x")
  switch(link,
    identity = x,
    log      = exp(x),
    logit    = plogis(x),
    probit   = pnorm(x),
    cloglog  = -expm1(-exp(x))
  )
}

#' Link function (response scale -> linear predictor)
#' @param x Numeric vector on response scale
#' @param link Character: link function name
#' @return Numeric vector on linear predictor scale
#' @keywords internal
link_fun <- function(x, link = c("identity", "log", "logit", "probit", "cloglog")) {
  link <- match.arg(link)
  .validate_numeric_vector(x, "x")
  eps <- .Machine$double.eps
  p <- pmin(pmax(x, eps), 1 - eps)
  switch(link,
    identity = x,
    log      = log(pmax(x, eps)),
    logit    = qlogis(p),
    probit   = qnorm(p),
    cloglog  = log(-log1p(-p))
  )
}

#' Bound probabilities away from 0 and 1
#' @keywords internal
bound_probability <- function(p, n, min_count = 0.5) {
  .validate_numeric_vector(p, "p")
  .validate_positive_numeric(n, "n")
  .validate_positive_numeric(min_count, "min_count")
  if (any(2 * min_count > n)) {
    stop("`min_count` must be no larger than n / 2.", call. = FALSE)
  }
  pmin(pmax(p, min_count / n), 1 - min_count / n)
}

#' Derivative of a binomial link with respect to probability
#' @keywords internal
link_derivative_response <- function(p, link = c("logit", "probit", "cloglog")) {
  link <- match.arg(link)
  .validate_numeric_vector(p, "p")
  p <- .bound_unit_interval(p)

  switch(link,
    logit = 1 / (p * (1 - p)),
    probit = 1 / dnorm(qnorm(p)),
    cloglog = 1 / ((1 - p) * (-log1p(-p)))
  )
}

#' Derivative of inverse link with respect to linear predictor
#' @keywords internal
inverse_link_derivative <- function(eta,
                                    p = inverse_link(eta, link),
                                    link = c("logit", "probit", "cloglog")) {
  link <- match.arg(link)
  .validate_numeric_vector(eta, "eta")
  .validate_numeric_vector(p, "p")
  p <- .bound_unit_interval(p)

  switch(link,
    logit = p * (1 - p),
    probit = dnorm(eta),
    cloglog = {
      exp_eta <- exp(eta)
      deriv <- exp_eta * exp(-exp_eta)
      deriv[is.infinite(exp_eta) & exp_eta > 0] <- 0
      deriv
    }
  )
}

#' Delta-method variance for a transformed binomial proportion
#' @keywords internal
binomial_link_variance <- function(p, n, link = c("logit", "probit", "cloglog")) {
  .validate_numeric_vector(p, "p")
  .validate_positive_numeric(n, "n")
  p <- .bound_unit_interval(p)
  p * (1 - p) * link_derivative_response(p, link)^2 / n
}

#' Emit package progress messages when enabled
#' @keywords internal
mlumr_message <- function(..., verbose = TRUE) {
  if (!is.logical(verbose) || length(verbose) != 1L || is.na(verbose)) {
    stop("`verbose` must be TRUE or FALSE.", call. = FALSE)
  }
  if (isTRUE(verbose)) {
    message(...)
  }
  invisible(NULL)
}


#' Validate a scalar link/family string
#' @keywords internal
.validate_link_string <- function(x, name) {
  if (!is.character(x) || length(x) != 1L || is.na(x) || !nzchar(x)) {
    stop(sprintf("`%s` must be a single non-missing string.", name),
         call. = FALSE)
  }
  x
}


#' Validate a numeric vector
#' @keywords internal
.validate_numeric_vector <- function(x, name) {
  if (!is.numeric(x)) {
    stop(sprintf("`%s` must be numeric.", name), call. = FALSE)
  }
  invisible(TRUE)
}


#' Validate positive finite numeric input
#' @keywords internal
.validate_positive_numeric <- function(x, name) {
  if (!is.numeric(x) || length(x) == 0L ||
        any(!is.finite(x)) || any(x <= 0)) {
    stop(sprintf("`%s` must contain positive finite values.", name),
         call. = FALSE)
  }
  invisible(TRUE)
}


#' Bound probabilities to the open unit interval
#' @keywords internal
.bound_unit_interval <- function(p) {
  eps <- .Machine$double.eps
  pmin(pmax(p, eps), 1 - eps)
}

# Tests for link function utilities

test_that("check_link returns correct defaults", {
  expect_equal(check_link("binomial")$link, "logit")
  expect_equal(check_link("binomial")$code, 1L)
  expect_equal(check_link("normal")$link, "identity")
  expect_equal(check_link("normal")$code, 1L)
  expect_equal(check_link("poisson")$link, "log")
  expect_equal(check_link("poisson")$code, 1L)
})

test_that("check_link returns canonical family name", {
  expect_equal(check_link("binomial")$family, "binomial")
  expect_equal(check_link("binary")$family, "binomial")
  expect_equal(check_link("count")$family, "binomial")
  expect_equal(check_link("normal")$family, "normal")
  expect_equal(check_link("continuous")$family, "normal")
  expect_equal(check_link("poisson")$family, "poisson")
  expect_equal(check_link("rate")$family, "poisson")
})

test_that("check_link resolves data-type aliases", {
  # binary -> binomial

  expect_equal(check_link("binary")$link, "logit")
  expect_equal(check_link("binary", "probit")$code, 2L)
  expect_equal(check_link("binary", "cloglog")$code, 3L)

  # count -> binomial (from multinma convention: r/n aggregate counts)
  expect_equal(check_link("count")$link, "logit")
  expect_equal(check_link("count", "probit")$link, "probit")

  # rate -> poisson
  expect_equal(check_link("rate")$link, "log")
  expect_equal(check_link("rate")$code, 1L)

  # continuous -> normal
  expect_equal(check_link("continuous")$link, "identity")
  expect_equal(check_link("continuous", "log")$code, 2L)
})

test_that("check_link returns correct codes for all valid links", {
  expect_equal(check_link("binomial", "logit")$code, 1L)
  expect_equal(check_link("binomial", "probit")$code, 2L)
  expect_equal(check_link("binomial", "cloglog")$code, 3L)

  expect_equal(check_link("normal", "identity")$code, 1L)
  expect_equal(check_link("normal", "log")$code, 2L)

  expect_equal(check_link("poisson", "log")$code, 1L)
})

test_that("check_link is case-insensitive", {
  expect_equal(check_link("binomial", "Logit")$link, "logit")
  expect_equal(check_link("binomial", "PROBIT")$link, "probit")
  expect_equal(check_link("normal", "LOG")$link, "log")
  expect_equal(check_link("Binary", "cloglog")$link, "cloglog")
  expect_equal(check_link("CONTINUOUS")$link, "identity")
})

test_that("check_link errors on invalid links", {
  expect_error(check_link("binomial", "identity"), "not valid for family")
  expect_error(check_link("binomial", "log"), "not valid for family")
  expect_error(check_link("normal", "logit"), "not valid for family")
  expect_error(check_link("poisson", "logit"), "not valid for family")
  expect_error(check_link("normal", "probit"), "not valid for family")
  expect_error(check_link("poisson", "identity"), "not valid for family")
})

test_that("check_link errors on unknown family", {
  expect_error(check_link("gamma"), "Unknown family")
  expect_error(check_link("ordered"), "Unknown family")
})

test_that("check_link rejects malformed scalar inputs cleanly", {
  expect_error(check_link(c("binomial", "normal")),
               "`family` must be a single non-missing string")
  expect_error(check_link(NA_character_),
               "`family` must be a single non-missing string")
  expect_error(check_link(""),
               "`family` must be a single non-missing string")
  expect_error(check_link(1),
               "`family` must be a single non-missing string")

  expect_error(check_link("binomial", c("logit", "probit")),
               "`link` must be a single non-missing string")
  expect_error(check_link("binomial", NA_character_),
               "`link` must be a single non-missing string")
  expect_error(check_link("binomial", ""),
               "`link` must be a single non-missing string")
})

test_that("inverse_link matches expected functions", {
  x <- seq(-3, 3, by = 0.5)
  expect_equal(inverse_link(x, "identity"), x)
  expect_equal(inverse_link(x, "log"), exp(x))
  expect_equal(inverse_link(x, "logit"), plogis(x))
  expect_equal(inverse_link(x, "probit"), pnorm(x))
  expect_equal(inverse_link(x, "cloglog"), 1 - exp(-exp(x)))
})

test_that("cloglog inverse link is stable for extreme predictors", {
  eta <- c(-1000, -50, 0, 50, 1000)
  out <- inverse_link(eta, "cloglog")

  expect_equal(out[[1L]], 0)
  expect_equal(out[[2L]], -expm1(-exp(-50)))
  expect_equal(out[[3L]], 1 - exp(-1))
  expect_equal(out[[4L]], 1)
  expect_equal(out[[5L]], 1)
  expect_true(all(is.finite(out)))
})

test_that("link_fun matches expected functions", {
  p <- seq(0.1, 0.9, by = 0.1)
  expect_equal(link_fun(p, "identity"), p)
  expect_equal(link_fun(p, "logit"), qlogis(p))
  expect_equal(link_fun(p, "probit"), qnorm(p))
  expect_equal(link_fun(exp(1:3), "log"), 1:3)
})

test_that("inverse_link and link_fun are inverses", {
  p <- seq(0.05, 0.95, by = 0.05)
  for (lnk in c("logit", "probit", "cloglog")) {
    expect_equal(inverse_link(link_fun(p, lnk), lnk), p, tolerance = 1e-10,
                 label = paste("roundtrip for", lnk))
  }

  x <- seq(-2, 2, by = 0.5)
  expect_equal(link_fun(inverse_link(x, "logit"), "logit"), x, tolerance = 1e-10)
  expect_equal(link_fun(inverse_link(x, "probit"), "probit"), x, tolerance = 1e-10)
  expect_equal(link_fun(inverse_link(x, "cloglog"), "cloglog"), x, tolerance = 1e-10)

  y <- exp(seq(0.1, 3, by = 0.5))
  expect_equal(inverse_link(link_fun(y, "log"), "log"), y, tolerance = 1e-10)
})

test_that("link_fun handles boundary values safely", {
  # Should not produce Inf or NaN
  expect_true(is.finite(link_fun(0, "logit")))
  expect_true(is.finite(link_fun(1, "logit")))
  expect_true(is.finite(link_fun(0, "probit")))
  expect_true(is.finite(link_fun(1, "probit")))
  expect_true(is.finite(link_fun(0, "cloglog")))
  expect_true(is.finite(link_fun(1, "cloglog")))
  expect_true(is.finite(link_fun(0, "log")))
})

test_that("link helpers reject non-numeric response inputs cleanly", {
  expect_error(inverse_link("x", "logit"), "`x` must be numeric")
  expect_error(link_fun("x", "logit"), "`x` must be numeric")
  expect_error(link_derivative_response("x", "logit"), "`p` must be numeric")
  expect_error(binomial_link_variance("x", 10, "logit"), "`p` must be numeric")
})

test_that("bound_probability validates counts and min_count", {
  expect_equal(bound_probability(c(0, 0.2, 1), n = 10), c(0.05, 0.2, 0.95))
  expect_error(bound_probability(0.2, n = 0),
               "`n` must contain positive finite values")
  expect_error(bound_probability(0.2, n = Inf),
               "`n` must contain positive finite values")
  expect_error(bound_probability(0.2, n = 10, min_count = 0),
               "`min_count` must contain positive finite values")
  expect_error(bound_probability(0.2, n = 10, min_count = 6),
               "`min_count` must be no larger than n / 2")
})

test_that("binomial link derivatives match existing formulas", {
  p <- 0.37
  n <- 123

  expect_equal(link_derivative_response(p, "logit"), 1 / (p * (1 - p)))
  expect_equal(link_derivative_response(p, "probit"), 1 / dnorm(qnorm(p)))
  expect_equal(
    link_derivative_response(p, "cloglog"),
    1 / ((1 - p) * (-log1p(-p)))
  )

  expect_equal(
    binomial_link_variance(p, n, "logit"),
    1 / (n * p * (1 - p))
  )
  expect_equal(
    binomial_link_variance(p, n, "probit"),
    p * (1 - p) / (n * dnorm(qnorm(p))^2)
  )
})

test_that("inverse link derivatives and binomial variances stay finite at boundaries", {
  eta <- c(-1000, 0, 1000)

  expect_equal(inverse_link_derivative(eta, link = "cloglog")[[1L]], 0)
  expect_equal(inverse_link_derivative(eta, link = "cloglog")[[3L]], 0)
  expect_true(all(is.finite(inverse_link_derivative(eta, link = "cloglog"))))

  vars <- binomial_link_variance(c(0, 0.5, 1), n = 100, link = "logit")
  expect_true(all(is.finite(vars)))
  expect_true(all(vars > 0))
  expect_error(binomial_link_variance(0.5, n = 0, link = "logit"),
               "`n` must contain positive finite values")
})

test_that("mlumr_message respects verbose flag", {
  expect_message(mlumr_message("visible", verbose = TRUE), "visible")
  expect_message(mlumr_message("hidden", verbose = FALSE), NA)
  expect_error(mlumr_message("bad", verbose = NA),
               "`verbose` must be TRUE or FALSE")
})

#' mlumr: Multilevel Unanchored Meta-Regression for Indirect Treatment
#' Comparisons
#'
#' Bayesian multilevel unanchored meta-regression (ML-UMR) for population-
#' adjusted indirect comparisons between a single-arm individual patient
#' data (IPD) study on the index treatment and aggregate data (AgD) on a
#' comparator. Two model variants are provided: a shared-prognostic-factor
#' (SPFA) model and a relaxed-SPFA model that allows treatment-specific
#' covariate coefficients. Supports binary (binomial), continuous (normal),
#' and count (Poisson) outcomes, each with appropriate link functions.
#' Frequentist outcome regression (Simulated Treatment Comparison) and a
#' naive unadjusted benchmark are included for side-by-side comparison.
#'
#' @section Typical workflow:
#' \enumerate{
#'   \item [set_ipd()] — declare the individual patient data and its
#'     covariates.
#'   \item [set_agd()] — declare the aggregate-data arm.
#'   \item [combine_data()] — join the two into an `mlumr_data` object.
#'   \item [add_integration()] — build the Gaussian-copula QMC
#'     integration points used to marginalize over the AgD covariate
#'     distribution.
#'   \item [mlumr()] — fit the Bayesian ML-UMR model (SPFA or relaxed).
#'   \item [prior_summary()], [check_integration()] — sanity-check the
#'     model before inferring from it.
#'   \item [predict()][predict.mlumr_fit()], [marginal_effects()],
#'     [conditional_effects()], [conditional_predict()] — extract
#'     population-level and profile-specific quantities.
#'   \item [calculate_dic()], [calculate_loo()], [calculate_waic()],
#'     [compare_models()] — compare SPFA vs relaxed or competing
#'     specifications.
#'   \item [prior_sensitivity()] — verify the posterior is data-driven
#'     rather than prior-driven.
#' }
#'
#' @section Priors:
#' The prior constructors [prior_normal()], [prior_student_t()],
#' [prior_cauchy()], and [prior_exponential()] all plug into [mlumr()] via
#' `prior_intercept`, `prior_beta`, and (normal family only) `prior_sigma`.
#' Defaults follow the Stan community's prior-choice recommendations
#' (Vehtari et al., 2025).
#'
#' @section Alternative methods:
#' [stc()] runs frequentist parametric G-computation (Simulated Treatment
#' Comparison). [naive()] computes an unadjusted crude-rate benchmark. Both
#' consume the same `mlumr_data` object as [mlumr()] so their estimands can
#' be compared directly.
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @useDynLib mlumr, .registration = TRUE
#' @import methods
#' @import Rcpp
#' @importFrom rstantools rstan_config
#' @importFrom RcppParallel RcppParallelLibs
#' @importFrom stats as.formula binomial coef complete.cases cor dbinom glm pbinom plogis predict qbinom qlogis qnorm quantile sd setNames var vcov weighted.mean
## usethis namespace: end
NULL

# Generated by rstantools.  Do not edit by hand.

# names of stan models
stanmodels <- c("mlumr_binary_relaxed", "mlumr_binary_spfa", "mlumr_normal_relaxed", "mlumr_normal_spfa", "mlumr_poisson_relaxed", "mlumr_poisson_spfa")

# load each stan module
Rcpp::loadModule("stan_fit4mlumr_binary_relaxed_mod", what = TRUE)
Rcpp::loadModule("stan_fit4mlumr_binary_spfa_mod", what = TRUE)
Rcpp::loadModule("stan_fit4mlumr_normal_relaxed_mod", what = TRUE)
Rcpp::loadModule("stan_fit4mlumr_normal_spfa_mod", what = TRUE)
Rcpp::loadModule("stan_fit4mlumr_poisson_relaxed_mod", what = TRUE)
Rcpp::loadModule("stan_fit4mlumr_poisson_spfa_mod", what = TRUE)

# instantiate each stanmodel object
stanmodels <- sapply(stanmodels, function(model_name) {
  # create C++ code for stan model
  stan_file <- if(dir.exists("stan")) "stan" else file.path("inst", "stan")
  stan_file <- file.path(stan_file, paste0(model_name, ".stan"))
  stanfit <- rstan::stanc_builder(stan_file,
                                  allow_undefined = TRUE,
                                  obfuscate_model_name = FALSE)
  stanfit$model_cpp <- list(model_cppname = stanfit$model_name,
                            model_cppcode = stanfit$cppcode)
  # create stanmodel object
  methods::new(Class = "stanmodel",
               model_name = stanfit$model_name,
               model_code = stanfit$model_code,
               model_cpp = stanfit$model_cpp,
               mk_cppmodule = function(x) get(paste0("rstantools_model_", model_name)))
})

---
title: "Comparing ML-UMR, STC, and Naive Methods"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Comparing ML-UMR, STC, and Naive Methods}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, purl = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  message = FALSE,
  warning = FALSE
)
```

## Why compare methods?

Unanchored indirect treatment comparisons (ITCs) rely on strong assumptions
that cannot be fully verified from data alone. Running multiple methods and
comparing their results is an important part of any ITC analysis:
a sensitivity check that reveals how conclusions depend on modeling choices.

mlumr provides three methods in a single package with a unified data
interface, making side-by-side comparison straightforward.

| Method | Adjustment | Framework | Key assumption |
|--------|-----------|-----------|----------------|
| Naive | None | Frequentist | Populations are exchangeable |
| STC | Outcome regression | Frequentist | Correct outcome model specification |
| ML-UMR SPFA | Joint Bayesian model | Bayesian | Shared prognostic effects (SPFA) |
| ML-UMR Relaxed | Joint Bayesian model | Bayesian | Correct specification and enough information/prior support for treatment-specific effects |

## Setup: shared data preparation

All three methods start from the same `mlumr_data` object. STC and ML-UMR
benefit from integration points; the naive method ignores them.

```{r, eval = TRUE, purl = FALSE}
library(mlumr)
set.seed(2026)

# --- Simulate IPD (index treatment) ---
n_A <- 500
age_A <- rbinom(n_A, 1, 0.40)
sex_A <- rbinom(n_A, 1, 0.55)

# True DGP: logit(p) = -0.5 + 0.8*age - 0.3*sex
logit_p_A <- -0.5 + 0.8 * age_A - 0.3 * sex_A
y_A <- rbinom(n_A, 1, plogis(logit_p_A))

ipd_df <- data.frame(
  trt = "Drug_A", outcome = y_A,
  age_group = age_A, sex = sex_A
)

# --- Simulate AgD (comparator) ---
# Same covariate effects, different intercept and covariate distribution
n_B <- 400
r_B <- 148  # pre-computed from true model with different covariates

agd_df <- data.frame(
  trt = "Drug_B", n_total = n_B, n_events = r_B,
  age_group_mean = 0.35, sex_mean = 0.50
)

# --- Prepare data ---
ipd <- set_ipd(ipd_df, treatment = "trt", outcome = "outcome",
               covariates = c("age_group", "sex"))
agd <- set_agd(agd_df, treatment = "trt",
               outcome_n = "n_total", outcome_r = "n_events",
               cov_means = c("age_group_mean", "sex_mean"),
               cov_types = c("binary", "binary"))

dat <- combine_data(ipd, agd)

# Add integration points (used by STC and ML-UMR)
dat <- add_integration(
  dat, n_int = 64,
  age_group = distr(qbern, prob = age_group_mean),
  sex = distr(qbern, prob = sex_mean)
)
```

## Running all three methods

### Naive estimate

```{r, eval = TRUE, purl = FALSE}
res_naive <- naive(dat)
print(res_naive)
```

The naive method ignores covariate differences between populations. It
serves as a reference: if naive and adjusted estimates agree, covariate
imbalance has little practical impact.

### STC estimate

```{r, eval = TRUE, purl = FALSE}
res_stc <- stc(dat)
print(res_stc)
```

STC fits a logistic regression on IPD and predicts counterfactual outcomes
for the comparator population via G-computation. The delta method provides
frequentist standard errors. STC is fast (sub-second) and a good default
when covariate adjustment is needed but a full Bayesian model is not
warranted.

### ML-UMR SPFA

```{r, eval = TRUE, purl = FALSE}
fit_spfa <- mlumr(
  dat, model = "spfa",
  prior_intercept = prior_normal(0, 10),
  prior_beta = prior_normal(0, 2.5),
  chains = 2, iter = 500, warmup = 250,
  seed = 42, refresh = 0, verbose = FALSE
)
summary(fit_spfa)
```

The SPFA model assumes shared covariate effects across treatments. It
jointly models both data sources and produces posterior distributions
for all parameters.

### ML-UMR Relaxed

```{r, eval = TRUE, purl = FALSE}
fit_relaxed <- mlumr(
  dat, model = "relaxed",
  prior_intercept = prior_normal(0, 10),
  prior_beta = prior_normal(0, 2.5),
  chains = 2, iter = 500, warmup = 250,
  seed = 43, refresh = 0, verbose = FALSE
)
summary(fit_relaxed)
```

The Relaxed model allows treatment-specific covariate coefficients,
capturing potential effect modification. Compare it with SPFA to assess
whether assuming shared effects is reasonable. With sparse AgD, relaxed-model
comparator coefficients can be prior-sensitive, so inspect `delta_beta` and
run prior-sensitivity checks when the relaxed model is central to the
interpretation.

## Building a comparison table

```{r, eval = TRUE, purl = FALSE}
# Extract ML-UMR marginal effects (LOR in comparator population)
me_spfa <- marginal_effects(fit_spfa, effect = "lor")
me_relaxed <- marginal_effects(fit_relaxed, effect = "lor")

# Comparator-population LORs from ML-UMR
lor_spfa <- me_spfa[me_spfa$population == "Comparator", ]
lor_relaxed <- me_relaxed[me_relaxed$population == "Comparator", ]

# Assemble results
comparison <- data.frame(
  Method = c("Naive", "STC", "ML-UMR SPFA", "ML-UMR Relaxed"),
  LOR = c(res_naive$link_effect, res_stc$link_effect, lor_spfa$mean, lor_relaxed$mean),
  SE = c(res_naive$se, res_stc$se, lor_spfa$sd, lor_relaxed$sd),
  CI_lower = c(res_naive$ci_lower, res_stc$ci_lower,
               lor_spfa$q2.5, lor_relaxed$q2.5),
  CI_upper = c(res_naive$ci_upper, res_stc$ci_upper,
               lor_spfa$q97.5, lor_relaxed$q97.5),
  stringsAsFactors = FALSE
)

# Add odds ratios for clinical interpretation
comparison$OR <- exp(comparison$LOR)
comparison$OR_lower <- exp(comparison$CI_lower)
comparison$OR_upper <- exp(comparison$CI_upper)

print(comparison, digits = 3)
```

## Interpreting differences between methods

### Naive vs STC

A large gap between naive and STC estimates indicates that covariate
imbalance is influencing the unadjusted comparison. The direction of the
shift reveals which population's covariates favor the outcome.

```{r, eval = TRUE, purl = FALSE}
bias_naive <- res_naive$link_effect - res_stc$link_effect
cat(sprintf("Apparent bias from covariate imbalance: %.3f (log OR scale)\n",
            bias_naive))
```

### STC vs ML-UMR SPFA

Under the shared prognostic factor assumption, STC and ML-UMR SPFA should
give similar point estimates because both adjust for the same covariates.
Differences arise because:

1. **STC** uses maximum likelihood; **ML-UMR** uses Bayesian inference with
   priors. With large samples, this difference is small.
2. **ML-UMR** jointly models both data sources; **STC** only uses AgD for
   prediction targets, not as a likelihood contribution.
3. ML-UMR uncertainty intervals tend to be wider because they account for
   more sources of uncertainty (prior, integration, joint modeling).

### SPFA vs Relaxed

```{r, eval = TRUE, purl = FALSE}
dic_comparison <- compare_models(fit_spfa, fit_relaxed)
print(dic_comparison)
```

If the Relaxed model gives markedly different LORs or substantially better
DIC, this *may* suggest effect modification -- covariate effects differ by
treatment. However, DIC is a rough metric with known limitations: it should
not be the sole basis for claiming effect modification. Always inspect
`delta_beta` credible intervals, prior sensitivity, and clinical plausibility.
If SPFA and Relaxed agree, the simpler SPFA model is preferred.

```{r, eval = TRUE, purl = FALSE}
# Directly compare delta_beta from the Relaxed model
# Non-zero delta_beta indicates effect modification
relaxed_summary <- fit_relaxed$summary
delta_rows <- grepl("^delta_beta", relaxed_summary$variable)
if (any(delta_rows)) {
  cat("Effect modification parameters (delta_beta):\n")
  print(relaxed_summary[delta_rows, c("variable", "mean", "2.5%", "97.5%")],
        row.names = FALSE)
}
```

```{r, eval = FALSE, purl = FALSE}
prior_sensitivity(fit_relaxed, prior_beta_scales = c(1, 2.5, 5, 10))
```

## Decision guide: which method to report?

The choice of primary analysis depends on the clinical and regulatory
context. This flowchart summarizes the main considerations:

1. **Are covariate distributions similar across trials?**
   - Yes: Naive estimate may suffice as primary, with STC as sensitivity.
   - No: Adjustment is needed. Proceed to step 2.

2. **Is the SPFA assumption plausible?**
   - Yes: ML-UMR SPFA is the recommended primary analysis. Report STC and
     Naive as sensitivity analyses.
   - Uncertain: Run both SPFA and Relaxed. If they agree, report SPFA.
     If they disagree, report Relaxed as primary and discuss the evidence
     for effect modification.
   - No (known effect modification): ML-UMR Relaxed as primary.

3. **Is Bayesian analysis acceptable to the audience?**
   - No: Report STC as primary with ML-UMR as sensitivity.
   - Yes: Report ML-UMR as primary with STC as sensitivity.

In all cases, report the naive estimate as a benchmark to quantify the
impact of covariate adjustment.

## Presenting results

For a complete ITC report, present:

1. **A summary table** (as above) with LOR, SE, and CI for each method
2. **Model diagnostics** for ML-UMR (divergences, Rhat, ESS)
3. **DIC comparison** between SPFA and Relaxed (if both were fit)
4. **A narrative** explaining which method is primary and why, with
   sensitivity analyses supporting the conclusions

```{r, eval = TRUE, purl = FALSE}
# Example narrative output
cat("Primary analysis: ML-UMR SPFA\n")
cat(sprintf("  LOR = %.3f (95%% CrI: %.3f to %.3f)\n",
            lor_spfa$mean, lor_spfa$q2.5, lor_spfa$q97.5))
cat(sprintf("  OR  = %.3f (95%% CrI: %.3f to %.3f)\n\n",
            exp(lor_spfa$mean), exp(lor_spfa$q2.5), exp(lor_spfa$q97.5)))

cat("Sensitivity analyses:\n")
cat(sprintf("  STC:    LOR = %.3f (95%% CI: %.3f to %.3f)\n",
            res_stc$link_effect, res_stc$ci_lower, res_stc$ci_upper))
cat(sprintf("  Naive:  LOR = %.3f (95%% CI: %.3f to %.3f)\n",
            res_naive$link_effect, res_naive$ci_lower, res_naive$ci_upper))
cat(sprintf("  Relaxed: LOR = %.3f (95%% CrI: %.3f to %.3f)\n",
            lor_relaxed$mean, lor_relaxed$q2.5, lor_relaxed$q97.5))
```

## Summary

Running all three methods and comparing their results strengthens any
unanchored ITC analysis. mlumr's unified data interface makes this
comparison straightforward -- the same `mlumr_data` object feeds all
methods, ensuring consistency in the inputs.

Key takeaways:

- **Always report the naive estimate** as a baseline to show the impact
  of adjustment.
- **STC is a fast, frequentist alternative** that adjusts for measured
  covariates via outcome regression.
- **ML-UMR SPFA** is the recommended primary analysis under the shared
  prognostic factor assumption, offering fully Bayesian uncertainty
  quantification.
- **ML-UMR Relaxed** is a useful sensitivity analysis when effect
  modification is plausible.
- **Compare SPFA and Relaxed via DIC** as a rough sensitivity check.
  DIC is a legacy metric with known limitations; treat large differences
  as suggestive rather than definitive evidence for effect modification.

---
title: "Introduction to mlumr"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to mlumr}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, purl = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  message = FALSE,
  warning = FALSE
)
```

## Overview

**mlumr** implements multilevel unanchored meta-regression (ML-UMR) for
indirect treatment comparisons using individual patient data (IPD) and
aggregate data (AgD). It supports binary, continuous (normal), and count
(Poisson) outcomes. It provides:

- **ML-UMR** (Bayesian): SPFA and Relaxed SPFA models via Stan
  (`rstan` by default, with optional `cmdstanr`)
- **STC**: Simulated treatment comparison via parametric G-computation
- **Naive**: Unadjusted comparison as a benchmark

Version 0.1.0 is a two-treatment package: one treatment is represented by
IPD (the index treatment) and one by AgD (the comparator treatment).
Multi-treatment unanchored networks are outside the current package scope.

## When to use mlumr

Use mlumr when you need to compare two treatments that have never been
directly compared in the same trial (disconnected network), and you have:

- **IPD** from one trial (index treatment)
- **AgD** from another trial (comparator treatment)
- Shared covariates between populations
- Binary, continuous, or count outcomes

## Installation

```{r, eval = FALSE, purl = FALSE}
# From GitHub (development version)
# install.packages("remotes")
remotes::install_github("choxos/mlumr")
```

## Quick start

The pipeline below runs on toy data so every chunk is self-contained.
For a more detailed worked analysis, see `vignette("worked-example")`.

```{r, eval = TRUE}
library(mlumr)
set.seed(2026)

# Toy IPD: trial A (index treatment, binary outcome)
n_a <- 300
trial_a_data <- data.frame(
  trt      = "Drug_A",
  response = rbinom(n_a, 1, 0.55),
  age_cat  = rbinom(n_a, 1, 0.40),
  sex      = rbinom(n_a, 1, 0.55)
)

# Toy AgD: trial B (comparator treatment)
trial_b_data <- data.frame(
  trt           = "Drug_B",
  n_total       = 400,
  n_events      = 160,
  age_cat_mean  = 0.35,
  sex_mean      = 0.50
)
```

```{r, eval = TRUE}
# 1. Prepare IPD
ipd <- set_ipd(
  data = trial_a_data,
  treatment = "trt",
  outcome = "response",
  covariates = c("age_cat", "sex")
)

# 2. Prepare AgD
agd <- set_agd(
  data = trial_b_data,
  treatment = "trt",
  outcome_n = "n_total",
  outcome_r = "n_events",
  cov_means = c("age_cat_mean", "sex_mean"),
  cov_types = c("binary", "binary")
)

# 3. Combine
dat <- combine_data(ipd, agd)

# 4. Add integration points (needed for ML-UMR)
dat <- add_integration(
  dat,
  n_int = 64,
  age_cat = distr(qbern, prob = age_cat_mean),
  sex = distr(qbern, prob = sex_mean)
)
```

```{r, eval = TRUE, purl = FALSE}
# 5. Fit ML-UMR
fit_spfa <- mlumr(
  dat, model = "spfa",
  chains = 2, iter = 500, warmup = 250,
  seed = 42, refresh = 0, verbose = FALSE
)
fit_relaxed <- mlumr(
  dat, model = "relaxed",
  chains = 2, iter = 500, warmup = 250,
  seed = 43, refresh = 0, verbose = FALSE
)

# 6. Compare
summary(fit_spfa)
compare_models(fit_spfa, fit_relaxed)

# 7. Frequentist benchmarks
naive_result <- naive(dat)
stc_result <- stc(dat)
print(naive_result)
print(stc_result)
```

Before using results in a real analysis, inspect integration quality and
posterior diagnostics:

```{r, eval = TRUE, purl = FALSE}
check_integration(
  dat,
  age_cat = distr(qbern, prob = age_cat_mean),
  sex = distr(qbern, prob = sex_mean)
)
prior_summary(fit_spfa)
fit_spfa$diagnostics
max(fit_spfa$summary$Rhat, na.rm = TRUE)
min(fit_spfa$summary$n_eff, na.rm = TRUE)
```

## Choosing a method

| Method | Adjusts for covariates? | Effect modification? | Inference |
|--------|------------------------|---------------------|-----------|
| ML-UMR SPFA | Yes | No (shared betas) | Bayesian |
| ML-UMR Relaxed | Yes | Yes (treatment-specific betas) | Bayesian |
| STC | Yes (outcome model only) | No | Frequentist |
| Naive | No | N/A | Frequentist |

- **ML-UMR SPFA** is appropriate when prognostic factors affect both
  treatments equally.
- **ML-UMR Relaxed** is appropriate when you suspect treatment-by-covariate
  interactions (effect modification).
- **STC** adjusts for imbalances through an outcome model and uses
  integration points when available; best when treatment effects do not vary
  with covariates.
- **Naive** serves as a benchmark only; do not use for inference in the
  presence of cross-trial differences.

---
title: "Data Preparation and Integration"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Data Preparation and Integration}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, purl = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  message = FALSE,
  warning = FALSE
)
```

## Overview

Before fitting any model in mlumr, you need to:

1. Set up the IPD with `set_ipd()`
2. Set up the AgD with `set_agd()`
3. Combine them with `combine_data()`
4. (For ML-UMR only) Add integration points with `add_integration()`

## Setting up IPD

IPD should be a data frame with one row per patient, containing:

- A treatment indicator column
- An outcome column (binary 0/1, continuous numeric, or non-negative integer
  counts depending on `family`)
- Covariate columns
- For Poisson family: an exposure column

```{r, eval = TRUE}
library(mlumr)
set.seed(2026)

# Example: Trial A data (index treatment)
trial_a <- data.frame(
  patient_id = 1:500,
  treatment = "Drug_A",
  response = rbinom(500, 1, 0.6),
  age_group = rbinom(500, 1, 0.4),   # 0 = young, 1 = old
  sex = rbinom(500, 1, 0.55)          # 0 = male, 1 = female
)

ipd <- set_ipd(
  data = trial_a,
  treatment = "treatment",
  outcome = "response",
  covariates = c("age_group", "sex")
)

ipd
```

### Continuous outcomes

For continuous outcomes, use `family = "normal"`. The IPD frame needs a
numeric outcome column:

```{r, eval = TRUE}
trial_a_normal <- data.frame(
  patient_id = 1:500,
  treatment = "Drug_A",
  score = rnorm(500, mean = 3.0, sd = 1.2),
  age_group = rbinom(500, 1, 0.4),
  sex = rbinom(500, 1, 0.55)
)

ipd_normal <- set_ipd(
  data = trial_a_normal,
  treatment = "treatment",
  outcome = "score",
  covariates = c("age_group", "sex"),
  family = "normal"
)
```

### Count outcomes

For Poisson count data, use `family = "poisson"` and supply an `exposure`
column:

```{r, eval = TRUE}
trial_a_poisson <- data.frame(
  patient_id = 1:500,
  treatment = "Drug_A",
  events = rpois(500, lambda = 0.8),
  person_years = runif(500, 0.5, 2.0),
  age_group = rbinom(500, 1, 0.4),
  sex = rbinom(500, 1, 0.55)
)

ipd_poisson <- set_ipd(
  data = trial_a_poisson,
  treatment = "treatment",
  outcome = "events",
  covariates = c("age_group", "sex"),
  family = "poisson",
  exposure = "person_years"
)
```

### Validation

`set_ipd()` checks that:

- All specified columns exist in the data
- The outcome matches the family (binary for binomial, numeric for normal,
  non-negative integers for Poisson)
- Missing values are handled (rows dropped with a warning)

```{r, eval = TRUE, error = TRUE, purl = FALSE}
# This will error: non-binary outcome with binomial family
bad_data <- data.frame(trt = "A", outcome = c(0, 1, 2), x = rnorm(3))
set_ipd(bad_data, "trt", "outcome", "x")
```

## Setting up AgD

AgD should be a data frame with one row per study/subgroup, containing:

- Treatment indicator
- Outcome summaries (depend on `family`):
  - **Binomial**: sample size (`outcome_n`) and number of events (`outcome_r`)
  - **Normal**: mean outcome (`outcome_mean`) and standard error (`outcome_se`),
    optionally sample size (`outcome_n`)
  - **Poisson**: number of events (`outcome_r`) and total exposure (`outcome_E`)
- Covariate means/proportions

```{r, eval = TRUE}
# Example: Trial B data (comparator treatment)
trial_b <- data.frame(
  study = "Trial_B",
  treatment = "Drug_B",
  n_total = 400,
  n_events = 160,
  age_group_mean = 0.35,   # proportion in "old" group
  sex_prop = 0.50           # proportion female
)

agd <- set_agd(
  data = trial_b,
  treatment = "treatment",
  outcome_n = "n_total",
  outcome_r = "n_events",
  cov_means = c("age_group_mean", "sex_prop"),
  cov_types = c("binary", "binary")
)
```

### AgD for continuous outcomes

```{r, eval = TRUE}
agd_normal <- set_agd(
  data = data.frame(
    trt = "Drug_B", y_mean = 3.2, se = 0.15, n = 400,
    age_group_mean = 0.35, sex_prop = 0.50
  ),
  treatment = "trt",
  family = "normal",
  outcome_mean = "y_mean",
  outcome_se = "se",
  outcome_n = "n",
  cov_means = c("age_group_mean", "sex_prop"),
  cov_types = c("binary", "binary")
)
```

**Scale requirement for normal outcomes.** `set_agd()` assumes the
AgD mean (`outcome_mean`) and SE (`outcome_se`) are on the
**arithmetic (original, untransformed) scale**, regardless of
whether `mlumr()` is later called with `link = "identity"` or
`link = "log"`. If a publication reports only a log-scale mean and
SD, or a geometric mean, back-transform before calling `set_agd()`
and propagate uncertainty via the delta method; passing log-scale
summaries silently biases the posterior because the Stan likelihood
is `normal(E[exp(eta)], se_agd)` under the log link, expecting the
AgD quantities already on the outcome scale.

### AgD for count outcomes

```{r, eval = TRUE}
agd_poisson <- set_agd(
  data = data.frame(
    trt = "Drug_B", n_events = 120, person_years = 800,
    age_group_mean = 0.35, sex_prop = 0.50
  ),
  treatment = "trt",
  family = "poisson",
  outcome_r = "n_events",
  outcome_E = "person_years",
  cov_means = c("age_group_mean", "sex_prop"),
  cov_types = c("binary", "binary")
)
```

### Covariate naming

`set_agd()` automatically strips `_mean` and `_prop` suffixes from covariate
names to match the IPD covariate names:

- `"age_group_mean"` -> covariate name `"age_group"`
- `"sex_prop"` -> covariate name `"sex"`

Make sure the resulting names match those in your IPD.

### Continuous covariates

For continuous covariates, also supply standard deviations:

```{r, eval = TRUE}
agd_continuous <- set_agd(
  data = data.frame(
    trt = "B", n_total = 400, n_events = 160,
    bmi_mean = 25.3, bmi_sd = 4.2
  ),
  treatment = "trt",
  outcome_n = "n_total",
  outcome_r = "n_events",
  cov_means = "bmi_mean",
  cov_sds = "bmi_sd",
  cov_types = "continuous"
)
```

## Combining data

```{r, eval = TRUE}
dat <- combine_data(ipd, agd)
print(dat)
```

## Adding integration points

Integration points are needed for ML-UMR's numerical integration over the
comparator population's covariate distribution. They are generated using
Sobol quasi-Monte Carlo sequences with a Gaussian copula to respect
covariate correlations.

### Basic usage

```{r, eval = TRUE}
dat <- add_integration(
  dat,
  n_int = 64,
  age_group = distr(qbern, prob = age_group_mean),
  sex = distr(qbern, prob = sex_mean)
)
```

Key points:

- **`n_int`**: Number of integration points (powers of 2 recommended: 32,
  64, 128, 256). More points = better accuracy but slower Stan fitting.
- **`distr()`**: Wraps a quantile function with parameters. Parameters can
  reference columns in the AgD data (e.g., `age_group_mean`).
- **`qbern`**: Bernoulli quantile function (provided by mlumr) for binary
  covariates. Use `qnorm` for continuous covariates.

### Distribution types

```{r, eval = FALSE, purl = FALSE}
# Binary covariate
age = distr(qbern, prob = age_mean)

# Continuous covariate (normal)
bmi = distr(qnorm, mean = bmi_mean, sd = bmi_sd)

# Continuous covariate (gamma)
biomarker = distr(qgamma, shape = 2, rate = 0.5)
```

### Correlation handling

By default, correlations between covariates are estimated from the IPD
using Spearman rank correlation, then adjusted for the Gaussian copula.
For binary and mixed discrete/continuous covariate pairs, this adjustment
is a pragmatic approximation — exact calibration of the latent Gaussian
correlation depends on marginal prevalences, which are not used in the
current transform. For most practical datasets this provides adequate
integration-point distributions, but users with strong prior information
on covariate correlations can supply their own matrix via the `cor`
argument.

```{r, eval = TRUE}
# Default: Spearman correlation from IPD
dat <- add_integration(dat, n_int = 64,
                       age_group = distr(qbern, prob = age_group_mean),
                       sex = distr(qbern, prob = sex_mean))

# Supply your own correlation matrix
my_cor <- matrix(c(1, 0.3, 0.3, 1), 2, 2)
dat <- add_integration(dat, n_int = 64, cor = my_cor,
                       age_group = distr(qbern, prob = age_group_mean),
                       sex = distr(qbern, prob = sex_mean))

# No correlation adjustment
dat <- add_integration(dat, n_int = 64, cor_adjust = "none",
                       age_group = distr(qbern, prob = age_group_mean),
                       sex = distr(qbern, prob = sex_mean))
```

### Inspecting integration points

```{r, eval = TRUE}
# Expand to long format
int_df <- unnest_integration(dat)
head(int_df)
```

Use `check_integration()` to compare the generated integration points against
the requested AgD marginal distributions and correlation structure:

```{r, eval = TRUE, purl = FALSE}
check_integration(
  dat,
  age_group = distr(qbern, prob = age_group_mean),
  sex = distr(qbern, prob = sex_mean)
)
```

This is especially important when there are several covariates, strong
correlations, binary covariates with prevalences near 0 or 1, or a small
number of integration points.

## Notes on STC and naive

The STC and naive methods do **not** require integration points. The STC
uses integration points if available (for better marginalization), but
falls back to AgD covariate means if they are not. You can call `stc()`
and `naive()` immediately after `combine_data()`.

```{r, eval = TRUE}
# These work without add_integration()
dat_no_int <- combine_data(ipd, agd)
naive_result <- naive(dat_no_int)
stc_result <- stc(dat_no_int)
```

---
title: "Fitting ML-UMR Models"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Fitting ML-UMR Models}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, purl = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  message = FALSE,
  warning = FALSE
)
```

## Overview

The `mlumr()` function fits Bayesian multilevel unanchored meta-regression
models using Stan. The default backend is `rstan`; `cmdstanr` can be used
when it is installed and configured. Three outcome families are supported:
binary (binomial), continuous (normal), and count (Poisson). Two model
types are available for each family:

- **SPFA** (Shared Prognostic Factor Assumption): shared regression
  coefficients across treatments
- **Relaxed SPFA**: treatment-specific regression coefficients, allowing
  effect modification

## Model specification

### Binary outcomes (binomial)

The SPFA model assumes that covariates affect outcomes equally for both
treatments. The model is:

$$
\text{logit}(P(Y_i = 1 | \text{Index})) = \mu_A + \mathbf{x}_i^T \boldsymbol{\beta}
$$

$$
\text{logit}(P(Y_j = 1 | \text{Comparator})) = \mu_B + \mathbf{x}_j^T \boldsymbol{\beta}
$$

where $\boldsymbol{\beta}$ is **shared** across both treatments. The AgD
likelihood integrates over the comparator population covariate distribution
using QMC points.

The Relaxed model allows treatment-specific effects:

$$
\text{logit}(P(Y_i = 1 | \text{Index})) = \mu_A + \mathbf{x}_i^T \boldsymbol{\beta}_A
$$

$$
\text{logit}(P(Y_j = 1 | \text{Comparator})) = \mu_B + \mathbf{x}_j^T \boldsymbol{\beta}_B
$$

This captures **effect modification** when covariate effects differ between
treatments. The difference $\boldsymbol{\delta} = \boldsymbol{\beta}_A - \boldsymbol{\beta}_B$
quantifies treatment-by-covariate interaction.

### Continuous outcomes (normal)

For continuous outcomes, the SPFA model is:

$$
Y_i | \text{Index} \sim \text{Normal}(\mu_A + \mathbf{x}_i^T \boldsymbol{\beta}, \sigma^2)
$$

The AgD likelihood uses the mean and standard error of the observed outcome.
The generated quantities include mean differences (`delta_index`,
`delta_comparator`) as the primary treatment effect measure.

An additional `prior_sigma` parameter controls the prior on the residual
standard deviation $\sigma$. The default is a half-normal prior induced by
Stan's `<lower=0>` constraint; half-Student-t and exponential priors are also
available through the prior constructors.

### Count outcomes (Poisson)

For count outcomes with exposure, the SPFA model uses a Poisson likelihood
with log link:

$$
Y_i | \text{Index} \sim \text{Poisson}(\exp(\mu_A + \mathbf{x}_i^T \boldsymbol{\beta}) \cdot E_i)
$$

where $E_i$ is the exposure (person-time). The AgD likelihood uses
log-sum-exp for numerical stability. The generated quantities include
rate ratios (`delta_index`, `delta_comparator`).

## Fitting models

The chunks below all use a small toy `dat` built from the same pipeline as
`vignette("data-preparation")`. Run this setup chunk first so the
subsequent fitting, summary, prediction, and comparison calls have
something to operate on.

```{r, eval = TRUE, purl = FALSE}
library(mlumr)
set.seed(2026)

trial_a_data <- data.frame(
  trt      = "Drug_A",
  response = rbinom(300, 1, 0.55),
  age_cat  = rbinom(300, 1, 0.40),
  sex      = rbinom(300, 1, 0.55)
)
trial_b_data <- data.frame(
  trt           = "Drug_B",
  n_total       = 400,
  n_events      = 160,
  age_cat_mean  = 0.35,
  sex_mean      = 0.50
)

ipd <- set_ipd(trial_a_data, treatment = "trt", outcome = "response",
               covariates = c("age_cat", "sex"))
agd <- set_agd(trial_b_data, treatment = "trt",
               outcome_n = "n_total", outcome_r = "n_events",
               cov_means = c("age_cat_mean", "sex_mean"),
               cov_types = c("binary", "binary"))
dat <- combine_data(ipd, agd)
dat <- add_integration(dat, n_int = 64,
                       age_cat = distr(qbern, prob = age_cat_mean),
                       sex = distr(qbern, prob = sex_mean))
```

```{r, eval = TRUE, purl = FALSE}
# SPFA model (default)
fit_spfa <- mlumr(
  dat, model = "spfa",
  chains = 2, iter = 500, warmup = 250,
  seed = 42, refresh = 0, verbose = FALSE
)

# Relaxed SPFA
fit_relaxed <- mlumr(
  dat, model = "relaxed",
  chains = 2, iter = 500, warmup = 250,
  seed = 43, refresh = 0, verbose = FALSE
)
```

### Controlling the sampler

```{r, eval = FALSE, purl = FALSE}
fit <- mlumr(
  dat,
  model = "spfa",
  chains = 4,               # Number of MCMC chains
  iter = 4000,               # Total iterations per chain
  warmup = 2000,             # Warmup iterations
  seed = 42,                 # For reproducibility
  adapt_delta = 0.99,        # Increase for divergences
  max_treedepth = 15,        # Maximum tree depth
  refresh = 500              # Print progress every 500 iterations
)
```

The backend can be selected per fit. The code below runs the same small
demonstration model with each available engine and records elapsed wall-clock
time. `rstan` uses the model compiled into the installed R package. The first
`cmdstanr` call may include external executable compilation or cache setup, so
the table reports cold and warm `cmdstanr` runs separately. For repeated
analyses, the warm-cache `cmdstanr` row is the more relevant comparison.

```{r, eval = TRUE, purl = FALSE}
run_backend_timed <- function(engine, run, note, seed = 2026, ...) {
  start_time <- proc.time()[["elapsed"]]
  fit <- tryCatch(
    mlumr(
      dat,
      model = "spfa",
      engine = engine,
      chains = 2,
      iter = 300,
      warmup = 150,
      seed = seed,
      refresh = 0,
      verbose = FALSE,
      ...
    ),
    error = function(e) e
  )
  elapsed <- proc.time()[["elapsed"]] - start_time

  if (inherits(fit, "error")) {
    return(data.frame(
      run = run,
      engine = engine,
      status = "failed",
      elapsed_seconds = round(elapsed, 2),
      lor_comparator_mean = NA_real_,
      max_rhat = NA_real_,
      min_ess = NA_real_,
      note = conditionMessage(fit),
      stringsAsFactors = FALSE
    ))
  }

  lor_comp <- marginal_effects(fit, effect = "lor", population = "comparator")
  data.frame(
    run = run,
    engine = engine,
    status = "ran",
    elapsed_seconds = round(elapsed, 2),
    lor_comparator_mean = round(lor_comp$mean, 3),
    max_rhat = round(max(fit$summary$Rhat, na.rm = TRUE), 3),
    min_ess = round(min(fit$summary$n_eff, na.rm = TRUE), 1),
    note = note,
    stringsAsFactors = FALSE
  )
}

cmdstanr_ready <- requireNamespace("cmdstanr", quietly = TRUE) &&
  isTRUE(tryCatch({
    nzchar(cmdstanr::cmdstan_path())
  }, error = function(e) FALSE))

backend_speed <- run_backend_timed(
  engine = "rstan",
  run = "rstan",
  note = "Package-compiled model, already used above"
)

if (cmdstanr_ready) {
  backend_speed <- rbind(
    backend_speed,
    run_backend_timed(
      engine = "cmdstanr",
      run = "cmdstanr_cold",
      note = "First call; may include compile/cache setup"
    ),
    run_backend_timed(
      engine = "cmdstanr",
      run = "cmdstanr_warm",
      note = "Second call; executable/cache already available"
    ),
    run_backend_timed(
      engine = "cmdstanr",
      run = "cmdstanr_warm_parallel",
      note = "Warm cache with two parallel chains",
      parallel_chains = 2
    )
  )
} else {
  backend_speed <- rbind(
    backend_speed,
    data.frame(
      run = "cmdstanr",
      engine = "cmdstanr",
      status = "skipped",
      elapsed_seconds = NA_real_,
      lor_comparator_mean = NA_real_,
      max_rhat = NA_real_,
      min_ess = NA_real_,
      note = "cmdstanr or CmdStan is not available in this R session",
      stringsAsFactors = FALSE
    )
  )
}

backend_speed
```

For production analyses, choose the backend based on your local toolchain,
diagnostics, and workflow. `cmdstanr` is often faster after the executable is
compiled, especially with parallel chains and longer runs, but this is not
guaranteed for tiny demonstration fits. In short examples, compilation, process
startup, and CSV readback can dominate the timing. Posterior estimates should be
similar up to Monte Carlo error when the same model, data, priors, and sampler
settings are used.

### Setting priors

By default, weakly informative `Normal(0, 10)` priors are used for
intercepts and `Normal(0, 2.5)` priors are used for regression coefficients.
Normal, Student-t, and Cauchy priors are available for intercepts and
regression coefficients. For normal-family residual standard deviations,
`prior_sigma` also supports an exponential prior.

```{r, eval = FALSE, purl = FALSE}
fit <- mlumr(
  dat,
  model = "spfa",
  prior_intercept = prior_normal(0, 10),   # Default
  prior_beta = prior_normal(0, 2.5)        # More informative for betas
)
```

Per-covariate beta priors and autoscaling are supported:

```{r, eval = FALSE, purl = FALSE}
fit <- mlumr(
  dat,
  model = "spfa",
  prior_beta = list(
    prior_normal(0, 2.5, autoscale = TRUE),
    prior_normal(0, 1.5, autoscale = TRUE)
  )
)
```

Inspect the priors actually passed to Stan, including autoscaled beta scales:

```{r, eval = FALSE, purl = FALSE}
prior_summary(fit)
```

## Inspecting results

### Print and summary

```{r, eval = TRUE, purl = FALSE}
# Quick overview
print(fit_spfa)

# Detailed summary
summary(fit_spfa)
```

### Marginal treatment effects

The main quantities of interest are population-level treatment effects,
marginalized over the covariate distribution:

```{r, eval = TRUE, purl = FALSE}
# All effects in both populations
marginal_effects(fit_spfa)

# Log odds ratios only
marginal_effects(fit_spfa, effect = "lor")

# In index population only
marginal_effects(fit_spfa, effect = "lor", population = "index")

# Full posterior draws (for custom summaries)
lor_draws <- marginal_effects(fit_spfa, effect = "lor", summary = FALSE)
head(lor_draws)
```

### Absolute predictions

```{r, eval = TRUE, purl = FALSE}
# Predicted probabilities P(Y=1) for each treatment in each population
predict(fit_spfa, population = "both")

# On logit scale
predict(fit_spfa, type = "link")

# Full posterior draws
pred_draws <- predict(fit_spfa, summary = FALSE)
head(pred_draws)
```

### Conditional predictions and effects

`predict()` and `marginal_effects()` report population-averaged quantities.
Use `conditional_predict()` and `conditional_effects()` when the estimand is a
specific covariate profile rather than the index or comparator population.

```{r, eval = TRUE, purl = FALSE}
profiles <- data.frame(
  age_cat = c(0, 1),
  sex = c(0, 1)
)

# Absolute event probabilities at each profile
conditional_predict(fit_spfa, newdata = profiles)

# Conditional link-scale, risk-difference, and risk-ratio effects
conditional_effects(fit_spfa, newdata = profiles)
```

For SPFA binary models, the conditional link-scale effect is constant across
profiles because the shared beta cancels on the fitted link scale. Risk
differences and risk ratios can still vary by profile because they depend on
the absolute baseline probabilities.

## Model comparison

Compare SPFA and Relaxed models using DIC:

```{r, eval = TRUE, purl = FALSE}
compare_models(fit_spfa, fit_relaxed)
```

You can also compute DIC individually:

```{r, eval = TRUE, purl = FALSE}
dic_spfa <- calculate_dic(fit_spfa)
print(dic_spfa)
```

## MCMC diagnostics

The package automatically warns about:

- Divergent transitions (increase `adapt_delta`)
- Maximum treedepth hits (increase `max_treedepth`)
- High Rhat values (> 1.01, run more iterations)
- Low ESS (< 400, run more iterations)

Check diagnostics manually:

```{r, eval = TRUE, purl = FALSE}
# Diagnostics stored in the fit object
fit_spfa$diagnostics$n_divergent
fit_spfa$diagnostics$n_max_treedepth

# Rhat and ESS from the summary table
max(fit_spfa$summary$Rhat, na.rm = TRUE)
min(fit_spfa$summary$n_eff, na.rm = TRUE)
```

For visual diagnostics, use the bayesplot package:

```{r, eval = TRUE, fig.width = 7, fig.height = 4, purl = FALSE}
library(bayesplot)
pars <- c("mu_index", "mu_comparator", "lor_index")
draws_array <- rstan::extract(fit_spfa$stanfit, pars = pars, permuted = FALSE)
mcmc_dens_overlay(draws_array, pars = pars)
mcmc_intervals(as.matrix(fit_spfa$draws[, c("lor_index", "lor_comparator")]))
```

## Troubleshooting

### Divergent transitions

Increase `adapt_delta` closer to 1:

```{r, eval = FALSE, purl = FALSE}
fit <- mlumr(dat, adapt_delta = 0.99)
```

### Slow convergence

Increase iterations and warmup:

```{r, eval = FALSE, purl = FALSE}
fit <- mlumr(dat, iter = 8000, warmup = 4000)
```

### Identifiability in Relaxed model

The Relaxed model has more parameters. With only one AgD row, the
comparator-specific coefficients may be weakly identified. The package
warns about this. Consider:

- Using the SPFA model if effect modification is not expected
- Providing multiple AgD subgroups
- Using more informative priors for betas
- Running `prior_sensitivity()` to assess how strongly relaxed-model
  conclusions depend on the beta prior

```{r, eval = FALSE, purl = FALSE}
prior_sensitivity(fit_relaxed, prior_beta_scales = c(1, 2.5, 5, 10))
```

---
title: "STC and Naive Benchmarks"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{STC and Naive Benchmarks}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, purl = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  message = FALSE,
  warning = FALSE
)
```

## Overview

In addition to ML-UMR, mlumr provides two frequentist methods:

- **STC** (Simulated Treatment Comparison): adjusts for cross-trial
  covariate differences using parametric G-computation
- **Naive**: unadjusted comparison of crude outcome summaries

These serve as important benchmarks alongside the Bayesian ML-UMR approach.

## Naive estimate

For binary outcomes, the naive method compares crude event rates without any
covariate adjustment:

$$
\text{LOR}_{\text{naive}} = \text{logit}(\hat{p}_A) - \text{logit}(\hat{p}_B)
$$

where $\hat{p}_A = \bar{Y}_{\text{IPD}}$ and $\hat{p}_B = r_B / n_B$ from
the AgD. The standard error is computed via the delta method:

$$
\text{SE} = \sqrt{\frac{1}{n_A \hat{p}_A(1-\hat{p}_A)} + \frac{1}{n_B \hat{p}_B(1-\hat{p}_B)}}
$$

### Usage

The chunks below operate on a small toy `dat`. Run this setup once first
so `naive()`, `stc()`, and the comparison block all have data to work
with.

```{r, eval = TRUE, purl = FALSE}
library(mlumr)
set.seed(2026)

trial_a_data <- data.frame(
  trt      = "Drug_A",
  response = rbinom(300, 1, 0.55),
  age_cat  = rbinom(300, 1, 0.40),
  sex      = rbinom(300, 1, 0.55)
)
trial_b_data <- data.frame(
  trt           = "Drug_B",
  n_total       = 400,
  n_events      = 160,
  age_cat_mean  = 0.35,
  sex_mean      = 0.50
)

ipd <- set_ipd(trial_a_data, treatment = "trt", outcome = "response",
               covariates = c("age_cat", "sex"))
agd <- set_agd(trial_b_data, treatment = "trt",
               outcome_n = "n_total", outcome_r = "n_events",
               cov_means = c("age_cat_mean", "sex_mean"),
               cov_types = c("binary", "binary"))
```

```{r, eval = TRUE, purl = FALSE}
# Prepare data (no integration points needed)
dat <- combine_data(ipd, agd)

result <- naive(dat)
print(result)
```

### Interpreting the naive estimate

The naive LOR is **biased** when covariate distributions differ between
the IPD and AgD populations. It provides a useful reference point:

- If the naive and adjusted estimates agree, covariate imbalance has little
  impact.
- If they disagree substantially, adjustment is important and the naive
  estimate should not be used for inference.

### Extreme rates

The function includes continuity-correction-style boundary guards for
extreme proportions (0% or 100% event rates), ensuring finite log odds
ratios even at the boundaries. Event-probability confidence intervals use
Wald standard errors and are bounded to `[0, 1]`.

## Simulated Treatment Comparison (STC)

STC uses parametric G-computation to adjust for covariate differences:

1. **Fit an outcome model** on IPD:
   $\text{logit}(P(Y_i = 1)) = \alpha + \mathbf{x}_i^T \boldsymbol{\beta}$
2. **Predict** counterfactual outcomes for the comparator population
3. **Marginalize**: $\hat{p}_{A|B} = \frac{1}{N} \sum_j \hat{P}(Y = 1 | \mathbf{x}_j)$
4. **Compute LOR**: $\text{LOR} = \text{logit}(\hat{p}_{A|B}) - \text{logit}(\hat{p}_B)$

The standard error uses the delta method, propagating parameter
uncertainty through the logit-of-mean transformation.

### Usage

```{r, eval = TRUE, purl = FALSE}
# Without integration points (uses AgD means)
result_stc <- stc(dat)

# With integration points (better marginalization)
dat <- add_integration(dat, n_int = 64,
                       age_cat = distr(qbern, prob = age_cat_mean),
                       sex = distr(qbern, prob = sex_mean))
result_stc <- stc(dat)

print(result_stc)
```

### With vs without integration points

- **Without integration points**: STC predicts only at the AgD covariate
  **means** (a single point). This can introduce bias for non-linear
  models (Jensen's inequality).
- **With integration points**: STC marginalizes over the full covariate
  distribution, giving a more accurate estimate. We recommend always using
  integration points.

For binary outcomes, event-probability confidence intervals are bounded to
`[0, 1]`. For Poisson outcomes, STC uses a 0.5 continuity correction for the
comparator log rate when the AgD event count is zero; the reported comparator
rate remains the observed rate.

### Accessing the GLM fit

The underlying logistic regression model is stored in the result:

```{r, eval = TRUE, purl = FALSE}
# Coefficients
coef(result_stc$glm_fit)

# Full GLM summary
summary(result_stc$glm_fit)

# Predicted probabilities
fitted(result_stc$glm_fit)
```

## Delta method variance

The delta method SE for STC accounts for uncertainty in:

1. **Outcome model parameters** (via the GLM's variance-covariance matrix)
2. **AgD event rate** (binomial sampling variance)

The gradient of $\text{logit}(\text{mean}(\sigma(\mathbf{X}\boldsymbol{\beta})))$
with respect to $\boldsymbol{\beta}$ is computed analytically:

$$
\frac{\partial}{\partial \boldsymbol{\beta}} \text{logit}\left(\frac{1}{N}\sum_j \sigma(\mathbf{x}_j^T\boldsymbol{\beta})\right) = \frac{1}{\bar{p}(1-\bar{p})} \cdot \frac{1}{N} \sum_j \sigma'(\eta_j) \mathbf{x}_j
$$

where $\sigma(\eta) = 1/(1 + e^{-\eta})$ and $\sigma'(\eta) = \sigma(\eta)(1 - \sigma(\eta))$.

## Continuous and count outcomes

The same `naive()` and `stc()` functions support normal and Poisson outcomes.
For normal outcomes, `naive()` compares the IPD mean against the inverse-
variance weighted AgD mean, while `stc()` fits a Gaussian GLM and
G-computes the index-treatment mean in the comparator population. For Poisson
outcomes, both methods report log rate ratios; zero observed event counts use
a 0.5 continuity correction on the log-rate scale so estimates remain finite.

```{r, eval = TRUE, purl = FALSE}
# Normal-family benchmark
ipd_normal <- set_ipd(
  data.frame(
    trt = "Drug_A",
    score = rnorm(120, mean = 3.0, sd = 1.0),
    age_cat = rbinom(120, 1, 0.40)
  ),
  treatment = "trt",
  outcome = "score",
  covariates = "age_cat",
  family = "normal"
)
agd_normal <- set_agd(
  data.frame(trt = "Drug_B", y_mean = 2.7, se = 0.12, age_cat_mean = 0.35),
  treatment = "trt",
  family = "normal",
  outcome_mean = "y_mean",
  outcome_se = "se",
  cov_means = "age_cat_mean",
  cov_types = "binary"
)
dat_normal <- combine_data(ipd_normal, agd_normal)
naive(dat_normal)
stc(dat_normal)
```

```{r, eval = TRUE, purl = FALSE}
# Poisson-family benchmark
exposure <- runif(120, 0.5, 2.0)
ipd_poisson <- set_ipd(
  data.frame(
    trt = "Drug_A",
    events = rpois(120, exp(0.2) * exposure),
    person_years = exposure,
    age_cat = rbinom(120, 1, 0.40)
  ),
  treatment = "trt",
  outcome = "events",
  covariates = "age_cat",
  family = "poisson",
  exposure = "person_years"
)
agd_poisson <- set_agd(
  data.frame(trt = "Drug_B", n_events = 40, person_years = 180,
             age_cat_mean = 0.35),
  treatment = "trt",
  family = "poisson",
  outcome_r = "n_events",
  outcome_E = "person_years",
  cov_means = "age_cat_mean",
  cov_types = "binary"
)
dat_poisson <- combine_data(ipd_poisson, agd_poisson)
naive(dat_poisson)
stc(dat_poisson)
```

## Comparing all methods

```{r, eval = TRUE, purl = FALSE}
# Fit all three methods
naive_result <- naive(dat)
stc_result <- stc(dat)
mlumr_result <- mlumr(
  dat, model = "spfa",
  chains = 2, iter = 500, warmup = 250,
  seed = 42, refresh = 0, verbose = FALSE
)

# Extract LORs for comparison
le_naive <- naive_result$link_effect
le_stc <- stc_result$link_effect
lor_mlumr <- marginal_effects(mlumr_result, effect = "lor",
                               population = "comparator")

cat("Method comparison (LOR in comparator population):\n")
cat(sprintf("  Naive:  %.3f [%.3f, %.3f]\n",
            naive_result$link_effect, naive_result$ci_lower, naive_result$ci_upper))
cat(sprintf("  STC:    %.3f [%.3f, %.3f]\n",
            stc_result$link_effect, stc_result$ci_lower, stc_result$ci_upper))
cat(sprintf("  ML-UMR: %.3f [%.3f, %.3f]\n",
            lor_mlumr$mean, lor_mlumr$q2.5, lor_mlumr$q97.5))
```

## Confidence level

Both `naive()` and `stc()` accept a `conf_level` parameter:

```{r, eval = TRUE, purl = FALSE}
# 90% confidence intervals
naive_90 <- naive(dat, conf_level = 0.90)
stc_90 <- stc(dat, conf_level = 0.90)
print(naive_90)
print(stc_90)
```

## Key differences from ML-UMR

| Feature | STC | Naive | ML-UMR |
|---------|-----|-------|--------|
| Covariate adjustment | Outcome model | None | Joint model |
| Population weighting | G-computation | None | QMC integration |
| Uncertainty | Delta method | Delta method | Posterior |
| Effect modification | Not captured | N/A | Relaxed model |
| Speed | Instant | Instant | Minutes |
| No. parameters | p+1 | 0 | 2+p (SPFA) or 2+2p (Relaxed) |

STC is faster but makes stronger modeling assumptions. ML-UMR jointly
models both data sources and naturally propagates all sources of
uncertainty through the posterior.

---
title: "Worked Example: Complete Analysis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Worked Example: Complete Analysis}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE, purl = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE,
  message = FALSE,
  warning = FALSE
)
```

## Scenario

We have IPD from a trial of **Drug A** (index treatment, N=500) and
published AgD from a trial of **Drug B** (comparator, N=400). We want
to estimate the treatment effect of Drug A vs Drug B on a binary endpoint.

Two binary covariates (age group and sex) are available in both trials
but their distributions differ:

- Drug A trial: 40% elderly, 55% female
- Drug B trial: 35% elderly, 50% female

## Step 1: Simulate example data

```{r, eval = TRUE, purl = FALSE}
library(mlumr)
set.seed(2026)

# --- IPD for Drug A ---
n_A <- 500
age_A <- rbinom(n_A, 1, 0.40)
sex_A <- rbinom(n_A, 1, 0.55)

# True model: logit(p) = -0.5 + 0.8*age - 0.3*sex
logit_p_A <- -0.5 + 0.8 * age_A - 0.3 * sex_A
y_A <- rbinom(n_A, 1, plogis(logit_p_A))

ipd_df <- data.frame(
  trt = "Drug_A",
  outcome = y_A,
  age_group = age_A,
  sex = sex_A
)

# --- AgD for Drug B ---
# True comparator model: logit(p) = -0.8 + 0.8*age - 0.3*sex
# (same covariate effects, different intercept)
n_B <- 400
age_B <- rbinom(n_B, 1, 0.35)
sex_B <- rbinom(n_B, 1, 0.50)
logit_p_B <- -0.8 + 0.8 * age_B - 0.3 * sex_B
y_B <- rbinom(n_B, 1, plogis(logit_p_B))

agd_df <- data.frame(
  trt = "Drug_B",
  n_total = n_B,
  n_events = sum(y_B),
  age_group_mean = mean(age_B),
  sex_mean = mean(sex_B)
)
```

## Step 2: Prepare data objects

```{r, eval = TRUE, purl = FALSE}
ipd <- set_ipd(ipd_df, treatment = "trt", outcome = "outcome",
               covariates = c("age_group", "sex"))

agd <- set_agd(agd_df, treatment = "trt",
               outcome_n = "n_total", outcome_r = "n_events",
               cov_means = c("age_group_mean", "sex_mean"),
               cov_types = c("binary", "binary"))

dat <- combine_data(ipd, agd)
print(dat)
```

## Step 3: Add integration points

```{r, eval = TRUE, purl = FALSE}
dat <- add_integration(
  dat,
  n_int = 64,
  age_group = distr(qbern, prob = age_group_mean),
  sex = distr(qbern, prob = sex_mean)
)

check_integration(
  dat,
  age_group = distr(qbern, prob = age_group_mean),
  sex = distr(qbern, prob = sex_mean)
)
```

## Step 4: Naive estimate (benchmark)

```{r, eval = TRUE, purl = FALSE}
naive_result <- naive(dat)
print(naive_result)
```

## Step 5: STC estimate

```{r, eval = TRUE, purl = FALSE}
stc_result <- stc(dat)
print(stc_result)
```

## Step 6: ML-UMR SPFA model

```{r, eval = TRUE, purl = FALSE}
fit_spfa <- mlumr(
  dat,
  model = "spfa",
  prior_intercept = prior_normal(0, 10),
  prior_beta = prior_normal(0, 2.5),
  chains = 2,
  iter = 500,
  warmup = 250,
  seed = 42,
  refresh = 0,
  verbose = FALSE
)

summary(fit_spfa)
prior_summary(fit_spfa)
```

## Step 7: ML-UMR Relaxed model

```{r, eval = TRUE, purl = FALSE}
fit_relaxed <- mlumr(
  dat,
  model = "relaxed",
  prior_intercept = prior_normal(0, 10),
  prior_beta = prior_normal(0, 2.5),
  chains = 2,
  iter = 500,
  warmup = 250,
  seed = 43,
  refresh = 0,
  verbose = FALSE
)

summary(fit_relaxed)
prior_summary(fit_relaxed)
```

## Step 8: Model comparison

```{r, eval = TRUE, purl = FALSE}
compare_models(fit_spfa, fit_relaxed)
```

## Step 9: Extract and compare results

```{r, eval = TRUE, purl = FALSE}
# ML-UMR marginal effects
me_spfa <- marginal_effects(fit_spfa, effect = "lor")
me_relaxed <- marginal_effects(fit_relaxed, effect = "lor")

# Build comparison table
results <- data.frame(
  Method = c("Naive", "STC",
             "ML-UMR SPFA (Index)", "ML-UMR SPFA (Comparator)",
             "ML-UMR Relaxed (Index)", "ML-UMR Relaxed (Comparator)"),
  LOR = c(
    naive_result$link_effect,
    stc_result$link_effect,
    me_spfa$mean[me_spfa$population == "Index"],
    me_spfa$mean[me_spfa$population == "Comparator"],
    me_relaxed$mean[me_relaxed$population == "Index"],
    me_relaxed$mean[me_relaxed$population == "Comparator"]
  ),
  SE = c(
    naive_result$se,
    stc_result$se,
    me_spfa$sd[me_spfa$population == "Index"],
    me_spfa$sd[me_spfa$population == "Comparator"],
    me_relaxed$sd[me_relaxed$population == "Index"],
    me_relaxed$sd[me_relaxed$population == "Comparator"]
  )
)
results$CI_lower <- c(
  naive_result$ci_lower,
  stc_result$ci_lower,
  me_spfa$q2.5[me_spfa$population == "Index"],
  me_spfa$q2.5[me_spfa$population == "Comparator"],
  me_relaxed$q2.5[me_relaxed$population == "Index"],
  me_relaxed$q2.5[me_relaxed$population == "Comparator"]
)
results$CI_upper <- c(
  naive_result$ci_upper,
  stc_result$ci_upper,
  me_spfa$q97.5[me_spfa$population == "Index"],
  me_spfa$q97.5[me_spfa$population == "Comparator"],
  me_relaxed$q97.5[me_relaxed$population == "Index"],
  me_relaxed$q97.5[me_relaxed$population == "Comparator"]
)

print(results, digits = 3)
```

## Step 10: Predicted probabilities

```{r, eval = TRUE, purl = FALSE}
# Predicted event probabilities for each treatment in each population
preds <- predict(fit_spfa, population = "both")
print(preds)
```

## Step 11: Conditional effects at covariate profiles

Population-level effects average over the covariate distribution. Conditional
effects evaluate the treatment effect at specific covariate values.

```{r, eval = TRUE, purl = FALSE}
profiles <- data.frame(
  age_group = c(0, 1),
  sex = c(0, 1)
)

conditional_predict(fit_spfa, newdata = profiles)
conditional_effects(fit_spfa, newdata = profiles)
```

## Interpretation

In this simulated example:

- The **true LOR** is approximately `logit(p_A) - logit(p_B)` where
  treatment effects are captured by the intercept difference (0.3 on logit
  scale).
- The **naive estimate** is biased because of covariate imbalance between
  populations.
- **STC** partially corrects for this by adjusting via outcome regression.
- **ML-UMR SPFA** provides population-specific treatment effects in both
  the index and comparator populations, fully accounting for covariate
  differences through QMC integration.
- **ML-UMR Relaxed** additionally allows for effect modification, though
  in this simulated case (same covariate effects), SPFA and Relaxed should
  give similar results.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.transform_integration_points}
\alias{.transform_integration_points}
\title{Transform uniform integration points to covariate scales}
\usage{
.transform_integration_points(u_cor, data, ds, cov_names, n_int, n_cov)
}
\description{
Transform uniform integration points to covariate scales
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/family_config.R
\name{family_config}
\alias{family_config}
\title{Family metadata registry}
\description{
Internal single-source-of-truth for family-specific Stan model names,
AgD weighting, prediction-variable prefixes, and supported links and
effect measures. Every R-side call site that hard-coded a per-family
branch now looks up the relevant field here. This file is deliberately
pure-R and makes no Stan calls; keeping it a data registry makes
rstantools regeneration of \code{R/stanmodels.R} independent.
}
\details{
Fields:
\describe{
\item{\code{stan_prefix}}{Prefix for the Stan model name (the full name
is \verb{<stan_prefix>_\{spfa,relaxed\}}).}
\item{\code{predict_prefix}}{Column prefix for generated-quantity variables
in \code{\link[=predict.mlumr_fit]{predict.mlumr_fit()}} (e.g. \code{"p"}, \code{"y"}, \code{"rate"}).}
\item{\code{link_default}}{The default link when the user passes
\code{link = NULL}.}
\item{\code{links}}{Vector of supported links. Should match the branches
in \code{\link[=check_link]{check_link()}}.}
\item{\code{effect_measures}}{Supported values of the \code{effect} argument in
\code{\link[=marginal_effects]{marginal_effects()}} (excluding \code{"all"}).}
\item{\code{marginal_effect_vars}}{Generated-quantity column names for each
effect measure, per population. Expanded in \code{\link[=marginal_effects]{marginal_effects()}}.}
\item{\code{comp_weight_field}}{Name of the Stan-data field used to
weight the comparator-population marginal predictions
(\code{n_agd} for binomial, \code{E_agd} for poisson, \code{NULL} for normal
= equal weights).}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_ipd_finite_columns}
\alias{.validate_ipd_finite_columns}
\title{Validate finite IPD outcome and exposure values}
\usage{
.validate_ipd_finite_columns(data, outcome, exposure, family)
}
\description{
Validate finite IPD outcome and exposure values
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{stan_prior_fields}
\alias{stan_prior_fields}
\title{Translate a scalar prior spec to the Stan data fields}
\usage{
stan_prior_fields(prior)
}
\arguments{
\item{prior}{A prior list.}
}
\value{
A list with \code{mean}, \code{sd}, \code{dist}, \code{df}.
}
\description{
Stan scalar-prior fields: \verb{prior_*_mean}, \verb{prior_*_sd},
\verb{prior_*_dist} (0 = normal, 1 = student_t, 2 = exponential for sigma
only), \verb{prior_*_df} (used only when dist == 1; positive placeholder
otherwise).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{.validate_mlumr_integer}
\alias{.validate_mlumr_integer}
\title{Validate an integer-like mlumr() argument}
\usage{
.validate_mlumr_integer(x, name, lower)
}
\description{
Validate an integer-like mlumr() argument
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.nonnegative_variance}
\alias{.nonnegative_variance}
\title{Truncate tiny negative variance estimates caused by numerical noise}
\usage{
.nonnegative_variance(x, name = "variance", tol = 1e-10)
}
\description{
Truncate tiny negative variance estimates caused by numerical noise
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{.stc_binomial_comparator_delta}
\alias{.stc_binomial_comparator_delta}
\title{Comparator-population delta-method terms for binomial STC}
\usage{
.stc_binomial_comparator_delta(
  fit,
  newdata,
  weights,
  beta_hat,
  V,
  link_resolved,
  p_A,
  p_B,
  n_B
)
}
\description{
Comparator-population delta-method terms for binomial STC
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.warn_constant_ipd_covariates}
\alias{.warn_constant_ipd_covariates}
\title{Warn when IPD covariates have no empirical variation}
\usage{
.warn_constant_ipd_covariates(data, covariates)
}
\description{
Warn when IPD covariates have no empirical variation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_single_treatment}
\alias{.validate_single_treatment}
\title{Validate a data source contains only one treatment}
\usage{
.validate_single_treatment(data, treatment, label)
}
\description{
Validate a data source contains only one treatment
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_poisson_outcomes}
\alias{.validate_agd_poisson_outcomes}
\title{Validate Poisson AgD outcomes}
\usage{
.validate_agd_poisson_outcomes(r_vals, E_vals)
}
\description{
Validate Poisson AgD outcomes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.bounded_wald_interval}
\alias{.bounded_wald_interval}
\title{Bound a Wald interval to a valid numerical range}
\usage{
.bounded_wald_interval(center, se, z, lower = -Inf, upper = Inf)
}
\description{
Bound a Wald interval to a valid numerical range
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.diagnostic_count}
\alias{.diagnostic_count}
\title{Read a non-negative scalar diagnostic count}
\usage{
.diagnostic_count(x)
}
\description{
Read a non-negative scalar diagnostic count
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.validate_integration_distributions}
\alias{.validate_integration_distributions}
\title{Validate integration distributions against the combined data}
\usage{
.validate_integration_distributions(ds, data)
}
\description{
Validate integration distributions against the combined data
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_effects.R
\name{.conditional_parameters}
\alias{.conditional_parameters}
\title{Extract posterior parameter draws for conditional summaries}
\usage{
.conditional_parameters(object, covariates)
}
\description{
Extract posterior parameter draws for conditional summaries
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{.resolve_mlumr_engine}
\alias{.resolve_mlumr_engine}
\title{Resolve and validate mlumr() backend engine}
\usage{
.resolve_mlumr_engine(engine)
}
\description{
Resolve and validate mlumr() backend engine
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{set_agd}
\alias{set_agd}
\title{Set up aggregate data (AgD)}
\usage{
set_agd(
  data,
  treatment,
  family = c("binomial", "normal", "poisson"),
  outcome_n = NULL,
  outcome_r = NULL,
  outcome_mean = NULL,
  outcome_se = NULL,
  outcome_E = NULL,
  cov_means,
  cov_sds = NULL,
  cov_types = NULL,
  study = NULL
)
}
\arguments{
\item{data}{Data frame containing AgD summary statistics}

\item{treatment}{Column name for treatment variable}

\item{family}{Outcome family: \code{"binomial"}, \code{"normal"}, or \code{"poisson"}}

\item{outcome_n}{Column name for sample size (required for binomial,
optional for normal)}

\item{outcome_r}{Column name for number of events (required for binomial
and poisson)}

\item{outcome_mean}{Column name for mean outcome (required for normal)}

\item{outcome_se}{Column name for standard error of outcome (required for
normal)}

\item{outcome_E}{Column name for total exposure (required for poisson)}

\item{cov_means}{Character vector of column names for covariate means/proportions}

\item{cov_sds}{Character vector of column names for covariate SDs
(\code{NA} for binary covariates)}

\item{cov_types}{Character vector specifying \code{"continuous"} or \code{"binary"} for
each covariate. If \code{NULL}, inferred from presence of SD.}

\item{study}{Column name for study identifier (optional)}
}
\value{
An object of class \code{mlumr_agd}
}
\description{
Prepare AgD from the comparator treatment for an unanchored indirect
comparison.
}
\details{
\strong{Scale assumptions for \code{family = "normal"}.} The AgD likelihood is
\code{y_agd ~ normal(E[exp(eta)], se_agd)} under \code{link = "log"} and
\code{y_agd ~ normal(E[eta], se_agd)} under \code{link = "identity"}. In both
cases \code{outcome_mean} and \code{outcome_se} must be on the \strong{arithmetic
(original, untransformed) scale}. If a publication reports only a
log-scale mean / SD or a geometric mean, back-transform before
calling \code{set_agd()} and propagate uncertainty via the delta method;
passing log-scale or geometric summaries silently misspecifies the
likelihood and biases the posterior.

\strong{Scale assumptions for \code{family = "poisson"}.} \code{outcome_r} is the
total count in each AgD row and \code{outcome_E} is the total
person-time (or other exposure). The Stan likelihood uses
\code{log(E_agd)} as an offset, so rates are modeled on the log scale
regardless of how \code{outcome_r} is tabulated.

\strong{Scale assumptions for \code{family = "binomial"}.} \code{outcome_r} /
\code{outcome_n} are counts of events and trials. The log-odds (or
probit / cloglog under alternative links) are formed from
\code{outcome_r / outcome_n}, so no scale conversion is required.
}
\examples{
\dontrun{
# Binary outcome
agd <- set_agd(
  data = trial_b,
  treatment = "trt",
  outcome_n = "n_total",
  outcome_r = "n_events",
  cov_means = c("age_mean", "sex_prop"),
  cov_sds = c("age_sd", NA),
  cov_types = c("continuous", "binary")
)

# Continuous outcome
agd <- set_agd(
  data = trial_b,
  treatment = "trt",
  family = "normal",
  outcome_mean = "mean_score",
  outcome_se = "se_score",
  outcome_n = "n_total",
  cov_means = c("age_mean", "sex_prop")
)

# Count outcome
agd <- set_agd(
  data = trial_b,
  treatment = "trt",
  family = "poisson",
  outcome_r = "n_events",
  outcome_E = "person_years",
  cov_means = c("age_mean", "sex_prop")
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{set_ipd}
\alias{set_ipd}
\title{Set up individual patient data (IPD)}
\usage{
set_ipd(
  data,
  treatment,
  outcome,
  covariates,
  family = c("binomial", "normal", "poisson"),
  exposure = NULL,
  study = NULL
)
}
\arguments{
\item{data}{Data frame containing IPD}

\item{treatment}{Column name for treatment variable}

\item{outcome}{Column name for outcome variable. For \code{family = "binomial"},
must be binary (0/1). For \code{family = "normal"}, any numeric. For
\code{family = "poisson"}, non-negative integer counts.}

\item{covariates}{Character vector of covariate column names}

\item{family}{Outcome family: \code{"binomial"}, \code{"normal"}, or \code{"poisson"}}

\item{exposure}{Column name for exposure/time-at-risk (required when
\code{family = "poisson"})}

\item{study}{Column name for study identifier (optional)}
}
\value{
An object of class \code{mlumr_ipd}
}
\description{
Prepare IPD from the index treatment for an unanchored indirect comparison.
}
\examples{
\dontrun{
# Binary outcome
ipd <- set_ipd(
  data = trial_a,
  treatment = "trt",
  outcome = "response",
  covariates = c("age", "sex")
)

# Continuous outcome
ipd <- set_ipd(
  data = trial_a,
  treatment = "trt",
  outcome = "score",
  covariates = c("age", "sex"),
  family = "normal"
)

# Count outcome with exposure
ipd <- set_ipd(
  data = trial_a,
  treatment = "trt",
  outcome = "events",
  covariates = c("age", "sex"),
  family = "poisson",
  exposure = "person_years"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{.validate_mlumr_fit_object}
\alias{.validate_mlumr_fit_object}
\title{Validate an mlumr fit object}
\usage{
.validate_mlumr_fit_object(object)
}
\description{
Validate an mlumr fit object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{binomial_link_variance}
\alias{binomial_link_variance}
\title{Delta-method variance for a transformed binomial proportion}
\usage{
binomial_link_variance(p, n, link = c("logit", "probit", "cloglog"))
}
\description{
Delta-method variance for a transformed binomial proportion
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.int_stats}
\alias{.int_stats}
\title{Compute summary statistics for integration points}
\usage{
.int_stats(X_int, cov_names, n_agd)
}
\description{
Compute summary statistics for integration points
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_binary_covariates}
\alias{.validate_agd_binary_covariates}
\title{Validate binary AgD covariate summaries}
\usage{
.validate_agd_binary_covariates(data, cov_means, cov_sds, cov_types)
}
\description{
Validate binary AgD covariate summaries
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.resolve_integration_cor}
\alias{.resolve_integration_cor}
\title{Resolve raw and copula correlation matrices for integration}
\usage{
.resolve_integration_cor(data, cov_names, n_cov, cor, cor_adjust, ds, verbose)
}
\description{
Resolve raw and copula correlation matrices for integration
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.check_required_columns}
\alias{.check_required_columns}
\title{Validate that required columns are present}
\usage{
.check_required_columns(data, required_cols)
}
\description{
Validate that required columns are present
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{is_single_prior}
\alias{is_single_prior}
\title{Is this object a single prior (vs a list of priors)?}
\usage{
is_single_prior(x)
}
\description{
Is this object a single prior (vs a list of priors)?
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.ensure_positive_definite_cor}
\alias{.ensure_positive_definite_cor}
\title{Ensure adjusted integration correlation is positive definite}
\usage{
.ensure_positive_definite_cor(copula_cor, n_cov)
}
\description{
Ensure adjusted integration correlation is positive definite
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{.validate_summary_flag}
\alias{.validate_summary_flag}
\title{Validate a scalar summary flag}
\usage{
.validate_summary_flag(summary)
}
\description{
Validate a scalar summary flag
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{pbern}
\alias{pbern}
\title{Bernoulli CDF}
\usage{
pbern(q, prob, lower.tail = TRUE, log.p = FALSE)
}
\arguments{
\item{q}{Vector of quantiles}

\item{prob}{Success probability}

\item{lower.tail}{Logical}

\item{log.p}{Logical}
}
\value{
Numeric vector
}
\description{
Bernoulli CDF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.validate_computed_integration_cor}
\alias{.validate_computed_integration_cor}
\title{Validate an IPD-derived correlation matrix before copula adjustment}
\usage{
.validate_computed_integration_cor(cor)
}
\description{
Validate an IPD-derived correlation matrix before copula adjustment
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{.validate_mlumr_sampling_args}
\alias{.validate_mlumr_sampling_args}
\title{Validate mlumr() sampler controls before backend dispatch}
\usage{
.validate_mlumr_sampling_args(
  chains,
  iter,
  warmup,
  seed,
  adapt_delta,
  max_treedepth,
  refresh
)
}
\description{
Validate mlumr() sampler controls before backend dispatch
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{.validate_numeric_vector}
\alias{.validate_numeric_vector}
\title{Validate a numeric vector}
\usage{
.validate_numeric_vector(x, name)
}
\description{
Validate a numeric vector
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_effects.R
\name{.conditional_profiles}
\alias{.conditional_profiles}
\title{Build covariate profiles for conditional summaries}
\usage{
.conditional_profiles(object, newdata = NULL)
}
\description{
Build covariate profiles for conditional summaries
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.attach_integration_points}
\alias{.attach_integration_points}
\title{Attach generated integration points to an mlumr_data object}
\usage{
.attach_integration_points(
  data,
  X_int_array,
  n_int,
  cor,
  copula_cor,
  cor_adjust
)
}
\description{
Attach generated integration points to an mlumr_data object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{validate_prior}
\alias{validate_prior}
\title{Validate prior specification}
\usage{
validate_prior(prior, param_name = "parameter")
}
\arguments{
\item{prior}{Prior specification list}

\item{param_name}{Parameter name for error messages}
}
\description{
Validate prior specification
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{.stc_agd_mean_newdata}
\alias{.stc_agd_mean_newdata}
\title{Build comparator-population covariates from AgD means}
\usage{
.stc_agd_mean_newdata(agd, cov_names)
}
\description{
Build comparator-population covariates from AgD means
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{combine_data}
\alias{combine_data}
\title{Combine IPD and AgD for unanchored comparison}
\usage{
combine_data(ipd, agd)
}
\arguments{
\item{ipd}{An \code{mlumr_ipd} object from \code{\link[=set_ipd]{set_ipd()}}}

\item{agd}{An \code{mlumr_agd} object from \code{\link[=set_agd]{set_agd()}}}
}
\value{
An object of class \code{mlumr_data}
}
\description{
Combine IPD and AgD for unanchored comparison
}
\examples{
\dontrun{
dat <- combine_data(ipd, agd)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.validate_diagnostic_choice}
\alias{.validate_diagnostic_choice}
\title{Validate a diagnostics/model-comparison choice}
\usage{
.validate_diagnostic_choice(x, choices, name)
}
\description{
Validate a diagnostics/model-comparison choice
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_effects.R
\name{.conditional_effect_choices}
\alias{.conditional_effect_choices}
\title{Conditional effect choices by family}
\usage{
.conditional_effect_choices(family)
}
\description{
Conditional effect choices by family
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{eval_distr}
\alias{eval_distr}
\title{Evaluate a mlumr_distr object with data context}
\usage{
eval_distr(d, p, data = list())
}
\arguments{
\item{d}{A \code{mlumr_distr} object}

\item{p}{Vector of probabilities}

\item{data}{A named list or data frame to evaluate expressions in}
}
\value{
Numeric vector of quantiles
}
\description{
Evaluate a mlumr_distr object with data context
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\name{.cmdstan_available}
\alias{.cmdstan_available}
\title{Check whether CmdStan is configured for cmdstanr}
\usage{
.cmdstan_available()
}
\description{
Check whether CmdStan is configured for cmdstanr
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_summary.R
\name{prior_summary}
\alias{prior_summary}
\alias{prior_summary.default}
\alias{prior_summary.mlumr_fit}
\title{Summary of priors used by a fitted ML-UMR model}
\usage{
prior_summary(object, ...)

\method{prior_summary}{default}(object, ...)

\method{prior_summary}{mlumr_fit}(object, digits = 3, ...)
}
\arguments{
\item{object}{An \code{mlumr_fit} object.}

\item{...}{Unused.}

\item{digits}{Number of significant digits for numeric values (default 3).}
}
\value{
Invisibly returns a list describing the priors; the side effect is
printing a formatted summary.
}
\description{
Print a human-readable summary of every prior that was used to fit an
\code{\link[=mlumr]{mlumr()}} model, including the effective per-coefficient scales after
autoscaling. Mirrors the spirit of \code{rstanarm::prior_summary()}.
}
\examples{
\dontrun{
fit <- mlumr(dat)
prior_summary(fit)
}
}
\seealso{
\code{\link[=prior_sensitivity]{prior_sensitivity()}} to quantify how much the posterior moves
under alternative \code{prior_beta} scales; \code{\link[=prior_normal]{prior_normal()}},
\code{\link[=prior_student_t]{prior_student_t()}}, \code{\link[=prior_cauchy]{prior_cauchy()}}, \code{\link[=prior_exponential]{prior_exponential()}} for the
prior constructors themselves.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{calculate_dic}
\alias{calculate_dic}
\title{Calculate DIC for model comparison}
\usage{
calculate_dic(object)
}
\arguments{
\item{object}{An \code{mlumr_fit} object}
}
\value{
A list of class \code{mlumr_dic} with components \code{DIC}, \code{pD}, \code{D_bar}
}
\description{
Computes the Deviance Information Criterion using the variance-based
effective-parameters formula from Gelman et al. (2004): \code{pD = 0.5 * Var(D)}.
This is more stable than the plug-in alternative for multimodal posteriors.
}
\details{
DIC is retained for backward compatibility and rough comparison. For
principled Bayesian model comparison, prefer \code{\link[=calculate_loo]{calculate_loo()}} or
\code{\link[=calculate_waic]{calculate_waic()}} (Vehtari, Gelman, Gabry 2017).
}
\examples{
\dontrun{
dic_spfa <- calculate_dic(fit_spfa)
dic_relaxed <- calculate_dic(fit_relaxed)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.sqrt_variance}
\alias{.sqrt_variance}
\title{Square root of a variance estimate with numerical guarding}
\usage{
.sqrt_variance(x, name = "variance", tol = 1e-10)
}
\description{
Square root of a variance estimate with numerical guarding
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.standardize_ipd_data}
\alias{.standardize_ipd_data}
\title{Standardize IPD to mlumr's internal column contract}
\usage{
.standardize_ipd_data(
  data,
  treatment,
  outcome,
  covariates,
  family,
  exposure = NULL,
  study = NULL
)
}
\description{
Standardize IPD to mlumr's internal column contract
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.agd_outcome_summary}
\alias{.agd_outcome_summary}
\title{Summarize AgD outcomes}
\usage{
.agd_outcome_summary(
  data,
  family,
  outcome_n,
  outcome_r,
  outcome_mean,
  outcome_se,
  outcome_E
)
}
\description{
Summarize AgD outcomes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{mlumr_message}
\alias{mlumr_message}
\title{Emit package progress messages when enabled}
\usage{
mlumr_message(..., verbose = TRUE)
}
\description{
Emit package progress messages when enabled
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_non_empty_data}
\alias{.validate_non_empty_data}
\title{Validate that input data contain at least one row}
\usage{
.validate_non_empty_data(data, label)
}
\description{
Validate that input data contain at least one row
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_binomial_outcomes}
\alias{.validate_agd_binomial_outcomes}
\title{Validate binomial AgD outcomes}
\usage{
.validate_agd_binomial_outcomes(r_vals, n_vals)
}
\description{
Validate binomial AgD outcomes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{default_priors}
\alias{default_priors}
\alias{default_prior_intercept}
\alias{default_prior_beta}
\alias{default_prior_sigma}
\title{Default priors used by \code{\link[=mlumr]{mlumr()}}}
\usage{
default_prior_intercept()

default_prior_beta()

default_prior_sigma()
}
\value{
A prior list (see \code{\link[=prior_normal]{prior_normal()}}).
}
\description{
These accessors return the current default priors used by \code{\link[=mlumr]{mlumr()}},
tagged with \verb{$default = TRUE} and the package version. \code{\link[=prior_summary]{prior_summary()}}
prints the version so cross-release reproducibility is diagnosable: if a
later release changes a default, fits produced with an older version
will still carry the correct \verb{$version} tag.
}
\examples{
default_prior_intercept()
default_prior_beta()
default_prior_sigma()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.summarize_draw_vector}
\alias{.summarize_draw_vector}
\title{Summarize a single draw vector into mean / sd / quantiles.}
\usage{
.summarize_draw_vector(x, probs)
}
\arguments{
\item{x}{Numeric vector of posterior draws.}

\item{probs}{Quantile probabilities.}
}
\value{
Named numeric vector: \verb{c(mean, sd, <named quantiles>)}.
}
\description{
Internal helper. Centralizes the (mean, sd, quantile) triplet used by
\code{\link[=predict.mlumr_fit]{predict.mlumr_fit()}}, \code{\link[=marginal_effects]{marginal_effects()}}, \code{\link[=conditional_effects]{conditional_effects()}} and
\code{\link[=conditional_predict]{conditional_predict()}} so that any later change to the canonical posterior
summary only needs to happen in one place.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{.mlumr_fit_backend}
\alias{.mlumr_fit_backend}
\title{Dispatch mlumr() sampling to the selected backend}
\usage{
.mlumr_fit_backend(
  engine,
  model_name,
  stan_data,
  chains,
  iter,
  warmup,
  seed,
  adapt_delta,
  max_treedepth,
  refresh,
  ...
)
}
\description{
Dispatch mlumr() sampling to the selected backend
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{stan_prior_fields_beta}
\alias{stan_prior_fields_beta}
\title{Translate a \code{prior_beta} spec to vector-valued Stan data fields}
\usage{
stan_prior_fields_beta(prior, n_cov, sd_x = NULL, covariate_names = NULL)
}
\arguments{
\item{prior}{A prior list from \code{\link[=prior_normal]{prior_normal()}} / \code{\link[=prior_student_t]{prior_student_t()}} /
\code{\link[=prior_cauchy]{prior_cauchy()}}, OR a list of such priors of length \code{n_cov}.}

\item{n_cov}{Number of covariates.}

\item{sd_x}{Optional numeric vector of covariate SDs (length \code{n_cov}).
Required when any prior has \code{autoscale = TRUE}; ignored otherwise.}

\item{covariate_names}{Optional character vector of covariate names
(length \code{n_cov}). Used only to produce informative warnings when
\code{autoscale = TRUE} meets a zero-SD covariate.}
}
\value{
A list with numeric vectors \code{mean} and \code{sd} (length \code{n_cov})
and scalars \code{dist}, \code{df}, and a logical vector \code{autoscale} recording
which elements were autoscaled (for \code{prior_summary()}).
}
\description{
The Stan models declare \code{prior_beta_mean} and \code{prior_beta_sd} as
\code{vector[n_cov]} so the same data contract supports:
}
\details{
\itemize{
\item (i) a single prior broadcast to all covariates (scalar user input),
\item (ii) per-coefficient priors (a list of prior lists, length \code{n_cov}),
\item (iii) autoscaling: each covariate's scale is divided by \code{sd(x_j)}
so the prior is weakly-informative on the standardized scale.
}

For a list of per-coefficient priors, all elements must use the same
\code{distribution} family and \code{df} (Stan branches on a single dist code).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.ordered_log_lik_columns}
\alias{.ordered_log_lik_columns}
\title{Ordered pointwise log-likelihood columns for one data source}
\usage{
.ordered_log_lik_columns(draws, source)
}
\description{
Ordered pointwise log-likelihood columns for one data source
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.int_cor_stats}
\alias{.int_cor_stats}
\title{Pairwise-correlation diagnostics for integration points}
\usage{
.int_cor_stats(X_orig, X_double, cov_names, n_agd, cor_target = NULL)
}
\description{
Pairwise-correlation diagnostics for integration points
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_effects.R
\name{conditional_predict}
\alias{conditional_predict}
\title{Conditional predictions}
\usage{
conditional_predict(
  object,
  newdata = NULL,
  type = c("response", "link"),
  summary = TRUE,
  probs = c(0.025, 0.5, 0.975)
)
}
\arguments{
\item{object}{An \code{mlumr_fit} object}

\item{newdata}{Data frame of covariate values. If \code{NULL}, uses IPD covariate
means.}

\item{type}{\code{"response"} for probabilities, means, or rates; \code{"link"} for
the fitted linear-predictor scale.}

\item{summary}{Return summary (\code{TRUE}) or full draws (\code{FALSE})}

\item{probs}{Quantiles for summary}
}
\value{
A data frame with predictions for each treatment at each profile
}
\description{
Generate absolute predictions at specific covariate values for both
treatments.
}
\examples{
\dontrun{
conditional_predict(fit)
conditional_predict(fit, newdata = data.frame(age = 60, sex = 1))
}
}
\seealso{
\code{\link[=conditional_effects]{conditional_effects()}} for covariate-conditional treatment
\emph{effects}; \code{\link[=predict.mlumr_fit]{predict.mlumr_fit()}} for population-level predictions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_outcome_args}
\alias{.validate_agd_outcome_args}
\title{Validate AgD outcome column arguments}
\usage{
.validate_agd_outcome_args(
  family,
  outcome_n,
  outcome_r,
  outcome_mean,
  outcome_se,
  outcome_E
)
}
\description{
Validate AgD outcome column arguments
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.validate_model_count}
\alias{.validate_model_count}
\title{Validate that model comparison has at least two candidates}
\usage{
.validate_model_count(models)
}
\description{
Validate that model comparison has at least two candidates
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_distribution_type}
\alias{get_distribution_type}
\title{Get distribution type (continuous, discrete, or binary)}
\usage{
get_distribution_type(..., data = list())
}
\arguments{
\item{...}{distr() objects}

\item{data}{Sample data for evaluation}
}
\value{
Named character vector
}
\description{
Get distribution type (continuous, discrete, or binary)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{prior_student_t}
\alias{prior_student_t}
\title{Specify a Student-t prior}
\usage{
prior_student_t(df = 5, mean = 0, sd = 2.5, autoscale = FALSE)
}
\arguments{
\item{df}{Degrees of freedom (must be positive). Values in \code{3:7} are
recommended for logistic-regression coefficients.}

\item{mean}{Prior location (default 0).}

\item{sd}{Prior scale (default 2.5).}

\item{autoscale}{See \code{\link[=prior_normal]{prior_normal()}}. Default \code{FALSE}.}
}
\value{
A list with components \code{distribution = "student_t"}, \code{df},
\code{mean}, \code{sd}, \code{autoscale}.
}
\description{
Heavier-tailed alternative to \code{\link[=prior_normal]{prior_normal()}}. For logistic-regression
coefficients with unit-scale predictors, the Stan community prior-choice
recommendations suggest Student-t with \code{df} between 3 and 7 as a robust
weakly informative prior (Gelman et al., 2008).
}
\examples{
# Gelman et al. 2008 recommendation for logistic-regression coefficients
prior_student_t(df = 5, mean = 0, sd = 2.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{prior_normal}
\alias{prior_normal}
\title{Specify a normal prior}
\usage{
prior_normal(mean = 0, sd = 10, autoscale = FALSE)
}
\arguments{
\item{mean}{Prior mean (default 0).}

\item{sd}{Prior standard deviation (default 10). The default matches
the historical "very weak" scale; explicit tighter values are
recommended for regression coefficients (see Details).}

\item{autoscale}{If \code{TRUE} and this prior is passed as \code{prior_beta},
the scale is divided by each covariate's empirical SD so the prior
is weakly-informative regardless of predictor scaling. Default
\code{FALSE} to preserve backward-compatible behavior; set to \code{TRUE}
explicitly when passing unstandardized predictors. Ignored for
\code{prior_intercept} and \code{prior_sigma}.}
}
\value{
A list with components \code{distribution}, \code{mean}, \code{sd}, \code{df},
\code{autoscale}.
}
\description{
Construct a normal prior for passing to \code{\link[=mlumr]{mlumr()}} via \code{prior_intercept},
\code{prior_beta}, or \code{prior_sigma}. The resulting list is consumed by the
Stan models.
}
\section{Choosing a scale}{


The Stan community's prior-choice wiki (Vehtari et al., 2025) describes
five broad categories, from least to most informative:

\enumerate{
\item Flat prior — not recommended.
\item Super-vague proper prior, e.g., \code{normal(0, 1e6)} — not
recommended.
\item Weakly informative, \strong{very weak}, e.g., \code{normal(0, 10)}.
\item Generic weakly informative, e.g., \code{normal(0, 1)}.
\item Specific informative, e.g., \code{normal(0.4, 0.2)}.
}

Those scales assume parameters are on roughly unit scale. In ML-UMR
models the natural scales are:

\describe{
\item{Treatment intercepts}{On the linear-predictor (link) scale. For
a binary outcome with logit link, the intercept is a baseline log
odds; \code{normal(0, 10)} spans ±20 log-odds at 95 percent and is
"very weak". It is the default because the data usually constrain
the intercept strongly. Tightening to \code{normal(0, 5)} is reasonable
when the expected event rate is far from the extremes.}
\item{Regression coefficients (\code{beta})}{On the link scale, per unit
of covariate. For logistic regression with predictors on unit
scale, Gelman et al. (2008) and the Stan wiki recommend
\code{student_t(df, 0, 2.5)} with \code{df} in \code{3:7}, or — as a practical
approximation — \code{normal(0, 2.5)}. That is the default used by
\code{\link[=mlumr]{mlumr()}}. Use \code{normal(0, 1)} if you expect small effects (e.g.,
standardized predictors in a normal-outcome model). If predictors
are on very different scales, set \code{autoscale = TRUE} so the scale
is divided by each covariate's SD.}
\item{Residual SD (\code{sigma}, normal family only)}{\code{prior_sigma} is
interpreted as a half-normal via the Stan \verb{<lower=0>} constraint.
The default \code{normal(0, 2.5)} (i.e., \code{half-normal(0, 2.5)}) is
weakly informative for residual SDs on the scale of the outcome.
Scale to the outcome if it is far from unit scale, or use
\code{\link[=prior_exponential]{prior_exponential()}}.}
}

Prior sensitivity is especially important for the relaxed model,
where \code{beta_comparator} is identified only by the AgD likelihood.
Run \code{\link[=prior_sensitivity]{prior_sensitivity()}} to quantify how much conclusions move under
alternative scales; see \code{vignette("mlumr-models")}.
}

\examples{
# Default weakly-very-weak intercept prior
prior_normal(mean = 0, sd = 10)

# Gelman 2008 default for logistic-regression coefficients
prior_normal(mean = 0, sd = 2.5)

# Autoscaled coefficient prior (dividing 2.5 by each covariate's SD)
prior_normal(mean = 0, sd = 2.5, autoscale = TRUE)
}
\references{
Gelman, A., Jakulin, A., Pittau, M. G., & Su, Y.-S. (2008). A weakly
informative default prior distribution for logistic and other
regression models. \emph{Annals of Applied Statistics}, 2(4), 1360–1383.

Vehtari, A. et al. Prior Choice Recommendations (Stan wiki):
\url{https://github.com/stan-dev/stan/wiki/Prior-Choice-Recommendations}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.validate_log_lik_matrix}
\alias{.validate_log_lik_matrix}
\title{Validate a pointwise log-likelihood matrix}
\usage{
.validate_log_lik_matrix(log_lik)
}
\description{
Validate a pointwise log-likelihood matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{.mlumr_prior_metadata}
\alias{.mlumr_prior_metadata}
\title{Store user priors plus the resolved Stan-scale beta prior}
\usage{
.mlumr_prior_metadata(
  data,
  family,
  prior_intercept,
  prior_beta,
  prior_sigma,
  beta_fields,
  sd_x
)
}
\description{
Store user priors plus the resolved Stan-scale beta prior
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_ipd_covariates}
\alias{.validate_ipd_covariates}
\title{Validate IPD covariates}
\usage{
.validate_ipd_covariates(data, covariates)
}
\description{
Validate IPD covariates
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_sensitivity.R
\name{prior_sensitivity}
\alias{prior_sensitivity}
\title{Prior sensitivity analysis for an ML-UMR fit}
\usage{
prior_sensitivity(
  fit,
  prior_beta_scales = c(0.5, 1, 2.5, 5, 10),
  probs = c(0.025, 0.5, 0.975),
  verbose = TRUE,
  ...
)
}
\arguments{
\item{fit}{A fitted \code{mlumr_fit} object to re-fit under alternative priors.}

\item{prior_beta_scales}{Numeric vector of scales for \code{prior_beta}.
Default \code{c(0.5, 1, 2.5, 5, 10)}.}

\item{probs}{Quantiles for summarizing each posterior
(default \code{c(0.025, 0.5, 0.975)}).}

\item{verbose}{Logical; if \code{FALSE}, suppresses progress messages and final
printed summary table.}

\item{...}{Additional arguments forwarded to \code{\link[=mlumr]{mlumr()}} on each refit
(e.g. \code{chains}, \code{iter}, \code{refresh}). Sampling defaults otherwise inherit
from the original fit.}
}
\value{
A data frame (tibble-style) with one row per
(scale, population, quantile) combination and columns \code{scale},
\code{parameter}, \code{mean}, \code{sd}, and the requested quantiles. Side effect:
prints a summary table at the end.
}
\description{
Refit an \code{\link[=mlumr]{mlumr()}} model across a grid of \code{prior_beta} scales (keeping the
family, mean, and df fixed) and summarize how the posterior for the
marginal treatment effects (\code{delta_index}, \code{delta_comparator}) moves. This
is the workflow recommended by Vehtari et al.'s prior-choice wiki for
judging how much of the posterior is driven by the data versus the prior.
}
\details{
Only the scale of the \code{prior_beta} family is varied; its distribution
(normal / student_t) and mean are preserved so comparisons are apples to
apples. \code{prior_intercept} and \code{prior_sigma} are carried through
unchanged from the original fit. Each value in \code{prior_beta_scales} is
used as the \strong{absolute} scale for every coefficient at that refit --- if
the original fit used per-coefficient priors, all coefficients are set
to the same scale (the sweep is deliberately homogeneous so the grid
reflects a single level of prior informativeness per refit, not a
rescaling of existing relative differences). If the original
\code{prior_beta} used an exponential family, it is swapped for a
\code{prior_normal(0, scale)} at each grid point since exponential has no
scale parameter to vary.
}
\examples{
\dontrun{
sens <- prior_sensitivity(fit_spfa, prior_beta_scales = c(1, 2.5, 5))
}
}
\seealso{
\code{\link[=prior_summary]{prior_summary()}} for a one-shot description of the priors on
a fit; \code{\link[=marginal_effects]{marginal_effects()}} for the posterior summary quantities this
sweep tracks.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_summary.R
\name{.validate_prior_summary_digits}
\alias{.validate_prior_summary_digits}
\title{Validate prior_summary digits}
\usage{
.validate_prior_summary_digits(digits)
}
\description{
Validate prior_summary digits
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{mlumr}
\alias{mlumr}
\title{Fit ML-UMR Model}
\usage{
mlumr(
  data,
  model = c("spfa", "relaxed"),
  link = NULL,
  prior_intercept = default_prior_intercept(),
  prior_beta = default_prior_beta(),
  prior_sigma = default_prior_sigma(),
  chains = 4,
  iter = 2000,
  warmup = 1000,
  seed = NULL,
  adapt_delta = 0.95,
  max_treedepth = 15,
  refresh = 200,
  engine = NULL,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{data}{An \code{mlumr_data} object with integration points (from
\code{\link[=add_integration]{add_integration()}})}

\item{model}{Model type: \code{"spfa"} (shared prognostic factor assumption) or
\code{"relaxed"} (treatment-specific coefficients). Default \code{"spfa"}.}

\item{link}{Link function. For binomial: \code{"logit"} (default), \code{"probit"},
or \code{"cloglog"}. For normal: \code{"identity"} (default) or \code{"log"}. For
poisson: \code{"log"} (default, only option). If \code{NULL}, uses the canonical
default for the family.}

\item{prior_intercept}{Prior for treatment intercepts. Default from
\code{\link[=default_prior_intercept]{default_prior_intercept()}} (\code{prior_normal(0, 10)}, weakly informative
on the linear-predictor scale). See \code{\link[=prior_normal]{prior_normal()}} for guidance.}

\item{prior_beta}{Prior for regression coefficients. May be a single
prior broadcast to all covariates, or a \code{list} of priors of length
\code{n_cov} for per-coefficient specification. All per-coefficient priors
must share the same family and (for Student-t) df. Default from
\code{\link[=default_prior_beta]{default_prior_beta()}} (\code{prior_normal(0, 2.5)}; Gelman et al. 2008;
Stan community prior-choice wiki). Set \code{autoscale = TRUE} on the
prior to divide the scale by each covariate's empirical SD — useful
when predictors are on very different scales.}

\item{prior_sigma}{Prior for residual SD (normal family only). Default
from \code{\link[=default_prior_sigma]{default_prior_sigma()}} (\code{prior_normal(0, 2.5)}, half-normal via
the Stan \verb{<lower=0>} constraint). \code{\link[=prior_exponential]{prior_exponential()}} is also
supported for sigma.}

\item{chains}{Number of MCMC chains (default 4)}

\item{iter}{Total iterations per chain (default 2000)}

\item{warmup}{Number of warmup iterations (default 1000)}

\item{seed}{Random seed for reproducibility}

\item{adapt_delta}{Target acceptance rate (default 0.95)}

\item{max_treedepth}{Maximum tree depth for NUTS (default 15)}

\item{refresh}{How often to print progress (0 = silent, default 200)}

\item{engine}{Stan backend: \code{"rstan"} (default) or \code{"cmdstanr"}. If \code{NULL},
uses the engine set by \code{\link[=mlumr_engine]{mlumr_engine()}}. See \code{\link[=mlumr_engine]{mlumr_engine()}} for setup.}

\item{verbose}{Logical; if \code{FALSE}, suppresses mlumr progress messages.
Stan sampler progress is still controlled by \code{refresh}.}

\item{...}{Additional arguments passed to the Stan sampling function
(\code{\link[rstan:sampling]{rstan::sampling()}} or cmdstanr's \verb{$sample()} method)}
}
\value{
An object of class \code{mlumr_fit}
}
\description{
Fit a Bayesian multilevel unanchored meta-regression model using individual
patient data (IPD) and aggregate data (AgD). Supports binary, continuous,
and count outcomes.
}
\details{
The model assumes that all AgD rows come from the same comparator treatment
and that, conditional on covariates, there is no between-study heterogeneity.
If AgD rows come from multiple studies with different designs or unmeasured
confounders, this assumption may not hold. No random effects for study-level
heterogeneity are included.

\strong{AgD scale assumptions (family = \code{"normal"}).} The AgD likelihood is
\code{y_agd ~ normal(E[exp(eta)], se_agd)} under \code{link = "log"} and
\code{y_agd ~ normal(E[eta], se_agd)} under \code{link = "identity"}. In both
cases \code{set_agd()} expects \code{outcome_mean} and \code{outcome_se} on the
\strong{arithmetic (original, untransformed) scale}, not log-scale or
geometric. Passing log-scale summaries silently misspecifies the
likelihood. See \code{\link[=set_agd]{set_agd()}} for details.

\strong{Comparator-population weighting is family-dependent.} Integrated
marginal predictions in the comparator population (\verb{*_comparator}
generated quantities) are weighted by:
\itemize{
\item \strong{binomial}: \code{n_agd[k]} (AgD sample size), so larger
AgD rows contribute more to the marginal mean.
\item \strong{normal}: equal weights across AgD rows (\verb{/ n_agd_rows}),
reflecting the normal likelihood's treatment of each row as a
single summary statistic.
\item \strong{poisson}: \code{E_agd[k]} (AgD exposure), matching the
rate-based likelihood.
}
Each weighting is natural for the corresponding likelihood; users
comparing marginal effects across families should be aware they are
not identically weighted.

\strong{Weakly-identified coefficients in the relaxed model} —
\code{beta_comparator} is identified only through AgD, so the relaxed
model needs informative priors (or many AgD rows) to estimate
effect modification reliably. \code{\link[=prior_sensitivity]{prior_sensitivity()}} is the
recommended diagnostic.
}
\examples{
\dontrun{
# Binary SPFA model
fit_spfa <- mlumr(dat, model = "spfa")

# Relaxed SPFA (allows effect modification)
fit_relaxed <- mlumr(dat, model = "relaxed")
}
}
\seealso{
\code{\link[=prior_sensitivity]{prior_sensitivity()}} for sensitivity of the posterior
to \code{prior_beta}; \code{\link[=set_agd]{set_agd()}} for AgD scale requirements;
\code{\link[=prior_summary]{prior_summary()}} for introspection of the priors actually used.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{.stc_formula}
\alias{.stc_formula}
\title{Build the STC GLM formula without assuming syntactic covariate names}
\usage{
.stc_formula(cov_names, family)
}
\description{
Build the STC GLM formula without assuming syntactic covariate names
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{.validate_predict_choice}
\alias{.validate_predict_choice}
\title{Validate an exact scalar prediction argument}
\usage{
.validate_predict_choice(x, choices, name)
}
\description{
Validate an exact scalar prediction argument
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{.mlumr_build_stan_data}
\alias{.mlumr_build_stan_data}
\title{Build the Stan data list for mlumr()}
\usage{
.mlumr_build_stan_data(
  data,
  family,
  link_info,
  prior_intercept,
  prior_beta,
  prior_sigma
)
}
\description{
Build the Stan data list for mlumr()
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/naive.R
\name{naive}
\alias{naive}
\title{Naive unadjusted indirect comparison}
\usage{
naive(data, link = NULL, conf_level = 0.95)
}
\arguments{
\item{data}{An \code{mlumr_data} object from \code{\link[=combine_data]{combine_data()}}}

\item{link}{Link function. For binomial: \code{"logit"} (default), \code{"probit"},
or \code{"cloglog"}. For normal/poisson: ignored (identity/log always used).
If \code{NULL}, uses the canonical default.}

\item{conf_level}{Confidence level for the interval (default 0.95)}
}
\value{
An object of class \code{mlumr_naive}
}
\description{
Compute an unadjusted (naive) indirect treatment comparison by comparing
crude outcomes from the IPD and AgD without any covariate adjustment.
Returns treatment effect on the link scale plus absolute predictions
(event probabilities, risk difference, risk ratio) for both populations.
}
\details{
For binomial outcomes, event-probability intervals use Wald standard errors
and are bounded to \verb{[0, 1]}. For Poisson outcomes, the log-rate contrast
uses a 0.5 continuity correction when an observed event count is zero.
}
\examples{
\dontrun{
result <- naive(dat)
print(result)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.strip_agd_cov_suffix}
\alias{.strip_agd_cov_suffix}
\title{Strip AgD covariate suffixes}
\usage{
.strip_agd_cov_suffix(cov_means)
}
\description{
Strip AgD covariate suffixes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_reserved_internal_names}
\alias{.validate_reserved_internal_names}
\title{Reject user names that would overwrite standardized internal columns}
\usage{
.validate_reserved_internal_names(user_names, reserved, label)
}
\description{
Reject user names that would overwrite standardized internal columns
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cor_adjust_spearman}
\alias{cor_adjust_spearman}
\title{Convert Spearman correlations to Gaussian copula correlations}
\usage{
cor_adjust_spearman(X, types)
}
\arguments{
\item{X}{Correlation matrix (Spearman)}

\item{types}{Character vector of distribution types}
}
\value{
Adjusted correlation matrix for Gaussian copula
}
\description{
Applies theoretical relationships between Spearman's rho and the
Gaussian copula parameter (Kurowicka & Cooke, 2006; Lebrun & Dutfoy, 2009):
\itemize{
\item Continuous-continuous: rho_copula = 2 * sin(pi * rho_S / 6) (exact)
\item Binary-binary: rho_copula = sin(pi * rho_S / 2) (heuristic; the exact
relationship depends on marginal prevalences, not accounted for here)
\item Continuous-binary: rho_copula = sqrt(2) * sin(pi * rho_S / (2*sqrt(3)))
(heuristic)
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{prior_exponential}
\alias{prior_exponential}
\title{Specify an exponential prior}
\usage{
prior_exponential(rate = 1)
}
\arguments{
\item{rate}{Rate parameter (default 1). Larger \code{rate} = tighter prior
concentrated near zero.}
}
\value{
A list with components \code{distribution = "exponential"}, \code{rate},
and placeholders (\code{mean = 0}, \code{sd = 1 / rate}) so the same
Stan-field translation works.
}
\description{
Exponential prior on a positive scalar. Currently supported for
\code{\link[=mlumr]{prior_sigma}} (normal-family residual SD) only; rejected for
unconstrained intercepts and regression coefficients.
\code{prior_exponential(rate = 1)} has mean 1 and is a reasonable
weakly-informative choice when the outcome is standardized.
}
\examples{
prior_exponential(rate = 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cor_adjust_pearson}
\alias{cor_adjust_pearson}
\title{Convert Pearson correlations to Gaussian copula correlations}
\usage{
cor_adjust_pearson(X, types)
}
\arguments{
\item{X}{Correlation matrix (Pearson)}

\item{types}{Character vector of distribution types}
}
\value{
Adjusted correlation matrix for Gaussian copula
}
\description{
For continuous-continuous pairs, Pearson rho equals the Gaussian copula
parameter only under normality of both margins. For non-normal continuous
covariates, this no-adjustment assumption introduces approximation error.
For binary and mixed pairs:
\itemize{
\item Binary-binary: rho_copula = sin(pi * rho_P / 2)
\item Continuous-binary: rho_copula = sqrt(pi/2) * rho_P
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_summary.R
\name{.resolved_prior_family_label}
\alias{.resolved_prior_family_label}
\title{Label a resolved Stan prior family code}
\usage{
.resolved_prior_family_label(dist)
}
\description{
Label a resolved Stan prior family code
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr-package.R
\docType{package}
\name{mlumr-package}
\alias{mlumr-package}
\title{mlumr: Multilevel Unanchored Meta-Regression for Indirect Treatment
Comparisons}
\description{
Bayesian multilevel unanchored meta-regression (ML-UMR) for population-
adjusted indirect comparisons between a single-arm individual patient
data (IPD) study on the index treatment and aggregate data (AgD) on a
comparator. Two model variants are provided: a shared-prognostic-factor
(SPFA) model and a relaxed-SPFA model that allows treatment-specific
covariate coefficients. Supports binary (binomial), continuous (normal),
and count (Poisson) outcomes, each with appropriate link functions.
Frequentist outcome regression (Simulated Treatment Comparison) and a
naive unadjusted benchmark are included for side-by-side comparison.
}
\section{Typical workflow}{

\enumerate{
\item \code{\link[=set_ipd]{set_ipd()}} — declare the individual patient data and its
covariates.
\item \code{\link[=set_agd]{set_agd()}} — declare the aggregate-data arm.
\item \code{\link[=combine_data]{combine_data()}} — join the two into an \code{mlumr_data} object.
\item \code{\link[=add_integration]{add_integration()}} — build the Gaussian-copula QMC
integration points used to marginalize over the AgD covariate
distribution.
\item \code{\link[=mlumr]{mlumr()}} — fit the Bayesian ML-UMR model (SPFA or relaxed).
\item \code{\link[=prior_summary]{prior_summary()}}, \code{\link[=check_integration]{check_integration()}} — sanity-check the
model before inferring from it.
\item \link[=predict.mlumr_fit]{predict()}, \code{\link[=marginal_effects]{marginal_effects()}},
\code{\link[=conditional_effects]{conditional_effects()}}, \code{\link[=conditional_predict]{conditional_predict()}} — extract
population-level and profile-specific quantities.
\item \code{\link[=calculate_dic]{calculate_dic()}}, \code{\link[=calculate_loo]{calculate_loo()}}, \code{\link[=calculate_waic]{calculate_waic()}},
\code{\link[=compare_models]{compare_models()}} — compare SPFA vs relaxed or competing
specifications.
\item \code{\link[=prior_sensitivity]{prior_sensitivity()}} — verify the posterior is data-driven
rather than prior-driven.
}
}

\section{Priors}{

The prior constructors \code{\link[=prior_normal]{prior_normal()}}, \code{\link[=prior_student_t]{prior_student_t()}},
\code{\link[=prior_cauchy]{prior_cauchy()}}, and \code{\link[=prior_exponential]{prior_exponential()}} all plug into \code{\link[=mlumr]{mlumr()}} via
\code{prior_intercept}, \code{prior_beta}, and (normal family only) \code{prior_sigma}.
Defaults follow the Stan community's prior-choice recommendations
(Vehtari et al., 2025).
}

\section{Alternative methods}{

\code{\link[=stc]{stc()}} runs frequentist parametric G-computation (Simulated Treatment
Comparison). \code{\link[=naive]{naive()}} computes an unadjusted crude-rate benchmark. Both
consume the same \code{mlumr_data} object as \code{\link[=mlumr]{mlumr()}} so their estimands can
be compared directly.
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/choxos/mlumr}
  \item Report bugs at \url{https://github.com/choxos/mlumr/issues}
}

}
\author{
\strong{Maintainer}: Ahmad Sofi-Mahmudi \email{a.sofimahmudi@gmail.com} (\href{https://orcid.org/0000-0001-6829-0823}{ORCID})

Authors:
\itemize{
  \item Ahmad Sofi-Mahmudi \email{a.sofimahmudi@gmail.com} (\href{https://orcid.org/0000-0001-6829-0823}{ORCID})
  \item Conor Chandler (\href{https://orcid.org/0000-0002-1365-9002}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{.validate_positive_numeric}
\alias{.validate_positive_numeric}
\title{Validate positive finite numeric input}
\usage{
.validate_positive_numeric(x, name)
}
\description{
Validate positive finite numeric input
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\name{get_engine}
\alias{get_engine}
\title{Get the current Stan engine (internal)}
\usage{
get_engine()
}
\description{
Get the current Stan engine (internal)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{.require_draw_columns}
\alias{.require_draw_columns}
\title{Require posterior draw columns}
\usage{
.require_draw_columns(draws, columns, context)
}
\description{
Require posterior draw columns
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{link_fun}
\alias{link_fun}
\title{Link function (response scale -> linear predictor)}
\usage{
link_fun(x, link = c("identity", "log", "logit", "probit", "cloglog"))
}
\arguments{
\item{x}{Numeric vector on response scale}

\item{link}{Character: link function name}
}
\value{
Numeric vector on linear predictor scale
}
\description{
Link function (response scale -> linear predictor)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.validate_integration_cor}
\alias{.validate_integration_cor}
\title{Validate a user-supplied integration correlation matrix}
\usage{
.validate_integration_cor(cor, n_cov, cov_names = NULL)
}
\description{
Validate a user-supplied integration correlation matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.generate_copula_uniforms}
\alias{.generate_copula_uniforms}
\title{Generate correlated uniform quasi-Monte Carlo points}
\usage{
.generate_copula_uniforms(n_int, n_cov, copula_cor)
}
\description{
Generate correlated uniform quasi-Monte Carlo points
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.summarize_draw_matrix}
\alias{.summarize_draw_matrix}
\title{Summarize a draws matrix column-wise into a tidy data frame.}
\usage{
.summarize_draw_matrix(draws, probs)
}
\arguments{
\item{draws}{A numeric matrix or data frame of posterior draws (one column
per quantity, one row per draw).}

\item{probs}{Quantile probabilities.}
}
\value{
Data frame with columns \code{mean}, \code{sd}, and one \code{qNN} column per
element of \code{probs}.
}
\description{
Applies \code{.summarize_draw_vector()} across columns of \code{draws} and renames
the quantile columns to \code{qNN} form (e.g., \code{q2.5}, \code{q50}, \code{q97.5}).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{.stc_comparator_data}
\alias{.stc_comparator_data}
\title{Build comparator-population covariates for STC prediction}
\usage{
.stc_comparator_data(data, cov_names, family)
}
\description{
Build comparator-population covariates for STC prediction
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/priors.R
\name{prior_cauchy}
\alias{prior_cauchy}
\title{Specify a Cauchy prior}
\usage{
prior_cauchy(mean = 0, sd = 2.5, autoscale = FALSE)
}
\arguments{
\item{mean}{Prior location (default 0).}

\item{sd}{Prior scale (default 2.5).}

\item{autoscale}{See \code{\link[=prior_normal]{prior_normal()}}. Default \code{FALSE}.}
}
\value{
A list with components \code{distribution = "student_t"}, \code{df = 1},
\code{mean}, \code{sd}, \code{autoscale}.
}
\description{
Cauchy is Student-t with \code{df = 1}; this constructor is a convenience
wrapper around \code{\link[=prior_student_t]{prior_student_t()}}. It has very heavy tails and should
be used with care — modern recommendations generally prefer
\verb{prior_student_t(df in 3:7, ...)} over Cauchy for regression
coefficients to keep sampling well-behaved (see Piironen & Vehtari on
the horseshoe; Ghosh et al. 2015).
}
\examples{
prior_cauchy(mean = 0, sd = 2.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{.stc_glm_parameters}
\alias{.stc_glm_parameters}
\title{Validate fitted STC GLM parameters before delta-method calculations}
\usage{
.stc_glm_parameters(fit)
}
\description{
Validate fitted STC GLM parameters before delta-method calculations
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{bound_probability}
\alias{bound_probability}
\title{Bound probabilities away from 0 and 1}
\usage{
bound_probability(p, n, min_count = 0.5)
}
\description{
Bound probabilities away from 0 and 1
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{compare_models}
\alias{compare_models}
\title{Compare fitted ML-UMR models}
\usage{
compare_models(..., criterion = c("dic", "loo", "waic"))
}
\arguments{
\item{...}{Two or more \code{mlumr_fit} objects. For DIC, \code{mlumr_dic}
objects are also accepted.}

\item{criterion}{One of \code{"dic"} (default), \code{"loo"}, or \code{"waic"}.
LOO and WAIC require the optional \code{loo} package.}
}
\value{
For \code{"loo"} / \code{"waic"}: a \code{compare.loo} table from
\code{\link[loo:loo_compare]{loo::loo_compare()}}. For \code{"dic"}: a data frame (invisibly) with
columns \code{Model}, \code{DIC}, \code{pD}, \code{Delta_DIC}.
}
\description{
Compare two or more \code{mlumr_fit} objects by DIC (default), LOO, or WAIC.
For LOO/WAIC, \code{\link[loo:loo_compare]{loo::loo_compare()}} is used under the hood; the output
is the standard \code{loo_compare} table. For DIC the return is a data
frame ordered by DIC.
}
\details{
DIC is the default for backward compatibility and because it has no
additional package dependencies. For principled Bayesian model
comparison, LOO (Vehtari, Gelman, Gabry 2017) is preferred and
requires the optional \code{loo} package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backend_cmdstanr.R
\name{fit_cmdstanr}
\alias{fit_cmdstanr}
\title{Fit a Stan model using cmdstanr}
\usage{
fit_cmdstanr(
  model_name,
  stan_data,
  chains,
  iter,
  warmup,
  seed,
  adapt_delta,
  max_treedepth,
  refresh,
  ...
)
}
\description{
Fit a Stan model using cmdstanr
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{predict.mlumr_fit}
\alias{predict.mlumr_fit}
\title{Predictions from ML-UMR model}
\usage{
\method{predict}{mlumr_fit}(
  object,
  population = c("both", "index", "comparator"),
  type = c("response", "link"),
  summary = TRUE,
  probs = c(0.025, 0.5, 0.975),
  ...
)
}
\arguments{
\item{object}{An \code{mlumr_fit} object}

\item{population}{Which population: \code{"both"}, \code{"index"}, or \code{"comparator"}}

\item{type}{Prediction type: \code{"response"} or \code{"link"}. For \code{"response"}:
probabilities (binomial), means (normal), or rates (poisson). For
\code{"link"}: mean linear predictor on the fitted link scale (logit, probit,
cloglog, log, or identity). The link-scale values are computed directly
from parameter draws as \code{E[eta]}, not as \code{link(E[g^{-1}(eta)])}, to
avoid Jensen's inequality bias.}

\item{summary}{Return summary statistics (\code{TRUE}) or full posterior draws (\code{FALSE})}

\item{probs}{Quantiles for summary (default \code{c(0.025, 0.5, 0.975)})}

\item{...}{Additional arguments (unused)}
}
\value{
A data frame with predictions. When \code{type = "link"}, values are
mean linear predictors computed directly from parameter draws (avoiding
Jensen's inequality bias).
}
\description{
Generate population-average absolute-outcome predictions in the index and
comparator populations.
}
\details{
\strong{Marginalization on non-identity links.} For \code{type = "response"} the
reported values are \code{E[g^{-1}(eta)]} — the posterior expectation of the
inverse-link-transformed linear predictor — \emph{not} \code{g^{-1}(E[eta])}. The
two differ whenever \code{g} is non-linear (logit, probit, cloglog, log) by
Jensen's inequality. In the index population the expectation is taken
over IPD individuals; in the comparator population it is taken over the
integration points constructed by \code{\link[=add_integration]{add_integration()}} from the AgD
moments. This is the correct population-average prediction for an
individual randomly drawn from that population, and it matches what the
Stan \verb{generated quantities} block computes. For the link scale
(\code{type = "link"}) the reported value is \code{E[eta]}, a linear functional,
and the two interpretations coincide.
}
\seealso{
\code{\link[=marginal_effects]{marginal_effects()}} for treatment-effect summaries;
\code{\link[=conditional_predict]{conditional_predict()}} and \code{\link[=conditional_effects]{conditional_effects()}} for predictions
at specific covariate profiles.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.agd_cov_info}
\alias{.agd_cov_info}
\title{Build AgD covariate metadata}
\usage{
.agd_cov_info(cov_names, cov_sds, cov_types)
}
\description{
Build AgD covariate metadata
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_summary.R
\name{.validate_resolved_beta_prior}
\alias{.validate_resolved_beta_prior}
\title{Validate resolved beta-prior metadata stored on a fit}
\usage{
.validate_resolved_beta_prior(br)
}
\description{
Validate resolved beta-prior metadata stored on a fit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.log_lik_column_index}
\alias{.log_lik_column_index}
\title{Extract integer indexes from Stan vector column names}
\usage{
.log_lik_column_index(cols, source)
}
\description{
Extract integer indexes from Stan vector column names
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{eval_distr_arg}
\alias{eval_distr_arg}
\title{Evaluate a single mlumr_distr argument expression}
\usage{
eval_distr_arg(expr, data)
}
\arguments{
\item{expr}{An unevaluated expression}

\item{data}{Data context}
}
\value{
Evaluated value
}
\description{
Evaluate a single mlumr_distr argument expression
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{qbern}
\alias{qbern}
\title{Bernoulli quantile function}
\usage{
qbern(p, prob, lower.tail = TRUE, log.p = FALSE)
}
\arguments{
\item{p}{Vector of probabilities}

\item{prob}{Success probability}

\item{lower.tail}{Logical; if TRUE, probabilities are P(X <= x)}

\item{log.p}{Logical; if TRUE, probabilities are given as log(p)}
}
\value{
Integer vector of 0s and 1s
}
\description{
Bernoulli quantile function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{.validate_effect_choice}
\alias{.validate_effect_choice}
\title{Validate a scalar marginal-effect choice}
\usage{
.validate_effect_choice(effect)
}
\description{
Validate a scalar marginal-effect choice
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\name{.validate_engine_name}
\alias{.validate_engine_name}
\title{Validate a Stan engine name}
\usage{
.validate_engine_name(engine)
}
\description{
Validate a Stan engine name
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backend_cmdstanr.R
\name{.cmdstanr_cache_root}
\alias{.cmdstanr_cache_root}
\title{Resolve a writable cmdstanr cache root}
\usage{
.cmdstanr_cache_root()
}
\description{
Resolve a writable cmdstanr cache root
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.mlumr_model_label}
\alias{.mlumr_model_label}
\title{Human-readable model label for an mlumr fit or DIC object}
\usage{
.mlumr_model_label(object)
}
\description{
Human-readable model label for an mlumr fit or DIC object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.ipd_required_columns}
\alias{.ipd_required_columns}
\title{Required source columns for IPD setup}
\usage{
.ipd_required_columns(
  treatment,
  outcome,
  covariates,
  exposure = NULL,
  study = NULL
)
}
\description{
Required source columns for IPD setup
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{check_link}
\alias{check_link}
\title{Validate and resolve link function for a given family}
\usage{
check_link(family, link = NULL)
}
\arguments{
\item{family}{Character: canonical (\code{"binomial"}, \code{"normal"}, \code{"poisson"})
or alias (\code{"binary"}, \code{"count"}, \code{"rate"}, \code{"continuous"}).}

\item{link}{Character or \code{NULL}. If \code{NULL}, uses default for family.}
}
\value{
List with components:
\describe{
\item{family}{Canonical family name (e.g. \code{"binomial"})}
\item{link}{Resolved link name (e.g. \code{"probit"})}
\item{code}{Integer code for Stan data block}
}
}
\description{
Checks that \code{link} is valid for \code{family} and returns the resolved link name
plus an integer code for Stan. Accepts canonical family names (\code{"binomial"},
\code{"normal"}, \code{"poisson"}) and data-type aliases (\code{"binary"}, \code{"count"},
\code{"rate"}, \code{"continuous"}).
}
\details{
The V1 likelihood/link matrix is:

\tabular{llll}{
\strong{Data type} \tab \strong{Family} \tab \strong{Likelihoods} \tab \strong{Link functions} \cr
Binary  \tab binomial \tab bernoulli (IPD), binomial (AgD)  \tab logit, probit, cloglog \cr
Count*  \tab binomial \tab bernoulli (IPD), binomial (AgD)  \tab logit, probit, cloglog \cr
Rate    \tab poisson  \tab poisson                          \tab log                    \cr
Continuous \tab normal \tab normal                           \tab identity, log          \cr
}

*\code{"count"} refers to count/total (binomial denominator) data, not Poisson
event counts. For Poisson rate or count outcomes, use \code{"poisson"} or \code{"rate"}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{dbern}
\alias{dbern}
\title{Bernoulli PMF}
\usage{
dbern(x, prob, log = FALSE)
}
\arguments{
\item{x}{Vector of values}

\item{prob}{Success probability}

\item{log}{Logical; if TRUE, return log-density}
}
\value{
Numeric vector
}
\description{
Bernoulli PMF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_summary.R
\name{.resolved_prior_broadcast_label}
\alias{.resolved_prior_broadcast_label}
\title{Format a homogeneous resolved beta prior}
\usage{
.resolved_prior_broadcast_label(br, digits)
}
\description{
Format a homogeneous resolved beta prior
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/family_config.R
\name{get_family_config}
\alias{get_family_config}
\title{Lookup helper for the family registry}
\usage{
get_family_config(family)
}
\arguments{
\item{family}{A single family string.}
}
\value{
The corresponding entry from \link{family_config}.
}
\description{
Fails fast with an informative message when a caller requests an
unregistered family, rather than returning \code{NULL} and letting a
downstream \code{$} chain produce a cryptic error.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{.bound_unit_interval}
\alias{.bound_unit_interval}
\title{Bound probabilities to the open unit interval}
\usage{
.bound_unit_interval(p)
}
\description{
Bound probabilities to the open unit interval
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{.validate_probs}
\alias{.validate_probs}
\title{Validate quantile probabilities}
\usage{
.validate_probs(probs)
}
\description{
Validate quantile probabilities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{inverse_link_derivative}
\alias{inverse_link_derivative}
\title{Derivative of inverse link with respect to linear predictor}
\usage{
inverse_link_derivative(
  eta,
  p = inverse_link(eta, link),
  link = c("logit", "probit", "cloglog")
)
}
\description{
Derivative of inverse link with respect to linear predictor
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/backend_cmdstanr.R
\name{.cmdstanr_compile_dir}
\alias{.cmdstanr_compile_dir}
\title{Cache directory for cmdstanr-compiled model executables}
\usage{
.cmdstanr_compile_dir(model_name, stan_file)
}
\description{
Cache directory for cmdstanr-compiled model executables
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.adjust_integration_cor}
\alias{.adjust_integration_cor}
\title{Adjust correlations to Gaussian-copula scale}
\usage{
.adjust_integration_cor(cor, cor_adjust, dtypes)
}
\description{
Adjust correlations to Gaussian-copula scale
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{check_integration}
\alias{check_integration}
\title{Check integration point adequacy}
\usage{
check_integration(
  data,
  ...,
  cor = NULL,
  cor_adjust = NULL,
  check_joint = TRUE,
  verbose = TRUE
)
}
\arguments{
\item{data}{An \code{mlumr_data} object with integration points}

\item{...}{Distribution specifications (same as passed to
\code{\link[=add_integration]{add_integration()}})}

\item{cor}{Correlation matrix (same as passed to \code{\link[=add_integration]{add_integration()}})}

\item{cor_adjust}{Adjustment method (same as passed to \code{\link[=add_integration]{add_integration()}})}

\item{check_joint}{If \code{TRUE} (default), also compare pairwise correlation
matrices between the current and doubled \code{n_int}, and the maximum
per-AgD-row absolute deviation from the user-supplied \code{cor}. The
pairwise comparison catches cases where marginals converge but joint
dependence structure does not (rare in practice for QMC with sensible
\code{cor_adjust} but worth flagging when \code{n_int} is small).}

\item{verbose}{Logical; if \code{FALSE}, suppresses printed diagnostic messages.}
}
\value{
A list with components \code{marginals} (the original data frame
returned by previous versions) and, if \code{check_joint = TRUE},
\code{correlations} -- a data frame of pairwise covariate correlations at
the current and doubled \code{n_int} for each AgD row. Printed with a
pass/warn verdict.
}
\description{
Compare integration results at the current \code{n_int} against a doubled
resolution to assess numerical accuracy. Large discrepancies indicate
that \code{n_int} should be increased.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.drop_missing_rows}
\alias{.drop_missing_rows}
\title{Drop rows with missing setup inputs}
\usage{
.drop_missing_rows(data, required_cols)
}
\description{
Drop rows with missing setup inputs
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_required_covariates}
\alias{.validate_required_covariates}
\title{Validate covariate argument shape before column lookup}
\usage{
.validate_required_covariates(covariates, arg_name)
}
\description{
Validate covariate argument shape before column lookup
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_sample_size}
\alias{.validate_agd_sample_size}
\title{Validate optional AgD sample sizes}
\usage{
.validate_agd_sample_size(n_vals)
}
\description{
Validate optional AgD sample sizes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{.stc_model_matrix}
\alias{.stc_model_matrix}
\title{Build a model matrix aligned with fitted GLM coefficients}
\usage{
.stc_model_matrix(fit, newdata)
}
\description{
Build a model matrix aligned with fitted GLM coefficients
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlumr.R
\name{.mlumr_log_fit_start}
\alias{.mlumr_log_fit_start}
\title{Log mlumr() fit metadata}
\usage{
.mlumr_log_fit_start(model_name, family, link, stan_data, engine, verbose)
}
\description{
Log mlumr() fit metadata
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{add_integration}
\alias{add_integration}
\title{Add numerical integration points}
\usage{
add_integration(
  data,
  n_int = 64,
  cor = NULL,
  cor_adjust = NULL,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{data}{An \code{mlumr_data} object from \code{\link[=combine_data]{combine_data()}}}

\item{n_int}{Number of integration points (default 64, powers of 2 recommended)}

\item{cor}{Correlation matrix for covariates. If \code{NULL}, computed from IPD.}

\item{cor_adjust}{Adjustment method: \code{"spearman"}, \code{"pearson"}, or \code{"none"}}

\item{verbose}{Logical; if \code{FALSE}, suppresses progress messages.}

\item{...}{Distribution specifications for each covariate using \code{\link[=distr]{distr()}}}
}
\value{
An \code{mlumr_data} object with integration points added
}
\description{
Generate quasi-Monte Carlo integration points using Sobol sequences and a
Gaussian copula to account for correlations between covariates in the AgD.
}
\examples{
\dontrun{
dat <- add_integration(
  dat,
  n_int = 64,
  x1 = distr(qnorm, mean = x1_mean, sd = x1_sd),
  x2 = distr(qbern, prob = x2_mean)
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.agd_covariate_spec}
\alias{.agd_covariate_spec}
\title{Normalize AgD covariate specification}
\usage{
.agd_covariate_spec(cov_means, cov_sds = NULL, cov_types = NULL)
}
\description{
Normalize AgD covariate specification
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_effects.R
\name{conditional_effects}
\alias{conditional_effects}
\title{Conditional treatment effects}
\usage{
conditional_effects(
  object,
  newdata = NULL,
  effect = "all",
  summary = TRUE,
  probs = c(0.025, 0.5, 0.975)
)
}
\arguments{
\item{object}{An \code{mlumr_fit} object}

\item{newdata}{Data frame of covariate values at which to compute effects.
Each row defines one covariate profile. Column names must match the
covariates used in fitting. If \code{NULL} (default), uses the covariate means
from the IPD as a single reference profile.}

\item{effect}{Which effect measure. For binomial: \code{"all"}, \code{"link_effect"},
\code{"rd"}, or \code{"rr"}. For normal: \code{"all"} or \code{"md"}. For Poisson: \code{"all"}
or \code{"rr"}. The legacy value \code{"lor"} is accepted as an alias for
\code{"link_effect"} when the fitted link is logit.}

\item{summary}{Return summary statistics (\code{TRUE}) or full posterior draws
(\code{FALSE})}

\item{probs}{Quantiles for summary (default \code{c(0.025, 0.5, 0.975)})}
}
\value{
A data frame. If \code{summary = TRUE}, contains columns \code{profile},
\code{effect}, \code{mean}, \code{sd}, and quantile columns. If \code{summary = FALSE},
returns a single combined data frame of full posterior draws with a
\code{profile} column indicating which covariate profile each draw belongs
to.
}
\description{
Compute conditional (individual-level) treatment effects at specific
covariate values from a fitted ML-UMR model. Unlike \code{\link[=marginal_effects]{marginal_effects()}},
which averages over a population's covariate distribution, conditional
effects evaluate the treatment effect at a particular covariate profile.
}
\details{
For SPFA models, the conditional link-scale treatment effect is constant
across all covariate values because the shared beta cancels in the treatment
contrast on the fitted link scale. However, risk difference (RD) and risk
ratio (RR) still vary with covariates because they depend on absolute
probability levels. For relaxed models, all conditional effects vary with
covariate values because the index and comparator treatments have different
regression coefficients.

The conditional link-scale effect is computed directly as
\code{eta_index - eta_comparator}. This avoids numerical distortion from
transforming extreme response-scale probabilities back through the link
function.

\strong{Conditional vs marginal on non-identity links.} Conditional effects are
evaluated at a single covariate profile, so there is no averaging over a
population and no Jensen's-inequality gap between the conditional and
marginal response. Compare with \code{\link[=marginal_effects]{marginal_effects()}} and
\code{\link[=predict.mlumr_fit]{predict.mlumr_fit()}}, which average over either the IPD individuals
(index population) or the AgD integration points (comparator population)
and therefore return \code{E[g^{-1}(eta)]}, not \code{g^{-1}(E[eta])}.
}
\examples{
\dontrun{
# Conditional effects at IPD covariate means (default)
conditional_effects(fit)

# At specific covariate values
conditional_effects(fit, newdata = data.frame(age = 60, sex = 1))

# Multiple profiles
profiles <- data.frame(age = c(50, 60, 70), sex = c(0, 0, 1))
conditional_effects(fit, newdata = profiles)
}
}
\seealso{
\code{\link[=marginal_effects]{marginal_effects()}} for population-averaged treatment effects;
\code{\link[=conditional_predict]{conditional_predict()}} for absolute predictions at specific profiles;
\code{\link[=predict.mlumr_fit]{predict.mlumr_fit()}} for population-level predictions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\name{.supported_engines}
\alias{.supported_engines}
\title{Supported Stan engines}
\usage{
.supported_engines()
}
\description{
Supported Stan engines
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_sensitivity.R
\name{.summarize_sensitivity}
\alias{.summarize_sensitivity}
\title{Summarize a sensitivity refit.}
\usage{
.summarize_sensitivity(fit, scale, probs)
}
\description{
Summarize a sensitivity refit.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.validate_conf_level}
\alias{.validate_conf_level}
\title{Validate a confidence level}
\usage{
.validate_conf_level(conf_level)
}
\description{
Validate a confidence level
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_complete_rows_remain}
\alias{.validate_complete_rows_remain}
\title{Validate that complete-case filtering left rows to analyze}
\usage{
.validate_complete_rows_remain(data, label)
}
\description{
Validate that complete-case filtering left rows to analyze
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\name{.message_engine_unchanged}
\alias{.message_engine_unchanged}
\title{Report that the current engine was not changed}
\usage{
.message_engine_unchanged()
}
\description{
Report that the current engine was not changed
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_normal_outcomes}
\alias{.validate_agd_normal_outcomes}
\title{Validate normal AgD outcomes}
\usage{
.validate_agd_normal_outcomes(y_vals, se_vals)
}
\description{
Validate normal AgD outcomes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_effects.R
\name{.safe_positive_ratio}
\alias{.safe_positive_ratio}
\title{Ratio with the same positive-denominator guard used by Stan generated quantities}
\usage{
.safe_positive_ratio(num, denom)
}
\description{
Ratio with the same positive-denominator guard used by Stan generated quantities
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.z_from_conf_level}
\alias{.z_from_conf_level}
\title{Convert a confidence level to a two-sided normal critical value}
\usage{
.z_from_conf_level(conf_level)
}
\description{
Convert a confidence level to a two-sided normal critical value
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.comparison_names}
\alias{.comparison_names}
\title{Use user-supplied comparison names when available}
\usage{
.comparison_names(models, fallback)
}
\description{
Use user-supplied comparison names when available
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{distr}
\alias{distr}
\title{Specify a marginal distribution}
\usage{
distr(qfun, ...)
}
\arguments{
\item{qfun}{Inverse CDF function (e.g., \code{qnorm}, \code{qbern})}

\item{...}{Parameters of the distribution, can reference column names in AgD}
}
\value{
A list with class \code{"mlumr_distr"} containing the distribution specification
}
\description{
Used to specify marginal distributions for covariates when adding integration
points. Wraps an inverse CDF (quantile) function with its parameters.
}
\examples{
# Normal distribution
distr(qnorm, mean = 0, sd = 1)

# Bernoulli distribution with probability 0.3
distr(qbern, prob = 0.3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_covariate_names}
\alias{.validate_agd_covariate_names}
\title{Validate AgD covariate names after suffix stripping}
\usage{
.validate_agd_covariate_names(cov_means)
}
\description{
Validate AgD covariate names after suffix stripping
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{extract_log_lik}
\alias{extract_log_lik}
\title{Extract the full pointwise log-likelihood matrix from an mlumr_fit}
\usage{
extract_log_lik(object)
}
\arguments{
\item{object}{An \code{mlumr_fit} object.}
}
\value{
A numeric matrix of dimension \verb{n_draws x (n_ipd + n_agd_rows)}.
IPD columns come first, then AgD rows.
}
\description{
Combines the IPD and AgD per-observation log-likelihood draws into a
single matrix with one row per posterior draw and one column per
observation. The result is suitable for direct use with
\code{\link[loo:loo]{loo::loo()}} / \code{\link[loo:waic]{loo::waic()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{unnest_integration}
\alias{unnest_integration}
\title{Expand integration points into a long-format data frame}
\usage{
unnest_integration(data)
}
\arguments{
\item{data}{An \code{mlumr_data} object with integration points}
}
\value{
A data frame with columns for each covariate plus \code{.int_id} and \code{.agd_row}
}
\description{
Expand integration points into a long-format data frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{.validate_link_string}
\alias{.validate_link_string}
\title{Validate a scalar link/family string}
\usage{
.validate_link_string(x, name)
}
\description{
Validate a scalar link/family string
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{link_derivative_response}
\alias{link_derivative_response}
\title{Derivative of a binomial link with respect to probability}
\usage{
link_derivative_response(p, link = c("logit", "probit", "cloglog"))
}
\description{
Derivative of a binomial link with respect to probability
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numerical_guards.R
\name{mlumr-numerical-guards}
\alias{mlumr-numerical-guards}
\alias{safe_logit}
\alias{safe_divide}
\title{Numerical guards in the Stan models}
\description{
\code{mlumr}'s Stan models clip a few quantities to avoid undefined arithmetic
in \verb{generated quantities}. This page documents the exact behavior so
users can assess when the guard matters for their analysis.
}
\section{Guards currently in place}{


\describe{
\item{\code{safe_logit(p)} (binomial models)}{Returns
\code{logit(clamp(p, 1e-10, 1 - 1e-10))}. The logit and log-odds-ratio
posterior summaries use this rather than the raw \code{logit(p)} so that
finite samples with \code{p} at the boundaries do not propagate
\code{Inf}/\code{NaN} into the posterior.}
\item{\code{safe_divide(num, denom)} (binomial and Poisson models)}{Returns
\code{num / max(denom, 1e-10)}. Used for risk ratios and rate ratios to
avoid division-by-zero when a comparator posterior draw has a
near-zero mean outcome.}
\item{Stan \verb{<lower=0>} constraints on \code{sigma} (normal family)}{Truncate
the prior to the positive half-line — the reason why
\code{prior_sigma = prior_normal(0, 2.5)} is interpreted as a
half-normal and why \code{\link[=prior_exponential]{prior_exponential()}} is also valid.}
}
}

\section{When the guards introduce bias}{


The guards are one-sided: they replace invalid values with extreme but
finite ones rather than with a symmetric distribution. In practice:

\itemize{
\item For binomial outcomes with event rates in the range
\verb{[1e-4, 1 - 1e-4]} the clipped output is indistinguishable from the
unclipped value at double precision.
\item For event rates outside that range — or Poisson models with
expected rates below \code{1e-4} events per unit exposure — the summary
of the posterior for LOR, RR, and related ratios may be biased
toward the clip boundary. The point estimate for the treatment
effect on the link scale (\code{delta_index} on log-odds, log-rate, or
mean-difference scale) is \emph{not} affected since it does not
go through the guards.
\item \strong{Direction of \code{safe_divide} bias.} When the comparator rate
(the denominator) is smaller than \code{1e-10}, the ratio is clipped
to \code{num / 1e-10}. Because rates and probabilities in mlumr are
non-negative, the clipped value is always \emph{larger} (more
positive) than the mathematical limit — so risk ratios and rate
ratios in near-zero-comparator regimes are \emph{upward-biased},
never downward. Expect the clipped posterior summary to
overstate, not understate, the relative treatment effect in
those draws.
\item For very-rare-event data, consider reframing the model in terms
of rates: an event count with exposure via \code{\link[=set_agd]{set_agd()}} \code{family = "poisson"} is numerically better behaved than Bernoulli /
Binomial at extreme probabilities.
}
}

\section{How to diagnose guard activation}{


Posterior draws of \verb{p_*_index} / \verb{p_*_comparator} or \verb{rate_*_*} are
returned by the Stan models and are available in \code{fit$draws}. If any
posterior draw sits below \code{5e-10} or above \code{1 - 5e-10} for binomial
probabilities (respectively below \code{5e-10} for Poisson rates), the
guards are active for that draw. A quick check:
\preformatted{
  any(fit$draws[, "p_comparator_comparator"] < 5e-10)
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engine.R
\name{mlumr_engine}
\alias{mlumr_engine}
\title{Get or Set the Stan Engine}
\usage{
mlumr_engine(engine = NULL)
}
\arguments{
\item{engine}{Character string: \code{"rstan"} or \code{"cmdstanr"}. If \code{NULL}
(default), returns the current engine without changing it.}
}
\value{
The current engine (character), returned invisibly when setting.
}
\description{
Control which Stan backend mlumr uses for model fitting. The default engine
is \code{"rstan"} (compiled C++ models via rstantools). Users who prefer cmdstanr
can switch engines after installation.
}
\details{
When switching to \code{"cmdstanr"}, this function checks whether the cmdstanr
package and CmdStan toolchain are installed. If either is missing, it offers
to install them interactively.

Engine names must be matched exactly. Partial strings such as \code{"c"} are not
accepted.

The engine preference is stored as \code{options(mlumr.stan_engine = ...)} and
persists for the current R session. To set a permanent default, add to your
\code{.Rprofile}:

\if{html}{\out{<div class="sourceCode">}}\preformatted{options(mlumr.stan_engine = "cmdstanr")
}\if{html}{\out{</div>}}
}
\examples{
# Check current engine
mlumr_engine()

# Switch to cmdstanr (interactive)
\dontrun{
mlumr_engine("cmdstanr")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/link_functions.R
\name{inverse_link}
\alias{inverse_link}
\title{Inverse link function (linear predictor -> response scale)}
\usage{
inverse_link(x, link = c("identity", "log", "logit", "probit", "cloglog"))
}
\arguments{
\item{x}{Numeric vector}

\item{link}{Character: link function name}
}
\value{
Numeric vector on response scale
}
\description{
Inverse link function (linear predictor -> response scale)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.diagnostic_value}
\alias{.diagnostic_value}
\title{Format a diagnostic setting for warning messages}
\usage{
.diagnostic_value(x)
}
\description{
Format a diagnostic setting for warning messages
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.warn_integration_size}
\alias{.warn_integration_size}
\title{Warn when integration resolution is low for the covariate dimension}
\usage{
.warn_integration_size(n_int, n_cov)
}
\description{
Warn when integration resolution is low for the covariate dimension
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/naive.R
\name{.naive_poisson}
\alias{.naive_poisson}
\title{Naive comparison for Poisson outcomes}
\usage{
.naive_poisson(data, ipd, agd, conf_level, z)
}
\description{
Naive comparison for Poisson outcomes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{calculate_waic}
\alias{calculate_waic}
\title{Calculate WAIC for an mlumr_fit}
\usage{
calculate_waic(object, ...)
}
\arguments{
\item{object}{An \code{mlumr_fit} object.}

\item{...}{Additional arguments passed to \code{\link[loo:waic]{loo::waic()}}.}
}
\value{
An object of class \code{waic} (see \code{\link[loo:waic]{loo::waic()}}).
}
\description{
Watanabe-Akaike Information Criterion (Watanabe 2010) based on the
pointwise log-likelihoods. WAIC is asymptotically equivalent to
LOO-CV; prefer \code{\link[=calculate_loo]{calculate_loo()}} when Pareto-k is well-behaved.
}
\note{
As with \code{\link[=calculate_loo]{calculate_loo()}}, each AgD row is treated as an
independent observation. WAIC will be optimistic for AgD rows
that share a study (see the note on \code{calculate_loo()}).
}
\examples{
\dontrun{
waic_spfa <- calculate_waic(fit_spfa)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_agd_cov_types}
\alias{.validate_agd_cov_types}
\title{Validate AgD covariate type labels}
\usage{
.validate_agd_cov_types(cov_types)
}
\description{
Validate AgD covariate type labels
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.validate_mlumr_data_object}
\alias{.validate_mlumr_data_object}
\title{Validate an mlumr_data object}
\usage{
.validate_mlumr_data_object(data)
}
\description{
Validate an mlumr_data object
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{marginal_effects}
\alias{marginal_effects}
\title{Marginal treatment effects}
\usage{
marginal_effects(
  object,
  population = c("both", "index", "comparator"),
  effect = "all",
  summary = TRUE,
  probs = c(0.025, 0.5, 0.975)
)
}
\arguments{
\item{object}{An \code{mlumr_fit} object}

\item{population}{Which population: \code{"both"}, \code{"index"}, or \code{"comparator"}}

\item{effect}{Which effect measure. For binomial: \code{"all"}, \code{"lor"}, \code{"rd"},
or \code{"rr"}. For normal: \code{"all"} or \code{"md"} (mean difference). For poisson:
\code{"all"} or \code{"rr"} (rate ratio).}

\item{summary}{Return summary (\code{TRUE}) or full draws (\code{FALSE})}

\item{probs}{Quantiles for summary}
}
\value{
A data frame
}
\description{
Extract marginal treatment effects from a fitted ML-UMR model. For
binomial: log odds ratio, risk difference, risk ratio. For normal:
mean difference. For poisson: rate ratio.
}
\examples{
\dontrun{
# All effect measures for both populations
marginal_effects(fit)

# Only the log odds ratio in the index population
marginal_effects(fit, population = "index", effect = "lor")

# Full posterior draws rather than summary statistics
marginal_effects(fit, summary = FALSE)
}
}
\seealso{
\code{\link[=predict.mlumr_fit]{predict.mlumr_fit()}} for absolute predictions;
\code{\link[=conditional_effects]{conditional_effects()}} for covariate-conditional effects at specific
profiles; \code{\link[=prior_sensitivity]{prior_sensitivity()}} to check how strongly the marginal
effect depends on \code{prior_beta}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.standardize_agd_data}
\alias{.standardize_agd_data}
\title{Standardize AgD to mlumr's internal column contract}
\usage{
.standardize_agd_data(
  data,
  treatment,
  family,
  outcome_n,
  outcome_r,
  outcome_mean,
  outcome_se,
  outcome_E,
  cov_means,
  cov_sds,
  cov_names,
  study = NULL
)
}
\description{
Standardize AgD to mlumr's internal column contract
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/integration.R
\name{.validate_integration_args}
\alias{.validate_integration_args}
\title{Validate top-level integration arguments}
\usage{
.validate_integration_args(data, n_int, cor_adjust, verbose, ds)
}
\description{
Validate top-level integration arguments
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.validate_ipd_outcome}
\alias{.validate_ipd_outcome}
\title{Validate IPD outcome and exposure columns}
\usage{
.validate_ipd_outcome(data, outcome, family, exposure = NULL)
}
\description{
Validate IPD outcome and exposure columns
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.relative_eff_from_log_lik}
\alias{.relative_eff_from_log_lik}
\title{Compute stable relative effective sample sizes from log-likelihoods}
\usage{
.relative_eff_from_log_lik(log_lik, chain_id)
}
\description{
Compute stable relative effective sample sizes from log-likelihoods
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_effects.R
\name{.conditional_eta}
\alias{.conditional_eta}
\title{Compute conditional linear predictors for one profile}
\usage{
.conditional_eta(params, x)
}
\description{
Compute conditional linear predictors for one profile
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{.finite_numeric_values}
\alias{.finite_numeric_values}
\title{Return finite numeric values from a summary column}
\usage{
.finite_numeric_values(x)
}
\description{
Return finite numeric values from a summary column
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{stc}
\alias{stc}
\title{Simulated treatment comparison via G-computation}
\usage{
stc(data, link = NULL, conf_level = 0.95)
}
\arguments{
\item{data}{An \code{mlumr_data} object from \code{\link[=combine_data]{combine_data()}}. Integration points
are not required for STC but covariate information from the AgD is used.}

\item{link}{Link function. For binomial: \code{"logit"} (default), \code{"probit"},
or \code{"cloglog"}. For normal: \code{"identity"} (default) or \code{"log"}. For
poisson: \code{"log"} (default). If \code{NULL}, uses the canonical default.}

\item{conf_level}{Confidence level for the interval (default 0.95)}
}
\value{
An object of class \code{mlumr_stc}
}
\description{
Perform an unanchored simulated treatment comparison (STC) using parametric
G-computation. Fits a regression on IPD, predicts counterfactual
outcomes in both the index and comparator populations, and computes
marginal treatment effects with delta-method standard errors.
}
\details{
For binomial outcomes, returns the treatment effect on the link scale plus
event probabilities, risk difference, and log risk ratio with SEs and CIs
for both populations. Event-probability intervals use Wald standard errors
and are bounded to \verb{[0, 1]}. For Poisson outcomes, the comparator log rate
uses a 0.5 continuity correction when the observed event count is zero.

The STC procedure is:
\enumerate{
\item Fit a GLM on IPD (binomial/gaussian/poisson as appropriate).
\item Predict on comparator-population covariates (from integration points or
AgD covariate means).
\item Marginalize predictions over the comparator population.
\item Predict on index-population covariates (IPD).
\item Compute treatment effects and SEs via the delta method.
}

STC is a parametric G-computation benchmark. It relies on the IPD outcome
model being correctly specified and transportable to the comparator
population. It does not model posterior uncertainty in population
covariate distributions or relax treatment-specific covariate effects.
When clinically meaningful effect modification is plausible, prefer
\code{mlumr(..., model = "relaxed")} as the primary analysis and use STC as a
sensitivity or benchmarking analysis.

For binomial outcomes, the comparator-population treatment contrast is
transported to the index population \strong{assuming the treatment contrast is
constant on the fitted link scale} (i.e., no effect modification on that
scale). Under this assumption, the index-population comparator
probability is computed as
\code{inv_link(link(p_A_index) - (link(p_A_comp) - link(p_B)))}, and its
uncertainty is propagated through the delta method. If effect modification
is expected, fit a Bayesian relaxed model with
\code{mlumr(..., model = "relaxed")} and use \code{predict(..., population = "index")} instead, which does not require this assumption.
}
\examples{
\dontrun{
result <- stc(dat)
print(result)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prior_sensitivity.R
\name{.rescale_prior_beta}
\alias{.rescale_prior_beta}
\title{Rescale a prior_beta's scale, preserving family / mean / df.}
\usage{
.rescale_prior_beta(prior, new_scale)
}
\description{
Rescale a prior_beta's scale, preserving family / mean / df.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predict.R
\name{.compute_mean_lp}
\alias{.compute_mean_lp}
\title{Compute mean linear predictors from parameter draws}
\usage{
.compute_mean_lp(object, pred_cols)
}
\arguments{
\item{object}{An mlumr_fit object}

\item{pred_cols}{Character vector of column names to compute}
}
\value{
Data frame with the same columns as pred_cols, on the link scale
}
\description{
Internal helper for predict.mlumr_fit with type="link". Computes
mean(eta_i) directly from parameter draws instead of applying the link
to marginalized response-scale draws, avoiding Jensen's inequality bias.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.agd_required_columns}
\alias{.agd_required_columns}
\title{AgD source columns required for setup}
\usage{
.agd_required_columns(
  treatment,
  cov_means,
  cov_sds,
  outcome_n = NULL,
  outcome_r = NULL,
  outcome_mean = NULL,
  outcome_se = NULL,
  outcome_E = NULL,
  study = NULL
)
}
\description{
AgD source columns required for setup
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{check_diagnostics}
\alias{check_diagnostics}
\title{Check MCMC diagnostics and warn if issues found}
\usage{
check_diagnostics(fit)
}
\arguments{
\item{fit}{An \code{mlumr_fit} object}
}
\description{
Check MCMC diagnostics and warn if issues found
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diagnostics.R
\name{calculate_loo}
\alias{calculate_loo}
\title{Calculate LOO-CV for an mlumr_fit}
\usage{
calculate_loo(object, ...)
}
\arguments{
\item{object}{An \code{mlumr_fit} object.}

\item{...}{Additional arguments passed to \code{\link[loo:loo.array]{loo::loo.array()}}.}
}
\value{
An object of class \code{psis_loo} (see \code{\link[loo:loo]{loo::loo()}}).
}
\description{
Computes approximate leave-one-out cross-validation (PSIS-LOO, Vehtari,
Gelman, Gabry 2017) using the pointwise log-likelihoods stored by the
Stan models. Returns a \code{loo} object from the \code{loo} package.
}
\details{
Pareto-k diagnostics: values > 0.7 indicate observations for which the
PSIS approximation is unreliable; the printed output flags these.
Typical remedies are running more iterations, using \code{moment_match = TRUE},
or (for highly influential AgD rows) refitting without the offending
observation to check sensitivity.
}
\note{
\strong{AgD rows are treated as independent observations.} Each AgD row
contributes one column to the pointwise \code{log_lik} matrix. If two or
more AgD rows come from the same study (e.g. subgroup summaries
within a single trial) the PSIS-LOO approximation does not account
for the within-study clustering; effective sample sizes are
inflated and Pareto-k warnings are understated. For clustered AgD,
corroborate with \code{\link[=prior_sensitivity]{prior_sensitivity()}} or refit omitting suspect
rows to check the influence on the posterior.
}
\examples{
\dontrun{
loo_spfa <- calculate_loo(fit_spfa)
print(loo_spfa)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.is_whole_number_count}
\alias{.is_whole_number_count}
\title{Check whether values are valid non-negative whole-number counts}
\usage{
.is_whole_number_count(x, allow_missing = FALSE)
}
\description{
Check whether values are valid non-negative whole-number counts
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stc.R
\name{.stc_normal}
\alias{.stc_normal}
\title{Normal-outcome STC estimator}
\usage{
.stc_normal(
  data,
  fit,
  ipd,
  agd,
  newdata,
  link_resolved,
  conf_level,
  z,
  beta_hat,
  V,
  n_int
)
}
\description{
Normal-outcome STC estimator
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_setup.R
\name{.ipd_outcome_summary}
\alias{.ipd_outcome_summary}
\title{Summarize standardized IPD outcomes}
\usage{
.ipd_outcome_summary(ipd_data, family)
}
\description{
Summarize standardized IPD outcomes
}
\keyword{internal}
